#!/usr/bin/wish

# Cosmo.tcl 
#
# John Doroshenko    Rutgers Univ., Physics   732-445-5034
# doroshenko@physics.rutgers.edu
# Bob Stone  stone@physics.rutgers.edu
#
# cosmo81 is first version with persistent i2c helper for faster speed

# internal version cosmo100.tcl -> version 1.1.0
#                                            100
#
# cosmo84.tcl -> cosmo100.tcl  Includes burst mode in ROC Init
#

#package require BLT

# cosmo200 was first released version with psi46 support under both win
# and linux
#
# cosmo201 will provide script hooks for Single Event Upset testing
#
# cosmo300 is a pci fec version derived from 2.5.2-p2
#


# USB 1st version begins in devel at 1.7.0
set softwareversion 3.5.4
# 3.0.26 still uses byte swapping

# Version 3.5.4 +  Collected some globals and moved to front.  Added auto
# script execution to allow resetting these.

# PCTARGET devel or release -- devel selects fake adc on my devel pc
set PCTARGET release
# Should be able to test for this, but let's set it here hard (linux | windows)
set OS linux

set slowfecchannel 1

set dogui 1

set usevme 1
    set vmeslot 6
    set mfec 8
set clkph 0

set fec_trigger_source 1
# 1-internal  0-external

#usbinit /dev/ttyUSB0

# --------------------------------------------------------------------
# Global declarations block - can be overwritten later via 
# /usr/local/cosmo/bin/cosmoparams.tcl
#
set adcloaded 0
set adcfileopen 0
set doutformat "show"

set lasthubaddress -1
set lastport -1
set iii -1
# COSMOSCRIPT 1 or 0 - control first revision of server
set COSMOSCRIPT 0

#set COSMOTCPSCRIPT 1 - control 2nd revision of server
set COSMOTCPSCRIPT 1

# init state of server
set serverstarted -1

#helpavailable - if 1 provide development help system 1 or 0
set helpavailable 0

set interface "usb"

set calibratelatency 139
set callatencybit enable
set FECSDA 2




# --------------------------------------------------------------------

#puts "Checking named extension loading"
#load "/home/doroshen/tbm/example.so"
#puts "DONE Loading extension"


#puts "Trying to load adc module via swig routine and stubs"
#set iii [setupcard 1 1 1 1]
#puts "Returned $iii"

#puts "testing mktcl created function"
#set iii [addup 5 6]
#puts "Returned $iii"

#puts "init of cyber card..."
#set iii [cyberinit 1]
#puts "returned $iii"

#puts "grab an event..."
#set ii [cybergetevent 240 500 1]
#puts "returned $iii"

#after 4000

#set roctype "psi43"
#set roctype "psi46"
set roctype "psi46"

if {$roctype == "psi43"} {
    set maxcols 52
    set maxrows 53
} elseif {$roctype == "psi46"} {
    set maxcols 52
    set maxrows 80
} else {
    puts "\nERROR: UNKNOWN ROC TYPE = $roctype\n"
    exit
}


# seu testing verbose stack count readbacks
set specialtest1 0


# added for PSI46v2, but control present in psi46v1, 1=0-1800mV,0=0-280mV
set pixcalrange 0

# ----------------------------------------------------------------------
# XML Routines begin

#########################################################################
#									#
# Script:	sxml.tcl (namespace sxml)				#
#  								        #
# Started:	20th October, 2000.					#
#									#
# Completed:	5th November, 2000.					#
#									#
# Author:	Simon Edwards, (C) Proprius Consulting Ltd		#
#									#
# Provides:	sxml namespace (init end register_routine parse)	#
#									#
#		These commands provide an event driven method of	#
#		parsing simple XML files.				#
#									#
# Credits:	Various sources on the web, such as devshed, where	#
#		I've learn't a little about XML.			#
#									#
# Limitations:	This is the initial version - it offers only basic	#
#		parsing functionality, though is good enough, if slow.	#
#									#
# Version:	@(#)0.8.0 Initially released version. (SE)>		#
# 		@(#)0.8.1 Empty element parse error resolved (SE)>	#
# 		@(#)0.8.2 Procedure args, CDATA support (SE)>		#
#									#
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - #
#									#
# Version:	0.8.2							#
# Date:		7th August, 2001.					#
# Author:	Simon Edwards, Proprius Consulting Ltd.			#
# Change:	Added support for optional command line arguments to	#
#		pass to a registered routine (Thanks to Jim Garrison 	#
#		(garrison@qualcomm.com).				#
#		Added support for CDATA attributes meaning that these	#
#		routines can handle "binary" data.			#
#		It expects the start of the block to have the following	#
#		definition:						#
#									#
#		<![CDATA[						#
#		At the end of the block the following definition is	#
#		expected:						#
#									#
#		]]>							#
#									#
#		If you need to include that sequence of characters in	#
#		the data then the following should be used instead:	#
#									#
#		]]&gt;							#
#									#
#		WARNING:	The details between the deliminators	#
#				for CDATA mean that EVERY BYTE is 	#
#				including in the element. This 		#
#				is as the XML specification (at least	#
#				as far as my basic understanding).	#
#									#
#		Support has also been added for the set_attr function	#
#		which is an interface to be able to set various 	#
#		functions which will affect the parsing of files.	#
#		The attributes are particular to each file to parse	#
#		and need to be set using the parse descriptor for a	#
#		particular file. All attributes are given suitable 	#
#		defaults.						#
#									#
#		Attribute	Values	Purpose				#
#		=========	======	=============================== #
#		trace		0/1/2	Whether to show trace info	#
#					when parsing a particular file.	#
#					Default is 0 (no).		#
#					2 Is the highest level of trace	#
#					whilst 1 gives more details 	#
#					than most people want anyway!	#
#		silent		0/1	Whether or not any errors 	#
#					found during the parse are sent	#
#					to the "stderr" channel. 	#
#					default is 0 (show errors).	#
#		pedantic	0/1	Whether to check when a proc is	#
#					registered whether it actually	#
#					exists and has the correct	#
#					number of arguments. Default is	#
#					0 (not pedantic).		#
#		extended	0/1	If set to 1 will look to see if	#
#					a event has been registered for	#
#					a "wildcard" at the latest	#
#					level of a element. Slower, but	#
#					allows improved parsing. Default#
#					is 0.				#
#									#
# Version:	0.8.1							#
# Date:		26th November, 2000.					#
# Author:	Simon Edwards, Proprius Consulting Ltd.			#
# Change:	Corrected assumption that an empty element tag		#
#		cannot have attributes, so now correctly handles	#
#		cases such as:						#
#		<testelement name="fred"/>				#
#									#
#########################################################################

###############################################################################
#    sxml.tcl - tcl module to provide basis XML parsing			      #
#    Copyright (C) 2000-2001 Simon Edwards, Proprius Consulting Ltd           #
#                                                                             #
#    This program is free software; you can redistribute it and/or modify     #
#    it under the terms of the GNU General Public License as published by     #
#    the Free Software Foundation; either version 2 of the License, or        #
#    (at your option) any later version.                                      #
#                                                                             #
#    This program is distributed in the hope that it will be useful,          #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of           #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
#    GNU General Public License for more details.                             #
#                                                                             #
#    You should have received a copy of the GNU General Public License        #
#    along with this program; if not, write to the Free Software              #
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA  #
#                                                                             #
#    If you wish to contact the author the following details can be used;     #
#                                                                             #
#    simon.edwards@proprius.co.uk                                             #
###############################################################################

namespace eval sxml {

namespace export init end register_routine parse set_attr

variable xml_invoc
variable xml_stack
variable xml_file
variable xml_procs
variable xml_hadtoplevel
variable xml_cdata_parse
variable xml_cdata_start
variable xml_cdata_end
variable xml_cdata_sub
variable xml_attrs

set xml_cdata_start {<![CDATA[}
set xml_cdata_end {]]>}
set xml_invoc 0

proc init {file} {
variable xml_invoc
variable xml_stack
variable xml_file
variable xml_procs
variable xml_attrs_stack
variable xml_hadtoplevel
variable xml_cdata_parse
variable xml_attrs

	if { [catch {set fd [open $file]}]  } {
		return -1
	}
	incr xml_invoc
	set xml_cdata_parse($xml_invoc) 0
	set xml_stack($xml_invoc) {}
	set xml_procs($xml_invoc) {}
	set xml_file($xml_invoc) $fd
	set xml_hadtoplevel($xml_invoc) 0

	#################################################################
	# Default the attributes currently supported.			#
	#################################################################
	set xml_attrs(${xml_invoc}_trace) 0
	set xml_attrs(${xml_invoc}_silent) 0
	set xml_attrs(${xml_invoc}_pedantic) 0
	set xml_attrs(${xml_invoc}_extended) 0

	return $xml_invoc
}

proc set_attr {id attr val} {
variable xml_attrs
variable xml_stack

	if { ! [info exists xml_stack(${id})] } {
		return -1
	}
	if { ! [info exists xml_attrs(${id}_$attr)] } {
		return -2
	}
	set xml_attrs(${id}_$attr) $val
	return 0
}

proc end {id} {
variable xml_stack
variable xml_file
variable xml_procs
variable xml_attrs_stack
variable xml_cdata_parse
variable xml_attrs

	if { ! [info exists xml_stack($id)] } {
		return -1
	}
	catch {close $xml_file($id)}
	unset xml_file($id)
	unset xml_stack($id)
	unset xml_procs($id)
	unset xml_cdata_parse($id)

	unset xml_attrs(${id}_trace)
	unset xml_attrs(${id}_silent)
	unset xml_attrs(${id}_pedantic)
	unset xml_attrs(${id}_extended)
	return 0
}

#########################################################################
# Construct an array containing all attributes details for all          #
# sub-elements in the current stack.                                    #
#########################################################################

proc xml_construct_attr_list {stack myarr} {
variable xml_attrs_stack
upvar $myarr xx

	catch {unset xx}
	set xx(_dummy_) ""
	if { ! [info exists xml_attrs_stack] } {
		return 0
	}

	foreach xxx [array names xml_attrs_stack "$stack:*"] {
		set xx($xxx) $xml_attrs_stack($xxx)
	}
	return 0
}

proc xml_construct_data_list {stack myarr} {
variable xml_data_stack
upvar $myarr xx

	catch {unset xx}
	set xx(_dummy_) ""

	if { ! [info exists xml_data_stack]} {
		return
	}
	foreach xxx [array names xml_data_stack "$stack:*"] {
		set xx($xxx) $xml_data_stack($xxx)
	}
}

proc register_routine {id path proc args} {
variable xml_stack
variable xml_procs
variable xml_attrs

	if { ! [info exists xml_stack($id)]} {
		return -1
	}
	#################################################################
	# We should also make sure that the specifed routine already	#
	# exists, and has the correct number of arguments.		#
	# But only if pedantic has been turned on...			#
	#################################################################

	if { $xml_attrs(${id}_pedantic) == 1 } {
		set valid [info procs $proc]
		if { "$valid" == "" } {
			if { $xml_attrs(${id}_silent) == 0 } {
				puts stderr "Error: Specified procedure to register \"$proc\" is not defined."
			}
			return -2
		}
		set args [info args $proc]
		if { [llength $args] != 6 } {
			if { $xml_attrs(${id}_silent) == 0 } {
				puts stderr "Error: Specified procedure to register \"$proc\" does not have valid argument count."
			}
			return -3
		}
	}

	set path [string tolower $path]
	set x [lsearch -exact $xml_procs($id) $path]
	if { $x !=-1 } {
		set xml_procs($id) [lreplace $xml_procs($id) $x [expr $x + 1] $path $proc]
		return 0
	}

	lappend xml_procs($id) $path $proc $args
	return 0
}

proc xml_tag_attrs_to_str {attrstr empty_t incdata} {
upvar $empty_t empty_tag

	set empty_tag 0
	set oattr $attrstr
	set attrs {}
	set x 1
	while {1} {
		set x [regexp { ([[:alnum:]\-_]+)="([^"]*)"} $attrstr junk a1 a2]
		if {$x == 0} break
		if { ! [validate_name $a1] } {
			puts stderr "Error: Invalid attribute name: $a1"
			return -code error 1
		}
		set a2 [xml_handle_bs_string $a2 $incdata]
		lappend attrs $a1 $a2
		regsub {([[:alnum:]\-_]+)="([^"]*)"} $attrstr {} xx
		set attrstr $xx
	}

	if { "[string trim "$attrstr"]" == "/" } {
		set empty_tag 1
	} elseif {[llength $attrstr] != 0} {
		puts stderr "Malformed attributes: $attrstr found in:\n$oattr"
		return -code error 1
	}
	return $attrs
}

#########################################################################
# The function below is used to valid the format of attribute and	#
# element names. These must begin with a letter or underscore and	#
# can consist of letters, underscore, dash and numbers only.		#
# Will return 1 if name is valid, 0 otherwise.				#
#########################################################################

proc validate_name {n} {
	set x [regexp {^[_\-[:alpha:]][_\-[:alnum:]]*} $n matched]
	if { $x == 0 } {
		return 0
	}
	if { "$matched" == "$n" } {
		return 1
	}
	return 0
}

#########################################################################
#									#
# The below function, when called with the current position and the	#
# being interpretted will return the character to place on the string	#
# and the pointer will be updated to the next character position.	#
#									#
# This routine handles escape characters and & special symbols.		#
#									#
# 1.8.2		When this routine is called it is now passed the	#
#		value of xml_cdata_parse as well. If set none of the	#
#		standard substitutions are performed. However the 	#
#		special CDATA case of "]]&gt;" will change to		#
#		"]]>" in such cases.					#
#									#
#########################################################################

proc xml_handle_bs {cptr cline incdata} {
upvar $cptr c

	set tcline [string range $cline $c end]
	if { $incdata == 1 } {
		if { [string first {]]&gt;} $tcline] == 0 } {
			incr c 6
			return {]]>}
		}
		set ch [string index $cline $c]
		incr c
		return $ch
	}

	if { [string first "&amp;" $tcline] == 0 } {
		incr c 5
		return {&}
	}
	if { [string first "&lt;" $tcline] == 0 } {
		incr c 4
		return {<}
	} 
	if { [string first "&quot;" $tcline] == 0 } {
		incr c 6
		return {"}
	} 
	if { [string first "&apos;" $tcline] == 0 } {
		incr c 6
		return {'}
	} 

	set ch [string index $cline $c]
	incr c
	return $ch
}

proc xml_handle_bs_string {str incdata} {
	set c 0
	set x [string length $str]
	set nstr ""
	while { $c <= $x } {
		append nstr [xml_handle_bs c $str $incdata]
	}
 	return $nstr
}

proc do_extended_proc_search {proclist cstack} {

	# We need to get the current stack and chop off the most 	#
	# recent...							#
	set els [split $cstack :]
	set y [llength $els]
	if { $y < 1 } {
		return -1
	}
	set searchfor [join [lrange $els 0 [expr $y - 2]] ":"]
	if { "$searchfor" == "" } {
		set searchfor "*"
	} else {
		append searchfor ":*"
	}
	#################################################################
	# If we have a stack then we generate the the possible global	#
	# function to handle this level of stack.			#
	#################################################################

	set x [lsearch -exact $proclist $searchfor]
	return $x
}

# xml_parse return codes:
# -1:	Id specified is not recognised.
# -2:	End of line encountered before end of current tag
# -3:	Current end tag does not match latest start tag
# -4:	An error occured whilst calling user-procedure
# -5:	End of file encountered during comment reading
# -6:	End of file encountered with non-empty stack
# -7:	More than one top level entity encountered in file
# -8:	Entity name is not well formed.
# -9:	Attribute name is not well formed.
# 

proc parse {id} {
variable xml_invoc
variable xml_stack
variable xml_file
variable xml_procs
variable xml_attrs_stack
variable xml_data_stack
variable xml_hadtoplevel
variable xml_cdata_parse
variable xml_cdata_start
variable xml_cdata_end
variable xml_attrs

	#################################################################
	# If the trace attribute has been set, then set the local	#
	# variable to indicate this fact.				#
	#################################################################

	set trace $xml_attrs(${id}_trace)

	set xml_s_len [expr [string length $xml_cdata_start] - 1]
	set xml_e_len [expr [string length $xml_cdata_end] - 1]

	#################################################################
	# Check for the special _default_ handler and ensure this	#
	# is made available during the parse.				#
	#################################################################

	set have_default [lsearch -exact $xml_procs($id) _default_]
	if { $have_default == -1 || [expr $have_default % 3] != 0 } {
		set defproc ""
	} else {
		incr have_default
		set defproc [lindex $xml_procs($id) $have_default]
		# Call back arguments, if defined
		set args [lindex $xml_procs($id) [expr {$have_default + 1}]]
	}

	set have_default 0
	if {! [info exists xml_stack($id)]} {
		return -1
	}
	
	#################################################################
	# Loop for all lines in the file to parse			#
	# Status is used to indicate current status of element... 	#
	# 0 - waiting to start element....				#
	# 1 - reading element start tag					#
	# 2 - Reading data for attribute...				#
	# 3 - Reading tag (end tag if 1st ch!= "/")			#
	# 4 - Reading embedded comment/code, ignoring...		#
	#								#
	# 1.8.2	Note:	When reading CDATA the status must be 2, since	#
	#		CDATA is only accepted in data.			#
	#################################################################

	set cstack {}
	set cfileline 0
	set status 0
	while {! [eof $xml_file($id)]} {
		gets $xml_file($id) cline
		incr cfileline
		if { "$trace" >= 1 } {
			puts "Trace: Read line ($status): $cline"
		}
		append cline "\n"
		#########################################################
		# The processing of <tag/> is performed by		#
		# pre-processing each line to convert <tag/> to		#
		# <tag></tag>.						#
		#########################################################
		if {[regsub -all {<([A-Za-z0-9_]+)/>} $cline {<\1></\1>} xx]} {
			set cline $xx
		}

		set c 0
		set ll [string length $cline]
		while { $c < $ll } {
			set ch [string index $cline $c]
			if { $status == 4 } {
				#################################################
				# Cope with --> end comments...			#
				#################################################
				set ch3 [string range $cline $c [expr $c + 2]]
				if { "$ch3" == "-->" } {
					set status $prev_status
					incr c 3; continue
				}
				if { "$ch" != "?" } {
					incr c; continue
				}
				set ch2 [string index $cline [expr $c + 1] ]
				if { "$ch2" == ">" } {
					set status $prev_status
					incr c 2; continue
				}
				incr c; continue
			}
			if { $status == 0 } {
				if { "$ch" == "<" } {
					set status 1
					set tag ""
					incr c; continue
				}
				#################################################
				# Ignore all other characters. . .		#
				#################################################
				incr c; continue
			}

			if { $status == 1 } {
				set empty_ctag 0
				#################################################
				# Look for <? which is treated as a		#
				# comment at the moment...			#
				#################################################
				if { "$tag" == "" } {
					if { "$ch" == "?" } {
						set status 4
						set prev_status 0
						set comstart $cfileline
						incr c; continue
					}

					#########################################
					# Look for <!-- comment start tag	#
					#########################################
					set ch3 [string range $cline $c [expr $c + 2]]
					if { "$ch3" == "!--" } {
						set status 4
						set prev_status 0
						set comstart $cfileline
						incr c 3; continue
					}

					if { "$ch" == "/" } {
						#################################
						# We've got an embedded end tag	#
						# rather than item start tag.   #
						#################################
						set status 3
						set etag $ch
						incr c; continue
					}
				}

				if { "$ch" != ">" } {
					#########################################
					# If we've got a space, then we expect  #
					# attributes - if tag is not empty. - - #
					#########################################
					if { "$tag" != "" && ("$ch" == " " || "$ch" == "\t") } {
						set xx [string range $cline $c end]
						set xx2 [string first ">" $xx]
						if { $xx2 == -1 } {
							if { $xml_attrs(${id}_silent) == 0 } {
								puts stderr "Error: End of line before end of tag encountered."
								puts stderr "Current line below: \n$cline"
							}
							return -2
						}
						set xx [string range $xx 0 [expr $xx2 - 1] ]
						if { [catch {set ctag_attrs [xml_tag_attrs_to_str $xx empty_ctag $xml_cdata_parse($id)]}] } {
							if { $xml_attrs(${id}_silent) == 0 } {
								puts stderr "Current stack: $cstack:$tag"
							}
							return -9
						}
						incr c $xx2
						# Move past tag and deal with it...
					} else {
						set ctag_attrs ""
						append tag $ch
						incr c; continue
					}
				}
				#################################################
				# We have ended the current tag...		#
				#################################################
				set tag [string tolower $tag]
				if { "$trace" >= 1 } {
					puts "Trace: Start-tag: $tag"
				}
				if { [string length $cstack] == 0 } {
					#################################################################
					# If we have ended the current tag and the stack is empty	#
					# then we should check to ensure this is the first top-level	#
					# entity and abort with an error if not.			#
					#################################################################
					if { $xml_hadtoplevel($id) == 1 } {
						if { $xml_attrs(${id}_silent) == 0 } {
							puts stderr "Error: Second top-level entity found! ($tag)"
						}
						return -7
					}
					if { ! [validate_name $tag] } {
						if { $xml_attrs(${id}_silent) == 0 } {
							puts stderr "Error: Entity name malformed: $tag"
						}
						return -8
					}
					set cstack $tag
					set xml_hadtoplevel($id) 1
				} else {
					if { ! [validate_name $tag] } {
						if { $xml_attrs(${id}_silent) == 0 } {
							puts stderr "Error: Entity name malformed: $tag"
						}
						return -8
					}
					append cstack ":$tag"
				}
				#################################################
				# Remove any elements that were sub-elements	#		
				# of an older element of the same stack level.	#
				#################################################
				foreach oldstent [array names xml_attrs_stack "$cstack:*"] {
					unset xml_attrs_stack($oldstent)
					unset xml_data_stack($oldstent)
				}
				set status 2
				set cdata ""
				set xml_attrs_stack($cstack) $ctag_attrs

				#################################################
				# If we have empty_ctag set, then we don't need	#
				# need to search for an end-tag, since it has	#
				# been provived with the trailing /...		#
				#################################################

				if { $empty_ctag } {
					set status 3
					set etag "/$tag"
					continue
				}

				incr c; continue
			}
			if { $status == 2 } {
				#################################################
				# 1.8.2						#
				# See if we have the special CDATA start symbol	#
				# at current position, or the end tag...	#
				#################################################

				if { "[string range $cline $c [expr $c + $xml_s_len]]" == $xml_cdata_start } {
					set xml_cdata_parse($id) 1
					incr c [expr $xml_s_len + 1]
					continue
				}

				if { "[string range $cline $c [expr $c + $xml_e_len]]" == $xml_cdata_end } {
					set xml_cdata_parse($id) 0
					incr c [expr $xml_e_len + 1]
					continue
				}

				if { $xml_cdata_parse($id) == 1 } {
					append cdata [xml_handle_bs c $cline $xml_cdata_parse($id)]
					continue
				}

				#################################################
				# 1.8.2		END OF DATA ALTERATIONS		#
				#################################################

				if { "$ch" == "<" } {
					set status 3
					set etag ""
					if { "$trace" >= 1 } {
						puts "Trace: Saving $cstack: $cdata"
					}
					if {[info exists xml_data_stack($cstack)]} {
						append xml_data_stack($cstack) $cdata
					} else {
						set xml_data_stack($cstack) $cdata
					}
					incr c; continue
				}
				append cdata [xml_handle_bs c $cline $xml_cdata_parse($id)]
				continue
			}
			if { $status == 3 } {
				if { "$etag" == "" && "$ch" == "?" } {
					set status 4
					set comstart $cfileline
					set prev_status 2
					incr c; continue
				}
				if { "$etag" == "" && "$ch" != "/" } {
				#########################################
				# We've got an embedded tag     	#
				# rather than item end tag.     	#
				#########################################

					set status 1
					set tag $ch
					incr c; continue
				}
				if { "$ch" != ">" } {
					append etag $ch
					incr c; continue
				}
				# if { "$ch" == "/" } { }
				#	incr c; continue
				# { }
				#########################################
				# Ok we've got a ">" indicating end of  #
				# deliminator tag, so see if we are	#
				# able to run call back for this	#
				# element.				#
				#########################################
				set etag [string tolower $etag]
				if { "$trace" >= 1 } {
					puts "Trace: End-tag: $etag"
				}
				set xx_el [llength [split $cstack :]]
				incr xx_el -1
				set xx_ll [lindex [split $cstack :] $xx_el]
				if { "$etag" != "/$xx_ll" } {
					if { $xml_attrs(${id}_silent) == 0 } {
						puts stderr "Error: End tag mismatch ($xx_ll -> $etag)"
						puts stderr "Current stack: $cstack"
					}
					return -3
				}
				set x [lsearch -exact $xml_procs($id) $cstack]
				if { "$trace" >= 2 } {
					puts "Trace: Searched $xml_procs($id) for $cstack - Result = $x"
				}
				if { [info exists xml_data_stack($cstack)] } {
					set cdata "$xml_data_stack($cstack)"
				}
				set xml_data_stack($cstack) "$cdata"
				if { ( $x ==-1 || [expr $x % 3] != 0 ) && $xml_attrs(${id}_extended) == 1 } {
					#################################
					# If wild carding has been 	#
					# enabled then perform an extra	#
					# check...			#
					#################################
					set x [do_extended_proc_search $xml_procs($id) $cstack]
				}
				if { $x ==-1 || [expr $x % 3] != 0 } {
					#################################
					# Check for default handler...	#
					#################################
					if { "$defproc" != "" } {
						xml_construct_attr_list $cstack myarr
						xml_construct_data_list $cstack myarr2
						set r [$defproc "$cstack" "$cdata" myarr2 "$xml_attrs_stack($cstack)" myarr $args]
						#########################
						# We need to purge or	#
						# error if required.	#
						#########################
						if { $r == "SXML_PURGE" } {
							set cdata {}
							set xml_data_stack($cstack) {}
						} elseif { ($r != "0" && $r != "SXML_OK") || $r == "SXML_ERROR" } {
							if { $xml_attrs(${id}_silent) == 0 } {
								puts stderr "Error: Returned error when calling: $defproc"
								puts stderr "Current stack: $cstack"
							}
							return -4
						}
					} else {
					#################################
					# Since we do not have a tag	#
					# for this element append it to #
					# the saved data....		#
					#################################
						if { "$trace" >= 1 } {
							puts "Trace: Added-saved: $tag=$cdata"
						}
					}
				} else {
					set proc [lindex $xml_procs($id) [expr $x + 1]]
					if { "$trace" >= 1 } {
						puts "Trace: Calling proc $proc"
					}
					xml_construct_attr_list $cstack myarr
					xml_construct_data_list $cstack myarr2
					set r [$proc "$cstack" "$cdata" myarr2 "$xml_attrs_stack($cstack)" myarr]
					#########################
					# We need to purge or	#
					# error if required.	#
					#########################
					if { $r == "SXML_PURGE" } {
						set cdata {}
						set xml_data_stack($cstack) {}
					} elseif { ($r != "0" && $r != "SXML_OK") || $r == "SXML_ERROR" } {
						if { $xml_attrs(${id}_silent) == 0 } {
							puts stderr "Error: Returned error when calling: $proc"
							puts stderr "Current stack: $cstack"
						}
						return -4
					}
				}
				set stack_split [split $cstack :]
				set e1 [llength $stack_split]
				incr e1 -2
				set cstack [join [lrange $stack_split 0 $e1] :]
				set cdata ""
				set ctag ""
				set etag ""
				set status 0
				incr c; continue
			}
			puts "stderr: Warning invalid state encountered!"
			incr c
		}
	}
	if { $status == 4 } {
		if { $xml_attrs(${id}_silent) == 0 } {
			puts stderr "Error: End of file during comment - comment started on line $comstart."
		}
		return -5
	}
	if { $status != 0 || [string length $cstack] > 0 } {
		if { $xml_attrs(${id}_silent) == 0 } {
			puts stderr "Error: Data exhausted, format not satisfied."
			puts stderr "Current stack: $cstack"
		}
		return -6
	}
	return 0
}

}


#########################################################################
#									#
# Script:	sxml_writer.tcl (namespace sxml::writer)		#
#									#
# Completed:	6th November, 2001.					#
#									#
# Updated:	17th November, 2001.					#
#									#
# Author:	Simon Edwards, (C) Proprius Consulting Ltd		#
#									#
# Provides:	sxml::writer namespace (init finalize get_error element	#
#					attribute data pop_element)	#
#									#
#		These commands provide a simple interface for 		#
#		generating a very simple XML file, which can be parsed	#
#		by the sxml module, or other XML parser.		#
#									#
# Credits:	Various sources on the web, such as devshed, where	#
#		I've learn't a little about XML.			#
#									#
# Limitations:	Definitly a Beta copy - not really tested too much,	#
#		so beware. Updates will be posted when bugs are found	#
#		and resolved.						#
#									#
# Version:	@(#)0.1 Initially released version. (SE)>		#
# Version:	@(#)0.2 Several improvements - see log. (SE)>		#
#									#
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - #
#									#
# Version:	0.2							#
# Date:		17th November, 2001.					#
# Author:	Simon Edwards, Proprius Consulting Ltd.			#
# Change:	Several small improvements:				#
#		1. Support to overwrite existing files.			#
#		2. Support for ">" character translation.		#
#		3. Added support for "original" or "compact" file	#
#		   writing, which alters the layout of the written	#
#		   file.						#
#									#
# Version:								#
# Date:									#
# Author:								#
# Change:								#
#									#
#########################################################################

###############################################################################
#    sxml_writer.tcl - tcl module to provide basic XML file generation.       #
#    Copyright (C) 2000-2001 Simon Edwards, Proprius Consulting Ltd           #
#                                                                             #
#    This program is free software; you can redistribute it and/or modify     #
#    it under the terms of the GNU General Public License as published by     #
#    the Free Software Foundation; either version 2 of the License, or        #
#    (at your option) any later version.                                      #
#                                                                             #
#    This program is distributed in the hope that it will be useful,          #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of           #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
#    GNU General Public License for more details.                             #
#                                                                             #
#    You should have received a copy of the GNU General Public License        #
#    along with this program; if not, write to the Free Software              #
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA  #
#                                                                             #
#    If you wish to contact the author the following details can be used;     #
#                                                                             #
#    simon.edwards@proprius.co.uk                                             #
###############################################################################

namespace eval sxml::writer {

namespace export init finalize get_error element attribute data
namespace export pop_element set_attr

variable xml_stack
variable xml_fd
variable xml_celement
variable xml_cattr
variable xml_wroteheader
variable xml_fdcount 0
variable xml_err
variable xml_swritten
variable xml_int_attrs

# Write out the header text if necessary

proc write_header_if_needed {fd} {
variable xml_fd
variable xml_wroteheader
variable xml_err

	set xfd $xml_fd($fd)
	if { $xml_wroteheader($fd) == 0 } {
		set x [catch {puts $xfd "<?xml version=\"1.0\"?>\n"}]
		if { $x } {
			set xml_err($fd) "Unable to write to file."
			return -1
		}
		set xml_wroteheader($fd) 1
	}
	return 0
}

#########################################################################
# Write out the specified data at the specifyied level of indentation.	#
#########################################################################

proc write_data {fd data level} {
variable xml_err

	set indent ""
	for {set x 0} {$x < $level} {incr x} {
		append indent "  "
	}

	set value [xml_stringify $data]
	set e [catch {puts $fd "$indent$data"}]
	if { $e } {
		set xml_err($fd) "Unable to write to file."
		return -1
	}
	return 0
}

# Take the details given as arguments and write them out to
# the given file descriptor

proc write_start_tag {fd tag cattrs level} {
variable xml_err
variable xml_fd
variable xml_int_attrs

	set xfd $xml_fd($fd)
	set indent ""
	for {set x 0} {$x < $level} {incr x} {
		append indent "  "
	}

	set x [llength $cattrs]
	set y 0

	#################################################################
	# If we have no attributes simply write out the start tag ...	#
	# but indent 2 x level 						#
	#################################################################

	if { $x == 0 } {
		if { $xml_int_attrs($fd,compact) == 0 } {
			set e [catch {puts $xfd "$indent<$tag>"}]
		} else {
			set e [catch {puts -nonewline $xfd "$indent<$tag>"}]
		}
		if { $e } {
			set xml_err($fd) "Unable to write to file."
			return -1
		}
		return 0
	}

	#################################################################
	# Other write out the tag, but don't close the quote		#
	#################################################################

	if { $xml_int_attrs($fd,compact) == 0 } {
		set e [catch {puts $xfd "$indent<$tag"}]
	} else {
		set e [catch {puts -nonewline $xfd "$indent<$tag"}]
	}
	if { $e } {
		set xml_err($fd) "Unable to write to file."
		return -1
	}
	set indent2 "$indent  "
	while { $y < $x } {
		set attr [lindex $cattrs $y]
		incr y
		set value [lindex $cattrs $y]
		incr y

		#########################################################
		# Convert the value to a valid XML string...		#
		#########################################################

		set value [xml_stringify $value]
		if { $xml_int_attrs($fd,compact) == 0 } {
			set e [catch {puts $xfd "$indent2$attr=\"$value\""}]
		} else {
			set e [catch {puts -nonewline $xfd " $attr=\"$value\""}]
		}
		if { $e } {
			set xml_err($fd) "Unable to write to file."
			return -1
		}
	}
	#################################################################
	# Finally write out the end of tag for this element...		#
	#################################################################
	if { $xml_int_attrs($fd,compact) == 0 } {
		set e [catch {puts $xfd "$indent>"}]
	} else {
		set e [catch {puts -nonewline $xfd ">"}]
	}
	if { $e } {
		set xml_err($fd) "Unable to write to file."
		return -1
	}
	return 0
}

# Take a normal string and convert to valid XML chars...

proc xml_stringify {str} {

	# Very basic at the moment!!!!!!!!!

	set rstr ""
	set x 0
	set y [string length $str]
	while { $x < $y } {
		set ch [string index $str $x]
		case $ch {
			{&}	{set ch "&amp;"}
			{<}	{set ch "&lt;"}
			{>}	{set ch "&gt;"}
			{"}	{set ch "&quot;"}
			{'}	{set ch "&apos;"}
		}
		# "
		append rstr $ch
		incr x
	}
	return $rstr
}

# Start the writing of a file - must supply file name to write...

proc init {fname {overwrite 0} } {
variable xml_stack
variable xml_celement
variable xml_cattr
variable xml_err
variable xml_fdcount
variable xml_fd
variable xml_wroteheader
variable xml_int_attrs

	if { [file exists $fname] && $overwrite == 0} {
		set xml_err(-1) "File \"$fname\" already exists."
		return -1
	}

	if { [catch {set fd [open $fname w]}] } {
		set xml_err(-1) "Unable to open \"$fname\" for writing."
		return -1
	}
	set xml_fd($xml_fdcount) $fd
	set xml_wroteheader($xml_fdcount) 0
	set xml_stack($xml_fdcount) {}
	set xml_celement($xml_fdcount) {}
	set xml_cattr($xml_fdcount) {}
	set xml_int_attrs($xml_fdcount,compact) 0
	set xml_int_attrs($xml_fdcount,needindent) 0
	set y $xml_fdcount
	incr xml_fdcount
	return $y
}

# The procedure allowing attributes to be set for a file...

proc set_attr {fd attr value} {
variable xml_fd
variable xml_int_attrs

	# Double check the fd is valid first

	if { ! [info exists xml_fd($fd)] } {
		set xml_err($fd) "Invalid descriptor specified."
		return -1
	}
	# If the variable name specified does not exist then we assume 
	# that the attribute name is invalid.

	if { ! [info exists xml_int_attrs($fd,$attr)] } {
		set xml_err($fd) "Invalid attribute specified."
		return -1
	}
	set xml_int_attrs($fd,$attr) $value
	return 0
}


# The finalize function closes of an existing file...

proc finalize {fd {alllevels 1} } {
variable xml_stack
variable xml_celement
variable xml_cattr
variable xml_err
variable xml_fdcount
variable xml_fd
variable xml_wroteheader
variable xml_swritten
variable xml_int_attrs

	# Double check the fd is valid first

	if { ! [info exists xml_fd($fd)] } {
		set xml_err($fd) "Invalid descriptor specified."
		return -1
	}
	
	# Ok the file is still open, so see if we've anything
	# to write - this will refuse to write an empty file...

	if { [llength $xml_stack($fd)] == 0 } {
		set xml_err($fd) "No data specified - will not write empty file!"
		return -2
	}

	# Ok, so we have some data - but do we still need to write out
	# the header line...
	
	if { [write_header_if_needed $fd] != 0 } {
		return -1
	}

	# Ok, the header has been written, so if we've any data
	# left to write we need to do it now...

	# For each element left on the stack we do some processing...

	set max_el [expr [llength $xml_stack($fd)] - 1]
	set max_el2 $max_el
	set i "  "

	#################################################################
	# Create the indent string first of all for showing data	#
	#################################################################

	for {set xx 0} {$xx < $max_el} {incr xx} {
		append i "  "
	}

	while { $max_el >= 0 } {
		set c_element [lindex $xml_stack($fd) $max_el]

		#########################################################
		# Write out the start tag - including any stored 	#
		# attributes if at deepest level.			#
		#########################################################
		
		if { ! [info exists xml_cattr($fd)] } {
			set xml_cattr($fd) {}
		}

		if { $xml_swritten($fd,$max_el) == 0 } {
			if { [write_start_tag $fd $c_element $xml_cattr($fd) $max_el] != 0 } {
				return -1
			}
			set xml_swritten($fd,$max_el) 1
			if { $xml_int_attrs($fd,compact) == 1 && $max_el != $max_el2 } {
				set e [catch {puts $xml_fd($fd) ""}]
				if { $e } {
					set xml_err($fd) "Unable to write to file."
					return -1
				}
			}
		}

		#########################################################
		# If at deepest level clear any stored attributes	#
		#########################################################

		if { $max_el == $max_el2 } {
			set xml_cattr($fd) {}
		}

		#########################################################
		# Write out any data stored at this level now...	#
		#########################################################

		set c_data [lindex $xml_celement($fd) $max_el]
		set c_data [xml_stringify $c_data]
		if { $xml_int_attrs($fd,compact) == 0 } {
			set e [catch {puts $xml_fd($fd) "$i$c_data"}]
		} else {
			set e [catch {puts -nonewline $xml_fd($fd) "$c_data"}]
		}
		if { $e } {
			set xml_err($fd) "Unable to write to file."
			return -1
		}
		#########################################################
		# Now write out the end tag for this level.		#
		#########################################################
		set i [string range $i 0 [expr ($max_el) * 2 - 1]]
		if { $xml_int_attrs($fd,compact) == 0 || $xml_int_attrs($fd,needindent) == 1 } {
			set e [catch {puts $xml_fd($fd) "$i</$c_element>"}]
		} else {
			set e [catch {puts $xml_fd($fd) "</$c_element>"}]
			set xml_int_attrs($fd,needindent) 1
		}
		if { $e } {
			set xml_err($fd) "Unable to write to file."
			return -1
		}

		#########################################################
		# Check if we just want to close the current level, and	#
		# if so, exit this level if necessary.			#
		#########################################################

		if { $alllevels == 0 } {
			set xml_stack($fd) [lrange $xml_stack($fd) 0 [expr $max_el - 1]]
			break
		}
		incr max_el -1
	}
	if { $alllevels == 0 } {
		if { [llength $xml_stack($fd)] == 0 } {
			catch {close $fd}
			unset xml_fd($fd)
			return 0
		}
		return 0
	}
	catch {close $fd}
	unset xml_fd($fd)
	return 0
}

#########################################################################
# This procedure is called when you wish to add a new element. It will	#
# add the element to stack, but will not actually write anything to 	#
# the file.								#
#########################################################################

proc element {fd name} {
variable xml_stack
variable xml_cattr
variable xml_celement
variable xml_fd
variable xml_err
variable xml_swritten
variable xml_int_attrs

	# Double check the fd is valid first

	if { ! [info exists xml_fd($fd)] } {
		set xml_err($fd) "Invalid descriptor specified."
		return -1
	}
	
	# If we've not got an empty stack, then now is the
	# time to write out the details...

	if { [llength $xml_stack($fd)] > 0 } {
		set i [llength $xml_stack($fd)]
		incr i -1
		if { ! [info exists xml_swritten($fd,$i)] } {
			set xml_swritten($fd,$i) 0
		}
		if { $xml_swritten($fd,$i) == 0 } {
			set tag [lindex $xml_stack($fd) $i]
			if { [write_header_if_needed $fd] != 0 } {
				return -1
			}
			set e [write_start_tag $fd $tag $xml_cattr($fd) $i]
			if { $e != 0 } {
				return -1
			}
			set xml_swritten($fd,$i) 1
			if { $xml_int_attrs($fd,compact) == 1 } {
				set e [catch {puts $xml_fd($fd) ""}]
				if { $e != 0 } {
					set xml_err($fd) "Unable to write to file."
					return -1
				}
			}
		}
		write_data $fd "[lindex $xml_celement($fd) $i]" $i
		set xml_int_attrs($fd,needindent) 0
	}
	# Add the element name to stack and clear out the
	# attribute details...

	lappend xml_stack($fd) $name
	set i [llength $xml_stack($fd)]

	incr i -1
	if { ! [info exists xml_celement($fd)] } {
		set xml_celement($fd) {}
	} else {
		set xml_celement($fd) "[lrange $xml_celement($fd) 0 [expr $i - 1]] {} [lrange $xml_celement($fd) [expr $i + 1] end]"
	}

	set xml_swritten($fd,$i) 0
	set xml_cattr($fd) {}
	return 0
}

proc attribute {fd name value} {
variable xml_stack
variable xml_cattr
variable xml_fd
variable xml_err

	# Double check the fd is valid first

	if { ! [info exists xml_fd($fd)] } {
		set xml_err($fd) "Invalid descriptor specified."
		return -1
	}

	# Make sure the stack size is greater than 0

	if { [llength $xml_stack($fd)] == 0 } {
		set xml_err($fd) "Attributes can only specifed after an element."
	}

	# Add this element details...

	lappend xml_cattr($fd) $name $value
	return 0
}

proc data {fd data} {
variable xml_stack
variable xml_celement
variable xml_fd
variable xml_err

	# Double check the fd is valid first

	if { ! [info exists xml_fd($fd)] } {
		set xml_err($fd) "Invalid descriptor specified."
		return -1
	}

	# Make sure the stack size is greater than 0

	if { [llength $xml_stack($fd)] == 0 } {
		set xml_err($fd) "Attributes can only specifed after an element."
	}

	# Now append this data for this item..
	set i [llength $xml_stack($fd)]
	incr i -1
	set x [lindex $xml_celement($fd) $i]
	append x $data
	set xml_celement($fd) "[lrange $xml_celement($fd) 0 [expr $i - 1]] {$x} [lrange $xml_celement($fd) [expr $i + 1] end]"
	return 0
}

# This takes the last element of the stack and writes out the
# details to the file. Similar to the finalise operation really.

proc pop_element {fd} {

	return [finalize $fd 0]
}

proc get_error {fd} {
variable xml_err

	if { [info exists xml_err($fd)] } {
		return $xml_err($fd)
	}
	return ""
}

}









# XML Routines end
# ----------------------------------------------------------------------




# set to 1 when actually enabled by menu
set cosmoipcscripting 0

# DisableAnalogOutput  has been removed, set to 0
set DisableAnalogOutput "0"
set DisableTriggerOutput "0"
set StackReadbackMode "0"
set IgnoreIncomingTriggers "0"
set PauseReadout "0"
set EnableAutoReset "0"
set ResetOnStackCount "0"
set DisableTokenPassing "0"
set NotUsedLine47 "0"
set ClearTokenOut "0"
set ClearStack "0"
set HardResetTBM "0"
set InjectPreCalTrigger "0"
set InjectResetTrigger "0"
set InjectSyncTrigger "0"
set InjectNormalTrigger "0"
set DisableTBMCLK "0"
set Readout40MHz "0"
set NotUsedLine65 "0"
set NotUsedLine64 "0"
set NotUsedLine63 "0"
set NotUsedLine62 "0"
set NotUsedLine61 "0"
set NotUsedLine60 "0"
set baseaddress 0xE0
set ForceReadoutClockOn "0"
set DisableTokenOutDrv "0"
set DisableAnalogOutputDrv "0"

set tbm2mode "Sync"
set ClearTriggerCounter "0"
set InjectTBMReset "0"
set InjectROCReset "0"

set ResetAnalogRegisters "255"
set AnalogInputAmplifierBias "160"
set AnalogOutputGain "105"
set AnalogOutputDriverBias "128"

set c00 "0"
set data ""
set hubaddress "012"
set portaddress "4"
set returnedbits ""
set lastreadbits ""
set numreturnedbits "0"

set paramout "1234"

set ctrl_reg 0
set wbc 134

set savedlastf0 "00000000"

set lptport 1
set portspeed 0

set pixreadoutspeed 20
set pixstopdaq 0
set pixchipaddress 0

set pixportaddress 0

set stackshown 0
set testshown 0

set maskbitstate 1
set trim 0
set calibrationon 0
set pixextraclocks 8

set D7position 0
set D0position 7

# 22 27
set D7positiontbm2 22
set D3positiontbm2 27

set forgetpixprestopbit no
#set datalength 10
set datalength 8

set colscalibrationon 0

# balloons ON|OFF sets the appearance of balloon help
set balloons "ON"

# verbose1: handy diagnostic of what is passed and returned from lpt helperproc
# verbose2: more detailed: show read data byte in 8bit form even when in 10bit
set verbose1 0
set verbose2 0
set verbose3 0
set verbose4 0

#adc globals
set dostop 0
set adcyoffset 2
set dataoutputfile ""
# outformat supported: paw24 (integer space delimeted, 24 values.)  1st data
#  for Bob's analysis
#  binary: (16bit little endian words.)  1st word is number of words to
#   follow.
set outformat "binary"
set nfileevents 0
set ioutevents 0
set icount 0
set suppressdisplay 0

#set calrowlinearray 0

set calrowlist ""
set calcollist ""
#

set initialdir [pwd]
global initialdir

proc setvmeslotandssid { } {
    global vmeslot
    toplevel .setssid -class Dialog
    wm title .setssid "Set PIXEL FEC Slot SSID"
    frame .setssid.row1
    frame .setssid.row2
    frame .setssid.row3
    label .setssid.row1.label -text "\n    Set Slot Number for Pixel FEC    \n"
    label .setssid.row2.l1 -text "VMESlot:" -font {Helvetica 10 {bold}}
    tk_optionMenu .setssid.row2.e1 vmeslot 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21
    button .setssid.row3.button -text "Set" -command {
	setssid $vmeslot
	destroy .setssid
    }
    pack .setssid.row1.label
    pack .setssid.row1
    pack .setssid.row2.l1 .setssid.row2.e1 -side left
    pack .setssid.row2
    pack .setssid.row3.button
    pack .setssid.row3
    set width [winfo screenwidth .]
    set height [winfo screenheight .]
    set x [expr ($width/2) - 200]
    set y [expr ($height/2) - 200]
    
    wm geometry .setssid  +$x+$y
    wm resizable .setssid 0 0
}

proc discoverhub2 { command } {
    global ctrl_reg
    global wbc
    global pixreadoutspeed
    global maskbitstate
    global trim
    global calibrationon

    global resetstatus
    global resetcalibration

    global hubaddress
    global portaddress

    global col
    global row
    global trim

    global PCTARGET

    toplevel .winhub -class Dialog 
    wm title .winhub "Discover HUB"

    frame .winhub.row1
    frame .winhub.row2
    frame .winhub.row3
    frame .winhub.row4
    frame .winhub.row5
    frame .winhub.row9

    frame .winhub.row1.f1
    label .winhub.row1.f1.label -text "Hub Search" -foreground "blue" \
	-padx 10 -font { Helvetica 18 {bold} } 

    pack .winhub.row1.f1.label

    frame .winhub.row2.f1
    label .winhub.row2.f1.l1 \
	-text "\nProbing attached Test-Station\n\nSearching for a hub with\na response on port 4\n\nHub address will be set\nto the discovered value.\n\n"
    pack .winhub.row2.f1.l1

    frame .winhub.row3.f1
    label .winhub.row3.f1.l1 -text "Searching for HUB..."
    pack .winhub.row3.f1.l1
##
    frame .winhub.row4.f1
    label .winhub.row4.f1.l1 -text "" -font { Helvetica 14 {bold} }
    pack .winhub.row4.f1.l1


    frame .winhub.row9.f1
    button .winhub.row9.f1.button1 -text "Dismiss window" -command {
##	resetiterate 1
	destroy .winhub
    }
    
    pack .winhub.row1.f1
    pack .winhub.row2.f1
    pack .winhub.row3.f1
    pack .winhub.row4.f1
    pack .winhub.row9.f1

    pack .winhub.row1
    pack .winhub.row2
    pack .winhub.row3
    pack .winhub.row4
    pack .winhub.row9


    set width [winfo screenwidth .]
    set height [winfo screenheight .]
    set x [expr ($width/2) - 200]
    set y [expr ($height/2) - 200]
    
    wm geometry .winhub  +$x+$y
    wm resizable .winhub 0 0

    set savedportaddress $portaddress
    set savedhubaddress $hubaddress

    set ih 0
    while { $ih < 32 } {
#	set ip 7
	# send something out to TBM and look for response
	set hubaddress $ih
	set portaddress 4
	.winhub.row4.f1.l1 config -text "$ih" -fg "violet"
	update
	set returnvalue [testwrite2 $ih]
	if {$returnvalue >= 0} {
	    # testwrite returns -1 if hub not found
	    # and 1 if it found a hub and 2 if it found a Si TBM with
	    # analog readback
	    # if found then exit this loop
#	    puts "Checking return value: $returnvalue"
#	    after 1000
	    .winhub.row4.f1.l1 config -text "$ih" -fg "\#44cf44"
	    set hubaddress $ih
	    set ip 99999
	    set ih 99999
	}
	incr ih
    }

    if {$PCTARGET == "devel"} {
	set ih 900
	set hubaddress 0
    }

    if {$ih < 1000} {
	# not found so set to 0
	set ih 1000
	set hubaddress 0
    } else {
	set ih $hubaddress
    }


    

    if {$ih > 256} {
	# not found
	puts "****************************************************\nAutoprobe for hub found no hub.  Setting HUB ID to 0\n****************************************************\n"
	.winhub.row2.f1.l1 config \
	    -text "\nPossible causes:\n\nTest-board power not connected\nBoard needs reset at pushbutton\n\n\n"
	pack .winhub.row9.f1.button1
	.winhub.row4.f1.l1 config -text "HUB NOT FOUND" -fg "red"
	set portaddress 4
	set hubaddress 0
    } else {
	if {$returnvalue == "1"} {
	    .winhub.row4.f1.l1 config -text "\n  FPGA HUB-TBM FOUND  \n  at address $hubaddress  \n" -fg "\#44cf44"
	}
	if {$returnvalue == "2"} {
	    .winhub.row4.f1.l1 config -text "\n  Silicon HUB-TBM FOUND  \n  at address $hubaddress  \n" -fg "\#44cf44"
	}
	#	pack .winhub.row9.f1.button1
	puts "****************************************************\nAutoprobe for hub found HUB with ID set to $hubaddress\n****************************************************\n"
	if {$returnvalue == 1} {
	    puts "************** FPGA HUB DETECTED ****************"
	} elseif {$returnvalue == 2} {
	    puts "************** Si TBM DETECTED ****************"
	}
	set portaddress $savedportaddress
	update
	after 2000 destroy .winhub
    }
}

proc chipload { mywidget } {
    global globalmodchipbutton
    global modulechip

    set fwin [focus]
    focus .busylock
    grab set .busylock
    set cursor [. cget -cursor]
    . configure -cursor watch

    memsave $globalmodchipbutton
    $globalmodchipbutton configure -fg red \
	-bg blue \
	-activebackground blue
    $modulechip($mywidget) configure -fg red \
	-bg green \
	-activebackground green
    memload $mywidget 1
    set globalmodchipbutton $mywidget

    . configure -cursor $cursor
    grab release .busylock
    focus $fwin

}

proc gauge_create { win { color "" }} {
    frame $win -class Gauge
    set len 200
    canvas $win.display -borderwidth 0 -background white \
	    -highlightthickness 0 -width $len -height 20
    pack $win.display -expand yes
    if { $color == "" } {
	set color [ option get $win color Color ]
    }
    $win.display create rectangle 0 0 0 20 \
	    -outline "" -fill $color -tags bar
    $win.display create text [ expr 0.5*$len ] 10 \
	    -anchor c -text "0%" -tags value
    return $win
}
proc gauge_value { win val } {
    if { $val < 0 || $val > 100 } {
	error "gauge routine called with bad value"
    }
    set msg [ format "%3.0f%%" $val ]
    $win.display itemconfigure value -text $msg
    set w [ expr 0.01*$val*[winfo width $win.display] ]
    set h [ winfo height $win.display ]
    $win.display coords bar 0 0 $w $h
    update
}

proc slowhubsend { myhaddress myhport myshaddress myshport \
		       myaddr mydata } {
# All params are in decimal here, not hex
    global slowhubstopshift
    global slowhubstartshift
    clearnibblearray "0000"
    set slowhubstopshift 0
    puts "SLOWHUBSEND"
    # first construct the data into dataentry()
    set slowhubaddress $myshaddress
    set slowhubport $myshport
    set myhub [expr $slowhubport + (8 * $slowhubaddress)]
    set imyhub [inttobits $myhub 8]
    set myhubhigh [string range $imyhub 0 3]
    set myhubhigh [bitstohex $myhubhigh]
    set myhublow [string range $imyhub 4 7]
    set myhublow [bitstohex $myhublow]
    
    set dataentry(0) "$myhubhigh$myhublow"

	set slowdata ""
# serial stream define 0="01" 1="10"
# insert a start, 24 x 0
    set slowhubstartshift 0
    if {$slowhubstartshift != 0} {
	puts "START SHIFT for slowhub=$slowhubstartshift"
	set ilimit [expr $slowhubstartshift + 23]
	for {set ii 0} {$ii < $ilimit} {incr ii} {
	    append slowdata "01"
	}
    } else {
	for {set ii 0} {$ii < 23} {incr ii} {
	    append slowdata "01"
	}
    }
    set nbytes 2
    for {set i 0} {$i < $nbytes} {incr i} {
	if {$i == 0} {
	    set dbyte $myaddr
	}
	if {$i == 1} {
	    set dbyte $mydata
	}
#	set slowhubhigh [string index $dbyte 0]
#	set slowhublow [string index $dbyte 1]
#	set slowhubhigh [hextoint $slowhubhigh]
#	set slowhublow [hextoint $slowhublow]
#	set islowhubdata [expr $slowhublow + 16*$slowhubhigh]
#	set dbyte [inttobits $islowhubdata 8]
	set dbyte [inttobits $dbyte 8]
	puts "DBYTE:$dbyte"
	for {set j 0} {$j < 8} {incr j} {
	    # now put the data in, 32 x N(i)
	    set dbit [string index $dbyte $j]
	    for {set ii 0} {$ii < 32} {incr ii} {
		if {$dbit == 1} {
		    append slowdata "10"
		} else {
		    append slowdata "01"
		}
	    }
	}
	# add in the 9th ACK data, 32 1s is 32 x 10
	for {set ii 0} {$ii < 32} {incr ii} {
	    append slowdata "10"
	}
	
    }

# insert a stop, M x 0 where m=19 for 1 byte, m=16 for 2B,m=18 for 3B,m=15
# for 4 bytes
    switch $nbytes {
	1 { set nzeros 19 }
	2 { set nzeros 16 }
	3 { set nzeros 18 }
	4 { set nzeros 15 }
    }
    if {$slowhubstopshift != 0} {
	set nzeros [expr $nzeros + $slowhubstopshift]
	puts "STOP SHIFT for slowhub=$slowhubstopshift   Stop M=$nzeros"
    }
    for {set ii 0} {$ii < $nzeros} {incr ii} {
	append slowdata "01"
    }
    
    puts "SLOWDATA RAW: $slowdata"
    set newslowdata ""
    set ilen [string length $slowdata]
    for {set k 0} {$k < $ilen} {incr k} {
	set mychar [string index $slowdata $k]
	set mev [expr ($k+1) % 5]
	if {$mev != 0} {
	    append newslowdata $mychar
	}
    }
    set slength [string length $newslowdata]
    puts "SLOWDATA length:$slength NEW: $newslowdata"
    
    
    set thisentry 1
    for {set m 0} {$m < [string length $newslowdata]} {set m [expr $m+8]} {
	set thishigh [string range $newslowdata $m [expr $m+3]]
	set thishigh [bitstohex $thishigh]
	set thislow [string range $newslowdata [expr $m+4] [expr $m+7]]
	set thislow [bitstohex $thislow]
	set thishex "$thishigh$thislow"
	set dataentry($thisentry) $thishex
	incr thisentry
    }
    
    set inum $thisentry
    puts "INUM:$inum"
    set myhex [format "%x" $inum]
    set itmp [string length $myhex]
    if {$itmp < 2} {
	set myhex "0$myhex"
    }
    set slowhubnum $myhex

    
    
    
    
    # next reset the pointer and send the data in dataentry (hex).
    sendresetpointer 1
    set temp1 [inttobits $myhaddress 5]
    # uport=4 is tbm
    set temp2 [inttobits $myhport 3]
    set temp3 "$temp1$temp2"
    set temp4 [bitstoint $temp3]
    set temp5 [inttobits $temp4 4]
    set tlow $temp5
    set temp6 [expr $temp4 / 16]
    set temp7 [inttobits $temp6 4]
    set thigh $temp7
    # Send the hub and port address
    sendbyte2 "$thigh" "$tlow"
    # Send the number of bytes (num) of data to follow
# when presented, global num should be hex    
    set highnum [string index $slowhubnum 0]
    set lownum [string index $slowhubnum 1]
    set ihighnum [hextoint $highnum]
    set ilownum [hextoint $lownum]
    set inum [expr $ilownum+$ihighnum*16]
#d    puts "Num is **$inum**: $ihighnum,$ilownum"
    set highnumbits [inttobits $ihighnum 4]
    set lownumbits [inttobits $ilownum 4]

    sendbyte2 "$highnumbits" "$lownumbits"
    # always two bytes of data
    puts "WRITE OF $inum bytes to SLOW HUB"
    set i 0
    while { $i < $inum } {
	set ttemp $dataentry($i)
	set thigh [hexhigh $ttemp]
	set tlow [hexlow $ttemp]
	sendbyte2 "$thigh" "$tlow"
	incr i
    }
    sendzero 1
    sendresetpointer 1
    # send get parity
    #    set checkingparity 1
    #    sendbyte "11110100"
    resetparitybyte
    ######    fixnibblearray 1
    sendnibblearray 0
    clearnibblearray "0000"

    sendgo 1
    ##    flush $usbdeviceid
    #    $uwin.f4.b1 config -image but0
    updatefeccmdindicators 1

    puts "SENT SLOWHUBDATA $myhaddress $myhport $myshaddress $myshport $myaddr $mydata"
}

proc updateslowdelayslide { mydelay value } {
    global slowhubdelay25data
#    puts "Slide $mydelay moved to $value"
    # set the 
    set slowhubdelay25data($mydelay) $value
#    puts "delay25 array: $slowhubdelay25data($mydelay), $mydelay"
}
set slowhubpllphasepolarity 1
set slowhubpllphase 0
proc updateslowpllslide { mypll value } {
    global slowhubpllphasepolarity
    global slowhubpllphase
    set slowhubpllphase $value
#    puts "Slide PLL $mypll moved to $value  POLARITY=$slowhubpllphasepolarity"
}

set slowhubdohI0 0
set slowhubdohI2 0
proc updateslowdohslideI0 { mydoh value } {
    global slowhubdohI0
    set temp [inttobits $value 2]
    set slowhubdohI0 $temp
#    puts "Slide DOH I0: $value    = $temp"
}
proc updateslowdohslideI2 { mydoh value } {
    global slowhubdohI2
    set temp [inttobits $value 2]
    set slowhubdohI2 $temp
#    puts "Slide DOH I2: $value    = $temp"
}

#slowhubdefaults:
#CLOCK default
set slowhubdelay25data(104) 20
# adr100 is sdata; changed default from 30 to 33 set slowhubdelay25data(100) 30
# Sdata default
set slowhubdelay25data(100) 23
#Trig default
set slowhubdelay25data(102) 20
# adr98 is retclock; changed default from 25to31 set slowhubdelay25data(98) 25
#Ret Clock default
set slowhubdelay25data(98) 34
#Ret Data default:
set slowhubdelay25data(96) 20
					
    

proc slowhubcontrol { command } {
    global slowhubdelay25data
    global slowhubpllphasepolarity slowhubpllphase
    global fecaddress
    global addresslookup
    global textwindow
    global textwindowline
    global fecleds
    global uwin
    global vmeslot mfec

    global fec_trigger_source
    global norm_loop_on
    global encalpulse
    global verbose3
    global disableautosend
    global uport
    global slowfecchannel

    global hubaddress port
    global hub0 hub1 hub2 hub3 hub4 hubaddresshigh hubaddresslow
    global port0 port1 port2 portlow
    global highnum lownum num inum
    global highnumbits lownumbits
    global currenttbm
    global roctype
    global baseaddress
    global clkph
    global delay25sdata delay25clock delay25trig delay25retclock delay25retdata
    global slowhubdohI0 slowhubdohI2

    toplevel .slowhub -class Dialog 
    wm title .slowhub "Slow HUB Control"
    label .slowhub.label -text "Slow Hub Control" \
	-font {Helvetica 14 {bold}}

    pack .slowhub.label

    button .slowhub.b1 -width 40 \
	-text "Send all adjustments to devices" -command {
	    puts "TEST slowhub"
	    
	    if {$delay25clock == "1"} {
		set dataval [expr $slowhubdelay25data(104) + 64]
	    } else {
		set dataval $slowhubdelay25data(104)
	    }
	    slowhubsend 31 0 31 2 104 $dataval
	    
	    if {$delay25sdata == "1"} {
		set dataval [expr $slowhubdelay25data(100) + 64]
	    } else {
		set dataval $slowhubdelay25data(100)
	    }
	    slowhubsend 31 0 31 2 100 $dataval

	    if {$delay25trig == "1"} {
		set dataval [expr $slowhubdelay25data(102) + 64]
	    } else {
		set dataval $slowhubdelay25data(102)
	    }
	    slowhubsend 31 0 31 2 102 $dataval
	    
	    if {$delay25retclock == "1"} {
		set dataval [expr $slowhubdelay25data(98) + 64]
	    } else {
		set dataval $slowhubdelay25data(98)
	    }
	    slowhubsend 31 0 31 2  98 $dataval
	    
	    if {$delay25retdata == "1"} {
		set dataval [expr $slowhubdelay25data(96) + 64]
	    } else {
		set dataval $slowhubdelay25data(96)
	    }
	    slowhubsend 31 0 31 2  96 $dataval
	    

	
	    set bslowhubpllphasepolarity [inttobits $slowhubpllphasepolarity 1]
	    set bslowhubpllphase [inttobits $slowhubpllphase 4]
	    puts "Slider:$bslowhubpllphase from $slowhubpllphase"
	    set slowhubplldata "000$bslowhubpllphasepolarity$bslowhubpllphase"
	    puts "bSLOWHUBPLLDATA: $slowhubplldata"
	    set slowhubplldata [bitstoint $slowhubplldata]
	    puts "bSLOWHUBPLLDATA: $slowhubplldata"
	    slowhubsend 31 0 31 2 66 $slowhubplldata


# DOH gain
	    set temp1 "00$slowhubdohI2"
	    set temp2 "00$slowhubdohI0"
	    set DOHgain "$temp1$temp2"
	    puts "DOHGain: $DOHgain"
	    set DOHgain [bitstoint $DOHgain]
	    slowhubsend 31 0 31 2 112 $DOHgain
	    
    }
    pack .slowhub.b1

    frame .slowhub.delay25 -relief groove -borderwidth 4 

    frame .slowhub.delay25.l1 -relief groove -borderwidth 2
    label .slowhub.delay25.l1.l1 -text "Delay 25 Control" \
	-font {Helvetica 10 {bold italic}}
    pack .slowhub.delay25.l1.l1

    frame .slowhub.delay25.l1.f0
    frame .slowhub.delay25.l1.f1
    label .slowhub.delay25.l1.f0.l1 -text "Enable" -width 6
    checkbutton .slowhub.delay25.l1.f0.cb1 -variable delay25clock -onvalue "1" \
	-offvalue "0"
    scale .slowhub.delay25.l1.f1.s1 -from 0 -to 63 -label "Clock:" -length 250 \
	-orient horizontal -tickinterval 10\
	-command { updateslowdelayslide 104 }

    pack .slowhub.delay25.l1.f0.l1 .slowhub.delay25.l1.f0.cb1

    pack .slowhub.delay25.l1.f1.s1
    pack .slowhub.delay25.l1.f0 .slowhub.delay25.l1.f1 -side left
    .slowhub.delay25.l1.f1.s1 set $slowhubdelay25data(104)
    set delay25clock 1
#--------


    frame .slowhub.delay25.l2 -relief groove -borderwidth 2
    frame .slowhub.delay25.l2.f2
    frame .slowhub.delay25.l2.f3
    label .slowhub.delay25.l2.f2.l1 -text "" -width 6
    checkbutton .slowhub.delay25.l2.f2.cb1 -variable delay25sdata -onvalue "1" \
	-offvalue "0"
    scale .slowhub.delay25.l2.f3.s1 -from 0 -to 63 -label "Sdata:" -length 250 \
	-orient horizontal -tickinterval 10\
	-command { updateslowdelayslide 100 }

    pack .slowhub.delay25.l2.f2.l1 .slowhub.delay25.l2.f2.cb1

    pack .slowhub.delay25.l2.f3.s1
    pack .slowhub.delay25.l2.f2 .slowhub.delay25.l2.f3 -side left
    .slowhub.delay25.l2.f3.s1 set $slowhubdelay25data(100)
    set delay25sdata 1
    
#--------

    frame .slowhub.delay25.l3 -relief groove -borderwidth 2
    frame .slowhub.delay25.l3.f2
    frame .slowhub.delay25.l3.f3
    label .slowhub.delay25.l3.f2.l1 -text "" -width 6
    checkbutton .slowhub.delay25.l3.f2.cb1 -variable delay25trig -onvalue "1" \
	-offvalue "0"
    scale .slowhub.delay25.l3.f3.s1 -from 0 -to 63 -label "Trig:" -length 250 \
	-orient horizontal -tickinterval 10\
	-command { updateslowdelayslide 102 }

    pack .slowhub.delay25.l3.f2.l1 .slowhub.delay25.l3.f2.cb1

    pack .slowhub.delay25.l3.f3.s1
    pack .slowhub.delay25.l3.f2 .slowhub.delay25.l3.f3 -side left
    .slowhub.delay25.l3.f3.s1 set $slowhubdelay25data(102)
    set delay25trig 1
#--------


    frame .slowhub.delay25.l4 -relief groove -borderwidth 2
    frame .slowhub.delay25.l4.f2
    frame .slowhub.delay25.l4.f3
    label .slowhub.delay25.l4.f2.l1 -text "" -width 6
    checkbutton .slowhub.delay25.l4.f2.cb1 -variable delay25retclock -onvalue "1" \
	-offvalue "0"
    scale .slowhub.delay25.l4.f3.s1 -from 0 -to 63 -label "Ret Clock:" -length 250 \
	-orient horizontal -tickinterval 10\
	-command { updateslowdelayslide 98 }

    pack .slowhub.delay25.l4.f2.l1 .slowhub.delay25.l4.f2.cb1

    pack .slowhub.delay25.l4.f3.s1
    pack .slowhub.delay25.l4.f2 .slowhub.delay25.l4.f3 -side left
    .slowhub.delay25.l4.f3.s1 set $slowhubdelay25data(98)
    set delay25retclock 1
#--------


    frame .slowhub.delay25.l5 -relief groove -borderwidth 2
    frame .slowhub.delay25.l5.f2
    frame .slowhub.delay25.l5.f3
    label .slowhub.delay25.l5.f2.l1 -text "" -width 6
    checkbutton .slowhub.delay25.l5.f2.cb1 -variable delay25retdata -onvalue "1" \
	-offvalue "0"
    scale .slowhub.delay25.l5.f3.s1 -from 0 -to 63 -label "Ret Data:" -length 250 \
	-orient horizontal -tickinterval 10\
	-command { updateslowdelayslide 96 }

    pack .slowhub.delay25.l5.f2.l1 .slowhub.delay25.l5.f2.cb1

    pack .slowhub.delay25.l5.f3.s1
    pack .slowhub.delay25.l5.f2 .slowhub.delay25.l5.f3 -side left
    .slowhub.delay25.l5.f3.s1 set $slowhubdelay25data(96)
    set delay25retdata 1
#--------

    pack .slowhub.delay25.l1
    pack .slowhub.delay25.l2
    pack .slowhub.delay25.l3
    pack .slowhub.delay25.l4
    pack .slowhub.delay25.l5


#--------

    frame .slowhub.pll -relief groove -borderwidth 4

    frame .slowhub.pll.l1 -relief groove -borderwidth 2
    label .slowhub.pll.l1.l1 -text "PLL" \
	-font {Helvetica 10 {bold italic}}
    pack .slowhub.pll.l1.l1
    frame .slowhub.pll.l1.f1 
    
    button .slowhub.pll.l1.f1.b1 -text "RESET PLL" -width 30 \
	-font { Helvetica 10 {bold}} -command {
	    puts "RESET PLL CALLED"
	    slowhubsend 31 0 31 2 64 0
	    slowhubsend 31 0 31 2 64 8
	    slowhubsend 31 0 31 2 64 0
	}
    pack .slowhub.pll.l1.f1.b1
    pack .slowhub.pll.l1.f1
    frame .slowhub.pll.l1.f2
    frame .slowhub.pll.l1.f3
    label .slowhub.pll.l1.f2.l1 -text "Polarity" -width 8
    checkbutton .slowhub.pll.l1.f2.cb1 -variable slowhubpllphasepolarity -onvalue "1" -offvalue "0"
    scale .slowhub.pll.l1.f3.s1 -from 0 -to 11 -label "Phase" -length 250 \
	-orient horizontal -tickinterval 1 -command { updateslowpllslide 66 }
    pack .slowhub.pll.l1.f2.l1 .slowhub.pll.l1.f2.cb1 
    pack .slowhub.pll.l1.f3.s1 -side left
    pack .slowhub.pll.l1.f2 .slowhub.pll.l1.f3 -side left
    pack .slowhub.pll.l1
    set slowhubpllphasepolarity 0
    pack .slowhub.delay25 .slowhub.pll
#----    
    frame .slowhub.doh -relief groove -borderwidth 4
    frame .slowhub.doh.l1 -relief groove -borderwidth 2
    label .slowhub.doh.l1.l1 -text "Linear Laser Driver - DoH" \
	-font {Helvetica 10 {bold italic}}
    pack .slowhub.doh.l1.l1

    frame .slowhub.doh.l1.f2
    scale .slowhub.doh.l1.f2.s1 -from 0 -to 3 -label "Clock:" -length 120 \
	-orient horizontal -tickinterval 1 -command { updateslowdohslideI0 115 }
    scale .slowhub.doh.l1.f2.s2 -from 0 -to 3 -label "Data:" -length 120 \
	-orient horizontal -tickinterval 1 -command { updateslowdohslideI2 115 }
    pack .slowhub.doh.l1.f2.s1 .slowhub.doh.l1.f2.s2 -side left
    pack .slowhub.doh.l1.f2
    pack .slowhub.doh.l1
    pack .slowhub.doh

    set width [winfo screenwidth .]
    set height [winfo screenheight .]
    set x [expr ($width/2) - 200]
    set y [expr ($height/2) - 200]
    
    wm geometry .slowhub  +$x+$y
    wm resizable .slowhub 0 0
    
    update


#    after 10000
#    destroy .fec
}



proc feccontrol { command } {
    global fecaddress
    global addresslookup
    global textwindow
    global textwindowline
    global fecleds
    global uwin
    global vmeslot mfec

    global fec_trigger_source
    global norm_loop_on
    global encalpulse
    global verbose3
    global disableautosend
    global uport
    global slowfecchannel

    global hubaddress port
    global hub0 hub1 hub2 hub3 hub4 hubaddresshigh hubaddresslow
    global port0 port1 port2 portlow
    global highnum lownum num inum
    global highnumbits lownumbits
    global currenttbm
    global roctype
    global baseaddress
    global clkph

    toplevel .fec -class Dialog 
    wm title .fec "FEC Control"
    label .fec.label -text " FEC CONTROL" -font {Helvetica 10 {bold}}

    pack .fec.label

    frame .fec.command
    frame .fec.command.row1 -relief groove -borderwidth 4
    label .fec.command.row1.l1 -text "VMESlot:" -font {Helvetica 10 {bold}}
    tk_optionMenu .fec.command.row1.e1 vmeslot 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21
    label .fec.command.row1.l2 -text "mFEC:" -font {Helvetica 10 {bold}}
    tk_optionMenu .fec.command.row1.e2 mfec 8 7 6 5 4 3 2 1
    label .fec.command.row1.l3 -text "   mFEC Channel:" -font {Helvetica 10 {bold}}
    tk_optionMenu .fec.command.row1.e3 slowfecchannel 1 2
    set slowfecchannel 1

    button .fec.command.row1.b1 -text "FEC Reset" \
	-font { Helvetica 10 {bold}} -command {
	    set nibblecount 0
	    setssid $vmeslot
	    if {$slowfecchannel == "1"} {
		pcifeccmdreg 3 1
		set textwindowline 0
	    } elseif {$slowfecchannel == "2"} {
		pcifeccmdreg 19 1
		set textwindowline 0
	    }
	    updatefeccmdindicators 1
	}

    button .fec.command.row1.b2 -text "Check Status" \
	-font { Helvetica 10 {bold}} -command {
	    updatefeccmdindicators 1
	}
    pack .fec.command.row1.l1 .fec.command.row1.e1 .fec.command.row1.l2 .fec.command.row1.e2 .fec.command.row1.l3 .fec.command.row1.e3 \
	.fec.command.row1.b1 .fec.command.row1.b2 -side left

# now construct the 2nd row
    frame .fec.command.row2
    frame .fec.command.row2.f1 -relief groove -borderwidth 4
    frame .fec.command.row2.f1.ftitle
    frame .fec.command.row2.f1.ftop
    frame .fec.command.row2.f1.fbot
    label .fec.command.row2.f1.ftitle.l1 -text "FEC Trigger Inject" -font {Helvetica 10 {bold}}
    pack .fec.command.row2.f1.ftitle.l1
    button .fec.command.row2.f1.ftop.b1 -text "CSRE" -width 5 -font {Helvetica 10 {bold}} \
	-command {
	    if {$verbose3=="1"} {
		puts "SENDING CSR REQUEST TO FEC"
	    }
	    set tt1 [wordtobits 00000008]
	    set tt2 [bitwordset $tt1 5 $fec_trigger_source]
	    set tt2 [bitwordset $tt2 6 $norm_loop_on]
	    set tt3 [bitstoword $tt2]
	    
	    sendpcifectriggerregister 3
	    updatefeccmdindicators 1
	}
    button .fec.command.row2.f1.ftop.b2 -text "RTBM" -width 5 -font {Helvetica 10 {bold}} \
	-command {
	    if {$verbose3=="1"} {
		puts "SENDING RTBM TO FEC"
	    }
	    set tt1 [wordtobits 00000004]
	    set tt2 [bitwordset $tt1 5 $fec_trigger_source]
	    set tt2 [bitwordset $tt2 6 $norm_loop_on]
	    set tt3 [bitstoword $tt2]
	    sendpcifectriggerregister 2
	    updatefeccmdindicators 1
	}
    button .fec.command.row2.f1.ftop.b3 -text "CAL LOOP" -width 12 -font {Helvetica 10 {bold}} \
	-command {
	    toggle_encalpulse 1
	    if {$verbose3=="1"} {
		puts "Changing encalpulse"
	    }
	    if {$callatencybit == "enabled"} {
		set temp1 "1"
	    } else {
		set temp1 "0"
	    }
	    
	    set temp2 "100$temp1"
	    set temp3 "001"
	    set temp4 "$temp2$temp3"
	    
	    if {$callatencybit == "enabled"} {
		set tt1 [wordtobits 00000010]
		set tt2 [bitwordset $tt1 7 $encalpulse]
		set tt2 [bitwordset $tt2 5 $fec_trigger_source]
		set tt2 [bitwordset $tt2 6 $norm_loop_on]
		set tt3 [bitstoword $tt2]
		sendpcifectriggerregister -1
	    } else {
		set tt1 [wordtobits 00000000]
		set tt1 [bitwordset $tt1 7 $encalpulse]
		set tt2 [bitwordset $tt1 5 $fec_trigger_source]
		set tt2 [bitwordset $tt2 6 $norm_loop_on]
		set tt3 [bitstoword $tt2]
		sendpcifectriggerregister -1
	    }
	    updatefeccmdindicators 1
    }

    button .fec.command.row2.f1.fbot.b1 -text "NORM" -width 5 -font {Helvetica 10 {bold}} \
	-command {
	    if {$verbose3=="1"} {
		puts "SENDING NORM TRIG REQUEST TO FEC"
	    }
	    set temp2 "0110100"
	    set tt1 [wordtobits 00000002]
	    set tt2 [bitwordset $tt1 7 $encalpulse]
	    set tt2 [bitwordset $tt2 5 $fec_trigger_source]
	    set tt2 [bitwordset $tt2 6 $norm_loop_on]
	    set tt3 [bitstoword $tt2]
	    sendpcifectriggerregister 1
	    updatefeccmdindicators 1
	}
    button .fec.command.row2.f1.fbot.b2 -text "RROC" -width 5 -font {Helvetica 10 {bold}} \
	-command {
	    if {$verbose3=="1"} {
		puts "SENDING ResetROC TO FEC"
	    }
	    set temp2 "0111000"
	    set tt1 [wordtobits 00000001]
	    set tt2 [bitwordset $tt1 7 $encalpulse]
	    set tt2 [bitwordset $tt2 5 $fec_trigger_source]
	    set tt2 [bitwordset $tt2 6 $norm_loop_on]
	    set tt3 [bitstoword $tt2]
	    sendpcifectriggerregister 0
	    updatefeccmdindicators 1
	}

    button .fec.command.row2.f1.fbot.b3 -text "NORMLOOP" -width 12 -font {Helvetica 10 {bold}} \
	-command {
	    toggle_norm_loop_on 1
	    sendpcifectriggerregister -1
	    updatefeccmdindicators 1
	}
    pack .fec.command.row2.f1.ftop.b1 .fec.command.row2.f1.ftop.b2 \
	.fec.command.row2.f1.ftop.b3 -side left
    pack .fec.command.row2.f1.fbot.b1 .fec.command.row2.f1.fbot.b2 \
	.fec.command.row2.f1.fbot.b3 -side left
    pack .fec.command.row2.f1.ftitle .fec.command.row2.f1.ftop \
	.fec.command.row2.f1.fbot

    frame .fec.command.row2.f2 -relief groove -borderwidth 4
    frame .fec.command.row2.f2.ftitle -relief flat
    frame .fec.command.row2.f2.ftop
    frame .fec.command.row2.f2.fbot
    label .fec.command.row2.f2.ftitle.l1 -text "TTC Control" -font {Helvetica 10 {bold}}
    pack .fec.command.row2.f2.ftitle.l1
    if {$fec_trigger_source == 1} {
	set initialtriggertext "TRIG Disabled"
    } else {
	set initialtriggertext "TRIG Enabled"
    }
    button .fec.command.row2.f2.ftop.b1 -text $initialtriggertext \
	-width 20 -font {Helvetica 10 {bold}} -command {
	toggle_trigger_source 1
	sendpcifectriggerregister -1
	    updatefeccmdindicators 1
    }
    
    set disableautosend 1
    button .fec.command.row2.f2.fbot.b1 -text "AutoSend Disabled" -width 20 \
	-font {Helvetica 10 {bold}} -command {
	    if {$disableautosend == "0"} {
		set disableautosend 1
		.fec.command.row2.f2.fbot.b1 configure \
		    -text "AutoSend Disabled"
	    } else {
		set disableautosend 0
		.fec.command.row2.f2.fbot.b1 configure \
		    -text "AutoSend Enabled"
	    }
	    if {$slowfecchannel == "1"} {
		pcifeccmdreg 6 $disableautosend
	    } elseif {$slowfecchannel == "2"} {
		pcifeccmdreg 22 $disableautosend
	    }
	    updatefeccmdindicators 1
	}
    pack .fec.command.row2.f2.ftop.b1 .fec.command.row2.f2.fbot.b1
    pack .fec.command.row2.f2.ftitle
    pack .fec.command.row2.f2.ftop 
    pack .fec.command.row2.f2.fbot 

# now do led table
set redled "
R0lGODlhEAAPAOcAAAAAAIAAAACAAICAAAAAgIAAgACAgICAgMDcwKbK8JkA
AMQAJv9mZv+Zmf/MzFMAXYFUkKWGs9S+3gAoVgA9hFxqpo+SwMzM/wBBawBj
pFmEvY+n0cbW8wAAiAAAqjOZ/47N8Nnu+SRNDDJsEU2mGYzAbLPTm0RZAF99
AJLAALjSa9HfnFBQHmZmM5mZZszMmf//1nlwAKqdAOjSWP/rAP/wnm5HAJpk
AP9mAP+ZAPezV//MM1Y1DW5KIKZ+Tsefbf/Mmebm5tnZ2b+/v6amppmZmYyM
jGZmZkBAQG+MnP//////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
///////////////////////////////////////////78KCgpICAgP8AAAD/
AP///////////////////yH5BAEAAP8ALAAAAAAQAA8AAAheAP8JHEiwoMGD
CDcAWAggCcJ/SQAsmLigocOCChfgwKGxYpKLA49oVKKEI8cjHwlKxEFy40SL
KjVuNOkR5D+RNDkuQGkzI82aNiFKpGgxqECFDD8aJah06cOnUP8FBAA7
"

set redledoff "
R0lGODlhEAAPAOcAAAAAAIAAAACAAICAAAAAgIAAgACAgICAgMDcwKbK8JkA
AMQAJv9mZv+Zmf/MzFMAXYFUkKWGs9S+3gAoVgA9hFxqpo+SwMzM/wBBawBj
pFmEvY+n0cbW8wAAiAAAqjOZ/47N8Nnu+SRNDDJsEU2mGYzAbLPTm0RZAF99
AJLAALjSa9HfnFBQHmZmM5mZZszMmf//1nlwAKqdAOjSWP/rAP/wnm5HAJpk
AP9mAP+ZAPezV//MM1Y1DW5KIKZ+Tsefbf/Mmebm5tnZ2b+/v6amppmZmYyM
jGZmZkBAQG+MnP//////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
///////////////////////////////////////////78KCgpICAgP8AAAD/
AP///////////////////yH5BAEAAP8ALAAAAAAQAA8AAAhTAP8JHEiwoMGD
CDcAWAggCcJ/SQAEmBigocOCCilStFjwyMQFGgMcSXJRoMSQEzkOPIlSpUCP
KEWSJJgxpMWSAiNqvHlQIUOSOA0CDfqwqNGBAQEAOw==
"

set greenled "
R0lGODlhEAAPAPf1AAAAAIAAAACAAICAAAAAgIAAgACAgICAgMDcwKbK8JkA
AMQAJv9mZv+Zmf/MzFMAXYFUkKWGs9S+3gAoVgA9hFxqpo+SwMzM/wBBawBj
pFmEvY+n0cbW8wAAiAAAqjOZ/47N8Nnu+SRNDDJsEU2mGYzAbLPTm0RZAF99
AJLAALjSa9HfnFBQHmZmM5mZZszMmf//1nlwAKqdAOjSWP/rAP/wnm5HAJpk
AP9mAP+ZAPezV//MM1Y1DW5KIKZ+Tsefbf/Mmebm5tnZ2b+/v6amppmZmYyM
jGZmZkBAQG+MnP//////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
///////////////////////////////////////////78KCgpICAgP8AAAD/
AP//AAAA//8A/wD//wAAACH5BAEAAP0ALAAAAAAQAA8AAAheAPsJHEiwoMGD
CDcAWAggCcJ+SQAImCigocOCCgXo06exYpKLA49oVKKEI8cjHwlK1Edy40SL
KjVuNOkRZD+RNDkKQGkzI82aNiFKpGgxqECFDD8aJah06cOnUPsFBAA7
"

set greenledoff "
R0lGODlhEAAPAMIAAI+n0QAAAG+MnACAAGZmZgD/AP///////yH5BAEAAAcA
LAAAAAAQAA8AAAM1eLrc/hCEGQQ8Iow9qm0Sx3kNsRXiQAifoqUbubywrJiw
yjJh6rWKjOj3kFBYQAcyeWk6FwkAOw==
"

set yellowled "
R0lGODlhEAAPAOcAAAAAAIAAAACAAICAAAAAgIAAgACAgICAgMDcwKbK8JkA
AMQAJv9mZv+Zmf/MzFMAXYFUkKWGs9S+3gAoVgA9hFxqpo+SwMzM/wBBawBj
pFmEvY+n0cbW8wAAiAAAqjOZ/47N8Nnu+SRNDDJsEU2mGYzAbLPTm0RZAF99
AJLAALjSa9HfnFBQHmZmM5mZZszMmf//1nlwAKqdAOjSWP/rAP/wnm5HAJpk
AP9mAP+ZAPezV//MM1Y1DW5KIKZ+Tsefbf/Mmebm5tnZ2b+/v6amppmZmYyM
jGZmZkBAQG+MnP//////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
///////////////////////////////////////////78KCgpICAgP8AAAD/
AP///////////////////yH5BAEAAP8ALAAAAAAQAA8AAAheAP8JHEiwoMGD
CDcAWAggCcJ/SQDsmLijocOCCnfQoKGxYpKLA49otGePI8cjHwlKpEFy40SL
KjVuNOkR5D+RNDnuQGkzI82aNiFKpGgxqECFDD8aJah06cOnUP8FBAA7
"

set yellowledoff "
R0lGODlhEAAPAOcAAAAAAIAAAACAAICAAAAAgIAAgACAgICAgMDcwKbK8JkA
AMQAJv9mZv+Zmf/MzFMAXYFUkKWGs9S+3gAoVgA9hFxqpo+SwMzM/wBBawBj
pFmEvY+n0cbW8wAAiAAAqjOZ/47N8Nnu+SRNDDJsEU2mGYzAbLPTm0RZAF99
AJLAALjSa9HfnFBQHmZmM5mZZszMmf//1nlwAKqdAOjSWP/rAP/wnm5HAJpk
AP9mAP+ZAPezV//MM1Y1DW5KIKZ+Tsefbf/Mmebm5tnZ2b+/v6amppmZmYyM
jGZmZkBAQG+MnP//////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
///////////////////////////////////////////78KCgpICAgP8AAAD/
AP///////////////////yH5BAEAAP8ALAAAAAAQAA8AAAhaAP8JHEiwoMGD
CDcAWAggCcJ/SQC0mNiiocOCCluwYEHRYsEjGmlsnMjiSJKLAgFsXLnS40AA
SFi2PEnwSEyZJWkOVIjTIkqBEVn6PKiQ4cmfBo8ifci06cCAADs=
"

#image create photo ledonimage -format GIF -file redled.gif
#image create photo ledoffimage -format GIF -file redledoff.gif
image create photo redledonimage -format GIF -data $redled
image create photo redledoffimage -format GIF -data $redledoff
image create photo greenledonimage -format GIF -data $greenled
image create photo greenledoffimage -format GIF -data $greenledoff
image create photo yellowledonimage -format GIF -data $yellowled
image create photo yellowledoffimage -format GIF -data $yellowledoff


    frame .fec.command.row2.f3
    frame .fec.command.row2.f3.r0
    frame .fec.command.row2.f3.r1
    frame .fec.command.row2.f3.r2
    frame .fec.command.row2.f3.r3
    frame .fec.command.row2.f3.r4
    label .fec.command.row2.f3.r0.l1 -text "CH1 CH2 -------------------" \
	-width 25 -font {Helvetica 8 {bold}}
    pack .fec.command.row2.f3.r0.l1 -side left

    label .fec.command.row2.f3.r1.led0 -image redledoffimage
    label .fec.command.row2.f3.r1.led1 -image redledoffimage
    label .fec.command.row2.f3.r1.l1 -text "Send Started" -width 16 \
	-font { Helvetica 8 {bold}}
    pack .fec.command.row2.f3.r1.led0 .fec.command.row2.f3.r1.led1 \
	.fec.command.row2.f3.r1.l1 -side left

    set fecleds .fec.command.row2.f3

    label .fec.command.row2.f3.r2.led0 -image redledoffimage
    label .fec.command.row2.f3.r2.led1 -image redledoffimage
    label .fec.command.row2.f3.r2.l1 -text "Rcv Complete" -width 16 \
	-font { Helvetica 8 {bold}}
    pack .fec.command.row2.f3.r2.led0 .fec.command.row2.f3.r2.led1 \
	.fec.command.row2.f3.r2.l1 -side left

    label .fec.command.row2.f3.r3.led0 -image redledoffimage
    label .fec.command.row2.f3.r3.led1 -image redledoffimage
    label .fec.command.row2.f3.r3.l1 -text "Hub Addr Err" -width 16 \
	-font { Helvetica 8 {bold}}
    pack .fec.command.row2.f3.r3.led0 .fec.command.row2.f3.r3.led1 \
	.fec.command.row2.f3.r3.l1 -side left

    label .fec.command.row2.f3.r4.led0 -image redledoffimage
    label .fec.command.row2.f3.r4.led1 -image redledoffimage
    label .fec.command.row2.f3.r4.l1 -text "Rcv Fail" -width 16 \
	-font { Helvetica 8 {bold}}
    pack .fec.command.row2.f3.r4.led0 .fec.command.row2.f3.r4.led1 \
	.fec.command.row2.f3.r4.l1 -side left

    pack .fec.command.row2.f3.r0
    pack .fec.command.row2.f3.r1
    pack .fec.command.row2.f3.r2
    pack .fec.command.row2.f3.r3
    pack .fec.command.row2.f3.r4

# now done with first two rows - non-expert commands
    frame .fec.expert
    frame .fec.expert.f1
    frame .fec.expert.f1.ftop
    scrollbar .fec.expert.f1.ftop.sbar -command { .fec.expert.f1.ftop.text \
						      yview }
    text .fec.expert.f1.ftop.text -width 8 -height 16 \
	-yscrollcommand { .fec.expert.f1.ftop.sbar set }
    pack .fec.expert.f1.ftop.sbar -side right -fill y
    pack .fec.expert.f1.ftop.text -side left -expand yes -fill both
    set textwindow .fec.expert.f1.ftop.text
    
    frame .fec.expert.f1.fbot
    button .fec.expert.f1.fbot.b1 -width 7 -text "Clear" -font { Helvetica 10 {bold}} \
	-command {
	    set linecount 1
	    $textwindow configure -state normal
	    $textwindow delete 1.0 end
	    $textwindow configure -state disabled
	    $textwindow configure -state normal
	}
    pack .fec.expert.f1.fbot.b1

    pack .fec.expert.f1.ftop
    pack .fec.expert.f1.fbot

    frame .fec.expert.f2
    frame .fec.expert.f2.r0
    label .fec.expert.f2.r0.l1 -text "Port:" -font { Helvetica 10 {bold}}
    set uport 4
    entry .fec.expert.f2.r0.e1 -textvariable uport -width 1 \
	-font { Helvetica 10 {bold}}

    label .fec.expert.f2.r0.l2 -text "Data:" -font { Helvetica 10 {bold}}
    pack .fec.expert.f2.r0.l1  .fec.expert.f2.r0.e1 .fec.expert.f2.r0.l2 \
	-side left
    
    frame .fec.expert.f2.r0.data
    set i 0
    while { $i < 10 } {
	entry .fec.expert.f2.r0.data.e$i -textvariable dataentry($i) \
	    -width 2 -font { Helvetica 10 {bold}}
	pack .fec.expert.f2.r0.data.e$i -side left
	incr i
    }
    pack .fec.expert.f2.r0.data -side left

    label .fec.expert.f2.r0.l3 -text "Num:" -font { Helvetica 10 {bold}}
    entry .fec.expert.f2.r0.e3 -textvariable num -width 2 \
	-font { Helvetica 10 {bold}}
    
    pack .fec.expert.f2.r0.l3 .fec.expert.f2.r0.e3 -side left

    frame .fec.expert.f2.r1
    button .fec.expert.f2.r1.b1 -text "Load" \
	-font { Helvetica 10 {bold}} -width 10 -command {
	    sendresetpointer 1
	    set lasthubaddress -1
	    bithubportaddress 1
	    set temp1 [inttobits $hubaddress 5]
	    set temp2 [inttobits $uport 3]
	    set temp3 "$temp1$temp2"
	    set temp4 [bitstoint $temp3]
	    set temp5 [inttobits $temp4 4]
	    set tlow $temp5
	    set temp6 [expr $temp4 / 16]
	    set temp7 [inttobits $temp6 4]
	    set thigh $temp7
	    # Send the hub and port address
	    sendbyte2 "$thigh" "$tlow"
	    # Send the number of bytes (num) of data to follow
	    sendbyte2 "$highnumbits" "$lownumbits"
	    set i 0
	    while { $i < $inum } {
		set ttemp $dataentry($i)
		set thigh [hexhigh $ttemp]
		set tlow [hexlow $ttemp]
		sendbyte2 "$thigh" "$tlow"
		incr i
	    }
	    sendzero 1
	    sendresetpointer 1
	    # send get parity
	    #    set checkingparity 1
	    #    sendbyte "11110100"
	    resetparitybyte
	    ######    fixnibblearray 1
	    sendnibblearray 0
	    clearnibblearray "0000"
	    ##    flush $usbdeviceid
	    #    $uwin.f4.b1 config -image but0
	    updatefeccmdindicators 1
	}    
    
    
    button .fec.expert.f2.r1.b2 -text "Send" \
	-font { Helvetica 10 {bold}} -width 5  -command {
	    set dostopsendgo 1
	    # sendgo for FEC 1 is in place
	    sendgo 1
	    updatefeccmdindicators 1
	}

    button .fec.expert.f2.r1.b3 -font { Helvetica 10 {bold}} \
	-text "Send Continuously" -width 20 -command {

        .fec.expert.f2.r1.b3 configure -fg "\#00ff00" -activeforeground "\#00ff00"
        .fec.expert.f2.r1.b4 configure -fg "\#ff0000" -activeforeground "\#ff0000"

	    set dostopsendgo 0
	    sendgo 1
	}
    
    button .fec.expert.f2.r1.b4 -text "Stop" \
	-font { Helvetica 10 {bold}} -width 5 -command {
        .fec.expert.f2.r1.b3 configure -fg "\#000000" -activeforeground "\#000000"
        .fec.expert.f2.r1.b4 configure -fg "\#000000" -activeforeground "\#000000"
	    set dostopsendgo 1
	    updatefeccmdindicators 1
	}

    pack .fec.expert.f2.r1.b1 .fec.expert.f2.r1.b2 .fec.expert.f2.r1.b3 \
	.fec.expert.f2.r1.b4 -side left


    frame .fec.expert.f2.r2
    button .fec.expert.f2.r2.b1 -text "Read 16 bytes from WR buf" \
	-font { Helvetica 10 {bold}}  -width 30 -command {
	    clearnibblearray "0000"
	    #    if {$slowfecchannel == "1"} {
	    #	pcifeccmdreg 1 1
	    #    } elseif {$slowfecchannel == "2"} {
	    #	pcifeccmdreg 17 1
	    #    }
	    for {set k 0} {$k<16} {set k [expr $k+4]} {
		if {$slowfecchannel == "1"} {
		    set returnedword [pcifecreadword 0]
		} else {
		    set returnedword [pcifecreadword 16]
		}
		set returnedword [padhexword $returnedword]
		set tempusbreaddata "$returnedword"
		set mytempstring [string range $tempusbreaddata 6 7]
		set myk [expr $k + $textwindowline]
		if {$myk < 10} {
		    $textwindow insert 1.0 "$myk  $mytempstring\n"
		} else {
		    $textwindow insert 1.0 "$myk $mytempstring\n"
		}	    
		set mytempstring [string range $tempusbreaddata 4 5]
		set myk [expr $k+1 + $textwindowline ]
		if {$myk < 10} {
		    $textwindow insert 1.0 "$myk  $mytempstring\n"
		} else {
		    $textwindow insert 1.0 "$myk $mytempstring\n"
		}	    
		set mytempstring [string range $tempusbreaddata 2 3]
		set myk [expr $k+2 + $textwindowline]
		if {$myk < 10} {
		    $textwindow insert 1.0 "$myk  $mytempstring\n"
		} else {
		    $textwindow insert 1.0 "$myk $mytempstring\n"
		}	    
		set mytempstring [string range $tempusbreaddata 0 1]
		set myk [expr $k+3 + $textwindowline]
		if {$myk < 10} {
		    $textwindow insert 1.0 "$myk  $mytempstring\n"
		} else {
		    $textwindow insert 1.0 "$myk $mytempstring\n"
		}	    
	    }
	    set textwindowline [expr $textwindowline + 16]
	    
	    updatefeccmdindicators 1
	}
    button .fec.expert.f2.r2.b2 -text "Dump Buffer to file" \
	-width 20 -font { Helvetica 10 {bold}} -command {
	    dumpfecregisters 1
	}
    pack .fec.expert.f2.r2.b1 .fec.expert.f2.r2.b2 -side left

    frame .fec.expert.f2.r3
    button .fec.expert.f2.r3.b1 -font { Helvetica 10 {bold}} \
	-width 30 -text "Read 16 bytes from RD buf" -command {
	    clearnibblearray "0000"
	    #    if {$slowfecchannel == "1"} {
	    #	pcifeccmdreg 2 1
	    #    } elseif {$slowfecchannel == "2"} {
	    #	pcifeccmdreg 18 1
	    #    }
	    for {set k 0} {$k<16} {set k [expr $k+4]} {
		if {$slowfecchannel == "1"} {
		    set returnedword [pcifecreadword 4]
		} else {
		    set returnedword [pcifecreadword 20]
		}
		set returnedword [padhexword $returnedword]
		set tempusbreaddata "$returnedword"
		set mytempstring [string range $tempusbreaddata 6 7]
		set myk [expr $k + $textwindowline]
		if {$myk < 10} {
		    $textwindow insert 1.0 "$myk  $mytempstring\n"
		} else {
		    $textwindow insert 1.0 "$myk $mytempstring\n"
		}	    
		set mytempstring [string range $tempusbreaddata 4 5]
		set myk [expr $k+1 + $textwindowline]
		if {$myk < 10} {
		    $textwindow insert 1.0 "$myk  $mytempstring\n"
		} else {
		    $textwindow insert 1.0 "$myk $mytempstring\n"
		}	    
		set mytempstring [string range $tempusbreaddata 2 3]
		set myk [expr $k+2 + $textwindowline]
		if {$myk < 10} {
		    $textwindow insert 1.0 "$myk  $mytempstring\n"
		} else {
		    $textwindow insert 1.0 "$myk $mytempstring\n"
		}	    
		set mytempstring [string range $tempusbreaddata 0 1]
		set myk [expr $k+3 + $textwindowline]
		if {$myk < 10} {
		    $textwindow insert 1.0 "$myk  $mytempstring\n"
		} else {
		    $textwindow insert 1.0 "$myk $mytempstring\n"
		}	    
	    }
	    set textwindowline [expr $textwindowline + 16]
	    updatefeccmdindicators 1
	}
    button .fec.expert.f2.r3.b2 -text "CLR BUF" \
	-width 20 -font { Helvetica 10 {bold}} -command {
	    
	    toplevel .clrfec -class Dialog 
	    wm title .clrfec "Zeroing FEC buffers..."
	    set width [winfo screenwidth .]
	    set height [winfo screenheight .]
	    set x [expr ($width/2) - 200]
	    set y [expr ($height/2) - 200]
	    
	    wm geometry .clrfec  +$x+$y
	    gauge_create .clrfec.g Pink
	    pack .clrfec.g -expand yes -fill both -padx 50 -pady 10
	    
	    # zero all 1k of the wr buff (0) and 2k for rd buff (4) (compr buf
	    # now shared with wr buf
	    for {set i 0} {$i < 512} {incr i} {
		if {$i < 256} {
		    pcifecwriteword 0 00000000
		}
		set iprint [expr $i/2]
##		set tteb [inttobits $iprint 8]
##		set tteb "$tteb$tteb$tteb$tteb"
##		set tteb [bitstoword $tteb]
		pcifecwriteword 4 00000000
##		pcifecwriteword 4 $tteb
		if {$i < 256} {
		    pcifecwriteword 16 00000000
		}
		pcifecwriteword 20 00000000
		gauge_value .clrfec.g [expr 100*$i/512.0]
	    }
	    destroy .clrfec.g
	    destroy .clrfec
	}
    pack .fec.expert.f2.r3.b1 .fec.expert.f2.r3.b2 -side left

    frame .fec.expert.f2.r4
    label .fec.expert.f2.r4.l1 -text "CLK Phase:" -font {Helvetica 10 {bold}}
    tk_optionMenu .fec.expert.f2.r4.option1 \
	clkph 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15
    button .fec.expert.f2.r4.l2 -text "SlowHubData" \
	-font { Helvetica 10 {bold}} -command {
	    # read file and fill byte array and set inum (int) and num (hex)
	    makeslowhubdata 1
	    bithubportaddress 1
	    puts "Makeslowhubdata"
	}
    button .fec.expert.f2.r4.l3 -text "Read Data File" \
	-font { Helvetica 10 {bold}} -command {
	    # read file and fill byte array and set inum (int) and num (hex)
	    loadrawdata 1
	    bithubportaddress 1
	    puts "READ DATA FROM FILE to FEC out memory"
	}
    pack .fec.expert.f2.r4.l1 .fec.expert.f2.r4.option1 \
	.fec.expert.f2.r4.l2 .fec.expert.f2.r4.l3 -side left

    frame .fec.expert.f2.r5
    set addresslookup(FEC0WRITE) 0
    set addresslookup(FEC0READ) 4
    set addresslookup(FEC0STATUSREG2) 8
    set addresslookup(FECCONTROLSTATUS) 12
    set addresslookup(FEC1WRITE) 16
    set addresslookup(FEC1READ) 20
    set addresslookup(FEC1STATUSREG2) 24
    set addresslookup(FECTRIGGER) 28
    tk_optionMenu .fec.expert.f2.r5.option1 \
	fecaddress  FEC0WRITE FEC0READ FEC0STATUSREG2 FECCONTROLSTATUS \
	FEC1WRITE FEC1READ FEC1STATUSREG2 FECTRIGGER
    button .fec.expert.f2.r5.b1 -text "WRITE" \
	-font { Helvetica 10 {bold}} -command {
	    set intfecaddress $addresslookup($fecaddress)
	    if {$verbose3 > 0} {
		puts "mFEC WR ADR=$intfecaddress ($fecaddress) $writehexdata"
	    }
	    pcifecwriteword $intfecaddress $writehexdata
	    updatefeccmdindicators 1
	}

    label .fec.expert.f2.r5.l1 -text "DATA:" -font { Helvetica 10 {bold}}
    entry .fec.expert.f2.r5.e1 -textvariable writehexdata \
	-width 8 -font { Helvetica 10 {bold}}
    button .fec.expert.f2.r5.b2 -text "READ" \
	-font { Helvetica 10 {bold}} -command {
	    set intfecaddress $addresslookup($fecaddress)
	    if {$verbose3 > 0} {
		puts "mFEC READ ADDRESS=$intfecaddress ($fecaddress)"
	    }
	    set returnedword [pcifecreadword $intfecaddress]
	    set returnedword [padhexword $returnedword]
	    set readhexdata $returnedword
	    if {$verbose3 > 0} {
		puts "VALUE RETURNED=$readhexdata"
	    }
	    updatefeccmdindicators 1
	}
    label .fec.expert.f2.r5.l2 -text "DATA:" -font { Helvetica 10 {bold}}
    entry .fec.expert.f2.r5.e2 -textvariable readhexdata \
	-width 8 -font { Helvetica 10 {bold}}

    pack .fec.expert.f2.r5.option1 .fec.expert.f2.r5.b1 .fec.expert.f2.r5.l1 \
	.fec.expert.f2.r5.e1 .fec.expert.f2.r5.b2 \
	.fec.expert.f2.r5.l2 .fec.expert.f2.r5.e2 -side left

    pack .fec.expert.f2.r0
    pack .fec.expert.f2.r1
    pack .fec.expert.f2.r2
    pack .fec.expert.f2.r3
    pack .fec.expert.f2.r4
    pack .fec.expert.f2.r5

    pack .fec.expert.f1 .fec.expert.f2 -side left

    pack .fec.command.row2.f1 .fec.command.row2.f2 .fec.command.row2.f3 \
	-side left
    pack .fec.command.row1 .fec.command.row2 -fill x


    pack .fec.command
#    pack .fec.expert

    balloon_help .fec.command.row2.f1.ftitle "FEC Inject Commands\nCSRE: Cal/Sync/RST EVENT (D3)\nRTBM: Reset TBM (D2)\nNORM: Normal Trig (D1)\nRROC: Reset ROC (D0)"
# --

    set i 0
    while { $i < 10 } {
	set dataentry($i) "0$i"
	#    set dataentry($i) "00"
	incr i
    }

    set num 01
    resetparitybyte



    set width [winfo screenwidth .]
    set height [winfo screenheight .]
    set x [expr ($width/2) - 200]
    set y [expr ($height/2) - 200]
    
    wm geometry .fec  +$x+$y
    wm resizable .fec 0 0
    
    update


#    after 10000
#    destroy .fec
}

set fecexpertshown 0
proc togglefecexpert { } {
    global fecexpertshown
    
    if {$fecexpertshown > 0} {
	set fecexpertshown 0
	if {[winfo exists .fec] == 1} {	pack forget .fec.expert }
    } else {
	set fecexpertshown 1
	if {[winfo exists .fec] == 1} { 
	    pack .fec.expert 
	}
    }
}


proc getproperties { command } {
    global lptport
    global portspeed
    global forgetpixprestopbit
    global datalength
    global balloons
    global FECSDA
    global callatencybit
    global localFECSDA
    global wdatalength

    toplevel .top -class Dialog 
    wm title .top "Properties"
    label .top.label -text "LPT Port Select: "
    tk_optionMenu .top.option iport 0x378 0x278 0x3BC
    set iport $lptport

    label .top.label2 -text "LPT Slow Clocking debug mode: \n(Really Slow: change discouraged) "

    set lspeed $portspeed

    checkbutton .top.checkbutton1 -variable lspeed -onvalue "555" \
	-offvalue "0" 

    if { $portspeed == "555" } {
	.top.checkbutton1 select
    }


    label .top.label3 -text "Last ACK (9th bit) to pix before i2c stop? \n(seek advice: change discouraged)"
    tk_optionMenu .top.option3  forgetbit high low drop
    set forgetbit $forgetpixprestopbit

    label .top.label4 -text "Data format length (bits): \n(seek advice: change discouraged)  "
    tk_optionMenu .top.option4  wdatalength 10 8
    set wdatalength $datalength

    label .top.label5 -text "Mouseover Balloon Help:"
    tk_optionMenu .top.option5  balloons "ON" "OFF"

    label .top.label6 -text " SDA Adjust (Expert\nsetting, FEC param) "
    tk_optionMenu .top.option6  localFECSDA 0 1 2 3 4 5 6 7
    set localFECSDA $FECSDA

    button .top.button -text "Set" -command { 
	switch $iport {
	    0x378 { set lptport 1 }
	    0x278 { set lptport 2 }
	    0x3BC { set lptport 0 }
	}
	set portspeed $lspeed
	set forgetpixprestopbit $forgetbit
	set datalength $wdatalength
	set FECSDA $localFECSDA
	puts "Set lptport: $lptport (0=0x3BC;1=0x378;2=0x278)"
	puts "Set portspeed: $portspeed"
	puts "Set dropped pix pre stop bit: $forgetpixprestopbit"
	puts "Set 10/8 bit data format to $datalength bit"
	puts "Set FEC SDA Adjust bit to $FECSDA"
	if {$callatencybit == "enabled"} {
	    set temp1 "1"
	} else {
	    set temp1 "0"
	}
	set sdatemp "000"
	set temp2 "100$temp1$sdatemp"
	.windac.col3.f66.d1.option1 configure -text $callatencybit
	sendbytewparity $temp2
	destroy .top
    }


    pack .top.label .top.option
    pack .top.label2 .top.checkbutton1

    pack .top.label3 .top.option3 
    pack .top.label4 .top.option4 
    pack .top.label5 .top.option5
    pack .top.label6 .top.option6

##    pack .top.button0 .top.button -side left
    pack .top.button

    set width [winfo screenwidth .]
    set height [winfo screenheight .]
    set x [expr ($width/2) - 200]
    set y [expr ($height/2) - 200]

    wm geometry .top  +$x+$y
    wm resizable .top 0 0
}

proc handleadc { myfileid } {
    global dostop
    global dataarray
    global adcrunmode
    global ioutevents
    global suppressdisplay
    global outformat

    global cosmoipcscripting

    set results [gets $myfileid]
    set icount $results
    set i 0
    while { $i < $icount } {
	set results [gets $myfileid]
	set dataarray($i) $results
	incr i
    }
    storedata $icount

    if {$suppressdisplay != "1"} {
	adcdrawdata2 .adc.f1.can1 0 150
    }

    if {$adcrunmode != "Continuous"} {
	.adc.f4.but1b config -text "Run\n($ioutevents/$adcrunmode)"
	if {$ioutevents >= $adcrunmode } {
	    set dostop 1
	    if {$cosmoipcscripting == 1} {
		update
		update idletasks
#		after 20
#		update
#		update idletasks
######		send cosmoscripter { nextpass 1 }
	    }
	}
    } else {
	.adc.f4.but1b config -text "Run\n($ioutevents)"
    }
    update 
    if {$dostop != 1} {
	if {$outformat == "binary2k"} {
	    puts $myfileid "3 2048 1000 1"
	} elseif {$outformat == "text1k"} {
	    puts $myfileid "3 1024 1000 1"
	} else {
	    puts $myfileid "3 256 1000 1"
	}
	flush $myfileid
    }
}

set pcifec ""
proc pcifecinit { mycommand } {
    global pcifec
    set pcifec [open "|/usr/local/cosmo/bin/cosmofechelper" "r+"]
}
pcifecinit 1

proc bitwordset { mybits mybitposition mybitvalue } {
# specify bit position counting from right, LSB
    set mybitstring ""

    set bitposition [expr 31-$mybitposition]
    for {set ii 0} {$ii < 32} {incr ii} {
	set t1 [string index $mybits $ii]
	if {$bitposition == $ii} {
	    if {$t1 != $mybitvalue} {
		set t1 $mybitvalue
	    }
	}
	append mybitstring $t1
    }
    return $mybitstring
}

proc wordtobits { myword } {
    global verbose3
    set mybitstring ""

    for {set ii 0} {$ii < 8} {incr ii} {
	set t1 [string index $myword $ii]
	set t2 [hextoint $t1]
	set t3 [inttobits $t2 4]
	append mybitstring $t3
    }
    if {$verbose3 > 0} { puts "WORDTOBITS:$mybitstring" }
    return $mybitstring
}
proc bitstoword { mybits } {
    global verbose3
    set myhexstring ""

    for {set ii 0} {$ii<8} {incr ii} {
	set lowindex [expr $ii*4]
	set highindex [expr $ii*4 + 3]
	set t1 [string range $mybits $lowindex $highindex]
	set t2 [bitstohex $t1]
	append myhexstring $t2
    }
    if {$verbose3 > 0} { puts "BITSTOWORD:$myhexstring" }
    return $myhexstring
}


# clear sendssid array and reset to set each once
for {set ii 1} {$ii <= 21} {incr ii} {
    set ssidsent($ii) 0
}
set lastvmeslot -1
proc setssid { slot } {
    global vmeslot mfec
    global pcifec
    global usevme
    set vmeslot $slot
    if {$usevme == 1} {
	set oldmfec $mfec
	vmefecsetaddresses $vmeslot 8
	puts "WRITING SSID register of FEC (vme slot $vmeslot) for PIXELS"
	# 772 is 304hex
	set ttemp5 [pcifecreadword 772]
	set ttemp5word [padhexword $ttemp5]
	if {$ttemp5word == "ffffffff" } {
	    set result [tk_messageBox -parent . \
			    -title { Error } -type ok \
			    -icon info \
			    -message "WARNING:  Does not appear to be\na VME FEC in slot:$vmeslot\n\nProceeding anyway..." ]
	}

	set tt0 [padhexword $ttemp5]
	set tt1 [wordtobits $tt0]
	puts "Original Register: $tt1"
	set tt1 [bitwordset $tt1 14 1]
	set tt1 [bitwordset $tt1 13 0]
	set tt2 [bitwordset $tt1 12 0]
	puts "Modified Register: $tt2"
	set tt3 [bitstoword $tt2]
	pcifecwriteword 772 $tt3
	# return to vmeslot and mfec
	vmefecsetaddresses $vmeslot $oldmfec
    }
}


proc pcifecwriteword { myoffset myword } {
    global verbose3
    global pcifec
    global usevme
    global vmeslot mfec
    global ssidsent
    global lastvmeslot
    if {$usevme == 1} {
	vmefecsetaddresses $vmeslot $mfec
	if {$vmeslot != $lastvmeslot} { #addressing diff card, check ssid
	    if {$ssidsent($vmeslot) == 0} {
		# havent set its ssid, so set it
		set lastvmeslot $vmeslot
		setssid $vmeslot
		set ssidsent($vmeslot) 1
	    }
	}
    }
    puts $pcifec "3 $myoffset $myword"
    flush $pcifec
    if {$verbose3 > 0} { puts "pcifecwriteword: $myoffset $myword" }
    gets $pcifec inputline
    if {$verbose3 > 0} { puts "GOT:$inputline" }
    scan $inputline "%d" myreturn
    # myreturn of -1 is write fail (ie, incorrect vmeslot)
    if {$verbose3 > 0} { puts "WRITE success ($myreturn)" }
}

proc pcifecreadword { myoffset } {
    global pcifec
    global usevme
    global vmeslot mfec
    global verbose3
    if {$usevme == 1} {
	vmefecsetaddresses $vmeslot $mfec
    }
    puts $pcifec "2 $myoffset 0"
    flush $pcifec
    gets $pcifec inputline
    scan $inputline "%s" hexdata
    if {$verbose3 > 0} { puts "pcifecreadword($myoffset): $hexdata" }
    return $hexdata
}

proc vmefecsetaddresses { myvmeslot mymfec } {
    global pcifec
    global vmeslot mfec
    global verbose3
    set vmeslot $myvmeslot
    set mfec $mymfec
    if {$verbose3 > 0} {
	puts "VME FEC Setting vmeslot:$vmeslot, mfec:$mfec"
    }
    puts $pcifec "4 $vmeslot 0"
    # program using hardware designated mfec number 8..1 up until here,
    # then convert to software desgn numbering 0..7
    set softmfec [expr 8-$mfec]
    puts $pcifec "5 $softmfec 0"
    flush $pcifec
}

set ch1receivetimeout "-"
set ch1rcvcounterr "-"
set ch1receivecomplete "-"
set ch1hubaddresserror "-"
set ch1sendstarted "-"
set ch2receivetimeout "-"
set ch2rcvcounterr "-"
set ch2receivecomplete "-"
set ch2hubaddresserror "-"
set ch2sendstarted "-"

proc pcifeccmdreg { bitint bitstate } {
    global usevme
    global vmeslot mfec
    if {$usevme == 1} {
	vmefecsetaddresses $vmeslot $mfec
    }
    set hextemp [pcifecreadword 12]
    set tt0 [padhexword $hextemp]
    set tt1 [wordtobits $tt0]
    set tt2 [bitwordset $tt1 $bitint $bitstate]


    set tt3 [bitstoword $tt2]
    pcifecwriteword 12 $tt3
}
    
#pcifecwriteword 0 0x12345678
#pcifecreadword 0

proc adcinitdata { mywidget mydata count} {
    global dataarray
    global lastcount
    global linearray
    global adcyoffset
    global adc
    global icount
    global dostop
    global PCTARGET
    global OS

    
    if {$PCTARGET =="release"} {
	if { $OS == "linux" } {
#	    set adc [open "|/usr/local/cosmo/bin/adccyber0" "r+"]
	} else {
#####	    set adc [open "| adccyber0" "r+"]
	}
    } elseif {$PCTARGET =="devel"} {
	if { $OS == "linux" } {
	    set adc [open "|/home/doroshen/tbm/radc1" "r+"]
#	    set adc [open "|/home/doroshen/bitscope/bs200" "r+"]
	} else {
	    set adc [open "| radc1" "r+"]
	}
    }


    set i 0
    set pwidth 5
    set pheight 256
    set lastx 0
    set lasty 0
    set xshift 0
    set lastcount $count
    set icount $count
    
    set i 0
    while { $i < 1024 } {
	set dataarray($i) 0
	incr i
    }

    set i 0
    while { $i < $icount } {
	set pheight $dataarray($i)
	set pheight [expr 256*($pheight/4096.0)]
	set pheight [expr 256-$pheight]
	set ix0 [expr $i*$pwidth+$xshift]
	set ix [expr $i*$pwidth+$xshift]
	set ix2 [expr $ix+$pwidth]
	set iy [expr $pheight+$adcyoffset]
	set aindex $i
	set linearray($aindex) [$mywidget create line \
				    $lastx $lasty $ix $iy $ix2 $iy \
				    -fill black -smooth 0 -width 2 ]
	set lastx $ix2
	set lasty $iy
	incr i
    }
}


proc adcdrawdata2 { mywidget mydata count} {
    global dataarray
    global lastcount
    global linearray
    global adcyoffset
    set i 0
    set pwidth 5
    set pheight 256
    set lastx 0
    set lasty 0
    set xshift 0
    set lastcount $count
    set icount $count
    set i 0
    while { $i < $icount } {
	set pheight $dataarray($i)
	set pheight [expr 256*($pheight/4096.0)]
	set pheight [expr 256-$pheight]
	set ix0 [expr $i*$pwidth+$xshift]
	set ix [expr $i*$pwidth+$xshift]
	set ix2 [expr $ix+$pwidth]
	set iy [expr $pheight+$adcyoffset]
	set aindex $i
	$mywidget coords $linearray($aindex) $lastx $lasty $ix $iy $ix2 $iy 
	set lastx $ix2
	set lasty $iy
	incr i
    }
}

proc getevent { command } {
    global dostop
    global adc
    global dataarray
    global lastcount
    global linearray
    global adcyoffset
    global outformat
    global adcrunmode
    global ioutevents
    global cosmoipcscripting
    global suppressdisplay
    global adcloaded 
    global doutformat
    global nfileevents

    set iii -2

    while {$dostop != 1} {
	if {$command >= 1} {
	    for {set ij 0} {$ij < $command} {incr ij} {
		if {$adcloaded == 1} {
		    if {$doutformat == "showdisk"} {
			set iii [cybergetevent 240 500 1 1]
			incr nfileevents
			set im [expr $ioutevents % 100]
			if {$im == 0} {
			    if {$iii > 0} {
				for {set i 0} {$i < $iii} {incr i} {
				    set dataarray($i) [cyberarray $i]
				}
				if {$suppressdisplay != "1"} {
				    adcdrawdata2 .adc.f1.can1 0 190
				}
			    }
			}
		    } elseif {$doutformat == "show"} {
			set iii [cybergetevent 240 500 1 0]
			if {$iii>200} {
			    set iii 200
			}
			set im [expr $ioutevents % 100]
			if {$im == 0} {
			    if {$iii > 0} {
				for {set i 0} {$i < $iii} {incr i} {
				    set dataarray($i) [cyberarray $i]
				}
				if {$suppressdisplay != "1"} {
				    adcdrawdata2 .adc.f1.can1 0 190
				}
			    }
			}
		    } elseif {$doutformat == "disk"} {
			set iii [cybergetevent 240 500 1 2]
			incr nfileevents
		    }
		    if {$iii == "0"} {
			puts "0 Timeout return from ADC - increase trig rate"
			return
		    }
		}
		incr ioutevents
	    }
	    if {$command > 1} {
		set dostop 1
	    }
	}

	if {$adcrunmode != "Continuous"} {
	    .adc.f4.but1b config -text "Run\n($ioutevents/$adcrunmode)"
	    .adc.f6.l1 config -text $nfileevents -fg black
	    set im [expr $ioutevents % 100]
	    if {$im == 0} {
		if {$suppressdisplay != "1"} {
		    adcdrawdata2 .adc.f1.can1 0 190
		}
	    }
	    if {$ioutevents >= $adcrunmode } {
		set dostop 1
		if {$cosmoipcscripting == 1} {
		    update
		    update idletasks
#		    after 1
#		    update
#		    update idletasks
		    ######		send cosmoscripter { nextpass 1 }
		}
	    }
	} else {
	    .adc.f6.l1 config -text $nfileevents -fg black
	    set im [expr $ioutevents % 100]
	    if {$im == 0} {
		if {$suppressdisplay != "1"} {
		    adcdrawdata2 .adc.f1.can1 0 190
		}
	    }
	    set im [expr $ioutevents % 50]
	    if {$im == 0} {
		.adc.f4.but1b config -text "Run\n($ioutevents)"
		update
	    }
	}
####	update 

#	if {$outformat == "binary2k"} {
#	    puts $adc "3 2048 1000 1"
#	} elseif {$outformat == "text1k"} {
#	    puts $adc "3 1024 1000 1"
#	} else {
#	    puts $adc "3 256 1000 1"
#	}
#	flush $adc
    }
    
}
proc cutdata { dcount ublevel } {
# return 0 for no ub found
# return -1 for shifted found
# return >0 for good event
# process dcount size event
# drop data back in dataarray 
    global dataarray
    
}

proc storedata { dcount } {
    global dataarray
    global dataoutputfile
    global outformat
    global fdataout
    global verbose3
    global nfileevents
    global ioutevents
    global dataarraycount

# set global length of datarray    
    set dataarraycount $dcount

#    incr ioutevents
    
    if {$dataoutputfile != ""} {

	incr nfileevents
	.adc.f6.l1 config -text $nfileevents -fg black
	if {$outformat == "paw24"} {
	    if {$verbose3==1} {
		puts "Paw24 write"
	    }
	    set iout 0
	    set outstring ""
	    set nout $dcount
	    if {$nout > 24} {
		set nout 24
	    }
	    while {$iout < $nout} {
		append outstring $dataarray($iout)
		append outstring " "
		incr iout
	    }
	    puts $fdataout $outstring
	} elseif {$outformat == "binary"} {
	    if {$verbose3==1} {
		puts "Binary write"
	    }
#	    fconfigure $fdataout -translation binary
	    set nout $dcount
	    if {$nout > 256} {
		set nout 256
	    }
	    set val [binary format s1 $nout]
	    puts -nonewline $fdataout $val
	    set iout 0
	    while {$iout < $nout} {
		set val [binary format s1 $dataarray($iout)]
		puts -nonewline $fdataout $val
		incr iout
	    }
	} elseif {$outformat == "binary2k"} {
	    if {$verbose3==1} {
		puts "Binary write"
	    }
#	    fconfigure $fdataout -translation binary
	    set nout $dcount
	    if {$nout > 2048} {
		set nout 2048
	    }
	    set val [binary format s1 $nout]
	    puts -nonewline $fdataout $val
	    set iout 0
	    while {$iout < $nout} {
		set val [binary format s1 $dataarray($iout)]
		puts -nonewline $fdataout $val
		incr iout
	    }
	} elseif {$outformat == "seu"} {
	    if {$verbose3==1} {
		puts "SEU testing write"
	    }
#	    fconfigure $fdataout -translation binary
	    set iout 0
	    set outstring "ADC $dcount "
	    set nout $dcount
	    if {$nout > 128} {
		set nout 128
	    }
	    while {$iout < $nout} {
		append outstring $dataarray($iout)
		if {$nout > 0} {
		    set itemp [expr $nout-1]
		    if {$iout < $itemp} {
			append outstring " "
		    }
		}
		incr iout
	    }
	    puts $fdataout $outstring
	} elseif {$outformat == "text1k"} {
	    if {$verbose3==1} {
		puts "TEXT ADC 1k length testing write"
	    }
#	    fconfigure $fdataout -translation binary
	    set iout 0
	    set outstring "ADC $dcount "
	    set nout $dcount
	    if {$nout > 1024} {
		set nout 1024
	    }
	    while {$iout < 512} {
		append outstring $dataarray($iout)
		if {$nout > 0} {
		    set itemp [expr $nout-1]
		    if {$iout < $itemp} {
			append outstring " "
		    }
		}
		incr iout
	    }
	    puts $fdataout $outstring

	    set outstring ""
	    while {$iout < $nout} {
		append outstring $dataarray($iout)
		if {$nout > 0} {
		    set itemp [expr $nout-1]
		    if {$iout < $itemp} {
			append outstring " "
		    }
		}
		incr iout
	    }
	    puts $fdataout $outstring
	}
    }
}

proc doadc { command } {
    global dostop
    global adc
    global dataarray
    global lastcount
    global linearray
    global adcyoffset
    global nfileevents
    global outformat
    global icount
    global ioutevents
    global adcrunmode
    global doutformat

    toplevel .adc -class Dialog 
    wm title .adc "Analog Readout"

    frame .adc.f0
    frame .adc.f1
    frame .adc.f2
    frame .adc.f3
    frame .adc.f4
    frame .adc.f5
    frame .adc.f6
    frame .adc.f7

    canvas .adc.f1.can1 -width 750 -height 260
    .adc.f1.can1 create rectangle 0 0 750 260 -fill lightgreen

    adcinitdata .adc.f1.can1 0 256

# Setup adc at f above PCI freq
####    puts $adc "1 1 1 1"
# Setup adc at f below PCI freq
#    puts $adc "1 0 1 1"
####    flush $adc

    tk_optionMenu .adc.f4.optionmenu2 adcrunmode \
	Continuous 10000 1000 500 100 10 1
    set adcrunmode Continuous

    button .adc.f4.but1 -text "Continuous" -command {
	set dostop 0
	getevent 1
    }

    button .adc.f4.but1b -text "Run" -command {
	set dostop 0
	set ioutevents 0
	getevent 1
    }
    button .adc.f4.but2 -text "Stop" -command {
	set dostop 1
    }

    label .adc.f4.optionlabel1 -text "Data Path:" 
    tk_optionMenu .adc.f4.optionmenu1 doutformat show showdisk disk
    set outformat show

    label .adc.f5.l1 -text "Output filename: - none -" \
	-foreground "gray" \
	-padx 10 -font { Helvetica 12 {bold} } 
    pack .adc.f5.l1 -side left

    label .adc.f6.l1 -font { Helvetica 12 {bold} } -fg "gray" -text 0
    pack .adc.f6.l1 -side left

    pack .adc.f1.can1
    pack .adc.f4.optionlabel1 .adc.f4.optionmenu1 \
	.adc.f4.optionmenu2 .adc.f4.but1b .adc.f4.but2 -side left
    pack .adc.f0 .adc.f1 .adc.f2 .adc.f3 .adc.f4 .adc.f5 .adc.f6 .adc.f7

    set width [winfo screenwidth .]
    set height [winfo screenheight .]
    set x [expr ($width/2) - 200]
    set y [expr ($height/2) - 200]

    wm geometry .adc  +$x+$y
    wm resizable .adc 0 0

}



proc cosmoabout { command } {
    global softwareversion
    set cosmodata "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"
    toplevel .helpwin1 -class Dialog 
    wm title .helpwin1 "Cosmo"

    image create photo cosmoimg -data $cosmodata

    frame .helpwin1.f1
    label .helpwin1.f1.l0 -image ring
    frame .helpwin1.f2
    frame .helpwin1.f2.left
    frame .helpwin1.f2.right
    label .helpwin1.f2.left.l -image cosmoimg
    label .helpwin1.f2.right.l1 -text "Cosmo\n" -foreground "blue" \
	-padx 10 -font { Helvetica 24 {bold} } 
    label .helpwin1.f2.right.l2 \
    -text "Tired of acronyms, this \nprogram was named\nafter Ed's dog." -foreground "black" \
	-padx 10 -font { Helvetica 16 {bold} } 
    label .helpwin1.f2.right.l3 \
    -text "Please see Ed's CMS web page\nat http://www.physics.rutgers.edu/~bartz/cms/ \nfor test-station instructions." -foreground "black" \
	-padx 10 -font { Helvetica 14 {italic} } 
    label .helpwin1.f2.right.l4 \
    -text "This is Cosmo Version $softwareversion" -foreground "darkblue" \
	-padx 10 -font { Helvetica 12 {bold} } 
    pack .helpwin1.f2.right.l1
    pack .helpwin1.f2.right.l2
    pack .helpwin1.f2.right.l3
    pack .helpwin1.f2.right.l4
    pack .helpwin1.f2.left.l
    pack .helpwin1.f2.left .helpwin1.f2.right -side left
    pack .helpwin1.f1.l0
    pack .helpwin1.f1 .helpwin1.f2
}

proc getpixarray { command } {
    global initialdir
    global hubaddress
    global portaddress
    global baseaddress
    global pixchipaddress
    global pixarraycal
    global pixarrayenabled
    global pixarraytrim
    global dcolstate

    toplevel .pixload -class Dialog 
    wm title .pixload "File Load ROC pixels"
    set width [winfo screenwidth .]
    set height [winfo screenheight .]
    set x [expr ($width/2) - 200]
    set y [expr ($height/2) - 200]

    wm geometry .pixload  +$x+$y


    set file_types {
	{ "ROC pixel settings" { .pix1 }}
	{ "All Files" * }
    }
    set filename [tk_getOpenFile -initialdir $initialdir \
		      -filetypes $file_types -title "Read ROC Pixel Settings (v1)" \
		      -parent .]
    if { $filename != "" } {
	set initialdir [ file dirname $filename ]
	set fileidread [ open $filename "r" ]
	gets $fileidread inputline
	scan $inputline "%d %d %d %d %d %d %d %d %d %d" \
	    var1 var2 var3 var4 var5 var6 var7 var8 var9 var10
#	puts "$var1 "
	set pixchipaddress $var1
	set idcol 0
	while {$idcol < 26} {
	    gets $fileidread dcolstate(.win0.fr1.b$idcol)
# changestatebutton is the target of the button-toggle switches, so have
# to call twice to set to value in array
	    changestatebutton .win0.fr1.b$idcol
	    changestatebutton .win0.fr1.b$idcol
	    incr idcol
	}

	while { [gets $fileidread inputline] >= 0 } {
	    set result [scan $inputline "%d %d %d %d %d" \
			    var1 var2 var3 var4 var5]
# In file, format is r c mask cal trim 	    
	    set aindex "$var1 $var2"
	    set pixarrayenabled($aindex) $var3
	    set pixarraycal($aindex) $var4
	    if {$var5<0} {
		set var5 "Unknown"
	    }
	    set pixarraytrim($aindex) $var5
#	    puts "$aindex $var2 $var3 $var4 $var5"
	    
	}
	close $fileidread

	gauge_create .pixload.g SkyBlue
	pack .pixload.g -expand yes -fill both -padx 10 -pady 10

	reassertpix 1
	destroy .pixload.g

    }
    destroy .pixload
    bell
}

    
proc putpixarray { command } {
    global initialdir
    global hubaddress
    global portaddress
    global baseaddress
    global pixchipaddress
    global pixarraycal
    global pixarrayenabled
    global pixarraytrim
    global dcolstate
    global maxcols
    global maxrows

    toplevel .pixput -class Dialog 
    wm title .pixput "Save ROC pixels"
    set width [winfo screenwidth .]
    set height [winfo screenheight .]
    set x [expr ($width/2) - 200]
    set y [expr ($height/2) - 200]

    wm geometry .pixput  +$x+$y

    set file_types {
	{ "ROC Pixel Settings (v1)" { .pix1 }}
	{ "All Files" * }
    }
    set filename [tk_getSaveFile -initialdir $initialdir \
		      -filetypes $file_types -title "Save ROC pixels (v1)" \
		      -parent .]
    if { $filename != "" } {

	set lasts [string last ".pix1" $filename]
	if {$lasts < 0} {
	    append filename ".pix1"
	}

	set initialdir [ file dirname $filename ]
	set fileidwrite [ open $filename "w" ]

	set var1 $pixchipaddress

	puts $fileidwrite "$var1 0 0 0 0 0 0 0 0 0"

	set idcol 0
	while {$idcol < 26} {
	    puts $fileidwrite $dcolstate(.win0.fr1.b$idcol)
	    incr idcol
	}

	gauge_create .pixput.g SkyBlue
	pack .pixput.g -expand yes -fill both -padx 10 -pady 10
	set icol 0
	while {$icol<$maxcols} {
	    set irow 0
	    while {$irow<$maxrows} {
		set aindex "$icol $irow"
		if {$pixarraytrim($aindex)=="Unknown"} {
		    set itrim -1
		} else {
		    set itrim $pixarraytrim($aindex)
		}
		puts $fileidwrite "$icol $irow $pixarrayenabled($aindex) $pixarraycal($aindex) $itrim"
		incr irow
	    }
	    gauge_value .pixput.g [expr 100*($icol/$maxcols.0) ]
	    incr icol
	}
	close $fileidwrite
	
    }

    destroy .pixput.g

    destroy .pixput
    bell
}

proc selectoutputfile { command } {
    global dataoutputfile
    set file_types {
	{{Pixel Data Output File} {.dat}}
    }
    set outfile [tk_getSaveFile -title "Output filename" -filetypes \
		     $file_types]
    if {$outfile != ""} {
	set lasts [string last ".dat" $outfile]
	if {$lasts < 0} {
	    append outfile ".dat"
	}

	set dataoutputfile $outfile
	if { [winfo exists .adc.f5.l1] } {
	    .adc.f5.l1 config -text "Output filename: $outfile" -fg black
	}
    }
}

    
proc exec_cmd { command } {
#    global bgexecStatus bgexecOutput
    set result ""
    set check_cmd [lindex $command 0]
    set cmd [info commands $check_cmd]
   if { [string length $cmd] > 0 } {
	set result [eval $command]
    } else {
	set result [eval exec $command]
    }

    return $result

}


##----------------------------------------------------------------------
##----------------------------------------------------------------------
# Cosmo-Startup screen introduction dialog.
# Cosmo-Startup screen from Graphical Apps w Tcl/Tk

  # Creates cosmo-startup screen dialog.
proc cosmo-startup_create { image product copyright msg } {

    global tcl_version tk_version tcl_platform
    global tcl_patchLevel


    # Get rid of any existing dialog.
    cosmo-startup_remove



    toplevel .cosmo-startup
    wm withdraw .cosmo-startup

    # Center on screen.
    dialog_position .cosmo-startup

    # Top area contains product info.
    set frm .cosmo-startup.top
    frame $frm -bd 2 -relief groove

    # Label for logo image
    label $frm.logo -image $image

    # Label for product version number.
    label $frm.product -text "$product"

    # Message widget for copyright info.
    message $frm.copyright -text "$copyright" -width 3i

    # Label for Tcl version number.
    label $frm.tclversion -text \
        "Tcl $tcl_version/Tk $tk_version, $tcl_patchLevel"

    # Label for OS version.
    label $frm.osversion -text \
        "$tcl_platform(os) $tcl_platform(osVersion)"

    pack $frm.logo \
        -side top -fill x -pady 4 -ipadx 4 -ipady 4

    pack $frm.product $frm.copyright \
        $frm.tclversion $frm.osversion \
        -side top -fill x 

    pack $frm -side top -fill x -padx 8 -pady 8

    # Bottom area contains current message
    set frm .cosmo-startup.bottom
    frame $frm -bd 2 -relief groove

    # Label for message
    label $frm.msg -text "$msg" -anchor w

    pack $frm.msg -side left -ipadx 6 -ipady 4
    pack $frm -side bottom -fill x -padx 8 -pady 8

    wm deiconify .cosmo-startup
    update
}


  # Updates text message in cosmo-startup screen dialog.
proc cosmo-startup_update { msg } {

    set name .cosmo-startup.bottom.msg

    if { [winfo exists $name] } {
        $name config -text "$msg"
        update
    }

}



  # Removes cosmo-startup screen dialog.
proc cosmo-startup_remove { } {

    if { [winfo exists .cosmo-startup] } {
        destroy .cosmo-startup
    }
}


  # Centers dialog in approximately screen center.
proc dialog_position { dlg } {

    set width  [winfo screenwidth  . ]
    set height [winfo screenheight . ]

    set x [expr ($width/2)  - 260]
    set y [expr ($height/2) - 260]

    wm geometry $dlg +$x+$y

}

# Cosmo-Startup image
set splashdata "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"
#image create photo ring -file /usr/local/ppfec/ppfeccosmo-startup.gif
image create photo ring -data $splashdata

cosmo-startup_create ring "COSMO Hardware Version 4 (PCI/VME)" \
                   "   Software v$softwareversion
Rutgers Univ Physics" \
"Loading ..."


# cosmo-startup.tcl


##----------------------------------------------------------------------

proc updatecolumnselector { value } {
    global resetcolumn
    set resetcolumn $value
}
				   

proc updatetrimslide { value } {
    global trim
    set trim $value

    .win0.fr2.g6.box3.line6.s1 set $trim
    .win0.fr2.g6.box3.line6.s1 configure -activebackground [trimcolor $trim]
#    .win0.fr2.can1.line6.s1 configure -highlightcolor [trimcolor $trim]
#    .win0.fr2.can1.line6.s1 configure -highlightbackground [trimcolor $trim]
    .win0.fr2.g6.box3.line6.s1 configure -troughcolor [trimcolor $trim]
#    .win0.fr2.can1.line6.s1 configure -background [trimcolor $trim]
#    .win0.fr2.can1.line6.s1 configure -foreground [trimcolor $trim]
}

proc updateresettrimslide { value } {
    global resettrim
    set resettrim $value
    .winreset.row3.f1.s1 configure -activebackground [trimcolor $resettrim]
    .winreset.row3.f1.s1 configure -troughcolor [trimcolor $resettrim]
}

proc updatecalslide { value } {
    global calibrationon
    .win0.fr2.can1.line7.s1 configure -activebackground red
    .win0.fr2.can1.line7.s1 configure -troughcolor red
    if {$value==1} {
	set calibrationon 1
	pack forget .win0.fr2.can1.line6.s1
	pack forget .win0.fr2.can1.line8.s1
    } else {
	set calibrationon 0
	pack .win0.fr2.can1.line6.s1
	pack .win0.fr2.can1.line8.s1
    }
#    .win0.fr2.can1.line6.s1 configure -activebackground [trimcolor $trim]
#    .win0.fr2.can1.line6.s1 configure -troughcolor [trimcolor $trim]
}
proc updateenabledslide { value } {
    global maskbitstate
    .win0.fr2.can1.line8.s1 configure -activebackground "\#808080"
    .win0.fr2.can1.line8.s1 configure -troughcolor "\#808080"
    if {$value==1} {
	set maskbitstate 0
	pack forget .win0.fr2.can1.line7.s1
    } else {
	set maskbitstate 1
	pack .win0.fr2.can1.line7.s1
    }
#    .win0.fr2.can1.line6.s1 configure -activebackground [trimcolor $trim]
#    .win0.fr2.can1.line6.s1 configure -troughcolor [trimcolor $trim]
}

proc updateslide1 { value } {
    global ResetAnalogRegisters
    set ResetAnalogRegisters $value
}
proc updateslide2 { value } {
    global AnalogOutputGain
    set AnalogOutputGain $value
}
proc updateslide3 { value } {
    global AnalogInputAmplifierBias
    set AnalogInputAmplifierBias $value
}
proc updateslide4 { value } {
    global AnalogOutputDriverBias
    set AnalogOutputDriverBias $value
}

proc inttobits { myint numbits } {
    set k 0
    if { [ expr $myint&128 ] == 128 } {
	set bit7 1
    } else {
	set bit7 0
    }
    if { [ expr $myint&64 ] == 64 } {
	set bit6 1
    } else {
	set bit6 0
    }
    if { [ expr $myint&32 ] == 32 } {
	set bit5 1
    } else {
	set bit5 0
    }
    if { [ expr $myint&16 ] == 16 } {
	set bit4 1
    } else {
	set bit4 0
    }
    if { [ expr $myint&8 ] == 8 } {
	set bit3 1
    } else {
	set bit3 0
    }
    if { [ expr $myint&4 ] == 4 } {
	set bit2 1
    } else {
	set bit2 0
    }
    if { [ expr $myint&2 ] == 2 } {
	set bit1 1
    } else {
	set bit1 0
    }
    if { [ expr $myint&1 ] == 1 } {
	set bit0 1
    } else {
	set bit0 0
    }
    switch $numbits {
	8 { set bitvalue [format "%s%s%s%s%s%s%s%s" $bit7 $bit6 $bit5 $bit4 \
		$bit3 $bit2 $bit1 $bit0] }
	7 { set bitvalue [format "%s%s%s%s%s%s%s" $bit6 $bit5 $bit4 \
		$bit3 $bit2 $bit1 $bit0] }
	6 { set bitvalue [format "%s%s%s%s%s%s" $bit5 $bit4 \
		$bit3 $bit2 $bit1 $bit0] }
	5 { set bitvalue [format "%s%s%s%s%s" $bit4 \
		$bit3 $bit2 $bit1 $bit0] }
	4 { set bitvalue [format "%s%s%s%s" $bit3 $bit2 $bit1 $bit0] }
	3 { set bitvalue [format "%s%s%s"  $bit2 $bit1 $bit0] }
	2 { set bitvalue [format "%s%s" $bit1 $bit0] }
	1 { set bitvalue [format "%s" $bit0] }
    }
    return $bitvalue
}



proc inttobase6 { myint mywidth } {
    set myresult ""
    set mybase 6
    set myval $myint
    while {$myval >= $mybase} {
	set itemp [expr $myval % $mybase]
	set myresult "$itemp$myresult"
	set myval [expr $myval / $mybase]
    }
    set myresult "$myval$myresult"
    set ilen [string length $myresult]
    set missed0 [expr $mywidth - $ilen]
    set icloop 0
    while {$icloop < $missed0} {
	set myresult "0$myresult"
	incr icloop
    }
    return $myresult
}

proc inttograycode { myint } {
    global verbose1
    set myresult [ expr $myint^($myint>>1) ]
    if {$verbose1==1} {
	puts "int:$myint  graycode: $myresult"
    }
    return $myresult
}
proc write_slider { widget myvalue myfunction } {
    global hubaddress
    global portaddress
    global returnedbits
    global tbm
    global baseaddress
    global datalength
    global verbose3
    global interface

#    if {$verbose3==1} {
#	puts "TBM Slider(B:$baseaddress,H:$hubaddress,P:$portaddress)<-$offset DATA:$myvalue $mydirection"
#    }

	set mydirection "write"
	switch $myfunction {
	    InputAmpBiasWrite { set addressoffset 0x5 }
	    InputAmpBiasRead { 
		set addressoffset 0x5
	        set mydirection "read"
		set myvalue 255
	    }
	    OutputDriverBiasWrite { set addressoffset 0x6 }
	    OutputDriverBiasRead {
		set addressoffset 0x6
		set mydirection "read"
		set myvalue 255
	    }
	    GainWrite  { set addressoffset 0x7 }
	    GainRead  {
		set addressoffset 0x7
	        set mydirection "read"
		set myvalue 255
	    }
	}
	set ivalue [inttobits $myvalue 8]
	transfer_byte $ivalue  $addressoffset $mydirection
	
	return
}

proc sendbytesplit { mybyte } {
	set temp4 [bitstoint $mybyte]
	set temp5 [inttobits $temp4 4]
	set tlow $temp5
	set temp6 [expr $temp4 / 16]
	set temp7 [inttobits $temp6 4]
	set thigh $temp7
	sendbyte2 "$thigh" "$tlow"
}


proc sendFECaddress4 {} {
    global FECSDA
    global callatencybit
    global verbose3

    if {$verbose3=="1"} {
	puts "SENDING FEDSDA and callatencybit"
    }
    set sdatemp "000"
    if {$callatencybit == "enabled"} {
	set temp1 "1"
    } else {
	set temp1 "0"
    }
    set temp2 "100$temp1$sdatemp"
    sendbytewparity $temp2
}

#TCL versions of these (which were C)
proc usbroccmd { myhub myport myrocaddress param0 param1 param2 param3 param4} {  global verbose3
    if {$verbose3 > 0}  { puts "Tcl (PCIFEC) version of usbroccmd called with ($myhub $myport $myrocaddress $param0 $param1 $param2 $param3 $param4 )" }
}

proc usbtbmcmd { mybase myhub myport myoffset mydatabyte mydirection } {
    global verbose3
    if {$verbose3 > 0} { puts "Tcl (PCIFEC) version of usbtbmcmd called with ($mybase $myhub $myport $myoffset $mydatabyte $mydirection )" }
}

proc pix_transfer_byte { pixcommand arg1 arg2 arg3 arg4 arg5 arg6 } {
    global datalength

    global hubaddress
    global pixportaddress
    global pixchipaddress
    global returnedbits
    global baseaddress
    global lastreadbits
    global maskbitstate
    global trim
    global calibrationon
    global forgetpixprestopbit
    global textstatus
    global textcalibration
    global verbose3
    global maxcols
    global maxrows
    global interface 
    global roctype
    global port
    global highnumbits
    global lownumbits
    global num
    global verbose4

    global usbdeviceid

    global resetcolumn
    global resetentirechip
    global resetcompression

    global slowfecchannel

    set ipixcommand 0

# arg4 will indicate to send a 2nd packet with address and additional clock
# bits
    if {$interface=="usb"} {
#	puts "PUT USB DATA"
    } else {
	puts "DEVELOPMENT no longer supports pport"
	exit
    }
    ####	set port $pixportaddress
    # convert to the bits we need to send
    set commandlength 1
    
    set port $pixportaddress 


##    set num $commandlength
##    bithubportaddress 1

# New with v350, set mode here to neutral (no compress or full col)
    if {$slowfecchannel == "1"} {
	pcifeccmdreg 4 0
	pcifeccmdreg 5 0
    } else {
	pcifeccmdreg 20 0
	pcifeccmdreg 21 0
    }

    switch $pixcommand {
	"ClrCal" {
	    if {$verbose4 == 1} {
		puts "usbroccmd $hubaddress $port $pixchipaddress 1 1 2 3 0"
	    }
	    set ttt [usbroccmd $hubaddress $port $pixchipaddress 1 1 2 3 0]
	}
	0x08 {
	    if {$verbose4 == 1} {
		puts "usbroccmd $hubaddress $port $pixchipaddress 8 $arg1 $arg2 3 0"
	    }
	    set ttt [usbroccmd \
			 $hubaddress $port $pixchipaddress 8 $arg1 $arg2 0 0]
	}
	0x44 {
# 4  col row enabled trim	    
	    if {$verbose4 == 1} {
		puts "usbroccmd $hubaddress $port $pixchipaddress 4 $arg1 $arg2 $arg3 $arg5"
	    }
	    set ttt [usbroccmd \
			 $hubaddress $port $pixchipaddress 4 $arg1 $arg2 $arg3 $arg5 ]
	}
	0x45 {
# 4  col row enabled trim	    
	    if {$verbose4 == 1} {
		puts "usbroccmd $hubaddress $port $pixchipaddress 2 $arg1 $arg2 $arg4 0"
	    }
	    set ttt [usbroccmd \
			 $hubaddress $port $pixchipaddress 2 $arg1 $arg2 $arg4 0 ]
	}
	0x95 {
# send entire col to ROC
	    if {$verbose4 == 1} {
		puts "usbroccmd $hubaddress $port $pixchipaddress 42 $arg1 $arg2 $arg4 0"
	    }
	    set ttt [usbroccmd \
			 $hubaddress $port $pixchipaddress 42 $arg1 $arg2 $arg4 0 ]
	}
	0x97 {
# init entire ROC to one value
# 0x97 new 1-06 Send reset entire chip to one data byte wia new pci fec cmd
	    if {$verbose4 == 1} {
		puts "usbroccmd $hubaddress $port $pixchipaddress 42 $arg1 $arg2 $arg4 0"
	    }
	    set ttt [usbroccmd \
			 $hubaddress $port $pixchipaddress 42 $arg1 $arg2 $arg4 0 ]
	}
	0x98 {
# init entire ROC column to one value
# 0x98 new 1-06 Send reset entire chip to 80 data byte wia new pci fec cmd
	    if {$verbose4 == 1} {
		puts "usbroccmd $hubaddress $port $pixchipaddress 42 $arg1 $arg2 $arg4 0"
	    }
	    set ttt [usbroccmd \
			 $hubaddress $port $pixchipaddress 42 $arg1 $arg2 $arg4 0 ]
	}
	0x25 {
	    if {$verbose4 == 1} {
		puts "usbroccmd $hubaddress $port $pixchipaddress 25 $arg1 $arg2 $arg3 0"
	    }
	    set ttt [usbroccmd \
			 $hubaddress $port $pixchipaddress 25 $arg1 $arg2 $arg3 0 ]
	}

    }


# adding pci capture    return

# REASSERT the setting in the FEC    
###6-1    sendFECaddress4
##    sendFECaddress4

    clearnibblearray "0000"
    
    switch $pixcommand {
	0x08 { 
	# 0x08 is Prog_DAC
	    set ipixcommand 8
	    set commandlength "03"
	}
	"ClrCal" { 
	    set commandlength "01"
	    set ipixcommand 1
	}
	0x44 { 
	    set commandlength "04"
	    set ipixcommand 4
	}
	0x45 {
	    set commandlength "04"
	    set ipixcommand 2
	}
	0x25 {
	    set ipixcommand 36
	    set commandlength "04"
	}
	0x95 {
	    # mult pixel command
	    set ipixcommand 4
	    # num of 162 in hex = A2
	    set commandlength "A2"
	}
	0x96 {
	    # set ipix to 4 for prog_pix(trim)
	    set ipixcommand 4
	    set commandlength "A2"
	}
	0x97 {
# 0x97 new pci fec cmd to set entire roc to one value
	    set ipixcommand 4
	    set commandlength "A2"
	}
	0x98 {
# 0x98 new pci fec cmd to set entire roc to one value
	    set ipixcommand 4
	    set commandlength "A2"
	}
    }
    
    # Send reset pointer
    sendresetpointer 1
    
    #send hub and pixel port address	
    set temp1 [inttobits $hubaddress 5]
    set temp2 [inttobits $pixportaddress 3]
    set temp3 "$temp1$temp2"
    set temp4 [bitstoint $temp3]
    set temp5 [inttobits $temp4 4]
    set tlow $temp5
    set temp6 [expr $temp4 / 16]
    set temp7 [inttobits $temp6 4]
    set thigh $temp7
####    puts "SENDING hub and port to TBM..."
    sendbyte2 "$thigh" "$tlow"
    
    
    # Send the number of bytes (num) of data to follow
    set num $commandlength
    bithubportaddress 1

    if {$verbose3 > 0} {
	puts "DEBUG: pixcommand:$pixcommand   hub:$hubaddress   num:$num ($highnumbits $lownumbits) commandlength:$commandlength"
    }

    if {$pixcommand == "0x97" } {
# send integer 52 for all 52 columns
	if {($resetentirechip == "EntireChip") && ($resetcompression == "ON")} {
	    sendbyte2 "0011" "0100"
	    puts "COL: 0011 0100"
	} elseif {($resetentirechip == "1st8ColumnCOMP") && ($resetcompression == "ON")} {
	    sendbyte2 "0000" "1000"
	    puts "COL: 0000 1000"
	} else {
	    sendbyte2 "0000" "0001"
	    puts "COL: 0000 0001"
	}
    } elseif {$pixcommand == "0x98"} {
# one column at a time in this mode
	if {$resetentirechip == "1st3ColumnsNOCOMP"} {
	    sendbyte2 "0000" "0011"
	    puts "COL: 0000 0011 for 0x98 3 col"
	} else {
	    sendbyte2 "0000" "0001"
	    puts "COL: 0000 0001    for 0x98"
	}
    } else {
# send num for all but fec compressed commands
	sendbyte2 $highnumbits $lownumbits
#	puts "NUM Sent"
    }
    
    set temp1 [inttobits $pixchipaddress 4]
    set temp2 [inttobits $ipixcommand 4]
####    puts "Sending ROC address and ROC command byte to ROC..."
    if {($pixcommand != "0x97") && ($pixcommand != "0x98")} {
	sendbyte2 "$temp1" "$temp2"
    }

    if {($pixcommand == "0x97") ||  ($pixcommand == "0x98")} {
	set highcmd $temp1
	set lowcmd $temp2
    }
    
    
    ####### command is sent, now send data bytes based on command
    switch $pixcommand {
	0x08 {
	    # 0x08 dac-address dac-value	    
	    if {$verbose3==1} {
		puts "PIX-08 DAC(H:$hubaddress,Px:$pixportaddress,CID:$pixchipaddress):$arg1<-$arg2"
	    }
	    ## Prog_DAC
	    ## send the DAC address (int 8 bit)	    
	    set temp5 [inttobits $arg1 4]
	    set tlow $temp5
	    set temp6 [expr $arg1 / 16]
	    set temp7 [inttobits $temp6 4]
	    set thigh $temp7
#	    puts "RAM: $thigh $tlow"
	    sendbyte2 "$thigh" "$tlow"
	    ## send the Value setting the dac to
	    set temp5 [inttobits $arg2 4]
	    set tlow $temp5
	    set temp6 [expr $arg2 / 16]
	    set temp7 [inttobits $temp6 4]
	    set thigh $temp7
#	    puts "RAM: $thigh $tlow"
	    sendbyte2 "$thigh" "$tlow"
	}
	0x95  {
	    # multiple pix command
	    if {$verbose3==1} {
		set tempstr "PIX-95(C:$arg1,R:$arg2,$arg3,$arg4,$arg5,CID:$pixchipaddress)<-"
		if {$arg4==1} {
		    append tempstr " CAL"
		} else {
		    if {$arg3==1} { 
			append tempstr "  ON"
		    } elseif {$arg3==0} {
			append tempstr " OFF"
		    }
		    append tempstr " TRB:$arg5"
		}
		puts $tempstr
	    }
# 0x95 for psi46 col-nongray npixels enabled cal trim extraclocks	    
# Set burst mode dcol of npixels to enabled cal and trim
	    # First byte, the chip address nibble and command nibble has
	    # already been sent.  
	    # works one col at a time (not one dcol at a time)

# Different method to calc column:
	    set coltemp0 $arg1
	    set coltemp1 [expr $coltemp0/2]
	    set coltemp1 [ inttograycode $coltemp1 ]
	    set coltemp1 [ expr $coltemp1*2 ]
	    set coltemp2 [ expr $coltemp1|($coltemp0 & 0x01) ]
	    set temp1 [inttobits $coltemp2 8]
	    ## Column address
#old	    set grayvalue [ inttograycode $arg1 ]
#old	    set temp1 [inttobits $grayvalue 8]
	    sendbytesplit $temp1
	    #
	    # Now we've got to send row-address/data repeatedly
	    ## This is a prog_pix (NO CAL) command
	    set temptrim [inttobits $arg5 4]
	    set tempenabled $arg3
	    set temptrim "000$temptrim"
	    set temptrim2 "$tempenabled$temptrim"
	    set temp2 [inttobits $temptrim2 8]

	    set krow 0

 # after the first column the FEC has the value in RAM for the rows and data
 #  all we need to do is suppress the sending of the row/data and then send
 #  a go at this stage
	    if {$arg1 == 0} {
		while {$krow < $arg2} {
		    ## Row address
		    set rowaddress [ inttograycode $krow ]
		    set temp1 [inttobits $rowaddress 8]
		    sendbytesplit $temp1
		    
		    sendbytesplit $temp2
		    
		    incr krow
		}
	    }
	    set ipixcommand 148 
	}
	0x96  {
	    if {$verbose3==1} {
		set tempstr "PIX-96(C:$arg1,R:$arg2,$arg3,$arg4,$arg5,CID:$pixchipaddress)<-"
		if {$arg4==1} {
		    append tempstr " CAL"
		} else {
		    if {$arg3==1} { 
			append tempstr "  CAP"
		    } elseif {$arg3==2} {
			append tempstr " BUMP"
		    } elseif {$arg3==0} {
			append tempstr " NOCAL"
		    }
		    append tempstr " TRB:$arg5"
		}
		puts $tempstr
	    }
# CAL PROG
# 0x96 for psi46 col-nongray npixels enabled cal trim extraclocks	    
# Set burst mode dcol of npixels to enabled cal and trim
	    # First byte, the chip address nibble and command nibble has
	    # already been sent.  
	    # works one col at a time (not one dcol at a time)
	    ## Column address
	    set grayvalue [ inttograycode $arg1 ]
	    set temp1 [inttobits $grayvalue 8]
	    sendbytesplit $temp1
	    #
	    # Now we've got to send row-address/data repeatedly
	    ## This is a prog_pix (NO CAL) command
	    set temptrim [inttobits $arg5 4]
	    set krow 0
	    while {$krow < $arg2} {
		## Row address
		set rowaddress [ inttograycode $krow ]
		set temp1 [inttobits $rowaddress 8]
		sendbytesplit $temp1
		
		## now send the mask bit and trim from arg3
		set tempenabled $arg3
		# send the cal data byte
		if {$arg3==0} {
		    set tempcal "00000000"
		} elseif {$arg3==1} {
		    set tempcal "00000001"
		} elseif {$arg3==2} {
		    set tempcal "00000010"
		}

		sendbytesplit $tempcal
		
		incr krow
	    }
	    set ipixcommand 149 
	}
	0x97  {
	    if {$verbose3==1} {
		set tempstr "PIX-97(C:$arg1,R:$arg2,$arg3,$arg4,$arg5,CID:$pixchipaddress)<-"
		if {$arg4==1} {
		    append tempstr " CAL"
		} else {
		    if {$arg3==1} { 
			append tempstr "  CAP"
		    } elseif {$arg3==2} {
			append tempstr " BUMP"
		    } elseif {$arg3==0} {
			append tempstr " NOCAL"
		    }
		    append tempstr " TRB:$arg5"
		}
		puts $tempstr
	    }
	    # First byte, the chip address nibble and command nibble has
	    # already been sent.  
	    # works one col at a time (not one dcol at a time)
	    ## Column address
	    # col aggress=-1 ==> reset entire chip in compressed mode
# v350,351
	    if {$slowfecchannel == "1"} {
		pcifeccmdreg 4 1
		pcifeccmdreg 5 1
	    } elseif {$slowfecchannel == "2"} {
		pcifeccmdreg 20 1
		pcifeccmdreg 21 1
	    }
		
	    if {$arg1 == -1} {
		for {set ijk 0} {$ijk < $maxcols} { incr ijk } {
		    # first, send chip address and command
		    sendbyte2 $highcmd $lowcmd
		    # next, send the column
		    set coltemp0 $ijk
#		    puts "                                 >>>>>>>>programming column $coltemp0"
		    set coltemp1 [expr $coltemp0/2]
		    set coltemp1 [ inttograycode $coltemp1 ]
		    set coltemp1 [ expr $coltemp1*2 ]
		    set coltemp2 [ expr $coltemp1|($coltemp0 & 0x01) ]
		    set grayvalue [inttobits $coltemp2 8]
		    sendbytesplit $grayvalue
		    # lastly, send the data byte which will be written to every
		    # pixel in that column
		    #
		    # Now we've got to send row-address/data repeatedly
		    ## This is a prog_pix (NO CAL) command
		    set temptrim [inttobits $arg5 4]
		    ## now send the mask bit and trim from arg3
		    set temptrim "000$temptrim"
		    set tempenabled $arg3
		    set tempout "$tempenabled$temptrim"
		    sendbytesplit $tempout
		}
	    } elseif {$arg1 == -2} {
		# col is -2 then do 1st four columns in compressed mode
		for {set ijk 0} {$ijk < 8} { incr ijk } {
		    # first, send chip address and command
		    sendbyte2 $highcmd $lowcmd
		    # next, send the column
		    set coltemp0 $ijk
#		    puts "                                 >>>>>>>>programming column $coltemp0"
		    set coltemp1 [expr $coltemp0/2]
		    set coltemp1 [ inttograycode $coltemp1 ]
		    set coltemp1 [ expr $coltemp1*2 ]
		    set coltemp2 [ expr $coltemp1|($coltemp0 & 0x01) ]
		    set grayvalue [inttobits $coltemp2 8]
		    sendbytesplit $grayvalue
		    # lastly, send the data byte which will be written to every
		    # pixel in that column
		    #
		    # Now we've got to send row-address/data repeatedly
		    ## This is a prog_pix (NO CAL) command
		    set temptrim [inttobits $arg5 4]
		    ## now send the mask bit and trim from arg3
		    set temptrim "000$temptrim"
		    set tempenabled $arg3
		    set tempout "$tempenabled$temptrim"
		    sendbytesplit $tempout
		}
	    } else {
		# do only one column if column=arg1 passed in with value
		set ijk $resetcolumn
#		    puts "                                 >>>>>>>>programming one column $ijk"
		# first, send chip address and command
		sendbyte2 $highcmd $lowcmd
		# next, send the column
		set coltemp0 $ijk
		set coltemp1 [expr $coltemp0/2]
		set coltemp1 [ inttograycode $coltemp1 ]
		set coltemp1 [ expr $coltemp1*2 ]
		set coltemp2 [ expr $coltemp1|($coltemp0 & 0x01) ]
		set grayvalue [inttobits $coltemp2 8]
		sendbytesplit $grayvalue
		# lastly, send the data byte which will be written to every
		# pixel in that column
		#
		# Now we've got to send row-address/data repeatedly
		## This is a prog_pix (NO CAL) command
		set temptrim [inttobits $arg5 4]
		## now send the mask bit and trim from arg3
		set temptrim "000$temptrim"
		set tempenabled $arg3
		set tempout "$tempenabled$temptrim"
		sendbytesplit $tempout
	    }
	    set ipixcommand 149 
	}
	0x98  {
	   # 0x98 is the compressed command that sends the full column of data
	    # right now we are sending the same command to every column
# v350,351
	    if {$slowfecchannel == "1"} {
		pcifeccmdreg 4 1
		pcifeccmdreg 5 0
	    } elseif {$slowfecchannel == "2"} {
		pcifeccmdreg 20 1
		pcifeccmdreg 21 0
	    }

	    if {$verbose3==1} {
		set tempstr "PIX-98(C:$arg1,R:$arg2,$arg3,$arg4,$arg5,CID:$pixchipaddress)<-"
		if {$arg4==1} {
		    append tempstr " CAL"
		} else {
		    if {$arg3==1} { 
			append tempstr "  CAP"
		    } elseif {$arg3==2} {
			append tempstr " BUMP"
		    } elseif {$arg3==0} {
			append tempstr " NOCAL"
		    }
		    append tempstr " TRB:$arg5"
		}
		puts $tempstr
	    }
	    # First byte, the chip address nibble and command nibble has
	    # already been sent.  
	    # works one col at a time (not one dcol at a time)
	    ## Column address

	    if {$arg1 == -2} {
		# col is -2 then do 1st 3 columns in one buffer
		for {set ijk 0} {$ijk < 3} { incr ijk } {
		    # first, send chip address and command
		    sendbyte2 $highcmd $lowcmd
		    # next, send the column
		    set coltemp0 $ijk
#		    puts "                                 >>>>>>>>programming column $coltemp0"
		    set coltemp1 [expr $coltemp0/2]
		    set coltemp1 [ inttograycode $coltemp1 ]
		    set coltemp1 [ expr $coltemp1*2 ]
		    set coltemp2 [ expr $coltemp1|($coltemp0 & 0x01) ]
		    set grayvalue [inttobits $coltemp2 8]
		    sendbytesplit $grayvalue
		    # lastly, send the data byte which will be written to every
		    # pixel in that column
		    #
		    # Now we've got to send row-address/data repeatedly
		    ## This is a prog_pix (NO CAL) command
		    set temptrim [inttobits $arg5 4]
		    ## now send the mask bit and trim from arg3
		    set temptrim "000$temptrim"
		    set tempenabled $arg3
		    set tempout "$tempenabled$temptrim"
		    for {set myrow 0} {$myrow < $maxrows} {incr myrow} {
			sendbytesplit $tempout
		    }
		}
	    } else {

		# first, send chip address and command
		sendbyte2 $highcmd $lowcmd

		# next, send the column
#		set grayvalue [ inttograycode $ijk ]
#		set temp1 [inttobits $grayvalue 8]

#		    puts "                                 >>>>>>>>programming single entire column $arg1"
		set coltemp0 [ inttograycode $arg1 ]
	    set coltemp1 [expr $coltemp0/2]
	    set coltemp1 [ inttograycode $coltemp1 ]
	    set coltemp1 [ expr $coltemp1*2 ]
	    set coltemp2 [ expr $coltemp1|($coltemp0 & 0x01) ]
	    set grayvalue [inttobits $coltemp2 8]
		sendbytesplit $grayvalue

		# lastly, send the data byte which will be written to every
		# pixel in that column

	    #
	    # Now we've got to send row-address/data repeatedly
	    ## This is a prog_pix (NO CAL) command
		set tempenabled $arg3
		set temptrim [inttobits $arg5 4]
		## now send the mask bit and trim from arg3
		set temptrim "000$temptrim"
		set tempout "$tempenabled$temptrim"
		for {set myrow 0} {$myrow < $maxrows} {incr myrow} {
		    sendbytesplit $tempout
		}
	    set ipixcommand 149 
	    }
	}
	0x44  {
       # this is 0x44 modified for psi46 chips
# 0x45 col-nongray row-nongray enabled cal trim extraclocks	    
# arg..    1             2       3      4   5      6
	    if {$verbose3==1} {
		set tempstr "PIX-45progpix(C:$arg1,R:$arg2,$arg3,$arg4,$arg5,$arg6)<-"
		if {$arg4==1} {
		    append tempstr " CAL CAP"
		} elseif {$arg4==2} {
		    append tempstr " CAL BUMP"
		} else {
		    if {$arg3==1} { 
			append tempstr "  ON"
		    } elseif {$arg3==0} {
			append tempstr " OFF"
		    }
		    append tempstr " TRB:$arg5"
		}
		puts $tempstr
	    }
	    ## Prog_Cal-USAGE: pix_transfer_byte col row enabled cal trim
	    ## Sets the bits in the byte sent to the chip
	    ## Needs to be called twice sometimes (as 2 ops are required
	    ## to set CAL and trim+enable)

# Different method to calc column:
	    set coltemp0 $arg1
	    set coltemp1 [expr $coltemp0/2]
#	    puts "XXXXXX0 $arg1 $coltemp1 "
	    set coltemp1 [ inttograycode $coltemp1 ]
#	    puts "XXXXXX1 $arg1 $coltemp1 "
	    set coltemp1 [ expr $coltemp1*2 ]
#	    puts "XXXXXX2 $arg1 $coltemp1 "
	    set coltemp2 [ expr $coltemp1|($coltemp0 & 0x01) ]
#	    puts "XXXXXX3 $arg1 $coltemp2 "
	    set temp1 [inttobits $coltemp2 8]
#	    puts "XXXXXX4 $coltemp0 $coltemp1 $coltemp2  - $temp1"

#old	    set grayvalue [ inttograycode $arg1 ]
#old	    set temp1 [inttobits $grayvalue 8]
#old	    puts "XXXXXXXX5 - $temp1"
	    sendbytesplit $temp1

	    set rowaddress [ inttograycode $arg2 ]
	    ## Row address
	    set temp1 [inttobits $rowaddress 8]
	    sendbytesplit $temp1


	    ## now use the arg3(enabled/disabled) and set trim using progpix
	    set tmask $arg3
	    set ttrim [ inttobits $arg5 4 ]
	    set tt "000"
	    set tbyte "$tmask$tt$ttrim"
	    sendbytesplit $tbyte
	}
	0x45  {
       # this is 0x45 modified for psi46 chips
# 0x45 col-nongray row-nongray enabled cal trim extraclocks	    
# arg..    1             2       3      4   5      6
	    if {$verbose3==1} {
		set tempstr "PIX-45calpix(C:$arg1,R:$arg2,$arg3,$arg4,$arg5,$arg6)<-"
		if {$arg4==1} {
		    append tempstr " CAL CAP"
		} elseif {$arg4==2} {
		    append tempstr " CAL BUMP"
		} else {
		    if {$arg3==1} { 
			append tempstr "  ON"
		    } elseif {$arg3==0} {
			append tempstr " OFF"
		    }
		    append tempstr " TRB:$arg5"
		}
		puts $tempstr
	    }
	    ## Prog_Cal-USAGE: pix_transfer_byte col row enabled cal trim
	    ## Sets the bits in the byte sent to the chip
	    ## Needs to be called twice sometimes (as 2 ops are required
	    ## to set CAL and trim+enable)
# Different method to calc column:
	    set coltemp0 $arg1
	    set coltemp1 [expr $coltemp0/2]
	    set coltemp1 [ inttograycode $coltemp1 ]
	    set coltemp1 [ expr $coltemp1*2 ]
	    set coltemp2 [ expr $coltemp1|($coltemp0 & 0x01) ]
	    set temp1 [inttobits $coltemp2 8]
#	    set grayvalue [ inttograycode $arg1 ]
#	    set temp1 [inttobits $grayvalue 8]
	    sendbytesplit $temp1

	    set rowaddress [ inttograycode $arg2 ]
	    ## Row address
	    set temp1 [inttobits $rowaddress 8]
	    sendbytesplit $temp1


	    ## now use the arg4(calcap=1 calbump=2) to set cal_pix
	    set tCd "0"
	    set tCb "0"
	    if {$arg4 == "1"} {
		set tCd "1"
	    } elseif {$arg4 == "2"} {
		set tCb "1"
	    }
	    set tt "000000"
	    set tbyte "$tt$tCb$tCd"
	    sendbytesplit $tbyte
	}
	0x25  {
	    # 0x25 is used to send arg3 to col (arg1) and row (arg2)
	    #  for psi46
	    if {$verbose3==1} {
		set tempstr $arg3
		if {$arg3==8} {
		    set tempstr "Calbit only set (value:8)"
		}
		puts "PIX-25(C:$arg1,R:$arg2)<-$tempstr"
	    }
	    ## Prog_Cal
	    ## Column address, msb must be set
	    set grayvalue [ inttograycode $arg1 ]
	    # shift out a bit and shift back 0
	    set tgrayvalue [ expr $grayvalue / 2 ]
	    set grayvalue [ expr $tgrayvalue * 2 ]
	    set temp1 [inttobits $grayvalue 8]	    
	    sendbytesplit $temp1
	    # send next arg
	    set temp1 [inttobits $arg2 8]
	    sendbytesplit $temp1
	    # send next arg
	    set temp1 [inttobits $arg3 8]
	    sendbytesplit $temp1
	    set ipixcommand 36 
	}
	0x26  {
	    # psi46 Cal_pix
	    # 0x26 is used to send arg3 to col (arg1) and row (arg2)
	    #  for psi46
	    if {$verbose3==1} {
		set tempstr $arg3
		if {$arg3==8} {
		    set tempstr "Calbit only set (value:8)"
		}
		puts "PIX-26 psi46CAL(C:$arg1,R:$arg2)<-$tempstr"
	    }
	    ## Prog_Cal
	    ## Column address, msb must be set
	    set grayvalue [ inttograycode $arg1 ]
	    set temp1 [inttobits $grayvalue 8]	    
	    sendbytesplit $temp1
	    # send next arg
	    set temp1 [inttobits $arg2 8]
	    sendbytesplit $temp1
	    # send next arg
	    if {$arg3 == "1"} {
		set temp1 "00000001"
	    } elseif {$arg3 == "2"} {
		set temp1 "00000010"
	    } else {
		set temp1 "00000000"
	    }
	    sendbytesplit $temp1
	    set ipixcommand 36 
	}
	"ClrCal" {
	    if {$verbose3==1} {
		puts "PIX-02<-CLRCAL"
	    }
	    ## ClrCal
	    ## no additional bytes need to be sent for ClrCal
	    set ipixcommand 2
	}
    }

    if {($pixcommand != "0x97") && ($pixcommand != "0x98")} {
	# send zeros at end
	sendbyte2 "0000" "0000"
    }

# Now send FF after final command in command buffer
    sendff 1

# Send reset pointer
##was4usb	sendbyte "11110010"

######        fixnibblearray 1

    updatefeccmdindicators 1

    if {$slowfecchannel == "1"} {
	# reset pointer
#	pcifeccmdreg 3 1
	pcifeccmdreg 3 1
    } elseif {$slowfecchannel == "2"} {
	# reset pointer
#	pcifeccmdreg 19 1
	pcifeccmdreg 19 1
    }

    
    if {($pixcommand != "0x97") && ($pixcommand != "0x98")} {
        sendnibblearray 0
    } else {
	# compressed buffer
        sendnibblearray 1
    }

        clearnibblearray "0000"


# Send GO
    sendgo 1


    switch $pixcommand {
	"ClrCal" {
	    set ttt [usbroccmd $hubaddress $port $pixchipaddress 1 1 2 3 0]
	}
	0x08 {
	    set ttt [usbroccmd \
			 $hubaddress $port $pixchipaddress 8 $arg1 $arg2 3 0]
	}
	0x44 {
# 4  col row enabled trim	    
	    set ttt [usbroccmd \
			 $hubaddress $port $pixchipaddress 4 $arg1 $arg2 $arg3 $arg5 ]
	}
	0x45 {
# 4  col row enabled trim	    
	    set ttt [usbroccmd \
			 $hubaddress $port $pixchipaddress 2 $arg1 $arg2 $arg4 0 ]
	}
	0x95 {
# send entire col to ROC
	    set ttt [usbroccmd \
			 $hubaddress $port $pixchipaddress 42 $arg1 $arg2 $arg4 0 ]
	}
	0x25 {
	    set ttt [usbroccmd \
			 $hubaddress $port $pixchipaddress 25 $arg1 $arg2 $arg3 0 ]
	}

    }
    


    if {($arg3 == "4") && ($pixcommand != "0x26")} {
# send the cal 
	puts "recurse cal programming"
	pix_transfer_byte "0x26" $arg1 $arg2 $arg3 $arg4 $arg5 $arg6
    }

    updatefeccmdindicators 1
}

set currenttbm A

proc bithubportaddress { command } {
    global hubaddress port
    global hub0 hub1 hub2 hub3 hub4 hubaddresshigh hubaddresslow
    global port0 port1 port2 portlow
    global highnum lownum num inum
    global highnumbits lownumbits
    global currenttbm
    global roctype
    global baseaddress

# This proc is a time hog, so check if we've done this conversion before 
# and if so, just return
    global lasthubaddress lastport


    if {$roctype == "psi46"} {
	if {$currenttbm == "A"} {
	    set baseaddress "0xE0"
	} elseif {$currenttbm == "B"} {
	    set baseaddress "0xF0"
	}
    }
    
# when presented, global num should be hex    
    set highnum [string index $num 0]
    set lownum [string index $num 1]
    set ihighnum [hextoint $highnum]
    set ilownum [hextoint $lownum]
    set inum [expr $ilownum+$ihighnum*16]
#d    puts "Num is **$inum**: $ihighnum,$ilownum"
    set highnumbits [inttobits $ihighnum 4]
    set lownumbits [inttobits $ilownum 4]

    if {($lasthubaddress == $hubaddress) && ($lastport == $port)} {
	return
    } else {
#	puts "RECALCULATED HUB AND PORT"
	set lasthubaddress $hubaddress
	set lastport $port
    }
    
    
    set hubhigh [string index $hubaddress 0]
    set hublow [string index $hubaddress 1]

    set hubhigh [hextoint $hubhigh]
    set hublow [hextoint $hublow]

    set hubaddresshigh [inttobits $hubhigh 1]
    set hubaddresslow [inttobits $hublow 4]

    set hub4 [string index $hubaddresshigh 0]
    set hub3 [string index $hubaddresslow 0]
    set hub2 [string index $hubaddresslow 1]
    set hub1 [string index $hubaddresslow 2]
    set hub0 [string index $hubaddresslow 3]

    set bytehub "000$hub4$hub3$hub2$hub1$hub0"
    set ihub [bitstoint $bytehub]
#d	puts "ihub: $ihub"
    if { ($ihub < 0) || ($ihub >=32) } {
	puts "ERROR - ihub out of range at $ihub"
    }

    set portnibble [string index $port 0]
    set portnibble [hextoint $portnibble]
    set portlow [inttobits $portnibble 3]

    set port2 [string index $portlow 0]
    set port1 [string index $portlow 1]
    set port0 [string index $portlow 2]

}

set linecount 1
set checkingparity 0

proc usbread { mycommand } {
# This is the current input/read handler for the USB port and is called every
# time a character is returned.  This will have to be expanded upon to 
# assemble individual returned chars into some sense.
    global globalparity
    global textwindow
    global usbdeviceid
    global usbreaddata
    global usbreadindex
# checkingparity flag is set when we want to look at parity then it disables
# the function.  So to use this, send the REQUEST RETURN OF PARITY BYTE, just
# after enabling checkingparity to 1.  The next byte coming back will then be
# checked against the global parity and the checking function will be turned
# off so that other data is displayed without flagging parity errors.
    global checkingparity
    global linecount
#    set usbtext [gets $fileid]
    set usbtext [read $usbdeviceid 1]
############    .wintbm2.f8.b1 config -activeforeground "#00FF00" -fg "#00FF00" -bg "#00FF00"
   $textwindow.b1 config -activeforeground "#00FF00" -fg "#00FF00" -bg "#00FF00"


#    puts "=$usbtext="
    if {$usbtext != ""} {
	set myreturn [format "(%x)" [chr $usbtext]]
	$textwindow configure -state normal
	set tempchar [chr $usbtext]
	set tempusbreaddata [inttobits $tempchar 8]
	$textwindow insert 1.0 "$linecount: $myreturn  $tempusbreaddata $usbreadindex\n"
	if {$usbreadindex == "1"} {
	    set usbreaddata $tempusbreaddata
	}
	incr linecount
	incr usbreadindex
	$textwindow configure -state disabled
	set returnedparity $usbtext

	if {$checkingparity == "1"} {
	    set checkingparity 0
	    if {$returnedparity == $globalparity} {
		#	    puts "*******************parity OKAY****************"
	    } else {
		set tttemp [format "(%x)" [chr $globalparity]]
		set result [tk_messageBox -parent . \
				-title { Error } -type ok \
				-icon info \
				-message "PARITY ERROR\nGot:$myreturn\nExpected:$$tttemp" ]
		#	    puts "*****FFFFF   PARITY MISMATCHED    FFFFF*******"
		#	    exit
	    }
	    #d	puts "**** ---->Got:$myreturn"
	    
	}
    }
}

proc init_usb { command } {
    global checkingparity
    global linecount
    global usbdeviceid
    global PCTARGET
    global OS
#windows###set usbdeviceid [open "com3:" w+]
#set usbdeviceid [open "com3:" w+]
#set fileid [open "com2:" w+]
#set fileid [open "com1:" w+]
##linux on serial port, so it runs ###set usbdeviceid [open "/dev/ttyS0" w+]
#set usbdeviceid [open "/dev/ttyS0" w+]

# linux on real USB
#set usbdeviceid [open "/dev/ttyUSB0" w+]

    if {$PCTARGET == "devel"} {
	set usbdeviceid [open "/dev/ttyS0" w+]
    } elseif {($PCTARGET == "release") && ($OS == "linux")} {
##6-3	set usbdeviceid [open "/dev/ttyUSB0" w+]
	set usbdeviceid [open "/dev/ttyS0" w+]
    } elseif {($PCTARGET == "release") && ($OS == "windows")} {
	set usbdeviceid [open "com3:" w+]
#	set usbdeviceid [open "com5:" w+]
    } else {
	put "\n\nBUILD FLAG ERROR\n\n"
    }

# until Jun3,05: set ttt [fconfigure $usbdeviceid -mode 9600,n,8,1 -buffering none -blocking false -translation binary]
set ttt [fconfigure $usbdeviceid -mode 9600,n,8,1 -buffering full -blocking false -translation binary]
set checkingparity 0

set linecount 1

############### fileevent $usbdeviceid readable "usbread 1"

}

########init_usb 1


proc sendresetpointer { command } {
    global slowfecchannel
    global textwindowline
    global verbose3
    if {$slowfecchannel == "1"} {
	if {$verbose3 > 0} {
	    puts "Reseting FEC channel 1/2 WR Pointer"
	}
	pcifeccmdreg 1 1
#	pcifecwriteword 12 00000002
	set textwindowline 0
    } elseif {$slowfecchannel == "2"} {
	if {$verbose3 > 0} {
	    puts "Reseting FEC channel 2/2 WR Pointer"
	}
	pcifeccmdreg 17 1
#	pcifecwriteword 12 00020000
	set textwindowline 0
    }
}
set usbreadindex 0
proc read16bytes { command } {
#USB function
    global usbreadindex
    set usbreadindex 0
    sendbyte "11111000"
    usbflush
}
proc sendzero { command } {
    global verbose3
    if {$verbose3 > 0} {
	puts "PCI SendZero called, inserting 0x00 into output"
    }
    sendbyte2 "0000" "0000"
#    sendbyte2 "1111" "1111"
}
proc sendff { command } {
    global verbose3
    if {$verbose3 > 0} {
	puts "PCI Sendff called, inserting 0xFF into output"
    }
    sendbyte2 "1111" "1111"
}
set dostopsendgo 1
proc sendgo { mycommand } {
   global dostopsendgo
    global slowfecchannel
    global verbose3
    if {$verbose3 > 0} { puts "FEC sending START(GO)" }
# send the actual go command

    if {$slowfecchannel == "1"} {
	# reset pointer
	pcifeccmdreg 3 1
	# send go
	pcifeccmdreg 0 1
#	pcifecwriteword 12 00000001
    } elseif {$slowfecchannel == "2"} {
	# reset pointer
	pcifeccmdreg 19 1
	# send go
	pcifeccmdreg 16 1
#	pcifecwriteword 12 00010000
    }
	    updatefeccmdindicators 1

    if { $dostopsendgo == 0 } {
	update idletasks
	after 100 sendgo 1
    }
}
proc sendbytewparity { my7bitbyte } {
# take a byte, actually lowest 7 bits, and return the parity bit 
    set b6 [string index $my7bitbyte 0]
    set b5 [string index $my7bitbyte 1]
    set b4 [string index $my7bitbyte 2]
    set b3 [string index $my7bitbyte 3]
    set b2 [string index $my7bitbyte 4]
    set b1 [string index $my7bitbyte 5]
    set b0 [string index $my7bitbyte 6]
    set myparity [expr 1^$b6^$b5^$b4^$b3^$b2^$b1^$b0]
    set mybyte "$myparity$my7bitbyte"

    sendbyte $mybyte
}

proc callatencysendbyte2 { highnib lownib bitdefault } {
    #d    puts "sendbyte2 called w highbyte:$highnib and lowbyte:$lownib"
    global encalpulse fec_trigger_source norm_loop_on callatencybit
    global clkph
    set hbit3 [string index $highnib 0]
    set hbit2 [string index $highnib 1]
    set hbit1 [string index $highnib 2]
    set hbit0 [string index $highnib 3]
    set lbit3 [string index $lownib 0]
    set lbit2 [string index $lownib 1]
    set lbit1 [string index $lownib 2]
    set lbit0 [string index $lownib 3]

    if {$callatencybit == "enabled"} {
	set tt1 [wordtobits 00000010]
    } else {
	set tt1 [wordtobits 00000000]
    }
    
    if {$bitdefault >= 0} {
	set tt1 [bitwordset $tt1 $bitdefault 1]
    }
    # unpack clkph bits
    set clkphbits [inttobits $clkph 4]
    set clkph0 [string index $clkphbits 3]
    set clkph1 [string index $clkphbits 2]
    set clkph2 [string index $clkphbits 1]
    set clkph3 [string index $clkphbits 0]
    set tt1 [bitwordset $tt1 16 $clkph0]
    set tt1 [bitwordset $tt1 17 $clkph1]
    set tt1 [bitwordset $tt1 18 $clkph2]
    set tt1 [bitwordset $tt1 19 $clkph3]



        set tt2 [bitwordset $tt1 7 $encalpulse]
        set tt2 [bitwordset $tt2 8 $lbit0]
        set tt2 [bitwordset $tt2 9 $lbit1]
        set tt2 [bitwordset $tt2 10 $lbit2]
        set tt2 [bitwordset $tt2 11 $lbit3]
        set tt2 [bitwordset $tt2 12 $hbit0]
        set tt2 [bitwordset $tt2 13 $hbit1]
        set tt2 [bitwordset $tt2 14 $hbit2]
        set tt2 [bitwordset $tt2 15 $hbit3]
        set tt2 [bitwordset $tt2 7 $encalpulse]
	set tt2 [bitwordset $tt2 5 $fec_trigger_source]
	set tt2 [bitwordset $tt2 6 $norm_loop_on]
	set tt3 [bitstoword $tt2]
	pcifecwriteword 28 $tt3
    

}
set nibblearray(0) 0
set nibblecount 0
for {set ijk 0} {$ijk < 800} {incr ijk} {
    set nibblearray($ijk) "0000"
}

proc clearnibblearray { myval } {
    global nibblearray
    global nibblecount
    set nibblecount 0
    for {set ijk 0} {$ijk < 800} {incr ijk} {
	set nibblearray($ijk) $myval
    }
}
proc sendbyte2 { highnib lownib } {
#d    puts "sendbyte2 called w highbyte:$highnib and lowbyte:$lownib"
    global nibblearray
    global nibblecount

    set hbit3 [string index $highnib 0]
    set hbit2 [string index $highnib 1]
    set hbit1 [string index $highnib 2]
    set hbit0 [string index $highnib 3]
    set lbit3 [string index $lownib 0]
    set lbit2 [string index $lownib 1]
    set lbit1 [string index $lownib 2]
    set lbit0 [string index $lownib 3]
##    set hpbit [expr 1^$hbit3^$hbit2^$hbit1^$hbit0]
##    set lpbit [expr 1^1^$lbit3^$lbit2^$lbit1^$lbit0]
    set txbytehigh "$hbit3$hbit2$hbit1$hbit0"
    set txbytelow  "$lbit3$lbit2$lbit1$lbit0"

    set nibblearray($nibblecount) $txbytehigh
    incr nibblecount
    set nibblearray($nibblecount) $txbytelow
    incr nibblecount

    set outstring ""
    for {set ijk 0} {$ijk < $nibblecount} {incr ijk} {
        set outstring "$outstring $nibblearray($ijk)"
    }

#    fixnibblearray 1

    return
}

proc sendnibblearray { compressedbuffer } {
# Sending nibblearray to PCI FEC number-argument 1 or 2
    global nibblearray
    global nibblecount
    global slowfecchannel
    global verbose3

#    puts "sendnibblearray with nibblecount: $nibblecount"
    set numwords [expr $nibblecount/8 + 1]
    for {set ijk 0} {$ijk < $numwords} {incr ijk} {
	set in8 [expr 8*$ijk]
	set n8 $nibblearray($in8)
	set hexn8 [bitstohex $n8]

	set in7 [expr 8*$ijk+1]
	set n7 $nibblearray($in7)
	set hexn7 [bitstohex $n7]

	set in6 [expr 8*$ijk+2]
	set n6 $nibblearray($in6)
	set hexn6 [bitstohex $n6]

	set in5 [expr 8*$ijk+3]
	set n5 $nibblearray($in5)
	set hexn5 [bitstohex $n5]

	set in4 [expr 8*$ijk+4]
	set n4 $nibblearray($in4)
	set hexn4 [bitstohex $n4]

	set in3 [expr 8*$ijk+5]
	set n3 $nibblearray($in3)
	set hexn3 [bitstohex $n3]

	set in2 [expr 8*$ijk+6]
	set n2 $nibblearray($in2)
	set hexn2 [bitstohex $n2]

	set in1 [expr 8*$ijk+7]
	set n1 $nibblearray($in1)
	set hexn1 [bitstohex $n1]

	set finalword "$hexn2$hexn1$hexn4$hexn3$hexn6$hexn5$hexn8$hexn7"
	if {$verbose3 > 0} { puts "sendnibblearray FEC $slowfecchannel: $finalword" }

	set fecoffset 0
# Compressed buffer and wr buffer now the same
#	if {$compressedbuffer == "1"} {
#	    set fecoffset 8
#	}
	if {$slowfecchannel == "1"} {
	    set fecoffset $fecoffset
	} elseif {$slowfecchannel == "2"} {
	    set fecoffset [expr $fecoffset + 16]
	}
	pcifecwriteword $fecoffset $finalword
    }
}
proc fixnibblearray { command } {
# fix the standard byte order of the nibble array (33 22 11 00) to 33 11 22 00 
# for fec ordering
# CHECK THIS
    global nibblearray
    global nibblecount

    if {$nibblecount == 0} {
	puts "fixnibblearray: 0 length for array"
	return
    }
    set numwords [expr $nibblecount/8 + 1]
    puts "fixnubblearray: numwords=$numwords  nibblecount=$nibblecount"
    for {set ijk 0} {$ijk < $numwords} {incr ijk} {
	set inexthigh [expr 8*$ijk+2]
	set inextlow [expr 8*$ijk+3]
	set temp1 $nibblearray($inexthigh)
	set temp2 $nibblearray($inextlow)
	set inexthigh [expr 8*$ijk+4]
	set inextlow [expr 8*$ijk+5]
	set temp3 $nibblearray($inexthigh)
	set temp4 $nibblearray($inextlow)
	set nibblearray($inexthigh) $temp1
	set nibblearray($inextlow) $temp2
	set inexthigh [expr 8*$ijk+2]
	set inextlow [expr 8*$ijk+3]
	set nibblearray($inexthigh) $temp3
	set nibblearray($inextlow) $temp4
    }
    set outstring ""
    for {set ijk 0} {$ijk < $nibblecount} {incr ijk} {
        set outstring "$outstring $nibblearray($ijk)"
    }
    puts "Current nibblearray: $outstring (exiting fixnibblearray)"

}
proc resetparitybyte {} {
    global p7 p6 p5 p4 p3 p2 p1 p0
    set p0 0
    set p1 0
    set p2 0
    set p3 0
    set p4 0
    set p5 0
    set p6 0
    set p7 0
}
resetparitybyte
proc hextoint { myhex } {
# convert one hex char to integer
    set ir -1
    switch $myhex {
	0 { set ir 0 }
	1 { set ir 1 }
	2 { set ir 2 }
	3 { set ir 3 }
	4 { set ir 4 }
	5 { set ir 5 }
	6 { set ir 6 }
	7 { set ir 7 }
	8 { set ir 8 }
	9 { set ir 9 }
	A { set ir 10 }
	B { set ir 11 }
	C { set ir 12 }
	D { set ir 13 }
	E { set ir 14 }
	F { set ir 15 }
	a { set ir 10 }
	b { set ir 11 }
	c { set ir 12 }
	d { set ir 13 }
	e { set ir 14 }
	f { set ir 15 }
    }
    if {$ir == -2} {
	puts "Error in hex conversion"
	set result [tk_messageBox -parent . \
			-title { Error } -type ok \
			-icon info \
			-message "ERROR in hex conversion\nWhat is:$myhex?"]

	set ir 0
    }
#d	puts "HEX:$myhex INT:$ir"

    return $ir
}
 proc inttobits { myint numbits } {
    set k 0
    if { [ expr $myint&128 ] == 128 } {
	set bit7 1
    } else {
	set bit7 0
    }
    if { [ expr $myint&64 ] == 64 } {
	set bit6 1
    } else {
	set bit6 0
    }
    if { [ expr $myint&32 ] == 32 } {
	set bit5 1
    } else {
	set bit5 0
    }
    if { [ expr $myint&16 ] == 16 } {
	set bit4 1
    } else {
	set bit4 0
    }
    if { [ expr $myint&8 ] == 8 } {
	set bit3 1
    } else {
	set bit3 0
    }
    if { [ expr $myint&4 ] == 4 } {
	set bit2 1
    } else {
	set bit2 0
    }
    if { [ expr $myint&2 ] == 2 } {
	set bit1 1
    } else {
	set bit1 0
    }
    if { [ expr $myint&1 ] == 1 } {
	set bit0 1
    } else {
	set bit0 0
    }
    switch $numbits {
	8 { set bitvalue [format "%s%s%s%s%s%s%s%s" $bit7 $bit6 $bit5 $bit4 \
		$bit3 $bit2 $bit1 $bit0] }
	7 { set bitvalue [format "%s%s%s%s%s%s%s" $bit6 $bit5 $bit4 \
		$bit3 $bit2 $bit1 $bit0] }
	6 { set bitvalue [format "%s%s%s%s%s%s" $bit5 $bit4 \
		$bit3 $bit2 $bit1 $bit0] }
	5 { set bitvalue [format "%s%s%s%s%s" $bit4 \
		$bit3 $bit2 $bit1 $bit0] }
	4 { set bitvalue [format "%s%s%s%s" $bit3 $bit2 $bit1 $bit0] }
	3 { set bitvalue [format "%s%s%s"  $bit2 $bit1 $bit0] }
	2 { set bitvalue [format "%s%s" $bit1 $bit0] }
	1 { set bitvalue [format "%s" $bit0] }
    }
    return $bitvalue
}
proc bitstohex { mybits } {
    set bit3 [string index $mybits 0]
    set bit2 [string index $mybits 1]
    set bit1 [string index $mybits 2]
    set bit0 [string index $mybits 3]

    set myvalue 0
    if {$bit3 == 1} then { set myvalue [expr $myvalue|8] }
    if {$bit2 == 1} then { set myvalue [expr $myvalue|4] }
    if {$bit1 == 1} then { set myvalue [expr $myvalue|2] }
    if {$bit0 == 1} then { set myvalue [expr $myvalue|1] }
    
    set myhex "?"
    switch $myvalue {
	0 { set myhex 0 }
	1 { set myhex 1 }
	2 { set myhex 2 }
	3 { set myhex 3 }
	4 { set myhex 4 }
	5 { set myhex 5 }
	6 { set myhex 6 }
	7 { set myhex 7 }
	8 { set myhex 8 }
	9 { set myhex 9 }
	10 { set myhex A }
	11 { set myhex B }
	12 { set myhex C }
	13 { set myhex D }
	14 { set myhex E }
	15 { set myhex F }
    }
    return $myhex
}
proc hexhigh { myval } {
# take a 2 hex char string and return the high nibble as a binary string
    set htemp [string index $myval 0]
    set ihex [hextoint $htemp]
    set mybits [inttobits $ihex 4]
    return $mybits
}
proc hexlow { myval } {
# take a 2 hex char string and return the low nibble as a binary string
    set htemp [string index $myval 1]
    set ihex [hextoint $htemp]
    set mybits [inttobits $ihex 4]
    return $mybits
}


proc bitstochar { mybyte } {
    set bit7 [string index $mybyte 0]
    set bit6 [string index $mybyte 1]
    set bit5 [string index $mybyte 2]
    set bit4 [string index $mybyte 3]
    set bit3 [string index $mybyte 4]
    set bit2 [string index $mybyte 5]
    set bit1 [string index $mybyte 6]
    set bit0 [string index $mybyte 7]

    set myvalue 0
    if {$bit7 == 1} then { set myvalue [expr $myvalue|128] }
    if {$bit6 == 1} then { set myvalue [expr $myvalue|64] }
    if {$bit5 == 1} then { set myvalue [expr $myvalue|32] }
    if {$bit4 == 1} then { set myvalue [expr $myvalue|16] }
    if {$bit3 == 1} then { set myvalue [expr $myvalue|8] }
    if {$bit2 == 1} then { set myvalue [expr $myvalue|4] }
    if {$bit1 == 1} then { set myvalue [expr $myvalue|2] }
    if {$bit0 == 1} then { set myvalue [expr $myvalue|1] }
    
    set tchar [asc $myvalue]
    return $tchar
}
proc bitstoint { mybyte } {
    set bit7 [string index $mybyte 0]
    set bit6 [string index $mybyte 1]
    set bit5 [string index $mybyte 2]
    set bit4 [string index $mybyte 3]
    set bit3 [string index $mybyte 4]
    set bit2 [string index $mybyte 5]
    set bit1 [string index $mybyte 6]
    set bit0 [string index $mybyte 7]

    set myvalue 0
    if {$bit7 == 1} then { set myvalue [expr $myvalue|128] }
    if {$bit6 == 1} then { set myvalue [expr $myvalue|64] }
    if {$bit5 == 1} then { set myvalue [expr $myvalue|32] }
    if {$bit4 == 1} then { set myvalue [expr $myvalue|16] }
    if {$bit3 == 1} then { set myvalue [expr $myvalue|8] }
    if {$bit2 == 1} then { set myvalue [expr $myvalue|4] }
    if {$bit1 == 1} then { set myvalue [expr $myvalue|2] }
    if {$bit0 == 1} then { set myvalue [expr $myvalue|1] }
    
    return $myvalue
}

# convert integer to ascii char
proc asc i { 
    if { $i<0 || $i>255 } { error "asc:Integer out of range 0-255" } 
    return [format %c $i ] 
}

proc chr c { 
    if {[string length $c] > 1 } { error "chr: arg should be a single char"}
#   set c [ string range $c 0 0] 
    set v 0; 
    scan $c %c v; return $v
}


proc sendbytechr { mychr } {
    global usbdeviceid
    set ttt [inttobits $mychr 8]
#   puts "SENDBYTE--:$ttt"
####dorosh    puts -nonewline $usbdeviceid [asc $mychr]
    usbq $mychr
    return

    puts -nonewline $usbdeviceid [asc $mychr]
#    puts -nonewline $usbdeviceid "0000000000"
}
proc sendbyte { mybyte } {
# takes the binary ascii representation

    global usbdeviceid
#    global globalparity

# keep here in case we put back parity check
#    set hbit3 [string index $mybyte 0]
#    set hbit2 [string index $mybyte 1]
#    set hbit1 [string index $mybyte 2]
#    set hbit0 [string index $mybyte 3]
#    set lbit3 [string index $mybyte 4]
#    set lbit2 [string index $mybyte 5]
#    set lbit1 [string index $mybyte 6]
#    set lbit0 [string index $mybyte 7]
#    set p7 [expr $p7^$hbit3]
#    set p6 [expr $p6^$hbit2]
#    set p5 [expr $p5^$hbit1]
#    set p4 [expr $p4^$hbit0]
#    set p3 [expr $p3^$lbit3] 
#    set p2 [expr $p2^$lbit2]
#    set p1 [expr $p1^$lbit1]
#    set p0 [expr $p0^$lbit0]
#    set highnibble "$p7$p6$p5$p4"
#    set lownibble "$p3$p2$p1$p0"
#    set bparity "$highnibble$lownibble"

##reinsert? if checkingparity == 1
#reingsert?    set globalparity [bitstochar $bparity]

#####    set ttt2 [bitstochar $mybyte]
####    set ttt3 [format "FINAL SEND: (%x)" [chr $ttt2]]
####    puts "$ttt3"

#    puts "SENDBYTE:$mybyte"
####    usbq [bitstoint $mybyte]
    sendbytesplit $mybyte

    return
	puts -nonewline $usbdeviceid [bitstochar $mybyte]
}

proc transfer_byte { myvalue offset mydirection } {
    global hubaddress
    global portaddress
    global hub0 hub1 hub2 hub3 hub4 hubaddresshigh hubaddresslow
    global port0 port1 port2 portlow
    global highnumbits lownumbits
    global port
    global returnedbits
    global tbm
    global baseaddress
    global lastreadbits
    global datalength
    global verbose3
    global interface
    global num inum
    global currenttbm
    global verbose4

    if {$currenttbm == "A"} {
	set baseaddress "0xE0"
    } elseif {$currenttbm == "B"} {
	set baseaddress "0xF0"
    }

    if {$verbose3==1} {
	puts "TBM(B:$baseaddress,H:$hubaddress,P:$portaddress)<-$offset DATA:$myvalue $mydirection"
    }


# test the C tbm write routine:
    if {$baseaddress == "0xE0"} {
	set param1 14
    } else {
	set param1 15
    }
    switch $offset {
	0x0 { set param4 0 }
	0x1 { set param4 1 }
	0x2 { set param4 2 }
	0x3 { set param4 3 }
	0x4 { set param4 4 }
	0x5 { set param4 5 }
	0x6 { set param4 6 }
	0x7 { set param4 7 }
    }
    set param5 [bitstoint $myvalue]
    if {$mydirection == "write"} {
	set param6 0
    } else {
	set param6 1
    }

#    puts "usbtbmcmd $param1 $hubaddress $portaddress $param4 $param5 $param6"
    
    if {$verbose4 == 1} {
	puts "usbtbmcmd $param1 $hubaddress $portaddress $param4 $param5 $param6"
    }    
    usbtbmcmd $param1 $hubaddress $portaddress $param4 $param5 $param6
    
#    return

    clearnibblearray "0000"

###6-1	sendFECaddress4
	set port $portaddress
# convert to the bits we need to send
	sendresetpointer 1
	set tempnum $num
	set num "02"
	bithubportaddress 1
	set num $tempnum
	if {$baseaddress == "0xE0"} {
	    set ibaseaddresshigh "1110"
	}
	if {$baseaddress == "0xF0"} {
	    set ibaseaddresshigh "1111"
	}
	switch $offset {
	    0x0 { set boffset "000" }
	    0x1  { set boffset "001" }
	    0x2 { set boffset "010" }
	    0x3 { set boffset "011" }
	    0x4 { set boffset "100" }
	    0x5 { set boffset "101" }
	    0x6 { set boffset "110" }
	    0x7 { set boffset "111" }
	}
	if {$mydirection == "write"} {
	    set bwrite "0"
	} else {
	    set bwrite "1"
	}
	set baddress "$boffset$bwrite"

	set temp1 [inttobits $hubaddress 5]
	set temp2 [inttobits $portaddress 3]
	set temp3 "$temp1$temp2"
	set temp4 [bitstoint $temp3]
	set temp5 [inttobits $temp4 4]
	set tlow $temp5
	set temp6 [expr $temp4 / 16]
	set temp7 [inttobits $temp6 4]
	set thigh $temp7
	# Send the hub and port address
	sendbyte2 "$thigh" "$tlow"

# Send the number of bytes (num) of data to follow
	sendbyte2 "$highnumbits" "$lownumbits"

	# Send the tbm and offset address
	sendbyte2 "$ibaseaddresshigh" "$baddress"

	set bb0 [string index $myvalue 7]
	set bb1 [string index $myvalue 6]
	set bb2 [string index $myvalue 5]
	set bb3 [string index $myvalue 4]
	set bb4 [string index $myvalue 3]
	set bb5 [string index $myvalue 2]
	set bb6 [string index $myvalue 1]
	set bb7 [string index $myvalue 0]
	set thigh "$bb7$bb6$bb5$bb4"
	set tlow "$bb3$bb2$bb1$bb0"
	sendbyte2 "$thigh" "$tlow"
	sendzero 1
        sendff 1
	sendresetpointer 1
# send get parity
#	set checkingparity 1
#	sendbyte "11110100"

######        fixnibblearray 1
        sendnibblearray 0
        clearnibblearray "0000"

	sendgo 1

    updatefeccmdindicators 1

##	resetparitybyte

}

proc testwrite2 { myvalue } {
# test hub of myvalue to test if existing
    global D7position
    global D0position
    global D7positiontbm2
    global D3positiontbm2
    global datalength
    global verbose2
    global hubaddress
    global portaddress
    global verbose3
    global usbreaddata
    global AnalogOutputGain


    set myreturn -1

    set tempportaddress $portaddress
    set portaddress 4
    set temphubaddress $hubaddress
    set hubaddress $myvalue
# We need to test that there is a hub connected so we search for
# a responsive hub/tbm and then determine if it is an FPGA tbm (does not)
# have a readable analogoutput gain register or the Si tbm (does have the
# register).

# First, we will send a 128 to hub, not send a go but read back the memory
# in the usb FEC.  If the FEC is there we will get the value 128 back
    set tempgain $AnalogOutputGain

    puts "TESTWRITE2 with hub,port: $hubaddress $portaddress"

    write_slider .wintbm2.fg3.f8.ff1.c1 $myvalue "GainWrite" 
    write_slider .wintbm2.fg3.f8.ff1.c1 $myvalue "GainWrite" 

#    sendresetpointer 1
#    read16bytes 1
    getusbreaddata 1
    set byteread $usbreaddata
    puts "Auto hub discover testwrite: $byteread"
    set ibyteread [bitstoint $byteread]
    puts "Testing for hub:$myvalue -- memory holds:$ibyteread"
    if {$ibyteread == $myvalue} {
	write_slider .wintbm2.fg3.f8.ff1.c1 128 "GainWrite" 
#	sendresetpointer 1
#	read16bytes 1
	getusbreaddata 1
	set byteread $usbreaddata
	#    puts "Auto hub discover testwrite: $byteread"
	if {$byteread == "10000000"} {
	    # read back from fec memory so hub/tbm found
	    puts "***************** tbm found ****************"
	    set myreturn 1
	}
    }

    # so if myreturn==1 we have atleast an FPGA hub.  Now check if it is
    # a Si tbm
    if {$myreturn == 1} {
	write_slider .wintbm2.fg3.f8.ff1.c1 128 "GainRead" 
#	sendresetpointer 1
#	read16bytes 1
	getusbreaddata 1
	set byteread $usbreaddata
	puts "Auto hub discover Si tbm testwrite: $byteread"
	if {$byteread == "10000000"} {
	    # read back from tbm memory so hub/tbm found
	    set myreturn 2
	    puts "**************** Si tbm found ************"
	    puts "****** resetting AnalogOutputGain ********"
	    write_slider .wintbm2.fg3.f8.ff1.c1 $tempgain "GainWrite" 
	}
    }
	

    set AnalogOutputGain $tempgain

    if {$myreturn >= 0} {
	if {$verbose3==1} {
	    puts "\n\n**************************************************\n   HUB DETECTED\n
	   hub:$hubaddress   port:$portaddress\n**************************************************\n"
	    after 500
	}
    }

    set portaddress $tempportaddress
    set hubaddress $temphubaddress

    return $myreturn

}

proc writeoutfilecheckpoint { command } {
    global fdataout
    global outformat

    if {($outformat=="seu")||($outformat=="paw24")||($outformat=="text1k")} {
	puts $fdataout "MARKER $command"
    }
}
set usbreaddata "00000000"
proc getusbreaddata { command } {
    global usbreaddata
    global usbreadindex
    global usbdeviceid
    global verbose3
    global outformat
    global fdataout

   sendresetpointer 1

#    update idletasks
#    update

    read16bytes 1
    set mytimeout 0

    set ttt [usbgetrd]
    if {$ttt != 16} {
	puts "TIMEOUT - got $ttt bytes instead of 16"
	set usbreaddata 0
    } else {
#	puts "USBGETRD returns $ttt"
	set tttt [usbdata 1]
#	puts "GOT THIS:$tttt"
	set usbreaddata [inttobits $tttt 8]
#	for {set k 0} {$k < $ttt} {incr k} {
#	    set ij [usbdata $k]
#	    puts "USBDATA $k = $ij"
#	}
    }

}
proc tbm2_read_f0 { command } {
    global D7position
    global D0position
    global D7positiontbm2
    global D3positiontbm2
    global datalength
    global verbose2
    global usbreaddata
    global slowfecchannel
    global verbose3

# write all 1s to data while reading    
    set bytewrite "11111111"
    transfer_byte $bytewrite "0x0" "read"    
#    set byteread [getdataticks 1]
##    set byteread [getreturnedbits 1]
####    getusbreaddata 1
    clearnibblearray "0000"
#reset the RD buffer before reading
    if {$slowfecchannel == "1"} {
	pcifeccmdreg 2 1
    } elseif {$slowfecchannel == "2"} {
	pcifeccmdreg 18 1
    }
    if {$slowfecchannel == "1"} {
	set hextemp [pcifecreadword 4]
    } else {
	set hextemp [pcifecreadword 20]
    }
    set hextemp [padhexword $hextemp]
#    set retdata [string range $hextemp 2 3]
    set retdata [string range $hextemp 4 5]
    
    set temphigh [hexhigh $retdata]
    set templow [hexlow $retdata]
    set byteread "$temphigh$templow"

    if {$verbose3 > 0} {
	puts "PCI TBM2_READ_F0 RETURN BYTE=$byteread  RETURNWORD=$hextemp"
    }

#    set byteread $usbreaddata

    .wintbm2.fg2.ft1.f07.rc07 deselect
    .wintbm2.fg2.ft1.f06.rc06 deselect
    .wintbm2.fg2.ft1.f05.rc05 deselect
    .wintbm2.fg2.ft1.f04.rc04 deselect
    .wintbm2.fg2.ft1.f03.rc03 deselect
    .wintbm2.fg2.ft1.f02.rc02 deselect
    .wintbm2.fg2.ft1.f01.rc01 deselect
    .wintbm2.fg2.ft1.f00.rc00 deselect
    set bit7 [string index $byteread 0]
    set bit6 [string index $byteread 1]
    set bit5 [string index $byteread 2]
    set bit4 [string index $byteread 3]
    set bit3 [string index $byteread 4]
    set bit2 [string index $byteread 5]
    set bit1 [string index $byteread 6]
    set bit0 [string index $byteread 7]
    if {$bit7 == 1} then { .wintbm2.fg2.ft1.f07.rc07 select }
    if {$bit6 == 1} then { .wintbm2.fg2.ft1.f06.rc06 select }
    if {$bit5 == 1} then { .wintbm2.fg2.ft1.f05.rc05 select }
    if {$bit4 == 1} then { .wintbm2.fg2.ft1.f04.rc04 select }
    if {$bit3 == 1} then { .wintbm2.fg2.ft1.f03.rc03 select }
# Note the flip in the Reset on stack count bit 
#    if {$bit2 == 0} then { .wintbm2.fg2.ft1.f02.rc02 select }
# Flip removed, 1-18-03
    if {$bit2 == 1} then { .wintbm2.fg2.ft1.f02.rc02 select }
    if {$bit1 == 1} then { .wintbm2.fg2.ft1.f01.rc01 select }
    if {$bit0 == 1} then { .wintbm2.fg2.ft1.f00.rc00 select }

}
proc tbm2_read_f1 { command } {
    global D7position
    global D0position
    global D7positiontbm2
    global D3positiontbm2
    global datalength
    global verbose2
    global usbreaddata
    global slowfecchannel
    global verbose3

# write all 1s to data while reading    
    set bytewrite "11111111"
    transfer_byte $bytewrite "0x1" "read"    
    clearnibblearray "0000"
    if {$slowfecchannel == "1"} {
	pcifeccmdreg 2 1
    } elseif {$slowfecchannel == "2"} {
	pcifeccmdreg 18 1
    }
    if {$slowfecchannel == "1"} {
	set hextemp [pcifecreadword 4]
    } else {
	set hextemp [pcifecreadword 20]
    }
    set hextemp [padhexword $hextemp]
#    set retdata [string range $hextemp 2 3]
    set retdata [string range $hextemp 4 5]

    set temphigh [hexhigh $retdata]
    set templow [hexlow $retdata]
    set byteread "$temphigh$templow"

    if {$verbose3 > 0} {
	puts "PCI TBM2_READ_F1 RETURN BYTE=$byteread  RETURNWORD=$hextemp"
    }


#    getusbreaddata 1
#    set byteread $usbreaddata

    set bit7 [string index $byteread 0]
    set bit6 [string index $byteread 1]
    set bit5 [string index $byteread 2]
    set bit4 [string index $byteread 3]
    set bit3 [string index $byteread 4]
    set bit2 [string index $byteread 5]
    set bit1 [string index $byteread 6]
    set bit0 [string index $byteread 7]

    set myvalue 0
    if {$bit5 == 1} then { set myvalue [expr $myvalue|32] }
    if {$bit4 == 1} then { set myvalue [expr $myvalue|16] }
    if {$bit3 == 1} then { set myvalue [expr $myvalue|8] }
    if {$bit2 == 1} then { set myvalue [expr $myvalue|4] }
    if {$bit1 == 1} then { set myvalue [expr $myvalue|2] }
    if {$bit0 == 1} then { set myvalue [expr $myvalue|1] }

    set tempstring "Stack\nCount:\n$myvalue"
    if {$bit7==0} then {
	set modestring     "Mode:\n\nSync           (0 $bit6)" 
    }
    if {$bit7==1} then {
	if {$bit6==0} then { set modestring "Mode:\n\nClr Trig Count (1 0)" }
	if {$bit6==1} then { set modestring "Mode:\n\nCalibration    (1 1)" }
    }
    .wintbm2.fg2.ft2.f17.f1.l1 configure -text $modestring
    if {$verbose2==1} {
	puts "Stack read int: $myvalue"
    }

    .wintbm2.fg2.ft2.f17.f2.l1 configure -text $tempstring
    .wintbm2.fg2.ft2.f17.f2.l1 configure -foreground "black"

}
proc tbm2_read_f2 { command } {
    global D7position
    global D0position
    global D7positiontbm2
    global D3positiontbm2
    global datalength
    global verbose2
    global usbreaddata
    global slowfecchannel
    global verbose3

# write all 1s to data while reading    
    set bytewrite "11111111"
    transfer_byte $bytewrite "0x2" "read"    
    clearnibblearray "0000"
#reset the RD buffer before reading
    if {$slowfecchannel == "1"} {
	pcifeccmdreg 2 1
    } elseif {$slowfecchannel == "2"} {
	pcifeccmdreg 18 1
    }
    if {$slowfecchannel == "1"} {
	set hextemp [pcifecreadword 4]
    } else {
	set hextemp [pcifecreadword 20]
    }
    set hextemp [padhexword $hextemp]
#    set retdata [string range $hextemp 2 3]
    set retdata [string range $hextemp 4 5]
    
    set temphigh [hexhigh $retdata]
    set templow [hexlow $retdata]
    set byteread "$temphigh$templow"

    if {$verbose3 > 0} {
	puts "PCI TBM2_READ_F2 RETURN BYTE=$byteread  RETURNWORD=$hextemp"
    }




#    getusbreaddata 1
#    set byteread $usbreaddata

    set tempint [ stringtoint $byteread ]
    set tempstring "EventNum:\n$tempint"
    .wintbm2.fg2.ft2.f27.l1 configure -text $tempstring
    .wintbm2.fg2.ft2.f27.l1 configure -foreground "black"
    
    set bit7 [string index $byteread 0]
    set bit6 [string index $byteread 1]
    set bit5 [string index $byteread 2]
    set bit4 [string index $byteread 3]
    set bit3 [string index $byteread 4]
    set bit2 [string index $byteread 5]
    set bit1 [string index $byteread 6]
    set bit0 [string index $byteread 7]

}
proc tbm2_read_f3 { commandd } {
    global D7position
    global D0position
    global D7positiontbm2
    global D3positiontbm2
    global datalength
    global verbose2
    global usbreaddata
    global slowfecchannel
    global verbose3

# write all 1s to data while reading    
    set bytewrite "11111111"
    transfer_byte $bytewrite "0x3" "read"    

    clearnibblearray "0000"
#reset the RD buffer before reading
    if {$slowfecchannel == "1"} {
	pcifeccmdreg 2 1
    } elseif {$slowfecchannel == "2"} {
	pcifeccmdreg 18 1
    }
    if {$slowfecchannel == "1"} {
	set hextemp [pcifecreadword 4]
    } else {
	set hextemp [pcifecreadword 20]
    }
    set hextemp [padhexword $hextemp]
#    set retdata [string range $hextemp 2 3]
    set retdata [string range $hextemp 4 5]
    
    set temphigh [hexhigh $retdata]
    set templow [hexlow $retdata]
    set byteread "$temphigh$templow"

    if {$verbose3 > 0} {
	puts "PCI TBM2_READ_F3 RETURN BYTE=$byteread  RETURNWORD=$hextemp"
    }


    .wintbm2.fg2.ft5.f57.c57 deselect
    .wintbm2.fg2.ft5.f56.c56 deselect
    .wintbm2.fg2.ft5.f55.c55 deselect
    .wintbm2.fg2.ft5.f54.c54 deselect
    .wintbm2.fg2.ft5.f53.c53 deselect
    .wintbm2.fg2.ft5.f52.c52 deselect
    .wintbm2.fg2.ft5.f51.c51 deselect
    .wintbm2.fg2.ft5.f50.c50 deselect
    set bit7 [string index $byteread 0]
    set bit6 [string index $byteread 1]
    set bit5 [string index $byteread 2]
    set bit4 [string index $byteread 3]
    set bit3 [string index $byteread 4]
    set bit2 [string index $byteread 5]
    set bit1 [string index $byteread 6]
    set bit0 [string index $byteread 7]

    if {$bit7 == 1} then { .wintbm2.fg2.ft5.f57.c57 select }
    if {$bit6 == 1} then { .wintbm2.fg2.ft5.f56.c56 select }
    if {$bit5 == 1} then { .wintbm2.fg2.ft5.f55.c55 select }
    if {$bit4 == 1} then { .wintbm2.fg2.ft5.f54.c54 select }
    if {$bit3 == 1} then { .wintbm2.fg2.ft5.f53.c53 select }
    if {$bit2 == 1} then { .wintbm2.fg2.ft5.f52.c52 select }
    if {$bit1 == 1} then { .wintbm2.fg2.ft5.f51.c51 select }
    if {$bit0 == 1} then { .wintbm2.fg2.ft5.f50.c50 select }
}
proc tbm2_read_f4 { command } {
    global D7position
    global D0position
    global D7positiontbm2
    global D3positiontbm2
    global datalength
    global verbose2
    global usbreaddata
    global slowfecchannel
    global verbose3

# write all 1s to data while reading    
    set bytewrite "11111111"
    transfer_byte $bytewrite "0x4" "read"    
    clearnibblearray "0000"
#reset the RD buffer before reading
    if {$slowfecchannel == "1"} {
	pcifeccmdreg 2 1
    } elseif {$slowfecchannel == "2"} {
	pcifeccmdreg 18 1
    }
    if {$slowfecchannel == "1"} {
	set hextemp [pcifecreadword 4]
    } else {
	set hextemp [pcifecreadword 20]
    }
    set hextemp [padhexword $hextemp]
#    set retdata [string range $hextemp 2 3]
    set retdata [string range $hextemp 4 5]
    
    set temphigh [hexhigh $retdata]
    set templow [hexlow $retdata]
    set byteread "$temphigh$templow"

    if {$verbose3 > 0} {
	puts "PCI TBM0_READ_F4 RETURN BYTE=$byteread  RETURNWORD=$hextemp"
    }


    .wintbm2.fg3.f7.ff1.f07.rc07x deselect
    .wintbm2.fg3.f7.ff1.f06.rc06x deselect
    .wintbm2.fg3.f7.ff1.f05.rc05x deselect

    set bit7 [string index $byteread 0]
    set bit6 [string index $byteread 1]
    set bit5 [string index $byteread 2]
    set bit4 [string index $byteread 3]
    set bit3 [string index $byteread 4]
    set bit2 [string index $byteread 5]
    set bit1 [string index $byteread 6]
    set bit0 [string index $byteread 7]

    if {$bit2 == 1} then { .wintbm2.fg3.f7.ff1.f07.rc07x select }
    if {$bit1 == 1} then { .wintbm2.fg3.f7.ff1.f06.rc06x select }
    if {$bit0 == 1} then { .wintbm2.fg3.f7.ff1.f05.rc05x select }

}
proc tbm2_write_f0 { command } {
    global savedlastf0
    global DisableTBMCLK
    global Readout40MHz
    global DisableAnalogOutput
    global DisableTriggerOutput
    global StackReadbackMode
    global IgnoreIncomingTriggers
    global PauseReadout
    global EnableAutoReset
    global ResetOnStackCount
    global DisableTokenPassing
# DisableAnalogOutput   DisableTriggerOutput  StackReadbackMode  IgnoreIncomingTriggers
# PauseReadout  ResetOnStackCount  DisableTBMCLK  Readout40MHz 
#    set bytewrite [format "%s%s%s%s%s%s%s%s" $DisableAnalogOutput 
    set bytewrite [format "%s%s%s%s%s%s%s%s" 0 \
	    $DisableTriggerOutput \
	    $StackReadbackMode \
	    $IgnoreIncomingTriggers \
	    $PauseReadout \
	    $ResetOnStackCount \
	    $DisableTBMCLK \
	    $Readout40MHz ]
    transfer_byte $bytewrite "0x0" "write"    

# save the last SENT command to 0x0 register
    set savedlastf0 $bytewrite

}
proc tbm2_write_f1 { command } {

    global ClearTriggerCounter
    global ClearTokenOut
    global ClearStack
    global InjectTBMReset
    global InjectPreCalTrigger
    global InjectROCReset
    global InjectSyncTrigger
    global InjectNormalTrigger

# ClearTriggerCounter ClearTokenOut ClearStack InjectTBMReset InjectPreCalTrigger  
# InjectROCReset InjectSyncTrigger InjectNormalTrigger
    set bytewrite [format "%s%s%s%s%s%s%s%s" $ClearTriggerCounter \
	    $ClearTokenOut \
	    $ClearStack \
	    $InjectTBMReset \
	    $InjectPreCalTrigger \
	    $InjectROCReset \
	    $InjectSyncTrigger \
	    $InjectNormalTrigger ]


#old     transfer_byte $bytewrite "0x2" "write"    
	if {$verbose3=="1"} {
	    puts "SENDING ResetROC TO FEC"
	}
	set temp2 "0111000"
##npci	sendbytewparity $temp2
	set tt1 [wordtobits 00000001]
	set tt2 [bitwordset $tt1 7 $encalpulse]
	set tt2 [bitwordset $tt2 5 $fec_trigger_source]
	set tt2 [bitwordset $tt2 6 $norm_loop_on]
	set tt3 [bitstoword $tt2]
	sendpcifectriggerregister 0


}
proc tbm2_write_f2 { command } {
    global DisableCalDelayedTrig
    global NotUsedLine64
    global NotUsedLine63
    global NotUsedLine62
    global NotUsedLine61
    global NotUsedLine60
    global tbm2mode

    if {$tbm2mode=="Sync"} {
	set modebit1 "0"
	set modebit0 "0"
    } elseif {$tbm2mode=="ClrTrigCount"} {
	set modebit1 "1"
	set modebit0 "0"
    } elseif {$tbm2mode=="Calibration"} {
	set modebit1 "1"
	set modebit0 "1"
    } else {
	puts "Unknown MODE Set - report error."
	bell
	bell
	bell
    }
    set bytewrite [format "%s%s%s%s%s%s%s%s" $modebit1 \
		       $modebit0 \
		       $DisableCalDelayedTrig \
		       $NotUsedLine64 \
		       $NotUsedLine63 \
		       $NotUsedLine62 \
		       $NotUsedLine61 \
		       $NotUsedLine60 ]
##old:    transfer_byte $bytewrite "0x3" "write"    

    transfer_byte $bytewrite "0x1" "write"    
}

proc tbm2_write_f4 { command } {

    global ForceReadoutClockOn
    global DisableTokenOutDrv
    global DisableAnalogOutputDrv

    set bytewrite [format "00000%s%s%s"  \
	    $ForceReadoutClockOn \
	    $DisableTokenOutDrv \
	    $DisableAnalogOutputDrv ]

    transfer_byte $bytewrite "0x4" "write"    

}
proc stringtoint { mydatabyte } {
    set myvalue 0

    if { [string index $mydatabyte 7] == "1" } {
	set myvalue [expr $myvalue|1]
    }
    if { [string index $mydatabyte 6] == "1" } {
	set myvalue [expr $myvalue|2]
    }
    if { [string index $mydatabyte 5] == "1" } {
	set myvalue [expr $myvalue|4]
    }
    if { [string index $mydatabyte 4] == "1" } {
	set myvalue [expr $myvalue|8]
    }
    if { [string index $mydatabyte 3] == "1" } {
	set myvalue [expr $myvalue|16]
    }
    if { [string index $mydatabyte 2] == "1" } {
	set myvalue [expr $myvalue|32]
    }
    if { [string index $mydatabyte 1] == "1" } {
	set myvalue [expr $myvalue|64]
    }
    if { [string index $mydatabyte 0] == "1" } {
	set myvalue [expr $myvalue|128]
    }

    return $myvalue
}

proc toggle_trigger_source { command } {
    global fec_trigger_source
    global uwin

    if {$fec_trigger_source == 1} {
	set fec_trigger_source 0
	.fec.command.row2.f2.ftop.b1 configure -text "TRIG Enabled"
    } else {
	set fec_trigger_source 1
	.fec.command.row2.f2.ftop.b1 configure -text "TRIG Disabled"
    }
}
set norm_loop_on 0
proc toggle_norm_loop_on { command } {
    global norm_loop_on
    global uwin
    if {$norm_loop_on == 1} {
	set norm_loop_on 0
	 .fec.command.row2.f1.fbot.b3 configure -text "NRMLOOP OFF"
    } else {
	set norm_loop_on 1
	.fec.command.row2.f1.fbot.b3 configure -text "NRMLOOP ON"
    }
}
set encalpulse 0
proc toggle_encalpulse { command } {
    global encalpulse
        global uwin

    if {$encalpulse == 1} {
	set encalpulse 0
	.fec.command.row2.f1.ftop.b3 configure -text "CAL LOOP OFF"
    } else {
	set encalpulse 1
	.fec.command.row2.f1.ftop.b3 configure -text "CAL LOOP ON"
    }
}

proc read_test { command } {
    global D7position
    global D0position
    global testshown

    if { $testshown == 1 } {
	destroy .readtest
    }

    toplevel .readtest -class Dialog 
    wm title .readtest "Looping Read Test"


    set i 0
    set readouttext "-------------  Loop test 01 ----------------\n"

    append readouttext "\n      L o o p i n g  .  .  .    \n"
    append readouttext "\n      Completed.  Push LOOP TEST for another 1000 iterations    \n"

    label .readtest.label -text $readouttext

    set iloop 0
    button .readtest.button0 -text "Dismiss" -command {
	set iloop 9999
	destroy .readtest
	set testshown 0
    }

    pack .readtest.label
    pack .readtest.button0

    set testshown 1

    set width [winfo screenwidth .]
    set height [winfo screenheight .]
    set x [expr ($width/2) - 200]
    set y [expr ($height/2) - 200]

    wm geometry .readtest  +$x+$y


    while { $iloop < 1000 } {
	incr iloop
# 4) Read Stack Status
	transfer_byte "11111111" "0x3" "read"
	set byteread2 [getreturnedbits 1]
	set byteread2 [string range $byteread2 $D7position $D0position]
	set valuebyteread2 [stringtoint $byteread2]
	if { $valuebyteread2 == 255 } {
	    puts "Error in read data.  Communication or noise?"
	    puts "Iterations at data=255 : "
	    puts $iloop
	    set iloop 9999
	}
    }
	

##    set result [tk_messageBox -parent . \
##	    -title { exec result } -type ok \
##	    -icon info \
##	    -message $readouttext ]
}

proc seuverbosestackcounton {} {
    global specialtest1
    set specialtest1 1
}

proc background_read_stack { command } {
    global D7position
    global D0position
    global stackshown
    global datalength
    global DisableTriggerOutput
    global ResetOnStackCount
    global DisableTBMCLK
    global Readout40MHz
    global D7positiontbm2
    global D3positiontbm2
    global verbose2
    global usbreaddata
    global outformat
    global fdataout
    global baseaddress
    global specialtest1

    set lastbaseaddress $baseaddress

# Set this to TBM 1 (1/2)    
    set baseaddress 0xE0

    set i 0
    set readouttext "STACK 1 "

# new 0) read lastsavedf0
    transfer_byte "11111111" "0x0" "read"
    getusbreaddata 1
    set savedf0 $usbreaddata

#    1) Set Pause Readout, Ignore Incoming Tracks, Stack Readback Mode:
    set bytewrite "0$DisableTriggerOutput"
    append bytewrite "111"
    append bytewrite "$ResetOnStackCount$DisableTBMCLK$Readout40MHz"

    transfer_byte $bytewrite "0x0" "write"    
# 2) Read Number of Items on Stack:
    transfer_byte "11111111" "0x1" "read"
##    set byteread [getreturnedbits 1]

    getusbreaddata 1
    set byteread $usbreaddata

    set numitems 0
    if { [string index $byteread 7] == "1" } {
	set numitems [expr $numitems|1]
    }
    if { [string index $byteread 6] == "1" } {
	set numitems [expr $numitems|2]
    }
    if { [string index $byteread 5] == "1" } {
	set numitems [expr $numitems|4]
    }
    if { [string index $byteread 4] == "1" } {
	set numitems [expr $numitems|8]
    }
    if { [string index $byteread 3] == "1" } {
	set numitems [expr $numitems|16]
    }
    if { [string index $byteread 2] == "1" } {
	set numitems [expr $numitems|32]
    }

    append readouttext "$numitems "

    set outstring "DESCSTK 1\n"

    while { $i < 32 } {
	incr i

# 3) Read Stack Status Bits (Stack D8-D13)
	transfer_byte "11111111" "0x3" "read"
	getusbreaddata 1
	set byteread1 $usbreaddata

	set itext " "

	set numflags 0

	set idata [bitstoint $byteread1]

	if {[string index $byteread1 0] == "1"} {
	    append itext "NTP "
	    incr numflags
	} else {
	    append itext " "
	}

	if {[string index $byteread1 1] == "1"} {
	    append itext "TBMRST "
	    incr numflags
	} else {
	    append itext " "
	}
 
	if {[string index $byteread1 2] == "1"} {
	    append itext "ROCRST "
	    incr numflags
	} else {
	    append itext " "
	}

	if { [string index $byteread1 3] == "1" } {
	    append itext "ST "
	    incr numflags
	} else {
	    append itext " "
	}
	if { [string index $byteread1 4] == "1" } {
	    append itext "SE "
	    incr numflags
	} else {
	    append itext " "
	}
	if { [string index $byteread1 5] == "1" } {
	    append itext "RST "
	    incr numflags
	} else {
	    append itext " "
	}
	if { [string index $byteread1 6] == "1" } {
	    append itext "PCT "
	    incr numflags
	} else {
	    append itext " "
	}
	if { [string index $byteread1 7] == "1" } {
	    append itext "SF "
	    incr numflags
	} else {
	    append itext " "
	}


#	if { $numflags > 5 } {
#	    append readouttext "\n*****ERROR STOP*****\n"
#	    set i 999
#	}

	set valuebyteread2 -1
	if { $i < 128 } {
# 4) Read Stack Event Number
	    transfer_byte "11111111" "0x2" "read"
	    getusbreaddata 1
	    set byteread2 $usbreaddata
	    set valuebyteread2 [stringtoint $byteread2]
	}
	if {$i < 31} {
	    append readouttext "$i $valuebyteread2 $idata "
	} else {
	    append readouttext "$i $valuebyteread2 $idata"
	}

#------------specialtest1

	if {$specialtest1 == 1} {
	    # Read Number of Items on Stack:
	    transfer_byte "11111111" "0x1" "read"
	    ##    set byteread [getreturnedbits 1]
	    
	    getusbreaddata 1
	    set byteread $usbreaddata
	    
	    set mynumitems 0
	    if { [string index $byteread 7] == "1" } {
		set mynumitems [expr $mynumitems|1]
	    }
	    if { [string index $byteread 6] == "1" } {
		set mynumitems [expr $mynumitems|2]
	    }
	    if { [string index $byteread 5] == "1" } {
		set mynumitems [expr $mynumitems|4]
	    }
	    if { [string index $byteread 4] == "1" } {
		set mynumitems [expr $mynumitems|8]
	    }
	    if { [string index $byteread 3] == "1" } {
		set mynumitems [expr $mynumitems|16]
	    }
	    if { [string index $byteread 2] == "1" } {
		set mynumitems [expr $mynumitems|32]
	    }
	    
	} 


#------------specialtest1
	if {$specialtest1 == 1} {
	    set outtext " $i $valuebyteread2 $itext   $mynumitems\n"
	} else {
	    set outtext " $i $valuebyteread2 $itext   $mynumitems\n"
	}
##	set outtext " $i $valuebyteread2 $itext\n"
	append outstring $outtext
#	if { $numitems > 0 } {
#	    if { $i == $numitems } {
#		append readouttext "\n ----------------------------"
#	    }
#	}
	


    } 
# 5) end of while

    if {$outformat == "seu"} {
	puts $fdataout $readouttext
	puts $fdataout $outstring
    }

# 6) Reset Pause Readout, Ingore Incoming Triggers, Stack Readback Mode
#    to the settings of the pushbuttons (same as pushing WRITE on top line)
    if { $i < 128 } {
	transfer_byte $savedf0 "0x0" "write"
    }

# ------------------------------- now 2nd TBM ------------------------



# Set this to TBM 2 (2/2)    
    set baseaddress 0xF0

    set i 0
    set readouttext "STACK 2 "

# new 0) read lastsavedf0
    transfer_byte "11111111" "0x0" "read"
    getusbreaddata 1
    set savedf0 $usbreaddata

#    1) Set Pause Readout, Ignore Incoming Tracks, Stack Readback Mode:
#    set bytewrite "$DisableAnalogOutput$DisableTriggerOutput"
    set bytewrite "0$DisableTriggerOutput"
    append bytewrite "111"
    append bytewrite "$ResetOnStackCount$DisableTBMCLK$Readout40MHz"

    transfer_byte $bytewrite "0x0" "write"    
# 2) Read Number of Items on Stack:
    transfer_byte "11111111" "0x1" "read"
##    set byteread [getreturnedbits 1]

    getusbreaddata 1
    set byteread $usbreaddata

    set numitems 0
    if { [string index $byteread 7] == "1" } {
	set numitems [expr $numitems|1]
    }
    if { [string index $byteread 6] == "1" } {
	set numitems [expr $numitems|2]
    }
    if { [string index $byteread 5] == "1" } {
	set numitems [expr $numitems|4]
    }
    if { [string index $byteread 4] == "1" } {
	set numitems [expr $numitems|8]
    }
    if { [string index $byteread 3] == "1" } {
	set numitems [expr $numitems|16]
    }
    if { [string index $byteread 2] == "1" } {
	set numitems [expr $numitems|32]
    }

    append readouttext "$numitems "

    set outstring "DESCSTK 2\n"

    while { $i < 32 } {
	incr i

# 3) Read Stack Status Bits (Stack D8-D13)
	transfer_byte "11111111" "0x3" "read"
	getusbreaddata 1
	set byteread1 $usbreaddata

	set itext " "

	set numflags 0

	set idata [bitstoint $byteread1]

	if {[string index $byteread1 0] == "1"} {
	    append itext "NTP "
	    incr numflags
	} else {
	    append itext " "
	}

	if {[string index $byteread1 1] == "1"} {
	    append itext "TBMRST "
	    incr numflags
	} else {
	    append itext " "
	}
 
	if {[string index $byteread1 2] == "1"} {
	    append itext "ROCRST "
	    incr numflags
	} else {
	    append itext " "
	}

	if { [string index $byteread1 3] == "1" } {
	    append itext "ST "
	    incr numflags
	} else {
	    append itext " "
	}
	if { [string index $byteread1 4] == "1" } {
	    append itext "SE "
	    incr numflags
	} else {
	    append itext " "
	}
	if { [string index $byteread1 5] == "1" } {
	    append itext "RST "
	    incr numflags
	} else {
	    append itext " "
	}
	if { [string index $byteread1 6] == "1" } {
	    append itext "PCT "
	    incr numflags
	} else {
	    append itext " "
	}
	if { [string index $byteread1 7] == "1" } {
	    append itext "SF "
	    incr numflags
	} else {
	    append itext " "
	}


#	if { $numflags > 5 } {
#	    append readouttext "\n*****ERROR STOP*****\n"
#	    set i 999
#	}

	set valuebyteread2 -1
	if { $i < 128 } {
# 4) Read Stack Event Number
	    transfer_byte "11111111" "0x2" "read"
	    getusbreaddata 1
	    set byteread2 $usbreaddata
	    set valuebyteread2 [stringtoint $byteread2]
	}
	if {$i < 31} {
	    append readouttext "$i $valuebyteread2 $idata "
	} else {
	    append readouttext "$i $valuebyteread2 $idata"
	}

#-------------specialtest1

	if {$specialtest1 == 1} {
	    # Read Number of Items on Stack:
	    transfer_byte "11111111" "0x1" "read"
	    ##    set byteread [getreturnedbits 1]
	    
	    getusbreaddata 1
	    set byteread $usbreaddata
	    
	    set mynumitems 0
	    if { [string index $byteread 7] == "1" } {
		set mynumitems [expr $mynumitems|1]
	    }
	    if { [string index $byteread 6] == "1" } {
		set mynumitems [expr $mynumitems|2]
	    }
	    if { [string index $byteread 5] == "1" } {
		set mynumitems [expr $mynumitems|4]
	    }
	    if { [string index $byteread 4] == "1" } {
		set mynumitems [expr $mynumitems|8]
	    }
	    if { [string index $byteread 3] == "1" } {
		set mynumitems [expr $mynumitems|16]
	    }
	    if { [string index $byteread 2] == "1" } {
		set mynumitems [expr $mynumitems|32]
	    }
	    
	} 


#-------------specialtest1
	if {$specialtest1 == 1} {
	    set outtext " $i $valuebyteread2 $itext   $mynumitems\n"
	} else {
	    set outtext " $i $valuebyteread2 $itext   $mynumitems\n"
	}

####	set outtext " $i $valuebyteread2 $itext\n"
	append outstring $outtext
#	if { $numitems > 0 } {
#	    if { $i == $numitems } {
#		append readouttext "\n ----------------------------"
#	    }
#	}
	


    } 
# 5) end of while

    if {$outformat == "seu"} {
	puts $fdataout $readouttext
	puts $fdataout $outstring
    }

# 6) Reset Pause Readout, Ingore Incoming Triggers, Stack Readback Mode
#    to the settings of the pushbuttons (same as pushing WRITE on top line)
    if { $i < 128 } {
	transfer_byte $savedf0 "0x0" "write"
    }


# Set the TBM number back to what 
    set baseaddress $lastbaseaddress

}
proc background_dumptbm {} {
    global baseaddress
    global outformat
    global fdataout
    global usbreaddata

    set lastbaseaddress $baseaddress

# Set this to TBM 1 (1/2)    
    set baseaddress 0xE0
    set readouttext "TBM 1 "
    transfer_byte "11111111" "0x0" "read"
    getusbreaddata 1
    set byteread2 $usbreaddata
    set itemp [bitstoint $byteread2]
    append readouttext " 0 $itemp"
    transfer_byte "11111111" "0x1" "read"
    getusbreaddata 1
    set byteread2 $usbreaddata
    set itemp [bitstoint $byteread2]
    append readouttext " 1 $itemp"
    transfer_byte "11111111" "0x4" "read"
    getusbreaddata 1
    set byteread2 $usbreaddata
    set itemp [bitstoint $byteread2]
    append readouttext " 4 $itemp"
    transfer_byte "11111111" "0x5" "read"
    getusbreaddata 1
    set byteread2 $usbreaddata
    set itemp [bitstoint $byteread2]
    append readouttext " 5 $itemp"
    transfer_byte "11111111" "0x6" "read"
    getusbreaddata 1
    set byteread2 $usbreaddata
    set itemp [bitstoint $byteread2]
    append readouttext " 6 $itemp"
    transfer_byte "11111111" "0x7" "read"
    getusbreaddata 1
    set byteread2 $usbreaddata
    set itemp [bitstoint $byteread2]
    append readouttext " 7 $itemp\n"

    if {$outformat == "seu"} {
	puts $fdataout $readouttext
    }


# Set this to TBM 2 (2/2)    
    set baseaddress 0xF0
    set readouttext "TBM 2 "
    transfer_byte "11111111" "0x0" "read"
    getusbreaddata 1
    set byteread2 $usbreaddata
    set itemp [bitstoint $byteread2]
    append readouttext " 0 $itemp"
    transfer_byte "11111111" "0x1" "read"
    getusbreaddata 1
    set byteread2 $usbreaddata
    set itemp [bitstoint $byteread2]
    append readouttext " 1 $itemp"
    transfer_byte "11111111" "0x4" "read"
    getusbreaddata 1
    set byteread2 $usbreaddata
    set itemp [bitstoint $byteread2]
    append readouttext " 4 $itemp"
    transfer_byte "11111111" "0x5" "read"
    getusbreaddata 1
    set byteread2 $usbreaddata
    set itemp [bitstoint $byteread2]
    append readouttext " 5 $itemp"
    transfer_byte "11111111" "0x6" "read"
    getusbreaddata 1
    set byteread2 $usbreaddata
    set itemp [bitstoint $byteread2]
    append readouttext " 6 $itemp"
    transfer_byte "11111111" "0x7" "read"
    getusbreaddata 1
    set byteread2 $usbreaddata
    set itemp [bitstoint $byteread2]
    append readouttext " 7 $itemp\n"

    if {$outformat == "seu"} {
	puts $fdataout $readouttext
    }

# Set the TBM number back to what 
    set baseaddress $lastbaseaddress

}


proc getpcireaddata { mycommand } {
    global usbreaddata
    global slowfecchannel
    clearnibblearray "0000"
    
    if {$slowfecchannel == "1"} {
	pcifeccmdreg 2 1
    } elseif {$slowfecchannel == "2"} {
	pcifeccmdreg 18 1
    }
    if {$slowfecchannel == "1"} {
	set hextemp [pcifecreadword 4]
    } else {
	set hextemp [pcifecreadword 20]
    }
    set hextemp [padhexword $hextemp]
#    set retdata [string range $hextemp 2 3]
    set retdata [string range $hextemp 4 5]
    
    set temphigh [hexhigh $retdata]
    set templow [hexlow $retdata]
    set byteread "$temphigh$templow"
    set usbreaddata $byteread
}


proc read_stack { command } {
    global D7position
    global D0position
    global stackshown
    global datalength
    global DisableTriggerOutput
    global ResetOnStackCount
    global DisableTBMCLK
    global Readout40MHz
    global D7positiontbm2
    global D3positiontbm2
    global verbose2
    global usbreaddata

    if { $stackshown == 1 } {
	destroy .stackdump
    }

    toplevel .stackdump -class Dialog 
    wm title .stackdump "Stack Readout"


    set i 0
    set readouttext "------------------Stack Contents:------------------\n"

# new 0) read lastsavedf0
    transfer_byte "11111111" "0x0" "read"
#    getusbreaddata 1
    getpcireaddata 1
    set savedf0 $usbreaddata

# 1) Set Pause Readout, Ignore Incoming Tracks, Stack Readback Mode:
#    set bytewrite "$DisableAnalogOutput$DisableTriggerOutput"
    set bytewrite "0$DisableTriggerOutput"
    append bytewrite "111"
    append bytewrite "$ResetOnStackCount$DisableTBMCLK$Readout40MHz"

    transfer_byte $bytewrite "0x0" "write"    
# 2) Read Number of Items on Stack:
    transfer_byte "11111111" "0x1" "read"
##    set byteread [getreturnedbits 1]

    getpcireaddata 1
    set byteread $usbreaddata

    set numitems 0
    if { [string index $byteread 7] == "1" } {
	set numitems [expr $numitems|1]
    }
    if { [string index $byteread 6] == "1" } {
	set numitems [expr $numitems|2]
    }
    if { [string index $byteread 5] == "1" } {
	set numitems [expr $numitems|4]
    }
    if { [string index $byteread 4] == "1" } {
	set numitems [expr $numitems|8]
    }
    if { [string index $byteread 3] == "1" } {
	set numitems [expr $numitems|16]
    }
    if { [string index $byteread 2] == "1" } {
	set numitems [expr $numitems|32]
    }

    append readouttext "\nStack Count: $numitems\n"

    while { $i < 32 } {
	incr i

# 3) Read Stack Status Bits (Stack D8-D13)
	transfer_byte "11111111" "0x3" "read"
	getpcireaddata 1
	set byteread1 $usbreaddata

	set itext ": "

	set numflags 0

	if {[string index $byteread1 0] == "1"} {
	    append itext "NoTokPass "
	    incr numflags
	} else {
	    append itext "          "
	}

	if {[string index $byteread1 1] == "1"} {
	    append itext "TBM-Reset "
	    incr numflags
	} else {
	    append itext "          "
	}
 
	if {[string index $byteread1 2] == "1"} {
	    append itext "ROC-Reset "
	    incr numflags
	} else {
	    append itext "          "
	}

	if { [string index $byteread1 3] == "1" } {
	    append itext "SyncTrig "
	    incr numflags
	} else {
	    append itext "          "
	}
	if { [string index $byteread1 4] == "1" } {
	    append itext "SyncErr "
	    incr numflags
	} else {
	    append itext "        "
	}
	if { [string index $byteread1 5] == "1" } {
	    append itext "Rst "
	    incr numflags
	} else {
	    append itext "    "
	}
	if { [string index $byteread1 6] == "1" } {
	    append itext "PreCalTrig "
	    incr numflags
	} else {
	    append itext "        "
	}
	if { [string index $byteread1 7] == "1" } {
	    append itext "StkFull "
	    incr numflags
	} else {
	    append itext "           "
	}


	if { $numflags > 5 } {
	    append readouttext "\n*****ERROR STOP*****\n"
	    set i 999
	}

	set valuebyteread2 -1
	if { $i < 128 } {
# 4) Read Stack Event Number
	    transfer_byte "11111111" "0x2" "read"
	    getpcireaddata 1
	    set byteread2 $usbreaddata
	    set valuebyteread2 [stringtoint $byteread2]
	}
	append readouttext "\n $i: $valuebyteread2 : $itext"
	if { $numitems > 0 } {
	    if { $i == $numitems } {
		append readouttext "\n ----------------------------"
	    }
	}
	


    } 
# 5) end of while

# 6) Reset Pause Readout, Ingore Incoming Triggers, Stack Readback Mode
#    to the settings of the pushbuttons (same as pushing WRITE on top line)
    if { $i < 128 } {
	transfer_byte $savedf0 "0x0" "write"
    }

    label .stackdump.label -text $readouttext


    button .stackdump.button0 -text "Dismiss" -command {
	destroy .stackdump
	set stackshown 0
    }

    pack .stackdump.label
    pack .stackdump.button0

    set stackshown 1

    set width [winfo screenwidth .]
    set height [winfo screenheight .]
    set x [expr ($width/2) - 200]
    set y [expr ($height/2) - 200]

    wm geometry .stackdump  +$x+$y

##    set result [tk_messageBox -parent . \
##	    -title { exec result } -type ok \
##	    -icon info \
##	    -message $readouttext ]
}

set menu_count 0
global menu_count


proc CreateMenu { menubar basename text mnemonic } {
    global tk_version menu_count

    if { $tk_version >= 8.0 } {
	set menu_name "$menubar.$basename"
	$menubar add cascade -label $text -menu $menu_name -underline $mnemonic
    } else {
	set menu_name "$menubar.menu$menu_count.$basename"
	menubutton $menubar.$menu_count -text $text -menu $menu_name -underline $mnemonic
	pack $menubar.$menu_count -side left
	set menu_count [expr $menu_count + 1]
    }

    menu $menu_name

    return $menu_name
}

proc CreateMenubar { toplevel menubar } {
    global tk_version

    if { $tk_version >= 8.0 } {
	menu $menubar -type menubar

	$toplevel configure -menu $menubar
    } else {
	frame $menubar -borderwidth 1 -relief raised
	pack $menubar -side top -fill x -expand true
    }
}
proc clrcalarray { command } {
    global pixarraycal
    global maxcols
    global maxrows
    set icol 0
    while {$icol < $maxcols} {
	set irow 0
	while {$irow < $maxrows } {
	    set aindex "$icol $irow"
	    set pixarraycal($aindex) 0
	    incr irow
	}
	incr icol
    }
}

proc ClrCal { command } {
    global pixarrayenabled
    global pixarraytrim
    global pixarraycal 
    global pixarray
    global calibrationon
    global maxcols
    global maxrows
    global dogui

###Takeout clrcal for tests#######    pix_transfer_byte "ClrCal" 0 0 0 0 0 0
    pix_transfer_byte "ClrCal" 0 0 0 0 0 0

# clear the array in memory for cal
#    clrcalarray 1

    
    set savedcalibrationon $calibrationon
    if {$command==1} {
	set calibrationon 0
	set icol 0
	while {$icol<$maxcols} {
	    set irow 0
	    while {$irow<$maxrows} {
		set aindex "$icol $irow"
		if {$pixarraycal($aindex)>0} {
		    set pixarraycal($aindex) 0
		    if {$dogui == 1} {
			if {$pixarrayenabled($aindex)==1} {
			    set pixelcolor [trimcolor $pixarraytrim($aindex)]
			} else {
			    set pixelcolor "\#808080"
			}
			.win0.fr1.can1 itemconfigure $pixarray($aindex) -fill $pixelcolor
		    }
		}
		incr irow
	    }
	    incr icol
	}
    }
    set calibrationon $savedcalibrationon

}


proc enumdacs { command } {
    global dacarray
    global ctrl_reg
    global wbc
    global hubaddress
    global pixportaddress
    global pixchipaddress
    global pixreadoutspeed
    global pixstopdaq
    global roctype

    parray dacarray
    set iindex [array startsearch dacarray]
    while { [array anymore dacarray $iindex] } {
	set inextdac [array nextelement dacarray $iindex]
	puts "dac: $inextdac should be set to integer value: $dacarray($inextdac)"
#### use above to guide sending dac values
####
	pix_transfer_byte "0x08" $inextdac $dacarray($inextdac) 0 0 0 0
    }
    array donesearch dacarray $iindex
    set temp3 0
    if { $pixreadoutspeed == "1" } {
	set temp3 [ expr ($temp3 | 0x01) ]
    }
    if { $pixstopdaq == "0" } {
	set temp3 [ expr ($temp3 | 0x02) ]
    }
    
#	pix_transfer_byte "0x08" 0xFE $wbc 0 0 0 0
	pix_transfer_byte "0x08" 254 $wbc 0 0 0 0

     puts "hubaddress: $hubaddress"
     puts "pixport: $pixportaddress"
     puts "pixchipaddress: $pixchipaddress"
#    puts $ctrl_reg
    puts "wbc: $wbc"
}
    

proc getdacfloat { mydac rmin rmax iresolution } {
    global dacarray
    # return real dac value
    set tempfloat $dacarray($mydac)
#    set tempfloat [ expr ($rmin + \
\#	    ($dacarray($mydac)*($rmax-$rmin)/(1.0*($iresolution-1)))) ]
    return $tempfloat
}

proc filegetdacdefaults { command } {
    global initialdir
    global dacarray
    global wbc
    global pixchipaddress
    global calibratelatency
    global currenttbm
    global pixportaddress

    set file_types {
	{ "DAC Settings Files (v1)" { .dacs1 }}
	{ "All Files" * }
    }
    set filename [tk_getOpenFile -initialdir $initialdir \
		      -filetypes $file_types -title "Read DAC Settings (v1)" \
		      -parent .]
    if { $filename != "" } {
	set initialdir [ file dirname $filename ]
	set fileidread [ open $filename "r" ]

	while { [gets $fileidread inputline] >= 0 } {
#	    puts $inputline
	    set result [scan $inputline "%d %d" var1 var2]
# Normal DACS are 0x04-0x81(129)	    
	    if {$var1 < 130} {
		set dacarray($var1) $var2 
	    } elseif {$var1 == 131} {
		set calibratelatency $var2
	    } elseif {$var1 == 130} {
		set wbc $var2
	    } elseif {$var1 == 132} {
		set currenttbm $var2
	    } elseif {$var1 == 133} {
		set pixportaddress $var2
	    } elseif {$var1 == 999} {
		set pixchipaddress $var2
	    }
	}
	close $fileidread

	enumdacs 1

# Inject ROC reset	
	if {$verbose3=="1"} {
	    puts "SENDING ResetROC TO FEC"
	}
	set temp2 "0111000"
##npci	sendbytewparity $temp2
	set tt1 [wordtobits 00000001]
	set tt2 [bitwordset $tt1 7 $encalpulse]
	set tt2 [bitwordset $tt2 5 $fec_trigger_source]
	set tt2 [bitwordset $tt2 6 $norm_loop_on]
	set tt3 [bitstoword $tt2]
	sendpcifectriggerregister 0

#old	puts "ROC reset."
#old	transfer_byte "00000100" "0x2" "write"    

    }
#    puts $filename
#    puts "get"
#    parray dacarray
    
}

proc fileputdacdefaults { command } {
    global initialdir
    global dacarray
    global wbc
    global pixchipaddress
    global calibratelatency
    global currenttbm
    global pixportaddress

    set file_types {
	{ "DAC Settings Files (v1)" { .dacs1 }}
	{ "All Files" * }
    }
    set filename [tk_getSaveFile -initialdir $initialdir \
		      -filetypes $file_types -title "Save DAC Settings (v1)" \
		      -parent .]
    if { $filename != "" } {

	set lasts [string last ".dacs1" $filename]
	if {$lasts < 0} {
	    append filename ".dacs1"
	}

	set initialdir [ file dirname $filename ]

	set fileidsave [ open $filename "w" ]
	set iindex [array startsearch dacarray]
	while { [array anymore dacarray $iindex] } {
	    set inextdac [array nextelement dacarray $iindex]
	    #	puts "dac: $inextdac should be set to integer value: $dacarray($inextdac)"
	    puts $fileidsave "$inextdac $dacarray($inextdac)"
	    #### use above to guide sending dac values
	    ####
	}
	array donesearch dacarray $iindex

# Write wbc	
	puts $fileidsave "130 $wbc"

# Write calibratelatency
	puts $fileidsave "131 $calibratelatency"

# Write tbm
	puts $fileidsave "132 $currenttbm"
# Write port	
	puts $fileidsave "133 $pixportaddress"

# Write pix chip address
	puts $fileidsave "999 $pixchipaddress"

	close $fileidsave
    }
#    puts "put"
#    parray dacarray    
}
proc setdacdefaults { command } {
	setdacdefaults46 $command
}

proc setdacdefaults46 { command } {
    global dacarray
    global wbc
    global calibratelatency
    set dacarray(1) 5
    set dacarray(2) 152
#    set sarray(4) [ getdacfloat 4 1.6 2.6 256 ]
#    set $sarray(4) 2.0
    set dacarray(3) 255
    set dacarray(4) 15
    set dacarray(5) 0
    set dacarray(6) 0
    set dacarray(7) 35
    set dacarray(8) 0
    set dacarray(9) 35

    set dacarray(10) 119
    set dacarray(11) 29
    set dacarray(12) 90
    set dacarray(13) 38

    set dacarray(14) 6
    set dacarray(15) 75
    set dacarray(16) 110
    set dacarray(17) 75

    set dacarray(18) 114
    set dacarray(19) 102
    # next was 188 until 2.2.1
    set dacarray(20) 60
    set dacarray(21) 200
    set dacarray(22) 100
    set dacarray(23) 100
    set dacarray(24) 100
    set dacarray(25) 20
    # next was 100 until 2.2.1
    set dacarray(26) 120
    set dacarray(27) 0

    set wbc 134
    set calibratelatency 139
}

proc incrcallatency { mywidget } {
    global calibratelatency
##    pix_transfer_byte "0x08" 0x82 $wbc 0 0 0 0
    incr calibratelatency
    if { $calibratelatency > 255 } {
	set calibratelatency 0
    }
#    $mywidget delete 0 end
#    $mywidget -textvariable wbc
}
proc deccallatency { mywidget } {
    global calibratelatency
##    pix_transfer_byte "0x08" 0x82 $wbc 0 0 0 0
    set calibratelatency [ expr ($calibratelatency - 1) ]
    if { $calibratelatency < 0 } {
	set calibratelatency 255
    }
}

proc incrwbc { mywidget } {
    global wbc
##    pix_transfer_byte "0x08" 0x82 $wbc 0 0 0 0
    incr wbc
    if { $wbc > 255 } {
	set wbc 0
    }
#    $mywidget delete 0 end
#    $mywidget -textvariable wbc
}
proc decwbc { mywidget } {
    global wbc
##    pix_transfer_byte "0x08" 0x82 $wbc 0 0 0 0
    set wbc [ expr ($wbc - 1) ]
    if { $wbc < 0 } {
	set wbc 255
    }
}

proc setwbc { mynewwbc } {
    global wbc
    global roctype

    set wbc $mynewwbc
    if {$roctype == "psi46"} {
#	pix_transfer_byte "0x08" 0xFE $wbc 0 0 0 0
	pix_transfer_byte "0x08" 254 $wbc 0 0 0 0
    } else {
	pix_transfer_byte "0x08" 0x82 $wbc 0 0 0 0
    }
	if {$verbose3=="1"} {
	    puts "SENDING ResetROC TO FEC"
	}
	set temp2 "0111000"
##npci	sendbytewparity $temp2
	set tt1 [wordtobits 00000001]
	set tt2 [bitwordset $tt1 7 $encalpulse]
	set tt2 [bitwordset $tt2 5 $fec_trigger_source]
	set tt2 [bitwordset $tt2 6 $norm_loop_on]
	set tt3 [bitstoword $tt2]
	sendpcifectriggerregister 0

#old    transfer_byte "00000100" "0x2" "write"    
}


proc scaleinfo { mydac myscalename mylabel myresolution myrmin myrmax \
		 mydacunits mydacfixedprecision } {
    global dacarray
    global dacscale
    global dacscalelabel
    global dacresolution
    global dacrmin
    global dacrmax
    global dacunits
    global dacfixedprecision

    set dacscale($mydac) $myscalename
    set dacscalelabel($mydac) $mylabel
    set dacresolution($mydac) $myresolution
    set dacrmin($mydac) $myrmin
    set dacrmax($mydac) $myrmax
    set dacunits($mydac) $mydacunits
    set dacfixedprecision($mydac) $mydacfixedprecision
#    puts "SI $mydac $myscalename $mylabel $myresolution $myrmin..$myrmax"
}

proc updatedacslide { mydac myresolution mymin mymax value2} {
    global dacarray
    global dacscale
    global dacscalelabel
    global verbose1
    global dacresolution
    global dacunits
    global dacfixedprecision
    global pixcalrange
    global roctype

##    set intvalue [ expr (0.5+($myresolution-1)*($value2-$mymin))/(1.0*$mymax-$mymin) ]
    set intvalue [ expr (($myresolution-1)*($value2-$mymin))/(1.0*$mymax-$mymin) ]
    if {$verbose1=="1"} {
	puts "intvalue:$intvalue  --> dacarray($mydac):$dacarray($mydac)"
    }
    set realvalue [ expr ($mymin + $value2*($mymax - $mymin)/(1.0*$myresolution-1)) ]
    if {($roctype == "psi46") && ($mydac=="25") && ($pixcalrange=="0")} {
	set realvalue [expr $realvalue*(280.0/1800)]
    }
    set tt $dacfixedprecision($mydac)
    case $tt {
	0 {  set realvalue [ format "%6.0f" $realvalue ] }
	1 {  set realvalue [ format "%6.1f" $realvalue ] }
	2 {  set realvalue [ format "%6.2f" $realvalue ] }
	3 {  set realvalue [ format "%6.3f" $realvalue ] }
	4 {  set realvalue [ format "%6.4f" $realvalue ] }
    }
######    puts "realvalue:$realvalue"
##    set dacarray($mydac) [ expr int($intvalue)]
##    set dacarray($mydac) [ expr int($intvalue + 0.5)]
    set dacarray($mydac) $value2

    set itemp [ expr int($intvalue)]

    if { $dacresolution($mydac) == "16" } {
	set resolutionstring "4bit"
    } elseif { $dacresolution($mydac) == "256" } {
	set resolutionstring "8bit"
    } else {
	set resolutionstring "NS"
    }
    set xxtemp "$dacscalelabel($mydac)   ($realvalue $dacunits($mydac))         $resolutionstring"
    $dacscale($mydac) configure -label $xxtemp

######    puts $itemp
    if { $mydac == 129 } {
	if { ($itemp >= 39) && ($itemp <= 99) } {
	    .windac.col3.f5.d2.s1 configure -activebackground "Red"
	} else {
	    .windac.col3.f5.d2.s1 configure -activebackground "Yellow"
	}
    }

#    puts $value2
#    puts $mydac
}
proc resetiterate { command } {
    if {$command == 1} {
	resetiterate46 1
    } elseif {$command == 2} {
	resetiterate46indiv 1
    } elseif {$command == 3} {
	resetiterate46maxcompressed 1
    } elseif {$command == 4} {
	resetiterate46maxcompressed 2
    }
}

proc resetiterate46maxcompressed { command } {
    global pixarray
    global resettrim
    global maskbitstate
    global calibrationon
    global resetcalibration
    global resetstatus
    global pixarraycal
    global pixarrayenabled
    global pixarraytrim
    global dcolstate
    global maxcols
    global maxrows
    global roctype
    global resetcolumn
    global resetentirechip
    global resetcompression

    gauge_create .winreset.g SkyBlue
    pack .winreset.g -expand yes -fill both -padx 10 -pady 10

    set tempmask $maskbitstate
    set tempcal $calibrationon

    if {$resetcalibration=="CAP"} {
	set calibrationon 1
    } elseif {$resetcalibration=="BUMP"} {
	set calibrationon 2
    } else {
	set calibrationon 0
    }

    if {$resetstatus=="ENABLED"} {
	set resetmaskbitstate 1
    } else {
	set resetmaskbitstate 0
    }

    if {$calibrationon == 0} {
	ClrCal 1
    }
    
# Now set the dcol periphery to off
    set idcol 0
    set numdcols [expr $maxcols / 2]
    while {$idcol < $numdcols} {
	# Kill dcol
	# send 32=0x20 and 0x00 -> set kill bit for dcol
	set dcolnotation [expr $idcol*2]
	.win0.fr1.can1 itemconfigure "$dcolnotation-back" -fill darkgray
	pix_transfer_byte "0x25" $dcolnotation 64 0 0 0 0

	.win0.fr1.b$idcol configure -bg darkgray \
	    -activeforeground darkgray \
	    -activebackground darkgray \
	    -foreground black 
	set dcolstate(.win0.fr1.b$idcol) 0

	incr idcol
#	update
    }
# Done setting the dcol periphery to off

    set initialresettrim $resettrim

# issue command, then update GUI
    # 97: update using datax1 compression
    # 98: use full column of data but here just setting to same value
    if {$command == 1} {
	if {$resetentirechip == "EntireChip"} {
	    pix_transfer_byte "0x97" -1 $maxrows $resetmaskbitstate $calibrationon $initialresettrim 0
	    puts "resetentirechip max compression  ((((((((((((((((((((((("
	} elseif {$resetentirechip == "1st8ColumnCOMP"} {
	    pix_transfer_byte "0x97" -2 $maxrows $resetmaskbitstate $calibrationon $initialresettrim 0
	    puts "reset single column max compression ((((((((((((((((((((((("
	} else {
	    pix_transfer_byte "0x97" $resetcolumn $maxrows $resetmaskbitstate $calibrationon $initialresettrim 0
	    puts "reset single column max compression ((((((((((((((((((((((("
	}	    
    } elseif {$command == 2} {
	if {$resetentirechip == "1st3ColumnsNOCOMP"} {
	    pix_transfer_byte "0x98" -2  $maxrows $resetmaskbitstate $calibrationon $initialresettrim 0
	} elseif {$resetentirechip == "EntireChip"} {
	    for {set thiscol 0} {$thiscol < $maxcols} {incr thiscol} {
		pix_transfer_byte "0x98" $thiscol $maxrows $resetmaskbitstate $calibrationon $initialresettrim 0
		puts "no compression reset all column ((((((((((((((((((((((("
	    }
	} else {
	    pix_transfer_byte "0x98" $resetcolumn $maxrows $resetmaskbitstate $calibrationon $initialresettrim 0
	    puts "no compression reset particular single column (((((((((((((((((((("
	}
    }	

# now iterate over each col and row
    set icol 0
    while { $icol < $maxcols } {
	# there are 80 rows  in PSI46 chip ***
# 0x95 col-nongray npixels enabled cal trim extraclocks	    
	    set maskbitstate $resetmaskbitstate
#puts "Reset: P($aindex): cal:$calibrationon  mask: $maskbitstate    trim:$initialresettrim"
	    

	    set irow 0
	    while {$irow < $maxrows} {
		set aindex "$icol $irow"
# the following colow chnage is for effect only and slightly slows things down
#	    .win0.fr1.can1 itemconfigure $pixarray($aindex) \
\#		-fill SkyBlue
#	    update
		set pixarrayenabled($aindex) $maskbitstate
		set pixarraycal($aindex) $calibrationon
		set pixarraytrim($aindex) $initialresettrim
		if {($calibrationon>0) && ($maskbitstate==1)} {
		    .win0.fr1.can1 itemconfigure $pixarray($aindex) \
			-fill red
		}
		if {($calibrationon>0) && ($maskbitstate==0)} {
		    .win0.fr1.can1 itemconfigure $pixarray($aindex) \
			-fill pink
		}
		if {($calibrationon==0) && ($maskbitstate==0)} {
		    .win0.fr1.can1 itemconfigure $pixarray($aindex) \
			-fill darkgray
		}
		if {($calibrationon==0) && ($maskbitstate==1)} {
		    .win0.fr1.can1 itemconfigure $pixarray($aindex) \
			-fill [trimcolor $initialresettrim]
		}
		#	    update
		incr irow
	    }

	    gauge_value .winreset.g [expr 100*($icol/51.0) ]

	    incr icol
	}
#    update


# clrcalgroups to clear cal/row lines
    if {$calibrationon == 0} {
	set calrowlist ""
	set calcollist ""
	set icol 0
	while {$icol < $maxcols} {
	    set irow 0
	    while {$irow < $maxrows} {
		set aindex "$icol $irow"
		set pixarraycal($aindex) $calibrationon
		incr irow
	    }
	    incr icol
	}
	
	clearcalgroups 1
    }

    update

    destroy .winreset.g

    set maskbitstate $tempmask
    set calibrationon $tempcal

}
proc resetiterate46 { command } {
    global pixarray
    global resettrim
    global maskbitstate
    global calibrationon
    global resetcalibration
    global resetstatus
    global pixarraycal
    global pixarrayenabled
    global pixarraytrim
    global dcolstate
    global maxcols
    global maxrows
    global roctype

    gauge_create .winreset.g SkyBlue
    pack .winreset.g -expand yes -fill both -padx 10 -pady 10

    set tempmask $maskbitstate
    set tempcal $calibrationon

    if {$resetcalibration=="CAP"} {
	set calibrationon 1
    } elseif {$resetcalibration=="BUMP"} {
	set calibrationon 2
    } else {
	set calibrationon 0
    }

    if {$resetstatus=="ENABLED"} {
	set resetmaskbitstate 1
    } else {
	set resetmaskbitstate 0
    }

    if {$calibrationon == 0} {
	ClrCal 1
    }
    
    set initialresettrim $resettrim

# Now clear dcol periphery
    set idcol 0
    set numdcols [expr $maxcols / 2]
    while {$idcol < $numdcols} {
	# Kill dcol
	# send 32=0x20 and 0x00 -> set kill bit for dcol
	set dcolnotation [expr $idcol*2]
	.win0.fr1.can1 itemconfigure "$dcolnotation-back" -fill darkgray
	pix_transfer_byte "0x25" $dcolnotation 64 0 0 0 0

	.win0.fr1.b$idcol configure -bg darkgray \
	    -activeforeground darkgray \
	    -activebackground darkgray \
	    -foreground black 
	set dcolstate(.win0.fr1.b$idcol) 0

	incr idcol
#	update
    }
# Done clear dcol periphery


# now iterate over each col and row
    set icol 0
    while { $icol < $maxcols } {
	# there are 80 rows  in PSI46 chip ***
# 0x95 col-nongray npixels enabled cal trim extraclocks	    
	    set maskbitstate $resetmaskbitstate
#puts "Reset: P($aindex): cal:$calibrationon  mask: $maskbitstate    trim:$initialresettrim"
	    
	    pix_transfer_byte "0x95" $icol $maxrows $maskbitstate $calibrationon $initialresettrim 0

	    set irow 0
	    while {$irow < $maxrows} {
		set aindex "$icol $irow"
# the following colow chnage is for effect only and slightly slows things down
#	    .win0.fr1.can1 itemconfigure $pixarray($aindex) \
\#		-fill SkyBlue
#	    update
		set pixarrayenabled($aindex) $maskbitstate
		set pixarraycal($aindex) $calibrationon
		set pixarraytrim($aindex) $initialresettrim
		if {($calibrationon>0) && ($maskbitstate==1)} {
		    .win0.fr1.can1 itemconfigure $pixarray($aindex) \
			-fill red
		}
		if {($calibrationon>0) && ($maskbitstate==0)} {
		    .win0.fr1.can1 itemconfigure $pixarray($aindex) \
			-fill pink
		}
		if {($calibrationon==0) && ($maskbitstate==0)} {
		    .win0.fr1.can1 itemconfigure $pixarray($aindex) \
			-fill darkgray
		}
		if {($calibrationon==0) && ($maskbitstate==1)} {
		    .win0.fr1.can1 itemconfigure $pixarray($aindex) \
			-fill [trimcolor $initialresettrim]
		}
		#	    update
		incr irow
	    }

	    gauge_value .winreset.g [expr 100*($icol/51.0) ]

	    incr icol
	}
#    update



# clrcalgroups to clear cal/row lines
    if {$calibrationon == 0} {
	set calrowlist ""
	set calcollist ""
	set icol 0
	while {$icol < $maxcols} {
	    set irow 0
	    while {$irow < $maxrows} {
		set aindex "$icol $irow"
		set pixarraycal($aindex) $calibrationon
		incr irow
	    }
	    incr icol
	}
	
	clearcalgroups 1
    }

    update

    destroy .winreset.g

    set maskbitstate $tempmask
    set calibrationon $tempcal

}
proc resetiterate46indiv { command } {
    global pixarray
    global resettrim
    global maskbitstate
    global calibrationon
    global resetcalibration
    global resetstatus
    global pixarraycal
    global pixarrayenabled
    global pixarraytrim
    global dcolstate
    global maxcols
    global maxrows
    global roctype

    gauge_create .winreset.g SkyBlue
    pack .winreset.g -expand yes -fill both -padx 10 -pady 10

    set tempmask $maskbitstate
    set tempcal $calibrationon

    if {$resetcalibration=="CAP"} {
	set calibrationon 1
    } elseif {$resetcalibration=="BUMP"} {
	set calibrationon 2
    } else {
	set calibrationon 0
    }

    if {$resetstatus=="ENABLED"} {
	set resetmaskbitstate 1
    } else {
	set resetmaskbitstate 0
    }

    if {$calibrationon == 0} {
	ClrCal 1
    }
    
# Now clear dcol periphery
    set idcol 0
    set numdcols [expr $maxcols / 2]
    while {$idcol < $numdcols} {
	# Kill dcol
	# send 32=0x20 and 0x00 -> set kill bit for dcol
	set dcolnotation [expr $idcol*2]
	.win0.fr1.can1 itemconfigure "$dcolnotation-back" -fill darkgray
	pix_transfer_byte "0x25" $dcolnotation 64 0 0 0 0

	.win0.fr1.b$idcol configure -bg darkgray \
	    -activeforeground darkgray \
	    -activebackground darkgray \
	    -foreground black 
	set dcolstate(.win0.fr1.b$idcol) 0

	incr idcol
	update
    }
# Done clear dcol periphery

    set initialresettrim $resettrim

# now iterate over each col and row
    set icol 0
    while { $icol < $maxcols } {
	# there are 80 rows  in PSI46 chip ***
# 0x95 col-nongray npixels enabled cal trim extraclocks	    
	    set maskbitstate $resetmaskbitstate
#puts "Reset: P($aindex): cal:$calibrationon  mask: $maskbitstate    trim:$initialresettrim"
	    
	    set irow 0
	    while {$irow < $maxrows} {
		pix_transfer_byte "0x44" $icol $irow $resetmaskbitstate 0 $resettrim 0
#		pix_transfer_byte "0x45" $icol $irow 1 $calibrationon $resettrim 0
		set aindex "$icol $irow"
# the following colow chnage is for effect only and slightly slows things down
#	    .win0.fr1.can1 itemconfigure $pixarray($aindex) \
\#		-fill SkyBlue
#	    update
		set pixarrayenabled($aindex) $maskbitstate
		set pixarraycal($aindex) $calibrationon
		set pixarraytrim($aindex) $initialresettrim
		if {($calibrationon>0) && ($maskbitstate==1)} {
		    .win0.fr1.can1 itemconfigure $pixarray($aindex) \
			-fill red
		}
		if {($calibrationon>0) && ($maskbitstate==0)} {
		    .win0.fr1.can1 itemconfigure $pixarray($aindex) \
			-fill pink
		}
		if {($calibrationon==0) && ($maskbitstate==0)} {
		    .win0.fr1.can1 itemconfigure $pixarray($aindex) \
			-fill darkgray
		}
		if {($calibrationon==0) && ($maskbitstate==1)} {
		    .win0.fr1.can1 itemconfigure $pixarray($aindex) \
			-fill [trimcolor $initialresettrim]
		}
		#	    update
		incr irow
	    }

	    gauge_value .winreset.g [expr 100*($icol/51.0) ]

	    incr icol
	}
    update


# clrcalgroups to clear cal/row lines
    if {$calibrationon == 0} {
	set calrowlist ""
	set calcollist ""
	set icol 0
	while {$icol < $maxcols} {
	    set irow 0
	    while {$irow < $maxrows} {
		set aindex "$icol $irow"
		set pixarraycal($aindex) $calibrationon
		incr irow
	    }
	    incr icol
	}
	
	clearcalgroups 1
    }

    destroy .winreset.g

    set maskbitstate $tempmask
    set calibrationon $tempcal

}

proc resetarrays { command } {
    global pixarray
    global maskbitstate
    global calibrationon
    global resetcalibration
    global pixarraycal
    global pixarrayenabled
    global pixarraytrim
    global dcolstate
    global maxrows
    global maxcols

    set tempcal $calibrationon
    set tempmask $maskbitstate

# now iterate over each col and row
    set icol 0
    while { $icol < $maxcols } {
	set irow 0
	while {$irow < $maxrows} {
	    set aindex "$icol $irow"
	    set maskbitstate 0
	    set calibrationon 0
	    set temptrim "Unknowns"
	    incr irow
	}
	incr icol
    }
    set idcol 0
    set numdcols [expr $maxcols / 2]
    while {$idcol < $numdcols} {
	set dcolstate(.win0.fr1.b$idcol) 0
	incr idcol
    }

# clrcalgroups to clear cal/row lines
    set calrowlist ""
    set calcollist ""
    set icol 0
    while {$icol < $maxcols} {
	set irow 0
	while {$irow < $maxrows} {
	    set aindex "$icol $irow"
	    set pixarraycal($aindex) 0
	    incr irow
	}
	incr icol
    }
    set maskbitstate $tempmask
    set calibrationon $tempcal

}

proc displayiterate { command } {
    global pixarray
    global maskbitstate
    global calibrationon
    global resetcalibration
    global pixarraycal
    global pixarrayenabled
    global pixarraytrim
    global dcolstate
    global maxcols
    global maxrows

#    gauge_create .winreset.g SkyBlue
#    pack .winreset.g -expand yes -fill both -padx 10 -pady 10

    set tempcal $calibrationon
    set tempmask $maskbitstate

# now iterate over each col and row
    set icol 0
    while { $icol < $maxcols } {
	set irow 0
	while {$irow < $maxrows} {
	    set aindex "$icol $irow"
	    set maskbitstate $pixarrayenabled($aindex)
	    set calibrationon $pixarraycal($aindex)
	    set temptrim $pixarraytrim($aindex)
	    if {($calibrationon==1) && ($maskbitstate==1)} {
		    .win0.fr1.can1 itemconfigure $pixarray($aindex) \
			-fill red
		}
		if {($calibrationon==1) && ($maskbitstate==0)} {
		    .win0.fr1.can1 itemconfigure $pixarray($aindex) \
			-fill pink
		}
		if {($calibrationon==0) && ($maskbitstate==0)} {
		    .win0.fr1.can1 itemconfigure $pixarray($aindex) \
			-fill darkgray
		}
		if {($calibrationon==0) && ($maskbitstate==1)} {
		    .win0.fr1.can1 itemconfigure $pixarray($aindex) \
			-fill [trimcolor $temptrim]
		}
		incr irow
	    }

#	    gauge_value .winreset.g [expr 100*($icol/51.0) ]
	    incr icol
	}

    set idcol 0
    set numdcols [expr $maxcols / 2]
    while {$idcol < $numdcols} {
	set dcolnotation [expr $idcol*2]
	if {$dcolstate(.win0.fr1.b$idcol)==0} {
	    .win0.fr1.can1 itemconfigure "$dcolnotation-back" -fill darkgray
	    .win0.fr1.b$idcol configure -bg darkgray \
		-activeforeground darkgray \
		-activebackground darkgray \
		-foreground black 
	} elseif {$dcolstate(.win0.fr1.b$idcol)==1} {
	    .win0.fr1.can1 itemconfigure "$dcolnotation-back" -fill blue
	    .win0.fr1.b$idcol configure -bg green \
		-activeforeground green \
		-activebackground green \
		-foreground black 
	}
	incr idcol
    }


# clrcalgroups to clear cal/row lines
    set calrowlist ""
    set calcollist ""
    set icol 0
    while {$icol < $maxcols} {
	set irow 0
	while {$irow < $maxrows} {
	    set aindex "$icol $irow"
##	    set pixarraycal($aindex) $calibrationon
	    incr irow
	}
	incr icol
    }

#    clearcalgroups 1

#    destroy .winreset.g

    set maskbitstate $tempmask
    set calibrationon $tempcal

}

proc docaldelete { mycol myrow } {
    global calrowlist
    global calcollist
    
    global pixarray
    global pixarraycal
    global pixarrayenabled
    global pixarraytrim
    global verbose2
    global maxcols
    global maxrows

## docaldelete called on a pixel we want to UNcalibrate
## Now go down each row and each col and turn the pixel either gray (disabled
## + disabled) or green (trim + enabled).
## So whereever was clicked, we change the color of that row/column.
    set jrow 0
    while {$jrow < $maxrows} {
	set aindex "$mycol $jrow"
	set maskstate $pixarrayenabled($aindex)
	set temptrim $pixarraytrim($aindex)
	set pixarraycal($aindex) 0
	if {$maskstate==0} {
	    .win0.fr1.can1 itemconfigure $pixarray($aindex) -fill darkgray
	} else {
	    .win0.fr1.can1 itemconfigure $pixarray($aindex) -fill [trimcolor $temptrim]
	}
	incr jrow
    }
    set jcol 0
    while {$jcol < $maxcols} {
	set aindex "$jcol $myrow"
	set maskstate $pixarrayenabled($aindex)
	set temptrim $pixarraytrim($aindex)
	set pixarraycal($aindex) 0
	if {$maskstate==0} {
	    .win0.fr1.can1 itemconfigure $pixarray($aindex) -fill darkgray
	} else {
	    .win0.fr1.can1 itemconfigure $pixarray($aindex) -fill [trimcolor $temptrim]
	}
	incr jcol
    }


## Now lets remove the pixel's row and col from the lists of calibrated rows
## and columns.  
    if {$verbose2==1} {
	puts "DOCALDELETE $mycol $myrow"
	puts "BEFORE deletion, calrowlist:$calrowlist    calcollist:$calcollist"
    }
# if the element is in the list find its position
    set ttemp [lsearch $calrowlist $myrow]
    set temprowlist2 ""
    if {$ttemp == 0} {
	set calrowlist [lrange $calrowlist 1 end]
    } elseif {$ttemp > 0} {
	set iend [expr ($ttemp-1)]
	set temprowlist [lrange $calrowlist 0 $iend]
	set istart [expr ($ttemp+1)]
	if {$istart <= [llength $calrowlist]} {
	    set temprowlist2 [lrange $calrowlist $istart end]
	}
	set calrowlist "$temprowlist $temprowlist2"
    }
#
    set ttemp [lsearch $calcollist $mycol]
    set tempcollist2 ""
    if {$ttemp == 0} {
	set calcollist [lrange $calcollist 1 end]
    } elseif {$ttemp > 0} {
	set iend [expr ($ttemp-1)]
	set tempcollist [lrange $calcollist 0 $iend]
	set istart [expr ($ttemp+1)]
	if {$istart <= [llength $calcollist]} {
	    set tempcollist2 [lrange $calcollist $istart end]
	}
	set calcollist "$tempcollist $tempcollist2"
    }
    set nlength [llength $calcollist]
    set nlength2 [llength $calrowlist]
    if { ($nlength == 0) || ($nlength2 == 0) } {
	set calrowlist ""
	set calcollist ""
    }
    if {$verbose2==1} {
	puts "AFTER deletion, calrowlist:$calrowlist    calcollist:$calcollist"
    }
}

proc docalcolrow { mycol myrow } {
    global calrowlist
    global calcollist
    global verbose2


# check if myrow is in the list and ifnot add it
    set ttemp [lsearch $calrowlist $myrow]
    if {$ttemp < 0} {
	set calrowlist [ linsert $calrowlist 0 $myrow ]
    }

    set ttemp [lsearch $calcollist $mycol]
    if {$ttemp < 0} {
	set calcollist [ linsert $calcollist 0 $mycol ]
    }

# Reassert a cal on rows using first entry in col
    set nitems [llength $calcollist]
    if {$nitems > 0} {
	set firstcolentry [lindex $calcollist 0]
	set iindex 0
	while {$iindex < [llength $calrowlist]} {
	    set irow [lindex $calrowlist $iindex]
	    .win0.fr1.can1 itemconfigure $irow-calrow \
		-fill red
	    incr iindex
	}
    }

# then reasert on cols for the first entry in row
    set nitems [llength $calrowlist]
    if {$nitems > 0} {
	set firstrowentry [lindex $calrowlist 0]
	set iindex 0
	while {$iindex < [llength $calcollist]} {
	    set icol [lindex $calcollist $iindex]
	    .win0.fr1.can1 itemconfigure $icol-calcol \
		-fill red
	    incr iindex
	}
    }
    if {$verbose2==1} {
	puts "CALrows:$calrowlist"
    }
}

proc dopixarraycal { mycol myrow mycal } {
    global pixarray
    global pixarraycal
    global pixarrayenabled
    global pixarraytrim
    global maxcols
    global maxrows

    set jcol 0
    while { $jcol < $maxcols } {
	# for the given row, go horizontally and change the cal array
	# to reflect the change of state of the entire row, respecting
	# enabled bit when lighting the pixel
	set aindex "$jcol $myrow"
	set maskstate $pixarrayenabled($aindex)
	set temptrim $pixarraytrim($aindex)
	set tempcal $pixarraycal($aindex)
	if { $jcol != $mycol } {
####	    set pixarraycal($aindex) $mycal
	    if {$maskstate==0} {
		.win0.fr1.can1 itemconfigure $pixarray($aindex) \
		    -fill darkgray
	    } else {
		.win0.fr1.can1 itemconfigure $pixarray($aindex) \
		    -fill [trimcolor $temptrim]
		if {$tempcal == 1} {
		    .win0.fr1.can1 itemconfigure $pixarray($aindex) \
			-fill red
		}
		if {$tempcal == 2} {
		    .win0.fr1.can1 itemconfigure $pixarray($aindex) \
			-fill violet
		}
	    }
	}
	incr jcol
    }
    #now up the column
    set jrow 0
    while { $jrow < $maxrows } {
	# for the given col, go vertically and change the cal array
	# to reflect the change of state of the entire row, respecting
	# enabled bit when lighting the pixel
	set aindex "$mycol $jrow"
	set maskstate $pixarrayenabled($aindex)
	set temptrim $pixarraytrim($aindex)
	set tempcal $pixarraycal($aindex)
	if { $jrow != $myrow } {
####	    set pixarraycal($aindex) $mycal
	    if {$maskstate==0} {
		.win0.fr1.can1 itemconfigure $pixarray($aindex) \
		    -fill darkgray
	    } else {
		.win0.fr1.can1 itemconfigure $pixarray($aindex) \
		    -fill [trimcolor $temptrim]
		if {$tempcal == 1} {
		    .win0.fr1.can1 itemconfigure $pixarray($aindex) \
			-fill red
		}
		if {$tempcal == 2} {
		    .win0.fr1.can1 itemconfigure $pixarray($aindex) \
			-fill violet
		}
	    }
	}
	incr jrow
    }
    update
}

proc calon { mycol myrow } {
    global calrowlist
    global calcollist
    global pixarraycal
    global pixarray
    global pixarraytrim
    global pixarrayenabled
    global verbose2

    set ireturn 0

    set ttemp [lsearch $calcollist $mycol]
    set ttemp2 [lsearch $calrowlist $myrow]
    if {($ttemp >= 0) || ($ttemp2 >= 0)} {
	if {$verbose2==1} {
	    puts "Coord is in the calibration set."
	}
	set ireturn 1
    }

    return $ireturn
}

proc findcals { command } {
    global calrowlist
    global calcollist
    global pixarraycal
    global pixarray
    global pixarraytrim
    global pixarrayenabled
    global verbose2

    if {$verbose2==1} {
	puts "FINDCALS: cols($calcollist)   rows($calrowlist)"
    }
    set icol 0
    set nitems [llength $calcollist]
    while {$icol < $nitems} {
	set colentry [lindex $calcollist $icol]
	set iindex 0
	while {$iindex < [llength $calrowlist]} {
	    set irow [lindex $calrowlist $iindex]
	    if {$verbose2==1} {
		puts "REASSERT CAL ***  on $colentry $irow    iindex: $iindex"
	    }
	    .win0.fr1.can1 itemconfigure $irow-calrow \
		-fill red
	    .win0.fr1.can1 itemconfigure $colentry-calcol \
		-fill red
	    set aindex "$colentry $irow"
#	    .win0.fr1.can1 itemconfigure $pixarray($aindex) -fill red
	    set maskstate $pixarrayenabled($aindex)
	    if {$maskstate==0} {
		.win0.fr1.can1 itemconfigure $pixarray($aindex) \
		    -fill pink
	    } else {
		# set pixel to calibrate, standard cap as default (not bump)
		# unclear as to how cap and bump cal interacts
		if {$pixarraycal($aindex) < 1} {
		    set pixarraycal($aindex) 1
		}
		.win0.fr1.can1 itemconfigure $pixarray($aindex) \
		    -fill red
	    }

	    incr iindex
	}
	incr icol
    }
}

proc clrcalintersections { command } {
    global calrowlist
    global calcollist
    global pixarray
    global verbose2
    global dogui

    if {$dogui != 1} { return }

    if {$verbose2==1} {
	puts "INTERSECTIONS: colsxrows of $calcollist x $calrowlist"
    }
    set nitems [llength $calcollist]
    set jindex 0
    while {$jindex < $nitems} {
	set icol [lindex $calcollist $jindex]
	set iindex 0
	while {$iindex < [llength $calrowlist]} {
	    set irow [lindex $calrowlist $iindex]
	    set aindex "$icol $irow"
	    .win0.fr1.can1 itemconfigure $pixarray($aindex) -fill darkgray
	    incr iindex
	}
	incr jindex
    }

}

proc clearcalgroups { command } {
    global calrowlist
    global calcollist
    global maxcols
    global maxrows

    # clear all the cal rows and cols lines
    set n 0
    while {$n < $maxrows} {
	.win0.fr1.can1 itemconfigure $n-calrow \
	    -fill ""
	incr n
    }
    set n 0
    while {$n < $maxcols} {
	.win0.fr1.can1 itemconfigure $n-calcol \
	    -fill ""
	incr n
    }
}

proc reassertcals { command } {
    global pixarray
    global maskbitstate
    global calibrationon
    global resetcalibration
    global resetstatus
    global pixarraycal
    global pixarrayenabled
    global pixarraytrim
    global trim
    
    global calrowlist
    global calcollist

    global calrowlinearray
    global verbose2

    global maxcols
    global maxrows

# On 3-25-03 we received description of proper cal functioning.  With v116
# on up we keep lists of col and rows set to cal and need not iterate for 
# each pixel.

    clearcalgroups 1
	
# Reassert a cal on rows using first entry in col
    set nitems [llength $calcollist]
    if {$nitems > 0} {
	set firstcolentry [lindex $calcollist 0]
	set iindex 0
	while {$iindex < [llength $calrowlist]} {
	    set irow [lindex $calrowlist $iindex]
	    if {$verbose2==1} {
		puts "REASSERT CAL on row:$irow and col:$firstcolentry"
	    }
	    .win0.fr1.can1 itemconfigure $irow-calrow \
		-fill red
	    incr iindex
	}
    }
# then reasert on cols for the first entry in row
    set nitems [llength $calrowlist]
    if {$nitems > 0} {
	set firstrowentry [lindex $calrowlist 0]
	set iindex 0
	while {$iindex < [llength $calcollist]} {
	    set icol [lindex $calcollist $iindex]
	    if {$verbose2==1} {
		puts "REASSERT CAL now on col:$icol and row:$firstrowentry"
	    }
	    .win0.fr1.can1 itemconfigure $icol-calcol \
		-fill red
	    incr iindex
	}
    }


# now iterate over each col and row
    set icol 0
    while { $icol < $maxcols } {
	set irow 0
	while { $irow < $maxrows } {
	    set aindex "$icol $irow"
	    set calstate $pixarraycal($aindex)
	    if {$calstate>"0"} {
		set maskstate $pixarrayenabled($aindex)
		set temptrim $pixarraytrim($aindex)
		if {$temptrim == "Unknown"} {
		    set temptrim $trim
		}
#		if icol in set of calcollist and irow in set of calrowlist
		set ttemp [lsearch $calcollist $icol]
		set ttemp2 [lsearch $calrowlist $irow]
		if {($ttemp >= 0) && ($ttemp2 >= 0)} {
		    if {$maskstate == "0"} {
			if { $command == "1" } {
			    pix_transfer_byte "0x45" $icol $irow 0 $calstate \
				$temptrim 0
			}
			.win0.fr1.can1 itemconfigure $pixarray($aindex) \
			    -fill pink
		    } else {
			if { $command == "1"} {
			    pix_transfer_byte "0x45" $icol $irow 1 $calstate \
				$temptrim 0
			}
			.win0.fr1.can1 itemconfigure $pixarray($aindex) \
			    -fill red
		    }
		}

	    } else {
# else the pixel should be pink, an intersection of red lines not enabled
# just gets turned pink and nothing more
#		if icol in set of calcollist and irow in set of calrowlist
		set ttemp [lsearch $calcollist $icol]
		set ttemp2 [lsearch $calrowlist $irow]
		if {($ttemp >= 0) && ($ttemp2 >= 0)} {
		    set maskstate $pixarrayenabled($aindex)
		    if {$maskstate == "0"} {
			.win0.fr1.can1 itemconfigure $pixarray($aindex) \
			    -fill pink
		    }
		}
	    }
#	    update
	    incr irow
	}
	incr icol
    }

}

proc memsave { command } {
    global pixarray
    global maskbitstate
    global calibrationon
    global resetcalibration
    global resetstatus
    global pixarraycal
    global pixarrayenabled
    global pixarraytrim
    global trim
    global wbc
    global ctrl_reg
    global dacarray
    global pixreadoutspeed
    global pixportaddress
    global pixchipaddress
    global pixstopdaq
    global dcolstate
    global currenttbm

    global calrowlist
    global calcollist

    global chiptouched

#arrays
    global memmodule$command-pixarraytrim
    global memmodule$command-pixarrayenabled
    global memmodule$command-pixarraycal
    global memmodule$command-dacarray
    global memmodule$command-dcolstate
    global memmodule_calrowlist
    global memmodule_calcollist

#vars
    global memmodule_calibrationon
    global memmodule_maskbitstate
    global memmodule_wbc
    global memmodule_ctrl_reg
    global memmodule_trim
    global memmodule_pixreadoutspeed
    global memmodule_pixportaddress
    global memmodule_pixchipaddress
    global memmodule_pixstopdaq
    global memmodule_currenttbm

    global resetstatus
    global resetcalibration


    global col
    global row
    global trim

    array set memmodule$command-pixarraytrim [array get pixarraytrim]
    array set memmodule$command-pixarrayenabled [array get pixarrayenabled]
    array set memmodule$command-pixarraycal [array get pixarraycal]
    array set memmodule$command-dacarray [array get dacarray]
    array set memmodule$command-dcolstate [array get dcolstate]
    set memmodule_calrowlist($command) $calrowlist
    set memmodule_calcollist($command) $calcollist

    set memmodule_calibrationon($command) $calibrationon
    set memmodule_maskbitstate($command) $maskbitstate
    set memmodule_wbc($command) $wbc
    set memmodule_ctrl_reg($command) $ctrl_reg
    set memmodule_trim($command) $trim
    set memmodule_pixreadoutspeed($command) $pixreadoutspeed
    set memmodule_pixportaddress($command) $pixportaddress
    set memmodule_pixchipaddress($command) $pixchipaddress
    set memmodule_pixstopdaq($command) $pixstopdaq
    set memmodule_currenttbm($command) $currenttbm

    if { [winfo exists .windac] == 1} {
	.windac.col3.f4.header configure -text "Chip ID:$pixchipaddress\n"
    }
 }

proc memload { command nodisplay } {
    global pixarray
    global maskbitstate
    global calibrationon
    global resetcalibration
    global resetstatus
    global pixarraycal
    global pixarrayenabled
    global pixarraytrim
    global trim
    global wbc
    global ctrl_reg
    global dacarray
    global pixreadoutspeed
    global pixportaddress
    global pixchipaddress
    global pixstopdaq
    global dcolstate
    global calrowlist
    global calcollist
    global roctype
    global currenttbm
#arrays
    global memmodule$command-pixarraytrim
    global memmodule$command-pixarrayenabled
    global memmodule$command-pixarraycal
    global memmodule$command-dacarray
    global memmodule$command-dcolstate
    global memmodule_calrowlist
    global memmodule_calcollist

#vars
    global memmodule_calibrationon
    global memmodule_maskbitstate
    global memmodule_wbc
    global memmodule_ctrl_reg
    global memmodule_trim
    global memmodule_pixreadoutspeed
    global memmodule_pixportaddress
    global memmodule_pixchipaddress
    global memmodule_pixstopdaq
    global memmodule_currenttbm
# clear the row/col lines (red)
    clearcalgroups 1

    array set pixarraytrim [array get memmodule$command-pixarraytrim]
    array set pixarrayenabled [array get memmodule$command-pixarrayenabled]
    array set pixarraycal [array get memmodule$command-pixarraycal]
    array set dacarray [array get memmodule$command-dacarray]
    array set dcolstate [array get memmodule$command-dcolstate]
    set calrowlist $memmodule_calrowlist($command)
    set calcollist $memmodule_calcollist($command)

    set calibrationon $memmodule_calibrationon($command)
    set maskbitstate  $memmodule_maskbitstate($command)
    set wbc $memmodule_wbc($command)
    set ctrl_reg $memmodule_ctrl_reg($command)
    set trim $memmodule_trim($command)
    set currenttbm $memmodule_currenttbm($command)
    updatetrimslide $trim
    set pixreadoutspeed $memmodule_pixreadoutspeed($command)
    set pixportaddress $memmodule_pixportaddress($command)
    set pixchipaddress $memmodule_pixchipaddress($command)
    set pixstopdaq $memmodule_pixstopdaq($command)

#    set iindex [array startsearch dacarray]
#    while { [array anymore dacarray $iindex] } {
#	set inextdac [array nextelement dacarray $iindex]
#	puts "dac: $inextdac should be set to integer value: $dacarray($inextdac)"
#    }
#    array donesearch dacarray $iindex

	.windac.col1.f1.d1.s1 set [ getdacfloat 2 800 1300 256 ]
	.windac.col1.f1.d2.s1 set [ getdacfloat 3 1000 2100 256 ]
	.windac.col1.f1.d3.s1 set [ getdacfloat 4 1800 2100 16 ]
	.windac.col1.f1.d4.s1 set [ getdacfloat 1 1700 2100 16 ]
	.windac.col1.f2.d1.s1 set [ getdacfloat 5 -700 0 256 ]
	.windac.col1.f2.d2.s1 set [ getdacfloat 6 0.0 500.0 16 ]
	.windac.col1.f2.d3.s1 set [ getdacfloat 7 500 1300 256 ]
	.windac.col1.f2.d4.s1 set [ getdacfloat 8 0 500 16 ]
	.windac.col1.f2.d5.s1 set [ getdacfloat 9 500 1300 256 ]
	.windac.col1.f2.d6.s1 set [ getdacfloat 10 -1500 -500 256 ]
	.windac.col1.f2.d7.s1 set [ getdacfloat 11 -1150 -400 256 ]
	.windac.col1.f2.d8.s1 set [ getdacfloat 12 -1500 -600 256 ]
	.windac.col2.f3.d1.s1 set [ getdacfloat 13 0 12 256 ]
	.windac.col2.f3.d2.s1 set [ getdacfloat 14 0 50 16 ]
	.windac.col2.f3.d3.s1 set [ getdacfloat 15 1000 1500 256 ]
	.windac.col2.f3.d4.s1 set [ getdacfloat 16 0 20 256 ]
	.windac.col2.f3.d5.s1 set [ getdacfloat 17 1000 1500 256 ]
	.windac.col2.f3.d6.s1 set [ getdacfloat 18 0 100 256 ]
	.windac.col2.f3.d7.s1 set [ getdacfloat 19 0 30 256 ]
	.windac.col2.f3.d8.s1 set [ getdacfloat 20 0 20 256 ]
	.windac.col2.f3.d9.s1 set [ getdacfloat 21 0 20 256 ]
	.windac.col3.f4.d1.s1 set [ getdacfloat 22 0 200 256 ]
	.windac.col3.f4.d2.s1 set [ getdacfloat 23 0 70 256 ]
	.windac.col3.f4.d3.s1 set [ getdacfloat 24 0 150 256 ]
	.windac.col3.f5.d1.s1 set [ getdacfloat 25 0 1800 256 ]
	.windac.col3.f5.d2.s1 set [ getdacfloat 26 55 205 256 ]
	.windac.col3.f5.d3.s1 set [ getdacfloat 27 -50 80 16 ]

    
    if { [winfo exists .windac] == 1} {
	.windac.col3.f4.header configure -text "Chip ID:$pixchipaddress"
    }

    if {$nodisplay != -1} {
	displayiterate 1
    }

# call w 0 to reassert cals on display but do not send i2c
    if {$nodisplay != -1} {
	reassertcals 0
   }

    # since some things may have changed patch up these control addresses
    bithubportaddress 1

#    parray memmodule$command-pixarraytrim
}

proc script_config { mycommand } {
    # anything settable in script has to be made global here to reach the
    # global var version.  "set" is prepended
# outformat paw24 or binary or seu
    global outformat
# dogui (do-gui or dog-ui) allows a script to turn off display updates,
# should do a reset after a script to get things back to normal, with this
# set back to 1
    global dogui

#    puts "S: config-> $mycommand"
    set commandstring "set $mycommand"
    set result [eval $commandstring]
}

proc script_getevent { } {
    set dostop 0
    set ioutevents 0
    getevent 1
}


# var arg list: proc script_systemreset { args } {
proc script_systemreset { } {
    doresetsystem 1
}
proc script_rocreset { } {
	if {$verbose3=="1"} {
	    puts "SENDING ResetROC TO FEC"
	}
	set temp2 "0111000"
##npci	sendbytewparity $temp2
	set tt1 [wordtobits 00000001]
	set tt2 [bitwordset $tt1 7 $encalpulse]
	set tt2 [bitwordset $tt2 5 $fec_trigger_source]
	set tt2 [bitwordset $tt2 6 $norm_loop_on]
	set tt3 [bitstoword $tt2]
	sendpcifectriggerregister 0

#old    transfer_byte "00000100" "0x2" "write"
}
proc script_clrcal { } {
    global calrowlist
    global calcollist
    global adcfileopen
    global dogui

    clrcalintersections 1
    set calrowlist ""
    set calcollist ""
    ClrCal 1
    if {$dogui != 1} { return }
    
    if {$adcfileopen == 1} { return }
    
    reassertcals 1
}

proc script_setmode { mymode } {
# Sync | ClrTrigCnt | Cal
    global tbm2mode
    if { $mymode == "Sync"} {
	set tbm2mode "Sync"
	.win0.fr2.g6.box1.f6.right.b3 configure -relief raised \
	    -fg black -activeforeground black
	.win0.fr2.g6.box1.f6.right.b2 configure -relief raised \
	    -fg black -activeforeground black
	.win0.fr2.g6.box1.f6.right.b1 configure -relief sunken \
	    -fg darkgreen -activeforeground darkgreen
	tbm2_write_f2 1
    } elseif { $mymode == "ClrTrigCnt" } {
	set tbm2mode "ClrTrigCount"
	.win0.fr2.g6.box1.f6.right.b3 configure -relief raised \
	    -fg black -activeforeground black
	.win0.fr2.g6.box1.f6.right.b1 configure -relief raised \
	    -fg black -activeforeground black
	.win0.fr2.g6.box1.f6.right.b2 configure -relief sunken \
	    -fg darkgreen -activeforeground darkgreen
	tbm2_write_f2 1
    } elseif { $mymode == "Cal" } {
	set tbm2mode "Calibration"
	.win0.fr2.g6.box1.f6.right.b1 configure -relief raised \
	    -fg black -activeforeground black
	.win0.fr2.g6.box1.f6.right.b2 configure -relief raised \
	    -fg black -activeforeground black
	.win0.fr2.g6.box1.f6.right.b3 configure -relief sunken \
	    -fg darkgreen -activeforeground darkgreen
	tbm2_write_f2 1
    } else {
	puts "Error in script: setmode to $mymode no defined."
    }
}


proc script_daqstate { mycommand } {
# ON | OFF
    global pixstopdaq
    if { $mycommand == "ON" } {
	set pixstopdaq 0
	dochangereadoutspeed 1 
    } elseif { $mycommand == "OFF" } {
	set pixstopdaq 1
	dochangereadoutspeed 1 
    } else {
	puts "ERROR in script: daqstate $mycommand undefined."
    }
}
proc script_slowhubsenddata { p0 p1 p2 p3 p4 p5} {
    global edserrors
    global edstotal
    slowhubsend $p0 $p1 $p2 $p3 $p4 $p5
    puts "SCRIPT $edserrors of $edstotal"
    set itemp [expr $p5 - 64]
    puts "SCRIPT slowhubsenddata $p0 $p1 $p2 $p3 $p4 $p5 ($itemp)"
}
proc script_setdcol { mydcol mystate } {
# 0..26, ON|OFF 
    global dcolstate
    
    if { ($mystate == "1") || ($mystate == "ON") } {
	set mybinarystate 1
    } elseif { ($mystate == "0") || ($mystate == "OFF") } {
	set mybinarystate 0
    }
    set dcolstate(.win0.fr1.b$mydcol) $mybinarystate
# changestatebutton is the target of the button-toggle switches, so have
# to call twice to set to value in array
    changestatebutton .win0.fr1.b$mydcol
    changestatebutton .win0.fr1.b$mydcol
}
proc script_setwbc { mywbc } {
    setwbc $mywbc
}

proc script_setreadoutspeed { myspeed } {
    global textreadoutspeed

    if {($myspeed == "20MHz") || ($myspeed == "40MHz")} {
	set textreadoutspeed $myspeed
	dochangereadoutspeed 1 
    } else {
	puts "ERROR script: setreadoutspeed $myspeed undefined."
    }
}

proc script_rocinit { mytrim mycal mystate } {
    global trim
    global calibrationon
    global maskbitstate
    global resetstatus
    global resettrim
    global resetcalibration

    if { [winfo exists .winreset] == 1} {
	wm withdraw .winreset
#	wm deiconify .winreset
    } else {
	resetchip 1 
	wm withdraw .winreset
    }

	ClrCal 1

    updateresettrimslide $mytrim
    if { ($mycal == "ON") || ($mycal == "1") || ($mycal == "CAP") } {
	set resetcalibration ON
    } else {
	set resetcalibration OFF
    }
    if { ($mystate == "ENABLED") || ($mystate == "enabled") } {
	set resetstatus ENABLED
    } else {
	set resetstatus DISABLED
    }
    

    	resetiterate 1

	clrcalintersections 1
	set calrowlist ""
	set calcollist ""

	destroy .winreset
}


proc script_adc_sample { nevents } {
    global adcrunmode
    global suppressdisplay
    global dostop

#    puts "S directed to sample $nevents events"
    set adcrunmode Continuous
#    set suppressdisplay 1
#    set suppressdisplay 0
#
	set dostop 0
	getevent $nevents
}


proc script_pixel_chipload { modchip } {
#
#    global globalmodchipbutton
#    puts "S chipload $modchip"
    if {$modchip <= 8} {
	chipload .win0.fr2.g2.line1.bb$modchip
    } else {
	set imodchip [expr ($modchip - 8)]
	chipload .win0.fr2.g2.line2.bb$imodchip
    }

#    return "S chipload $modchip done"

}
proc script_pixel_calibrate { pcol prow penabled } {
    global pixarray
    global pixarrayenabled
    global pixarraycal
    global pixarraytrim
    global trim
    global dogui
    
    set saindex "$pcol $prow"
    set temptrim $pixarraytrim($saindex)
    if {$temptrim == "Unknown"} {
	set temptrim $trim
    }
    if {$penabled == "1"} {
	set isenabled "1"
    } elseif {$penabled == "0"} {
	set isenabled "0"
    } else {
	puts "Script error 1"
    }

    set pixarrayenabled($saindex) $isenabled
    set pixarraycal($saindex) "1"

    if {$dogui == 1} {
	docalcolrow $pcol $prow
	dopixarraycal $pcol $prow 1
    }

    if {$dogui == 1} {
	if { $isenabled == "1" } {
	    .win0.fr1.can1 itemconfigure $pixarray($saindex) \
		-fill red
	} elseif { $isenabled == "0" } {
	    .win0.fr1.can1 itemconfigure $pixarray($saindex) \
		-fill pink
	}
    }

    if {$isenabled == "1"} {
	pix_transfer_byte "0x44" $pcol $prow 1 0 $temptrim 0
	pix_transfer_byte "0x45" $pcol $prow 1 1 $temptrim 0
    } elseif {$isenabled == "0"} {
	pix_transfer_byte "0x44" $pcol $prow 0 0 $temptrim 0
	pix_transfer_byte "0x45" $pcol $prow 0 1 $temptrim 0
    }
}

proc test_pix_transfer_byte { command } {
    global dogui
    set dogui 0
    puts "\nBEGIN TEST_PIX_TRANSFER_BYTE with 10000 (100byte) USBFEC OPS\n"
    puts "Sending 10000 usbtbmcmd 14 16 4 5 64 1"
# 4160
    for {set m 0} {$m < 10000} {incr m} {
	set ttt [expr $m % 1000]
	if {$ttt == 0} {
	    puts $m
	    update
	}
	for {set n 0} {$n < 1} {incr n} {
		set ttt [usbroccmd 16 0 15 1 1 2 3 0]
	    if {$m == 4763} { 
		set ttt [usbroccmd 16 0 15 1 1 2 3 0]

	    }
	}
	
#    sendbytechr [bitstoint "10000000"]
    }

    usbtbmcmd 14 16 4 7 250 0

#    usbclose
    puts "\nEND TEST_PIX_TRANSFER_BYTE\n"
    set dogui 1
}
proc script_pixel_settrim { pcol prow ptrimvalue penabled } {
    global pixarray
    global pixarrayenabled
    global pixarraytrim
    global trim
    global dogui

    set saindex "$pcol $prow"
    set temptrim $ptrimvalue

    if {$penabled == "1"} {
	set isenabled "1"
    } elseif {$penabled == "0"} {
	set isenabled "0"
    } else {
	puts "Script error 2"
    }

    set pixarrayenabled($saindex) $isenabled
    set pixarraytrim($saindex) $temptrim
    
    if {$dogui == 1} {
	if { $isenabled == "1" } {
	    .win0.fr1.can1 itemconfigure $pixarray($saindex) \
		-fill [trimcolor $temptrim]
	} elseif { $isenabled == "0" } {
	    .win0.fr1.can1 itemconfigure $pixarray($saindex) \
		-fill gray
	}
    }

    if {($isenabled == "1") | ($isenabled == "0")} {
	pix_transfer_byte "0x44" $pcol $prow $isenabled 0 $temptrim 0
    }
}
proc script_dacset_byaddress { dacaddress dacvalue } {
    global dacarray
    global dacresolution
    global dacrmin
    global dacrmax
    global roctype
    global dogui

    set dacarray($dacaddress) $dacvalue

#    updatedacslide $dacaddress $dacresolution($dacaddress) $dacrmin($dacaddress) $dacrmax($dacaddress)
#    puts "UPS: $dacaddress $dacresolution($dacaddress)	$dacrmin($dacaddress) $dacrmax($dacaddress) $dacvalue"

    pix_transfer_byte "0x08" $dacaddress $dacvalue 0 0 0 0

    if {$dogui != 1} { return }

# now, psi46 only...
    switch $dacaddress {
	2 { .windac.col1.f1.d1.s1 set [ getdacfloat 2 800 1300 256 ] }
	3 { .windac.col1.f1.d2.s1 set [ getdacfloat 3 1000 2100 256 ] }
	4 { .windac.col1.f1.d3.s1 set [ getdacfloat 4 1800 2100 16 ] }
	1 { .windac.col1.f1.d4.s1 set [ getdacfloat 1 1700 2100 16 ] }
	5 { .windac.col1.f2.d1.s1 set [ getdacfloat 5 -700 0 256 ] }
	6 { .windac.col1.f2.d2.s1 set [ getdacfloat 6 0.0 500.0 16 ] }
	7 { .windac.col1.f2.d3.s1 set [ getdacfloat 7 500 1300 256 ] }
	8 { .windac.col1.f2.d4.s1 set [ getdacfloat 8 0 500 16 ] }
	9 { .windac.col1.f2.d5.s1 set [ getdacfloat 9 500 1300 256 ] }
	10 { .windac.col1.f2.d6.s1 set [ getdacfloat 10 -1500 -500 256 ] }
	11 { .windac.col1.f2.d7.s1 set [ getdacfloat 11 -1150 -400 256 ] }
	12 { .windac.col1.f2.d8.s1 set [ getdacfloat 12 -1500 -600 256 ] }
	13 { .windac.col2.f3.d1.s1 set [ getdacfloat 13 0 12 256 ] }
	14 { .windac.col2.f3.d2.s1 set [ getdacfloat 14 0 50 16 ] }
	15 { .windac.col2.f3.d3.s1 set [ getdacfloat 15 1000 1500 256 ] }
	16 { .windac.col2.f3.d4.s1 set [ getdacfloat 16 0 20 256 ] }
	17 { .windac.col2.f3.d5.s1 set [ getdacfloat 17 1000 1500 256 ] }
	18 { .windac.col2.f3.d6.s1 set [ getdacfloat 18 0 100 256 ] }
	19 { .windac.col2.f3.d7.s1 set [ getdacfloat 19 0 30 256 ] }
	20 { .windac.col2.f3.d8.s1 set [ getdacfloat 20 0 20 256 ] }
	21 { .windac.col2.f3.d9.s1 set [ getdacfloat 21 0 20 256 ] }
	22 { .windac.col3.f4.d1.s1 set [ getdacfloat 22 0 200 256 ] }
	23 { .windac.col3.f4.d2.s1 set [ getdacfloat 23 0 70 256 ] }
	24 { .windac.col3.f4.d3.s1 set [ getdacfloat 24 0 150 256 ] }
	25 { .windac.col3.f5.d1.s1 set [ getdacfloat 25 0 1800 256 ] }
	26 { .windac.col3.f5.d2.s1 set [ getdacfloat 26 55 205 256 ] }
	27 { .windac.col3.f5.d3.s1 set [ getdacfloat 27 -50 80 16 ] }
    }
#    puts "S: set dac to $dacvalue"
}

proc script_dacset_byname { dacname dacvalue } {
    global dacarray
    global dacresolution
    global dacrmin
    global dacrmax
    global roctype

    global dacscalelabel
    global dogui

    set ilocation -1
    set iindex [array startsearch dacscalelabel]
    while { [array anymore dacscalelabel $iindex] } {
	set inextdac [array nextelement dacscalelabel $iindex]
	if {$dacscalelabel($inextdac) == $dacname} {
#	    puts "FOUND match in $dacname as position $inextdac"
	    set ilocation $inextdac
	} 
    }
    array donesearch dacscalelabel $iindex
    if {$ilocation < 0} {
	puts "ERROR in script, dac named $dacname not found"
	return "Error"
    }

    set dacaddress $ilocation

    set dacarray($dacaddress) $dacvalue

#    updatedacslide $dacaddress $dacresolution($dacaddress) $dacrmin($dacaddress) $dacrmax($dacaddress)
#    puts "UPS: $dacaddress $dacresolution($dacaddress) $dacrmin($dacaddress) $dacrmax($dacaddress) $dacvalue"

    pix_transfer_byte "0x08" $dacaddress $dacvalue 0 0 0 0

    edstest

    if {$dogui != 1} { return }

# now, psi46 only...
    switch $dacaddress {
	2 { .windac.col1.f1.d1.s1 set [ getdacfloat 2 800 1300 256 ] }
	3 { .windac.col1.f1.d2.s1 set [ getdacfloat 3 1000 2100 256 ] }
	4 { .windac.col1.f1.d3.s1 set [ getdacfloat 4 1800 2100 16 ] }
	1 { .windac.col1.f1.d4.s1 set [ getdacfloat 1 1700 2100 16 ] }
	5 { .windac.col1.f2.d1.s1 set [ getdacfloat 5 -700 0 256 ] }
	6 { .windac.col1.f2.d2.s1 set [ getdacfloat 6 0.0 500.0 16 ] }
	7 { .windac.col1.f2.d3.s1 set [ getdacfloat 7 500 1300 256 ] }
	8 { .windac.col1.f2.d4.s1 set [ getdacfloat 8 0 500 16 ] }
	9 { .windac.col1.f2.d5.s1 set [ getdacfloat 9 500 1300 256 ] }
	10 { .windac.col1.f2.d6.s1 set [ getdacfloat 10 -1500 -500 256 ] }
	11 { .windac.col1.f2.d7.s1 set [ getdacfloat 11 -1150 -400 256 ] }
	12 { .windac.col1.f2.d8.s1 set [ getdacfloat 12 -1500 -600 256 ] }
	13 { .windac.col2.f3.d1.s1 set [ getdacfloat 13 0 12 256 ] }
	14 { .windac.col2.f3.d2.s1 set [ getdacfloat 14 0 50 16 ] }
	15 { .windac.col2.f3.d3.s1 set [ getdacfloat 15 1000 1500 256 ] }
	16 { .windac.col2.f3.d4.s1 set [ getdacfloat 16 0 20 256 ] }
	17 { .windac.col2.f3.d5.s1 set [ getdacfloat 17 1000 1500 256 ] }
	18 { .windac.col2.f3.d6.s1 set [ getdacfloat 18 0 100 256 ] }
	19 { .windac.col2.f3.d7.s1 set [ getdacfloat 19 0 30 256 ] }
	20 { .windac.col2.f3.d8.s1 set [ getdacfloat 20 0 20 256 ] }
	21 { .windac.col2.f3.d9.s1 set [ getdacfloat 21 0 20 256 ] }
	22 { .windac.col3.f4.d1.s1 set [ getdacfloat 22 0 200 256 ] }
	23 { .windac.col3.f4.d2.s1 set [ getdacfloat 23 0 70 256 ] }
	24 { .windac.col3.f4.d3.s1 set [ getdacfloat 24 0 150 256 ] }
	25 { .windac.col3.f5.d1.s1 set [ getdacfloat 25 0 1800 256 ] }
	26 { .windac.col3.f5.d2.s1 set [ getdacfloat 26 55 205 256 ] }
	27 { .windac.col3.f5.d3.s1 set [ getdacfloat 27 -50 80 16 ] }
    }
#    puts "SN: set dac to $dacvalue"
}	
proc reassertpix { command } {
    global pixarray
    global maskbitstate
    global calibrationon
    global resetcalibration
    global resetstatus
    global pixarraycal
    global pixarrayenabled
    global pixarraytrim
    global trim
    global maxcols
    global maxrows
    global dogui

# now iterate over each col and row
    set icol 0
    while { $icol < $maxcols } {
	set irow 0
	while { $irow < $maxrows } {
	    set aindex2 "$icol $irow"
	    set mouseovertrim $pixarraytrim($aindex2) 
	    set mouseovercal $pixarraycal($aindex2) 
	    set mouseoverenabled $pixarrayenabled($aindex2) 

	    if {$dogui == 1} { .win0.fr1.can1 itemconfigure $pixarray($aindex2) \
				  -fill SkyBlue }
#	    update

	    if {$mouseovertrim=="Unknown"} {
		set mouseovertrim 0
	    }
	    if {($mouseovercal==1) && ($mouseoverenabled==1)} {
		pix_transfer_byte "0x44" $icol $irow 1 0 $mouseovertrim 0
		pix_transfer_byte "0x45" $icol $irow 1 $mouseovercal \
		    $mouseovertrim 0
		if {$dogui == 1} { .win0.fr1.can1 itemconfigure $pixarray($aindex2) \
				       -fill red }
	    }
	    if {($mouseovercal==1) && ($mouseoverenabled==0)} {
		pix_transfer_byte "0x44" $icol $irow 0 0 $mouseovertrim 0
		pix_transfer_byte "0x45" $icol $irow 0 $mouseovercal \
		    $mouseovertrim 0
		if {$dogui == 1} { .win0.fr1.can1 itemconfigure $pixarray($aindex2) \
				       -fill pink }
	    }
	    if {($mouseovercal==0) && ($mouseoverenabled==0)} {
		pix_transfer_byte "0x44" $icol $irow 0 0 $mouseovertrim 0
		if {$dogui == 1} { .win0.fr1.can1 itemconfigure $pixarray($aindex2) \
				       -fill darkgray }
	    }
	    if {($mouseovercal==0) && ($mouseoverenabled==1)} {
		pix_transfer_byte "0x44" $icol $irow 1 0 $mouseovertrim 0
		if {$dogui == 1} { .win0.fr1.can1 itemconfigure $pixarray($aindex2) \
				       -fill [trimcolor $mouseovertrim] }
	    }
#	    update
	    incr irow
	}
	if {$command==1} {
	    gauge_value .pixload.g [expr 100*($icol/$maxcols.0) ]
	} else {
	    gauge_value .pixput.g [expr 100*($icol/$maxcols.0) ]
	}

	incr icol
    }

}

proc resetchip { command } {
    global ctrl_reg
    global wbc
    global pixreadoutspeed
    global maskbitstate
    global trim
    global calibrationon

    global resetstatus
    global resetcalibration

    global col
    global row
    global trim
    global roctype

    global slowfecchannel

    toplevel .winreset -class Dialog 
    wm title .winreset "ROC Init"

    frame .winreset.row1
    frame .winreset.row2
    frame .winreset.row3
    frame .winreset.row4
    frame .winreset.row5
    frame .winreset.row9

    frame .winreset.row1.f1
    if {$roctype == "psi46"} {
	set tempstring "PSI46"
    } else {
	set tempstring "?????"
    }
    label .winreset.row1.f1.label -text "Reset $tempstring Chip:" \
	-foreground "blue" \
	-padx 10 -font { Helvetica 18 {bold} } -fg "black"

    pack .winreset.row1.f1.label

    frame .winreset.row2.f1
    label .winreset.row2.f1.l1 \
	-text "Click Initialize to Reset\nthe entire chip.\n\nAll pixels will be\nset to the values below."
    pack .winreset.row2.f1.l1

    frame .winreset.row3.f1
    if {$roctype == "psi46"} {
    scale .winreset.row3.f1.s1 -from 0 -to 15 \
	 -label "Trim Setting:" \
	 -tickinterval 5 \
    -length 100 -orient horizontal -command "updateresettrimslide"
    } else {
    scale .winreset.row3.f1.s1 -from 0 -to 7 \
	 -label "Trim Setting:" \
    -length 100 -orient horizontal -tickinterval 1 -command "updateresettrimslide"
    }

    pack .winreset.row3.f1.s1

    frame .winreset.row3.f2

    label .winreset.row3.f2.s0 -text "Apply to:"
    tk_optionMenu .winreset.row3.f2.option1 resetentirechip \
	EntireChip SingleColumn 1st8ColumnCOMP 1st3ColumnsNOCOMP

    scale .winreset.row3.f2.s1 -from 0 -to 51 \
	 -label "Column to reset:" \
	 -tickinterval 20 \
    -length 100 -orient horizontal -command "updatecolumnselector"

    pack .winreset.row3.f2.s0 .winreset.row3.f2.option1 .winreset.row3.f2.s1 -side left

##
    frame .winreset.row4.f1
    label .winreset.row4.f1.label -text "Compression:"
    tk_optionMenu .winreset.row4.f1.option1 resetcompression \
	ON OFF
    set resetcompression ON
    pack .winreset.row4.f1.label .winreset.row4.f1.option1 -side left

##
    frame .winreset.row5.f1
    label .winreset.row5.f1.label -text "Pixels:"
    tk_optionMenu .winreset.row5.f1.option1 resetstatus \
	ENABLED DISABLED
    pack .winreset.row5.f1.label .winreset.row5.f1.option1 -side left
    set resetstatus DISABLED

    frame .winreset.row9.f1
    button .winreset.row9.f1.button1 -text "\nInitialize\n" -command {
#	resetiterate 1

	# calib always set to 0 to do a clrcal
	set resetcalibration 0

	ClrCal 1

# call new max compressed routine
	if {$resetcompression == "ON"} {
	    puts "COMPRESSION ON*******************************************"
# Depending on which fec channel we're using, set the mode to max compression
	    if {$slowfecchannel == "1"} {
		pcifeccmdreg 4 1
		pcifeccmdreg 5 1
	    } else {
		pcifeccmdreg 20 1
		pcifeccmdreg 21 1
	    }
	    resetiterate 3
	    # Now turn OFF max copression mode
	    if {$slowfecchannel == "1"} {
##9-06		pcifeccmdreg 4 0
##9-06		pcifeccmdreg 5 0
		pcifeccmdreg 4 0
		pcifeccmdreg 5 0
		puts "Completed TX"
	    } else {
#		pcifeccmdreg 20 0
#		pcifeccmdreg 21 0
		pcifeccmdreg 20 0
		pcifeccmdreg 21 0
	    }
	    
	} else {
	    puts "COMPRESSION NONE-----------------------------------------"
#not max compression
# Depending on which fec channel we're using, set the mode
	    if {$slowfecchannel == "1"} {
		pcifeccmdreg 4 1
		pcifeccmdreg 5 0
	    } else {
		pcifeccmdreg 20 1
		pcifeccmdreg 21 0
	    }
	    # call new colx80 compressed routine
	    resetiterate 4

	    # Now turn OFF mode
	    if {$slowfecchannel == "1"} {
#sept6		pcifeccmdreg 4 0
		pcifeccmdreg 4 0
	    } else {
#sept6		pcifeccmdreg 20 0
		pcifeccmdreg 20 0
	    }
	}


	clrcalintersections 1
	set calrowlist ""
	set calcollist ""

	destroy .winreset
    }

#old	resetiterate 2
#old	clrcalintersections 1
#old	set calrowlist ""
#old	set calcollist ""
#old	ClrCal 1


    pack .winreset.row9.f1.button1 -side left

    pack .winreset.row1.f1
    pack .winreset.row2.f1
    pack .winreset.row3.f1
    pack .winreset.row3.f2
    pack .winreset.row4.f1
    pack .winreset.row5.f1
    pack .winreset.row9.f1

    pack .winreset.row1
    pack .winreset.row2
    pack .winreset.row3
    pack .winreset.row4
    pack .winreset.row5
    pack .winreset.row9


    set width [winfo screenwidth .]
    set height [winfo screenheight .]
    set x [expr ($width/2) - 200]
    set y [expr ($height/2) - 200]

    wm geometry .winreset  +$x+$y
    wm resizable .winreset 0 0
}    

proc dacsettings { command } {
    global roctype
	dacsettings46 $command
}


proc sendpcifectriggerregister { command } {
    global calibratelatency
    global verbose3
	set temphigh [expr $calibratelatency/16]
	set temphighbits [inttobits $temphigh 4]
	set temphigh [expr $temphigh*16]
	set templow [expr $calibratelatency - $temphigh]
	set templowbits [inttobits $templow 4]
    if {$verbose3 > 0} { puts "******* temphigh,templow: $temphigh,$templow   temphighbits,templowbits: $temphighbits,$templowbits" }
	callatencysendbyte2 $temphighbits $templowbits $command
# command: bit position to set (inject) this one time
}

proc dacsettings46 { command } {
    global ctrl_reg
    global wbc
    global pixreadoutspeed
    global dacarray
    global pixchipaddress
    global callatencybit
    global FECSDA
    global pixcalrange

    toplevel .windac -class Dialog 
    wm title .windac "PSI46 Pixel Chip DAC Settings"

    frame .windac.col1
    frame .windac.col2
    frame .windac.col3

    frame .windac.col1.f1

    setdacdefaults 1

    frame .windac.col3.f7

    button .windac.col3.f7.button0 -text "Set DAC defaults" -command {
	setdacdefaults 1
	.windac.col1.f1.d1.s1 set [ getdacfloat 2 800 1300 256 ]
	.windac.col1.f1.d2.s1 set [ getdacfloat 3 1000 2100 256 ]
	.windac.col1.f1.d3.s1 set [ getdacfloat 4 1800 2100 16 ]
	.windac.col1.f1.d4.s1 set [ getdacfloat 1 1700 2100 16 ]
	.windac.col1.f2.d1.s1 set [ getdacfloat 5 -700 0 256 ]
	.windac.col1.f2.d2.s1 set [ getdacfloat 6 0.0 500.0 16 ]
	.windac.col1.f2.d3.s1 set [ getdacfloat 7 500 1300 256 ]
	.windac.col1.f2.d4.s1 set [ getdacfloat 8 0 500 16 ]
	.windac.col1.f2.d5.s1 set [ getdacfloat 9 500 1300 256 ]
	.windac.col1.f2.d6.s1 set [ getdacfloat 10 -1500 -500 256 ]
	.windac.col1.f2.d7.s1 set [ getdacfloat 11 -1150 -400 256 ]
	.windac.col1.f2.d8.s1 set [ getdacfloat 12 -1500 -600 256 ]
	.windac.col2.f3.d1.s1 set [ getdacfloat 13 0 12 256 ]
	.windac.col2.f3.d2.s1 set [ getdacfloat 14 0 50 16 ]
	.windac.col2.f3.d3.s1 set [ getdacfloat 15 1000 1500 256 ]
	.windac.col2.f3.d4.s1 set [ getdacfloat 16 0 20 256 ]
	.windac.col2.f3.d5.s1 set [ getdacfloat 17 1000 1500 256 ]
	.windac.col2.f3.d6.s1 set [ getdacfloat 18 0 100 256 ]
	.windac.col2.f3.d7.s1 set [ getdacfloat 19 0 30 256 ]
	.windac.col2.f3.d8.s1 set [ getdacfloat 20 0 20 256 ]
	.windac.col2.f3.d9.s1 set [ getdacfloat 21 0 20 256 ]
	.windac.col3.f4.d1.s1 set [ getdacfloat 22 0 200 256 ]
	.windac.col3.f4.d2.s1 set [ getdacfloat 23 0 70 256 ]
	.windac.col3.f4.d3.s1 set [ getdacfloat 24 0 150 256 ]
	.windac.col3.f5.d1.s1 set [ getdacfloat 25 0 1800 256 ]
	.windac.col3.f5.d2.s1 set [ getdacfloat 26 55 205 256 ]
	.windac.col3.f5.d3.s1 set [ getdacfloat 27 -50 80 16 ]
	if {$callatencybit == "enabled"} {
	    set temp1 "1"
	} else {
	    set temp1 "0"
	}
	set sdatemp "000"
	set temp2 "100$temp1$sdatemp"
	sendbytewparity $temp2
    }
#	parray dacarray


    button .windac.col3.f7.button2 -text "Load DACs from file" -command {
	filegetdacdefaults 1
	.windac.col1.f1.d1.s1 set [ getdacfloat 2 800 1300 256 ]
	.windac.col1.f1.d2.s1 set [ getdacfloat 3 1000 2100 256 ]
	.windac.col1.f1.d3.s1 set [ getdacfloat 4 1800 2100 16 ]
	.windac.col1.f1.d4.s1 set [ getdacfloat 1 1700 2100 16 ]
	.windac.col1.f2.d1.s1 set [ getdacfloat 5 -700 0 256 ]
	.windac.col1.f2.d2.s1 set [ getdacfloat 6 0.0 500.0 16 ]
	.windac.col1.f2.d3.s1 set [ getdacfloat 7 500 1300 256 ]
	.windac.col1.f2.d4.s1 set [ getdacfloat 8 0 500 16 ]
	.windac.col1.f2.d5.s1 set [ getdacfloat 9 500 1300 256 ]
	.windac.col1.f2.d6.s1 set [ getdacfloat 10 -1500 -500 256 ]
	.windac.col1.f2.d7.s1 set [ getdacfloat 11 -1150 -400 256 ]
	.windac.col1.f2.d8.s1 set [ getdacfloat 12 -1500 -600 256 ]
	.windac.col2.f3.d1.s1 set [ getdacfloat 13 0 12 256 ]
	.windac.col2.f3.d2.s1 set [ getdacfloat 14 0 50 16 ]
	.windac.col2.f3.d3.s1 set [ getdacfloat 15 1000 1500 256 ]
	.windac.col2.f3.d4.s1 set [ getdacfloat 16 0 20 256 ]
	.windac.col2.f3.d5.s1 set [ getdacfloat 17 1000 1500 256 ]
	.windac.col2.f3.d6.s1 set [ getdacfloat 18 0 100 256 ]
	.windac.col2.f3.d7.s1 set [ getdacfloat 19 0 30 256 ]
	.windac.col2.f3.d8.s1 set [ getdacfloat 20 0 20 256 ]
	.windac.col2.f3.d9.s1 set [ getdacfloat 21 0 20 256 ]
	.windac.col3.f4.d1.s1 set [ getdacfloat 22 0 200 256 ]
	.windac.col3.f4.d2.s1 set [ getdacfloat 23 0 70 256 ]
	.windac.col3.f4.d3.s1 set [ getdacfloat 24 0 150 256 ]
	.windac.col3.f5.d1.s1 set [ getdacfloat 25 0 1800 256 ]
	.windac.col3.f5.d2.s1 set [ getdacfloat 26 55 205 256 ]
	.windac.col3.f5.d3.s1 set [ getdacfloat 27 -50 80 16 ]
	if {$callatencybit == "enabled"} {
	    set temp1 "1"
	} else {
	    set temp1 "0"
	}
	set sdatemp "000"
	set temp2 "100$temp1$sdatemp"
	sendbytewparity $temp2
#	parray dacarray
    }
    button .windac.col3.f7.button3 -text "Save DACs to file" -command {
	fileputdacdefaults 1
    }
    
    label .windac.col1.f1.label -text Supply: -foreground "blue"

    frame .windac.col1.f1.d1 -relief groove -borderwidth 4

    set vresolution 1
    scale .windac.col1.f1.d1.s1 -from 0 -to 255 -resolution $vresolution \
	    -digits 3 -label "Vana"  -width 12 \
	    -length 250 -orient horizontal \
	    -command { updatedacslide 2 256 800 1300 }
    scaleinfo 2 .windac.col1.f1.d1.s1 "Vana" 256 800 1300 "mV" 0
#	.windac.col1.f1.d1.s1 configure -activebackground "Red"
    bind .windac.col1.f1.d1.s1 <ButtonRelease> {
	pix_transfer_byte "0x08" 2 $dacarray(2) 0 0 0 0
	.windac.col1.f1.d1.s1 configure -troughcolor "DarkGray"
	.windac.col1.f1.d1.s1 configure -activebackground "Green"
	update
#	after 10
	.windac.col1.f1.d1.s1 configure -activebackground "Yellow"
    }



#-command "updateVa"
    
    frame .windac.col1.f1.d2 -relief groove -borderwidth 4
    scale .windac.col1.f1.d2.s1 -from 0 -to 255 -resolution $vresolution \
	    -digits 3 -label "Vsf"  -width 12 \
	    -length 250 -orient horizontal \
	    -command { updatedacslide 3 256 1000 2100  }
    scaleinfo 3 .windac.col1.f1.d2.s1 "Vsf" 256 1000 2100 "mV" 0
    bind .windac.col1.f1.d2.s1 <ButtonRelease> {
	pix_transfer_byte "0x08" 3 $dacarray(3) 0 0 0 0
    }
    bind .windac.col1.f1.d2.s1 <ButtonRelease> {
	pix_transfer_byte "0x08" 3 $dacarray(3) 0 0 0 0
	.windac.col1.f1.d2.s1 configure -troughcolor "DarkGray"
	.windac.col1.f1.d2.s1 configure -activebackground "Green"
	update
#	after 10
	.windac.col1.f1.d2.s1 configure -activebackground "Yellow"
    }

# -command "updateVsf"
    frame .windac.col1.f1.d3 -relief groove -borderwidth 4
    scale .windac.col1.f1.d3.s1 -from 0 -to 15 -resolution $vresolution \
	    -digits 2 -label "Vcomp"  -width 12 \
	    -length 250 -orient horizontal \
	    -command { updatedacslide 4 16 1800 2100 }
    scaleinfo 4 .windac.col1.f1.d3.s1 "Vcomp" 16 1800 2100 "mV" 0
    bind .windac.col1.f1.d3.s1 <ButtonRelease> {
	pix_transfer_byte "0x08" 4 $dacarray(4) 0 0 0 0
	.windac.col1.f1.d3.s1 configure -troughcolor "DarkGray"
	.windac.col1.f1.d3.s1 configure -activebackground "Green"
	update
#	after 10
	.windac.col1.f1.d3.s1 configure -activebackground "Yellow"
    }
# -command "updateVsf"
    frame .windac.col1.f1.d4 -relief groove -borderwidth 4
    scale .windac.col1.f1.d4.s1 -from 0 -to 15 -resolution $vresolution \
	    -digits 2 -label "Vdd"  -width 12 \
	    -length 250 -orient horizontal \
	    -command { updatedacslide 1 16 1700 2100 }
    scaleinfo 1 .windac.col1.f1.d4.s1 "Vdd" 16 1700 2100 "mV" 0
    bind .windac.col1.f1.d4.s1 <ButtonRelease> {
	pix_transfer_byte "0x08" 1 $dacarray(1) 0 0 0 0
	.windac.col1.f1.d4.s1 configure -troughcolor "DarkGray"
	.windac.col1.f1.d4.s1 configure -activebackground "Green"
	update
#	after 10
	.windac.col1.f1.d4.s1 configure -activebackground "Yellow"
    }
# -command "updateVsf"

    

#   -length 250 -orient horizontal -tickinterval 50 -command "updateslide1"
    .windac.col1.f1.d1.s1 set [ getdacfloat 2 800 1300 256 ]
    .windac.col1.f1.d2.s1 set [ getdacfloat 3 1000 2100 256 ]
    .windac.col1.f1.d3.s1 set [ getdacfloat 4 1800 2100 16 ]
    .windac.col1.f1.d4.s1 set [ getdacfloat 1 1700 2100 16 ]
    

    frame .windac.col1.f2
    label .windac.col1.f2.label -text "Analog PUC:" -foreground "blue"

    frame .windac.col1.f2.d1 -relief groove -borderwidth 4
    scale .windac.col1.f2.d1.s1 -from 0 -to 255 -resolution $vresolution \
	    -digits 3 -label "Vleak_comp"  -width 12 \
	    -length 250 -orient horizontal \
	    -command { updatedacslide 5 256 -700 0 }
    scaleinfo 5 .windac.col1.f2.d1.s1 "Vleak_comp" 256 -700 0 "mV" 0
    bind .windac.col1.f2.d1.s1 <ButtonRelease> {
	pix_transfer_byte "0x08" 5 $dacarray(5) 0 0 0 0
	.windac.col1.f2.d1.s1 configure -troughcolor "DarkGray"
	.windac.col1.f2.d1.s1 configure -activebackground "Green"
	update
#	after 10
	.windac.col1.f2.d1.s1 configure -activebackground "Yellow"
    }
#-command "updateVa"
    
    frame .windac.col1.f2.d2 -relief groove -borderwidth 4
    scale .windac.col1.f2.d2.s1 -from 0 -to 15 -resolution $vresolution \
	    -digits 2 -label "VrgPr"  -width 12 \
	    -length 250 -orient horizontal \
	    -command { updatedacslide 6 16 0.0 500.0 }
    scaleinfo 6 .windac.col1.f2.d2.s1 "VrgPr" 16 0.0 500.0 "mV" 0
    bind .windac.col1.f2.d2.s1 <ButtonRelease> {
	pix_transfer_byte "0x08" 6 $dacarray(6) 0 0 0 0
	.windac.col1.f2.d2.s1 configure -troughcolor "DarkGray"
	.windac.col1.f2.d2.s1 configure -activebackground "Green"
	update
#	after 10
	.windac.col1.f2.d2.s1 configure -activebackground "Yellow"
    }
# -command "updateVsf"
    frame .windac.col1.f2.d3 -relief groove -borderwidth 4
    scale .windac.col1.f2.d3.s1 -from 0 -to 255 -resolution $vresolution \
	    -digits 3 -label "VwllPr"  -width 12 \
	    -length 250 -orient horizontal \
	    -command { updatedacslide 7 256 500 1300 }
    scaleinfo 7 .windac.col1.f2.d3.s1 "VwllPr" 256 500 1300 "mV" 0
    bind .windac.col1.f2.d3.s1 <ButtonRelease> {
	pix_transfer_byte "0x08" 7 $dacarray(7) 0 0 0 0
	.windac.col1.f2.d3.s1 configure -troughcolor "DarkGray"
	.windac.col1.f2.d3.s1 configure -activebackground "Green"
	update
#	after 10
	.windac.col1.f2.d3.s1 configure -activebackground "Yellow"
    }
# -command "updateVsf"
    frame .windac.col1.f2.d4 -relief groove -borderwidth 4
    scale .windac.col1.f2.d4.s1 -from 0 -to 15 -resolution $vresolution \
	    -digits 2 -label "VrgSh"  -width 12 \
	    -length 250 -orient horizontal \
	    -command { updatedacslide 8 16 0 500 }
    scaleinfo 8  .windac.col1.f2.d4.s1 "VrgSh" 16 0 500 "mV" 0
    bind .windac.col1.f2.d4.s1 <ButtonRelease> {
	pix_transfer_byte "0x08" 8 $dacarray(8) 0 0 0 0
	.windac.col1.f2.d4.s1 configure -troughcolor "DarkGray"
	.windac.col1.f2.d4.s1 configure -activebackground "Green"
	update
#	after 10
	.windac.col1.f2.d4.s1 configure -activebackground "Yellow"
    }
# -command "updateVsf"
    frame .windac.col1.f2.d5 -relief groove -borderwidth 4
    scale .windac.col1.f2.d5.s1 -from 0 -to 255 -resolution $vresolution \
	    -digits 3 -label "VwllSh"  -width 12 \
	    -length 250 -orient horizontal \
	    -command { updatedacslide 9 256 500 1300 }
    scaleinfo 9  .windac.col1.f2.d5.s1 "VwllSh" 256 500 1300 "mV" 0
    bind .windac.col1.f2.d5.s1 <ButtonRelease> {
	pix_transfer_byte "0x08" 9 $dacarray(9) 0 0 0 0
	.windac.col1.f2.d5.s1 configure -troughcolor "DarkGray"
	.windac.col1.f2.d5.s1 configure -activebackground "Green"
	update
#	after 10
	.windac.col1.f2.d5.s1 configure -activebackground "Yellow"
    }
# -command "updateVsf"
    frame .windac.col1.f2.d6 -relief groove -borderwidth 4
    scale .windac.col1.f2.d6.s1 -from 0 -to 255 -resolution $vresolution \
	    -digits 3 -label "VHldDel"  -width 12  \
	    -length 250 -orient horizontal \
	    -command { updatedacslide 10 256 -1500 -500 }
    scaleinfo 10  .windac.col1.f2.d6.s1 "VHldDel" 256 -1500 -500 "mV" 0
    bind .windac.col1.f2.d6.s1 <ButtonRelease> {
	pix_transfer_byte "0x08" 10 $dacarray(10) 0 0 0 0
	.windac.col1.f2.d6.s1 configure -troughcolor "DarkGray"
	.windac.col1.f2.d6.s1 configure -activebackground "Green"
	update
#	after 10
	.windac.col1.f2.d6.s1 configure -activebackground "Yellow"
    }
# -command "updateVsf"
    frame .windac.col1.f2.d7 -relief groove -borderwidth 4
    scale .windac.col1.f2.d7.s1 -from 0 -to 255 -resolution $vresolution \
	    -digits 3 -label "Vtrim"  -width 12 \
	    -length 250 -orient horizontal \
	    -command { updatedacslide 11 256 -1150 -400 }
    scaleinfo 11  .windac.col1.f2.d7.s1 "Vtrim" 256 -1150 -400 "mV" 0
    bind .windac.col1.f2.d7.s1 <ButtonRelease> {
	pix_transfer_byte "0x08" 11 $dacarray(11) 0 0 0 0
	.windac.col1.f2.d7.s1 configure -troughcolor "DarkGray"
	.windac.col1.f2.d7.s1 configure -activebackground "Green"
	update
#	after 10
	.windac.col1.f2.d7.s1 configure -activebackground "Yellow"
    }
    frame .windac.col1.f2.d8 -relief groove -borderwidth 4
#    set vresolution [ expr 1 ]
    scale .windac.col1.f2.d8.s1 -from 0 -to 255 -resolution $vresolution \
	    -digits 3 -label "VthrComp" -width 12 \
	    -length 250 -orient horizontal \
	    -command { updatedacslide 12 256 -1500 -600 }
    scaleinfo 12  .windac.col1.f2.d8.s1 "VthrComp" 256 -1500 -600 "mV" 0
    bind .windac.col1.f2.d8.s1 <ButtonRelease> {
	pix_transfer_byte "0x08" 12 $dacarray(12) 0 0 0 0
	.windac.col1.f2.d8.s1 configure -troughcolor "DarkGray"
	.windac.col1.f2.d8.s1 configure -activebackground "Green"
	update
#	after 10
	.windac.col1.f2.d8.s1 configure -activebackground "Yellow"
    }
# -command "updateVsf"
    .windac.col1.f2.d1.s1 set [ getdacfloat 5 -700 0 256 ]
    .windac.col1.f2.d2.s1 set [ getdacfloat 6 0.0 500.0 16 ]
    .windac.col1.f2.d3.s1 set [ getdacfloat 7 500 1300 256 ]
    .windac.col1.f2.d4.s1 set [ getdacfloat 8 0 500 16 ]
    .windac.col1.f2.d5.s1 set [ getdacfloat 9 500 1300 256 ]
    .windac.col1.f2.d6.s1 set [ getdacfloat 10 -1500 -500 256 ]
    .windac.col1.f2.d7.s1 set [ getdacfloat 11 -1150 -400 256 ]
    .windac.col1.f2.d8.s1 set [ getdacfloat 12 -1500 -600 256 ]


    frame .windac.col2.f3
    label .windac.col2.f3.label -text "Pixel Readout:" -foreground "blue"

    frame .windac.col2.f3.d1 -relief groove -borderwidth 4
    scale .windac.col2.f3.d1.s1 -from 0 -to 255 -resolution $vresolution \
	    -digits 2 -label "VIBias_Bus"   -width 12 \
	    -length 250 -orient horizontal \
	    -command { updatedacslide 13 256 0 12 }
    scaleinfo 13 .windac.col2.f3.d1.s1  "VIBias_Bus" 256 0 12 "uA" 1
    bind .windac.col2.f3.d1.s1 <ButtonRelease> {
	pix_transfer_byte "0x08" 13 $dacarray(13) 0 0 0 0
	.windac.col2.f3.d1.s1 configure -troughcolor "DarkGray"
	.windac.col2.f3.d1.s1 configure -activebackground "Green"
	update
#	after 10
	.windac.col2.f3.d1.s1 configure -activebackground "Yellow"
    }

    
    frame .windac.col2.f3.d2 -relief groove -borderwidth 4
    scale .windac.col2.f3.d2.s1 -from 0 -to 15 -resolution $vresolution \
	    -digits 2 -label "Vbias_sf"   -width 12 \
	    -length 250 -orient horizontal \
	    -command { updatedacslide 14 16 0 50 }
    scaleinfo 14  .windac.col2.f3.d2.s1 "Vbias_sf" 16 0 50 "uA" 0
    bind .windac.col2.f3.d2.s1 <ButtonRelease> {
	pix_transfer_byte "0x08" 14 $dacarray(14) 0 0 0 0
	.windac.col2.f3.d2.s1 configure -troughcolor "DarkGray"
	.windac.col2.f3.d2.s1 configure -activebackground "Green"
	update
#	after 10
	.windac.col2.f3.d2.s1 configure -activebackground "Yellow"
    }

    label .windac.col2.f3.label2 -text "Double Column Readout:" -foreground "blue"

    frame .windac.col2.f3.d3 -relief groove -borderwidth 4
    scale .windac.col2.f3.d3.s1 -from 0 -to 255 -resolution $vresolution \
	    -digits 3 -label "VoffsetOp"   -width 12 \
	    -length 250 -orient horizontal \
	    -command { updatedacslide 15 256 1000 1500 }
    scaleinfo 15 .windac.col2.f3.d3.s1  "VoffsetOp" 256 1000 1500 "mV" 0
    bind .windac.col2.f3.d3.s1 <ButtonRelease> {
	pix_transfer_byte "0x08" 15 $dacarray(15) 0 0 0 0
	.windac.col2.f3.d3.s1 configure -troughcolor "DarkGray"
	.windac.col2.f3.d3.s1 configure -activebackground "Green"
	update
#	after 10
	.windac.col2.f3.d3.s1 configure -activebackground "Yellow"
    }

    frame .windac.col2.f3.d4 -relief groove -borderwidth 4
    scale .windac.col2.f3.d4.s1 -from 0 -to 255 -resolution $vresolution \
	    -digits 3 -label "VIbiasOp"   -width 12 \
	    -length 250 -orient horizontal \
	    -command { updatedacslide 16 256 0 20 }
    scaleinfo 16 .windac.col2.f3.d4.s1  "VIbiasOp" 256 0 20 "uA" 1
    bind .windac.col2.f3.d4.s1 <ButtonRelease> {
	pix_transfer_byte "0x08" 16 $dacarray(16) 0 0 0 0
	.windac.col2.f3.d4.s1 configure -troughcolor "DarkGray"
	.windac.col2.f3.d4.s1 configure -activebackground "Green"
	update
#	after 10
	.windac.col2.f3.d4.s1 configure -activebackground "Yellow"
    }

    frame .windac.col2.f3.d5 -relief groove -borderwidth 4
    scale .windac.col2.f3.d5.s1 -from 0 -to 255 -resolution $vresolution \
	    -digits 3 -label "VoffsetRO"   -width 12 \
	    -length 250 -orient horizontal \
	    -command { updatedacslide 17 256 1000 1500 }
    scaleinfo  17 .windac.col2.f3.d5.s1 "VoffsetRO" 256 1000 1500 "mV" 1
    bind .windac.col2.f3.d5.s1 <ButtonRelease> {
	pix_transfer_byte "0x08" 17 $dacarray(17) 0 0 0 0
	.windac.col2.f3.d5.s1 configure -troughcolor "DarkGray"
	.windac.col2.f3.d5.s1 configure -activebackground "Green"
	update
#	after 10
	.windac.col2.f3.d5.s1 configure -activebackground "Yellow"
    }

    frame .windac.col2.f3.d6 -relief groove -borderwidth 4
    scale .windac.col2.f3.d6.s1 -from 0 -to 255 -resolution $vresolution \
	    -digits 3 -label "VIon"   -width 12 \
	    -length 250 -orient horizontal \
	    -command { updatedacslide 18 256 0 100 }
    scaleinfo  18 .windac.col2.f3.d6.s1 "VIon" 256 0 100 "uA" 1
    bind .windac.col2.f3.d6.s1 <ButtonRelease> {
	pix_transfer_byte "0x08" 18 $dacarray(18) 0 0 0 0
	.windac.col2.f3.d6.s1 configure -troughcolor "DarkGray"
	.windac.col2.f3.d6.s1 configure -activebackground "Green"
	update
#	after 10
	.windac.col2.f3.d6.s1 configure -activebackground "Yellow"
    }

    label .windac.col2.f3.label3 -text "Chip Readout:" -foreground "blue"

    frame .windac.col2.f3.d7 -relief groove -borderwidth 4
    scale .windac.col2.f3.d7.s1 -from 0 -to 255 -resolution $vresolution \
	    -digits 3 -label "VIbias_PH"   -width 12 \
	    -length 250 -orient horizontal \
	    -command { updatedacslide 19 256 0 30 }
    scaleinfo 19 .windac.col2.f3.d7.s1  "VIbias_PH" 256 0 30 "uA" 1
    bind .windac.col2.f3.d7.s1 <ButtonRelease> {
	pix_transfer_byte "0x08" 19 $dacarray(19) 0 0 0 0
	.windac.col2.f3.d7.s1 configure -troughcolor "DarkGray"
	.windac.col2.f3.d7.s1 configure -activebackground "Green"
	update
#	after 10
	.windac.col2.f3.d7.s1 configure -activebackground "Yellow"
    }

    frame .windac.col2.f3.d8 -relief groove -borderwidth 4
#    puts "dac35 vres: $vresolution"
    scale .windac.col2.f3.d8.s1 -from 0 -to 255 -resolution $vresolution \
	    -digits 3 -label "Ibias_DAC"   -width 12 \
	    -length 250 -orient horizontal \
	    -command { updatedacslide 20 256 0 20 }
    scaleinfo  20  .windac.col2.f3.d8.s1 "Ibias_DAC" 256 0 20 "uA" 1
    bind .windac.col2.f3.d8.s1 <ButtonRelease> {
	pix_transfer_byte "0x08" 20 $dacarray(20) 0 0 0 0
	.windac.col2.f3.d8.s1 configure -troughcolor "DarkGray"
	.windac.col2.f3.d8.s1 configure -activebackground "Green"
	update
#	after 10
	.windac.col2.f3.d8.s1 configure -activebackground "Yellow"
    }

    frame .windac.col2.f3.d9 -relief groove -borderwidth 4
#    puts "dac35 vres: $vresolution"
    scale .windac.col2.f3.d9.s1 -from 0 -to 255 -resolution $vresolution \
	    -digits 3 -label "VIbias_roc"   -width 12 \
	    -length 250 -orient horizontal \
	    -command { updatedacslide 21 256 0 20 }
    scaleinfo  21  .windac.col2.f3.d9.s1 "VIbias_roc" 256 0 20 "uA" 1
    bind .windac.col2.f3.d9.s1 <ButtonRelease> {
	pix_transfer_byte "0x08" 21 $dacarray(21) 0 0 0 0
	.windac.col2.f3.d9.s1 configure -troughcolor "DarkGray"
	.windac.col2.f3.d9.s1 configure -activebackground "Green"
	update
#	after 10
	.windac.col2.f3.d9.s1 configure -activebackground "Yellow"
    }


    

#   -length 250 -orient horizontal -tickinterval 50 -command "updateslide1"
    .windac.col2.f3.d1.s1 set [ getdacfloat 13 0 12 256 ]
    .windac.col2.f3.d2.s1 set [ getdacfloat 14 0 50 16 ]
    .windac.col2.f3.d3.s1 set [ getdacfloat 15 1000 1500 256 ]
    .windac.col2.f3.d4.s1 set [ getdacfloat 16 0 20 256 ]
    .windac.col2.f3.d5.s1 set [ getdacfloat 17 1000 1500 256 ]
    .windac.col2.f3.d6.s1 set [ getdacfloat 18 0 100 256 ]
    .windac.col2.f3.d7.s1 set [ getdacfloat 19 0 30 256 ]
    .windac.col2.f3.d8.s1 set [ getdacfloat 20 0 20 256 ]
    .windac.col2.f3.d9.s1 set [ getdacfloat 21 0 20 256 ]
    

    frame .windac.col3.f4
    label .windac.col3.f4.header -text "Chip ID: $pixchipaddress" \
	-foreground "blue" \
	-font { Helvetica 16 {bold italic} }
    label .windac.col3.f4.label -text "Fast Trigger:" -foreground "blue"

    frame .windac.col3.f4.d1 -relief groove -borderwidth 4
    scale .windac.col3.f4.d1.s1 -from 0 -to 255 -resolution $vresolution \
	    -digits 3 -label "VIColOr" -width 12 \
	    -length 250 -orient horizontal \
	    -command { updatedacslide 22 256 0 200 }
    scaleinfo  22  .windac.col3.f4.d1.s1 "VIColOr" 256 0 200 "uA" 1

    button .windac.col3.f4.d1.b1 -text "Set" -command {
	pix_transfer_byte "0x08" 22 $dacarray(22) 0 0 0 0
    }
    bind .windac.col3.f4.d1.s1 <ButtonRelease> {
	pix_transfer_byte "0x08" 22 $dacarray(22) 0 0 0 0
	.windac.col3.f4.d1.s1 configure -troughcolor "DarkGray"
	.windac.col3.f4.d1.s1 configure -activebackground "Green"
	update
#	after 10
	.windac.col3.f4.d1.s1 configure -activebackground "Yellow"
    }
    
    frame .windac.col3.f4.d2 -relief groove -borderwidth 4
    scale .windac.col3.f4.d2.s1 -from 0 -to 255 -resolution $vresolution \
	    -digits 3 -label "Vnpix" -width 12 \
	    -length 250 -orient horizontal \
	    -command { updatedacslide 23 256 0 70 }
    scaleinfo  23 .windac.col3.f4.d2.s1 "Vnpix" 256 0 70 "uA" 1

    button .windac.col3.f4.d2.b1 -text "Set" -command {
	pix_transfer_byte "0x08" 23 $dacarray(23) 0 0 0 0
    }
    bind .windac.col3.f4.d2.s1 <ButtonRelease> {
	pix_transfer_byte "0x08" 23 $dacarray(23) 0 0 0 0
	.windac.col3.f4.d2.s1 configure -troughcolor "DarkGray"
	.windac.col3.f4.d2.s1 configure -activebackground "Green"
	update
#	after 10
	.windac.col3.f4.d2.s1 configure -activebackground "Yellow"
    }

    frame .windac.col3.f4.d3 -relief groove -borderwidth 4
# Needed to change this to 4bit dac.  Discovered it was 4 not 8bit
# on 3-4-03 -- back again for psi46 per specs to 8bit
    scale .windac.col3.f4.d3.s1 -from 0 -to 255 -resolution $vresolution \
	    -digits 3 -label "VsumCol" -width 12 \
	    -length 250 -orient horizontal \
	    -command { updatedacslide 24 256 0 150 }
    scaleinfo  24 .windac.col3.f4.d3.s1 "VsumCol" 256 0 150 "uA" 1

    button .windac.col3.f4.d3.b1 -text "Set" -command {
	pix_transfer_byte "0x08" 24 $dacarray(24) 0 0 0 0
    }
    bind .windac.col3.f4.d3.s1 <ButtonRelease> {
	pix_transfer_byte "0x08" 24 $dacarray(24) 0 0 0 0
	.windac.col3.f4.d3.s1 configure -troughcolor "DarkGray"
	.windac.col3.f4.d3.s1 configure -activebackground "Green"
	update
#	after 10
	.windac.col3.f4.d3.s1 configure -activebackground "Yellow"
    }

    .windac.col3.f4.d1.s1 set [ getdacfloat 22 0 200 256 ]
    .windac.col3.f4.d2.s1 set [ getdacfloat 23 0 70 256 ]
    .windac.col3.f4.d3.s1 set [ getdacfloat 24 0 150 256 ]

    frame .windac.col3.f5
    label .windac.col3.f5.label -text "Others:" -foreground "blue"
    frame .windac.col3.f5.d1 -relief groove -borderwidth 4
    scale .windac.col3.f5.d1.s1 -from 0 -to 255 -resolution $vresolution \
	    -digits 3 -label "VCal" -width 12 \
	    -length 250 -orient horizontal \
	    -command { updatedacslide 25 256 0 1800 }
    scaleinfo  25 .windac.col3.f5.d1.s1  "VCal" 256 0 1800 "mV" 0

    button .windac.col3.f5.d1.b1 -text "Set" -command {
	pix_transfer_byte "0x08" 25 $dacarray(25) 0 0 0 0
    }
    bind .windac.col3.f5.d1.s1 <ButtonRelease> {
	pix_transfer_byte "0x08" 25 $dacarray(25) 0 0 0 0

	edstest
	.windac.col3.f5.d1.s1 configure -troughcolor "DarkGray"
	.windac.col3.f5.d1.s1 configure -activebackground "Green"
	update
#	after 10
	.windac.col3.f5.d1.s1 configure -activebackground "Yellow"
    }


    frame .windac.col3.f5.d11 -relief groove -borderwidth 4
    label .windac.col3.f5.d11.label -text "VCal Range:"
    button .windac.col3.f5.d11.button -text " 0 - 280mV  " -command {
	if {$pixcalrange == "0"} {
	    set pixcalrange "1"
	    dochangereadoutspeed 1
	    .windac.col3.f5.d11.button configure -text "0 - 1800mV"
	} else {
	    set pixcalrange "0"
	    dochangereadoutspeed 1
	    .windac.col3.f5.d11.button configure -text " 0 - 280mV"
	}
    }
    balloon_help .windac.col3.f5.d11 "For PSI46v2, sets range of calibrate signal(VCal)\nbetween 0-1800mV or 0-280mV.\nPush button to change range.
"
    
    frame .windac.col3.f5.d2 -relief groove -borderwidth 4
    scale .windac.col3.f5.d2.s1 -from 0 -to 255 -resolution $vresolution \
	    -digits 3 -label "CalDel" -width 12 \
	    -length 250 -orient horizontal \
	    -command { updatedacslide 26 256 55 205 }
    scaleinfo  26  .windac.col3.f5.d2.s1 "CalDel" 256 55 205 "nsec" 1
    button .windac.col3.f5.d2.b1 -text "Set" -command {
	pix_transfer_byte "0x08" 26 $dacarray(26) 0 0 0 0
    }
    bind .windac.col3.f5.d2.s1 <ButtonRelease> {
	pix_transfer_byte "0x08" 26 $dacarray(26) 0 0 0 0
	.windac.col3.f5.d2.s1 configure -troughcolor "DarkGray"
	.windac.col3.f5.d2.s1 configure -activebackground "Green"
	update
#	after 10
	.windac.col3.f5.d2.s1 configure -activebackground "Yellow"
    }

    frame .windac.col3.f5.d3 -relief groove -borderwidth 4
    scale .windac.col3.f5.d3.s1 -from 0 -to 15 -resolution $vresolution \
	    -digits 2 -label "Temp_Range" -width 12 \
	    -length 250 -orient horizontal \
	    -command { updatedacslide 27 16 -50 80 }
    scaleinfo  27  .windac.col3.f5.d3.s1 "Temp_Range" 16 -50 80 "C" 0
    button .windac.col3.f5.d3.b1 -text "Set" -command {
	pix_transfer_byte "0x08" 27 $dacarray(27) 0 0 0 0
    }
    bind .windac.col3.f5.d3.s1 <ButtonRelease> {
	pix_transfer_byte "0x08" 27 $dacarray(27) 0 0 0 0
	.windac.col3.f5.d3.s1 configure -troughcolor "DarkGray"
	.windac.col3.f5.d3.s1 configure -activebackground "Green"
	update
#	after 10
	.windac.col3.f5.d3.s1 configure -activebackground "Yellow"
    }

    .windac.col3.f5.d1.s1 set [ getdacfloat 25 0 1800 256 ]
    .windac.col3.f5.d2.s1 set [ getdacfloat 26 55 205 256 ]
    .windac.col3.f5.d3.s1 set [ getdacfloat 27 -50 80 16 ]
# -----------
    frame .windac.col3.f66
    label .windac.col3.f66.label -text "FEC Register:" -foreground "blue"
    frame .windac.col3.f66.d1 -relief groove -borderwidth 4
    label .windac.col3.f66.d1.l2 -text "Calibrate latency:" 
    entry .windac.col3.f66.d1.e2 -textvariable calibratelatency -width 3

    button .windac.col3.f66.d1.b1 -text "+" -command {
	incrcallatency .windac.col3.f66.d1.b1
    }
    balloon_help .windac.col3.f66.d1.b1 "Increment Calibrate Latency Value\nPush Set to set\nThis value is set in the FEC"
    button .windac.col3.f66.d1.b2 -text "-" -command {
	deccallatency .windac.col3.f66.d1.b2
    }
    balloon_help .windac.col3.f66.d1.b2 "Decrement Calibrate Latency Value\nPush Set to set\nThis value is set in the FEC"
    button .windac.col3.f66.d1.b01 -text "    Set     " -command {
	set temphigh [expr $calibratelatency/16]
	set temphighbits [inttobits $temphigh 4]
	set temphigh [expr $temphigh*16]
	set templow [expr $calibratelatency - $temphigh]
	set templowbits [inttobits $templow 4]
	puts "******* temphigh,templow: $temphigh,$templow   temphighbits,templowbits: $temphighbits,$templowbits"
	callatencysendbyte2 $temphighbits $templowbits -1
    }
    balloon_help  .windac.col3.f66.d1.b01 "Send Calibrate Latency to FEC"

    set tempbit "0"
    set callatencybit "enabled"
    button .windac.col3.f66.d1.option1 -text $callatencybit -command {
	if {$callatencybit == "enabled"} {
	    set callatencybit "disabled"
	} else {
	    set callatencybit "enabled"
	}
	if {$callatencybit == "enabled"} {
	    set temp1 "1"
	} else {
	    set temp1 "0"
	}
#	set sdatemp "000"
#	set temp2 "100$temp1$sdatemp"
	.windac.col3.f66.d1.option1 configure -text $callatencybit
#	sendbytewparity $temp2

	set temphigh [expr $calibratelatency/16]
	set temphighbits [inttobits $temphigh 4]
	set temphigh [expr $temphigh*16]
	set templow [expr $calibratelatency - $temphigh]
	set templowbits [inttobits $templow 4]
	puts "******* temphigh,templow: $temphigh,$templow   temphighbits,templowbits: $temphighbits,$templowbits"
	callatencysendbyte2 $temphighbits $templowbits -1
    }

    balloon_help .windac.col3.f66.d1.option1 "Enable or disable the calibrate latency value function in FEC"

# -----------
    frame .windac.col3.f6
    label .windac.col3.f6.label -text "Digital Register:" -foreground "blue"
    frame .windac.col3.f6.d1 -relief groove -borderwidth 4

    label .windac.col3.f6.d1.l01 -text "40MHz Readout Speed: "
    checkbutton .windac.col3.f6.d1.cb1 -variable pixreadoutspeed -onvalue "1" \
	    -offvalue "0"
    label .windac.col3.f6.d1.l02 -text "DAQ Stop: "
    checkbutton .windac.col3.f6.d1.cb2 -variable pixstopdaq -onvalue "0" \
	    -offvalue "1"

    button .windac.col3.f6.d1.b00 -text "Set ctrl_reg" -command {
	set temp3 0
	if { $pixreadoutspeed == "1" } {
	    set temp3 [ expr ($temp3 | 0x01) ]
	}
	if { $pixstopdaq == "0" } {
	    set temp3 [ expr ($temp3 | 0x02) ]
	}
	
#	pix_transfer_byte "0x08" 0x83 $temp3 0 0 0 0
	pix_transfer_byte "0x08" 131 $temp3 0 0 0 0
    }

    label .windac.col3.f6.d1.l2 -text "WBC (trigger latency):" 
    entry .windac.col3.f6.d1.e2 -textvariable wbc -width 3

    button .windac.col3.f6.d1.b1 -text "+" -command {
	incrwbc .windac.col3.f6.d1.b1
    }
    balloon_help .windac.col3.f6.d1.b1 "Increment WBC\nPush Set-WBC to set"
    button .windac.col3.f6.d1.b2 -text "-" -command {
	decwbc .windac.col3.f6.d1.b2
    }
    balloon_help .windac.col3.f6.d1.b2 "Decrement WBC\nPush Set-WBC to set"
    button .windac.col3.f6.d1.b01 -text "  Set WBC   " -command {
	pix_transfer_byte "0x08" 254 $wbc 0 0 0 0
	if {$verbose3=="1"} {
	    puts "SENDING ResetROC TO FEC"
	}
	set temp2 "0111000"
##npci	sendbytewparity $temp2
	set tt1 [wordtobits 00000001]
	set tt2 [bitwordset $tt1 7 $encalpulse]
	set tt2 [bitwordset $tt2 5 $fec_trigger_source]
	set tt2 [bitwordset $tt2 6 $norm_loop_on]
	set tt3 [bitstoword $tt2]
	sendpcifectriggerregister 0

#old	transfer_byte "00000100" "0x2" "write"    
    }
    balloon_help  .windac.col3.f6.d1.b01 "Send WBC and Reset-Inject"
    button .windac.col3.f6.d1.b02 -text "ROC Reset" -command {
	if {$verbose3=="1"} {
	    puts "SENDING ResetROC TO FEC"
	}
	set temp2 "0111000"
##npci	sendbytewparity $temp2
	set tt1 [wordtobits 00000001]
	set tt2 [bitwordset $tt1 7 $encalpulse]
	set tt2 [bitwordset $tt2 5 $fec_trigger_source]
	set tt2 [bitwordset $tt2 6 $norm_loop_on]
	set tt3 [bitstoword $tt2]
	sendpcifectriggerregister 0

#old	transfer_byte "00000100" "0x2" "write"    
    }
    balloon_help .windac.col3.f6.d1.b02 "Inject Reset via TBM"


    label .windac.col3.label -text " "

    button .windac.col3.button -text "Set displayed DAC values" -command { 
	enumdacs 1

# send callatencybit
	if {$callatencybit == "enabled"} {
	    set temp1 "1"
	} else {
	    set temp1 "0"
	}
	set sdatemp "000"
	set temp2 "100$temp1$sdatemp"
	.windac.col3.f66.d1.option1 configure -text $callatencybit
	sendbytewparity $temp2

	if {$callatencybit == "enabled"} {
	    set temphigh [expr $calibratelatency/16]
	    set temphighbits [inttobits $temphigh 4]
	    set temphigh [expr $temphigh*16]
	    set templow [expr $calibratelatency - $temphigh]
	    set templowbits [inttobits $templow 4]
	    callatencysendbyte2 $temphighbits $templowbits -1
	}

# set wbc
	pix_transfer_byte "0x08" 254 $wbc 0 0 0 0
# send ROC reset via TBM
	if {$verbose3=="1"} {
	    puts "SENDING ResetROC TO FEC"
	}
	set temp2 "0111000"
##npci	sendbytewparity $temp2
	set tt1 [wordtobits 00000001]
	set tt2 [bitwordset $tt1 7 $encalpulse]
	set tt2 [bitwordset $tt2 5 $fec_trigger_source]
	set tt2 [bitwordset $tt2 6 $norm_loop_on]
	set tt3 [bitstoword $tt2]
	sendpcifectriggerregister 0

#old	transfer_byte "00000100" "0x2" "write"    


##	destroy .windac
    }

    balloon_help  .windac.col3.button "Send the DAC settings\ndisplayed to\nthe Read Out Chip\nthen send callatency, wbc, and ROC Reset"

    pack .windac.col1.f1.label
    pack .windac.col1.f1.d1.s1 -side left
    pack .windac.col1.f1.d1
    pack .windac.col1.f1.d2.s1 -side left
    pack .windac.col1.f1.d2
    pack .windac.col1.f1.d3.s1 -side left
    pack .windac.col1.f1.d3
    pack .windac.col1.f1.d4.s1 -side left
    pack .windac.col1.f1.d4
    pack .windac.col1.f2.label
    pack .windac.col1.f2.d1.s1 -side left
    pack .windac.col1.f2.d1
    pack .windac.col1.f2.d2.s1 -side left
    pack .windac.col1.f2.d2
    pack .windac.col1.f2.d3.s1 -side left
    pack .windac.col1.f2.d3
    pack .windac.col1.f2.d4.s1 -side left
    pack .windac.col1.f2.d4
    pack .windac.col1.f2.d5.s1 -side left
    pack .windac.col1.f2.d5
    pack .windac.col1.f2.d6.s1 -side left
    pack .windac.col1.f2.d6
    pack .windac.col1.f2.d7.s1 -side left
    pack .windac.col1.f2.d7
    pack .windac.col1.f2.d8.s1 -side left
    pack .windac.col1.f2.d8

    pack .windac.col2.f3.label
    pack .windac.col2.f3.d1.s1 -side left
    pack .windac.col2.f3.d1
    pack .windac.col2.f3.d2.s1 -side left
    pack .windac.col2.f3.d2
    pack .windac.col2.f3.label2
    pack .windac.col2.f3.d3.s1 -side left
    pack .windac.col2.f3.d3
    pack .windac.col2.f3.d4.s1 -side left
    pack .windac.col2.f3.d4
    pack .windac.col2.f3.d5.s1 -side left
    pack .windac.col2.f3.d5
    pack .windac.col2.f3.d6.s1 -side left
    pack .windac.col2.f3.d6
    pack .windac.col2.f3.label3
    pack .windac.col2.f3.d7.s1 -side left
    pack .windac.col2.f3.d7
    pack .windac.col2.f3.d8.s1 -side left
    pack .windac.col2.f3.d8
    pack .windac.col2.f3.d9.s1 -side left
    pack .windac.col2.f3.d9

    pack .windac.col3.f4.header
    pack .windac.col3.f4.label
    pack .windac.col3.f4.d1.s1 -side left
    pack .windac.col3.f4.d1
    pack .windac.col3.f4.d2.s1 -side left
    pack .windac.col3.f4.d2
    pack .windac.col3.f4.d3.s1 -side left
    pack .windac.col3.f4.d3

    pack .windac.col3.f5.label
    pack .windac.col3.f5.d1.s1 -side left
    pack .windac.col3.f5.d1
    pack .windac.col3.f5.d11.label .windac.col3.f5.d11.button -side left
    pack .windac.col3.f5.d11
    pack .windac.col3.f5.d2.s1 -side left
    pack .windac.col3.f5.d2
    pack .windac.col3.f5.d3.s1 -side left
    pack .windac.col3.f5.d3

    pack .windac.col3.f6.d1.l2 
    pack .windac.col3.f6.d1.e2 .windac.col3.f6.d1.b1 .windac.col3.f6.d1.b2 -side left
    pack .windac.col3.f6.d1.b01 .windac.col3.f6.d1.b02
    pack .windac.col3.f6.label
    pack .windac.col3.f6.d1

    pack .windac.col3.f66.d1.l2 
    pack .windac.col3.f66.d1.e2 .windac.col3.f66.d1.b1 .windac.col3.f66.d1.b2 -side left
    pack .windac.col3.f66.d1.b01 .windac.col3.f66.d1.option1
#    pack .windac.col3.f66.d1.b01
    pack .windac.col3.f66.label
    pack .windac.col3.f66.d1

    pack .windac.col1.f1
    pack .windac.col1.f2

    pack .windac.col2.f3

    pack .windac.col3.f4
    pack .windac.col3.f5
    pack .windac.col3.f6 
    pack .windac.col3.f66
    pack .windac.col3.label
    pack .windac.col3.button

    pack .windac.col3.f7.button2 .windac.col3.f7.button3 .windac.col3.f7.button0
    pack .windac.col3.f7

    pack .windac.col1 .windac.col2 .windac.col3 -side left


    set width [winfo screenwidth .]
    set height [winfo screenheight .]
    set x [expr ($width/2) - 200]
    set y [expr ($height/2) - 200]

    wm geometry .windac  +$x+$y
    wm resizable .windac 0 0
    

}

 proc balloon_help {w msg} {
        bind $w <Enter> "after 1000 \"balloon_aux %W [list $msg]\""
        bind $w <Leave> "after cancel \"balloon_aux %W [list $msg]\"
                         after 100 {catch {destroy .balloon_help}}"
 }



proc balloon_aux  {wid msg} {
    global balloons

    if {$balloons=="ON"} {
	set t .balloon_help
	catch {destroy $t}
	toplevel $t
	wm overrideredirect $t 1
	pack [label $t.l -text $msg -relief groove \
		  -bd 1 -bg gold] -fill both
	set x [expr [winfo rootx $wid]+6+[winfo width $wid]/2]
	set y [expr [winfo rooty $wid]+6+[winfo height $wid]/2]
	wm geometry $t +$x\+$y
	bind $t <Enter> {after cancel {catch {destroy .balloon_help}}}
	bind $t <Leave> "catch {destroy .balloon_help}"
    }
}

    # demo
#    pack [button .b -text tryme -command {puts "you did it!"}]
#    balloon_help .b "Text that describes\nwhat the button does"




#---------------------------------------------------------------------
proc changestatebutton { mywin } {
    global dcolstate
    global roctype

    set temp $dcolstate($mywin)
    scan $mywin ".win0.fr1.b%d" var1
    set evencol [expr $var1*2]
    set oddcol [expr $var1*2+1]
    if {  $temp == 0 } {
# turn on dcol (turns green)
	set temp 1
	$mywin configure -bg green \
	    -activeforeground green \
	    -activebackground green \
	    -foreground black 
#	.win0.fr1.can1 itemconfigure "$evencol-back" -fill green
#	.win0.fr1.can1 itemconfigure "$oddcol-back" -fill green
	update
# Enable dcol in doublehit mode
	set dcolnotation [expr $var1*2]
	.win0.fr1.can1 itemconfigure "$dcolnotation-back" -fill blue
	pix_transfer_byte "0x25" $dcolnotation 64 128 0 0 0
    } elseif { $temp == 1 } {
# turn off dcol (gray it)	
	set temp 0
	$mywin configure -bg darkgray \
	    -activeforeground darkgray \
	    -activebackground darkgray \
	    -foreground black 
#	$mywin itemconfigure -fill gray
#	.win0.fr1.can1 itemconfigure "$evencol-back" -fill darkgray
#	.win0.fr1.can1 itemconfigure "$oddcol-back" -fill darkgray
	update
# Kill dcol
	set dcolnotation [expr $var1*2]
	.win0.fr1.can1 itemconfigure "$dcolnotation-back" -fill darkgray
	pix_transfer_byte "0x25" $dcolnotation 64 0 0 0 0
    }
	
    set dcolstate($mywin) $temp
    

}

proc trimcolor { iinteger } {
    global roctype
    set ireturn "\#bbbbbb"
    switch $iinteger {
	0 { set ireturn "\#d2ffd2" }
	1 { set ireturn "\#c0ffc0" }
	2 { set ireturn "\#aaffaa" }
	3 { set ireturn "\#a0ffa0" }
	4 { set ireturn "\#66ff66" }
	5 { set ireturn "\#55ff55" }
	6 { set ireturn "\#55ef55" }
	7 { set ireturn "\#44ef44" }
	8 { set ireturn "\#44df44" }
	9 { set ireturn "\#33df33" }
	10 { set ireturn "\#33cf33" }
	11 { set ireturn "\#22cf22" }
	12 { set ireturn "\#22bf22" }
	13 { set ireturn "\#11bf11" }
	14 { set ireturn "\#22a022" }
	15 { set ireturn "\#119011" }
	16 { set ireturn "gray" }
	17 { set ireturn "gray" }
	18 { set ireturn "white" }
	19 { set ireturn "white" }
    }
    return $ireturn
}

proc dochangereadoutspeed { command } {
    #This is a 10bit only command
    global pixreadoutspeed
    global textreadoutspeed
    global pixreadoutspeed
    global pixstopdaq
    global Readout40MHz
    global verbose2
    global roctype
    global pixcalrange

    if {$textreadoutspeed=="40MHz"} {
	set pixreadoutspeed 1
    } else {
	set pixreadoutspeed 0
    }

    if {$roctype == "psi46"} {
	set temp3 0
	if {$pixreadoutspeed != "1"} {
	    set temp3 [ expr ($temp3 | 0x01) ]
	}
	if {$pixstopdaq == "1"} {
	    set temp3 [ expr ($temp3 | 0x02) ]
	}
# added for PSI46v2, but control present in psi46v1, 1=0-1800mV,0=0-280mV
	if {$pixcalrange == "1"} {
	    set temp3 [ expr ($temp3 | 0x04) ]
	}
    }

    set Readout40MHz $pixreadoutspeed
    if {$Readout40MHz==1} {
	tbm2_write_f0 1
	if {$roctype == "psi46"} {
#	    pix_transfer_byte "0x08" 0xFD $temp3 0 0 0 0
	    pix_transfer_byte "0x08" 253 $temp3 0 0 0 0
	} else {
	    pix_transfer_byte "0x08" 0x83 $temp3 0 0 0 0
	}
	if {$verbose2==1} {
	    puts "System Readout speed changed to 40MHz."
	}
	.win0.fr2.g6.box1.f1.right.b2 configure -relief sunken \
	    -fg darkgreen -activeforeground darkgreen
	.win0.fr2.g6.box1.f1.right.b1 configure -relief raised \
	    -fg black -activeforeground black
    } else {
	if {$roctype == "psi46"} {
#	    pix_transfer_byte "0x08" 0xFD $temp3 0 0 0 0
	    pix_transfer_byte "0x08" 253 $temp3 0 0 0 0
	} else {
	    pix_transfer_byte "0x08" 0x83 $temp3 0 0 0 0
	}
	tbm2_write_f0 1
	if {$verbose2==1} {
	    puts "System Readout speed changed to 20MHz."
	}
	.win0.fr2.g6.box1.f1.right.b2 configure -relief raised \
	    -fg black -activeforeground black
	.win0.fr2.g6.box1.f1.right.b1 configure -relief sunken \
	    -fg darkgreen -activeforeground darkgreen
    }	

# update state of daq button because it is in the same word
    if {$pixstopdaq=="0"} {
	.win0.fr2.g6.box1.f4.right.b2 configure -relief raised \
	    -fg black -activeforeground black
	.win0.fr2.g6.box1.f4.right.b1 configure -relief sunken \
	    -fg darkgreen -activeforeground darkgreen
    } else {
	.win0.fr2.g6.box1.f4.right.b1 configure -relief raised \
	    -fg black -activeforeground black
	.win0.fr2.g6.box1.f4.right.b2 configure -relief sunken \
	    -fg darkgreen -activeforeground darkgreen
    }
    
}

proc doresetsystem { command } {
    global textreadoutspeed
    global pixreadoutspeed
    global InjectTBMReset
    global dcolstate
    global chiptouched
    global buttext
    global globalmodchipbutton
    global verbose2
    global maxcols

    if {$verbose2=="1"} {
	parray chiptouched
    }

    memsave $globalmodchipbutton
    set iline 1
    while {$iline <= 2} {
	set ibutton 1
	while {$ibutton <= 8} {
	    set buttext [expr (($iline-1)*8 + $ibutton)]
	    if {$buttext <= 16} {
#		gauge_value .modulesave.g [ expr 100*($buttext/16.0) ]
	    }
	    set iiline $iline
	    if {$chiptouched(.win0.fr2.g2.line$iiline.bb$ibutton)=="1"} {
		memload .win0.fr2.g2.line$iiline.bb$ibutton 1


# Reset block for currently loaded chip

		set savedreset $InjectTBMReset

		#Clear CAL
		ClrCal 1
		#disable columns
		set idcol 0
		set numdcols [expr $maxcols / 2]
		while {$idcol < $numdcols} {
		    # Kill dcol
		    # send 32=0x20 and 0x00 -> set kill bit for dcol
		    set dcolnotation [expr $idcol*2]
		    pix_transfer_byte "0x25" $dcolnotation 64 0 0 0 0
		    
		    .win0.fr1.can1 itemconfigure "$dcolnotation-back" -fill skyblue
		    .win0.fr1.b$idcol configure -bg skyblue \
			-activeforeground skyblue \
			-activebackground skyblue \
			-foreground skyblue
#may25		    update
#		    after 20
		    .win0.fr1.can1 itemconfigure "$dcolnotation-back" -fill darkgray
		    .win0.fr1.b$idcol configure -bg darkgray \
			-activeforeground darkgray \
			-activebackground darkgray \
			-foreground black 
		    
		    set dcolstate(.win0.fr1.b$idcol) 0
		    
		    incr idcol
#may25		    update
		}
		
		#TBM Reset
		set InjectTBMReset 1
		tbm2_write_f1 1
		set InjectTBMReset $savedreset
		# set speed
		dochangereadoutspeed 1

# Reset block for currently loaded chip

	    }
	    incr ibutton
	}
	incr iline
    }

#set readout speed to 20MHz initially
set textreadoutspeed 20MHz
dochangereadoutspeed 1 
# set mode to sync initially
set tbm2mode "Sync"
.win0.fr2.g6.box1.f6.right.b3 configure -relief raised \
    -fg black -activeforeground black
.win0.fr2.g6.box1.f6.right.b2 configure -relief raised \
    -fg black -activeforeground black
.win0.fr2.g6.box1.f6.right.b1 configure -relief sunken \
    -fg darkgreen -activeforeground darkgreen
tbm2_write_f2 1
#



}

proc mymouseover { mypixel } {
    global mouseovercol
    global mouseoverrow
    puts "MOUSEOVER: $mypixel"
    scan $mypixel "%d %d" mycol myrow
    set mouseovercol $mycol
    set mouseoverrow $myrow
#    .win0.fr2.line1.l2 -configure -text $mycol -font $fontname2
    bell
    update
}


proc disksavechip { command myx myy type } {
    # type is either flat or xml
    global dacarray
    global dacscale
    global dacscalelabel
    global wbc
    global fileidsave
    global pixchipaddress

    global fileidsave

    global chiptouched

    global chipcounter
    global roctype

    global currenttbm
    global pixportaddress
    global calibratelatency

    memload $command -1

    if {$type == "xml"} {
	sxml::writer::element $fileidsave "detector"

	if {$roctype == "psi46"} {
	    set tempstring "PSI46"
	} else {
	    set tempstring "?????"
	}

	sxml::writer::data $fileidsave "$tempstring Read Out Chip"
	sxml::writer::attribute $fileidsave "type" "$roctype"
	
	sxml::writer::element $fileidsave "location"
	sxml::writer::attribute $fileidsave "id" "$chipcounter"
	sxml::writer::attribute $fileidsave "x" "$myx"
	sxml::writer::attribute $fileidsave "y" "$myy"
	sxml::writer::data $fileidsave "$chipcounter"
	sxml::writer::pop_element $fileidsave

    }
    
    set iindex [array startsearch dacarray]
    while { [array anymore dacarray $iindex] } {
	set inextdac [array nextelement dacarray $iindex]
	    #	puts "dac: $inextdac should be set to integer value: $dacarray($inextdac)"
	if {$type == "flat"} {
	    puts $fileidsave "$inextdac $dacarray($inextdac)"
	} elseif {$type == "xml"} {
	    sxml::writer::element $fileidsave "dac"
	    sxml::writer::attribute \
		$fileidsave "name" "$dacscalelabel($inextdac)"
	    sxml::writer::attribute $fileidsave "value" "$dacarray($inextdac)"
	    sxml::writer::attribute $fileidsave "address" "$inextdac"
	    sxml::writer::data $fileidsave \
		"$dacscalelabel($inextdac)=$dacarray($inextdac)"
	    sxml::writer::pop_element $fileidsave
	}
	
    }
    array donesearch dacarray $iindex

    
    if {$type == "flat"} {
	# Write wbc	
	puts $fileidsave "130 $wbc"
	# Write calibratelatency
	puts $fileidsave "131 $calibratelatency"
	# Write tbm	
	puts $fileidsave "132 $currenttbm"
	# Write pixportaddress	
	puts $fileidsave "133 $pixportaddress"
	# Write pix chip address
	puts $fileidsave "999 $pixchipaddress"
    } elseif {$type == "xml"} {
	# Write calibratelatency
	sxml::writer::element $fileidsave "register"
	sxml::writer::attribute \
	    $fileidsave "name" "calibratelatency"
	sxml::writer::attribute \
	    $fileidsave "value" "$calibratelatency"
	sxml::writer::attribute \
	    $fileidsave "address" "131"
	sxml::writer::data $fileidsave \
	    "calibratelatency = $calibratelatency"
	sxml::writer::pop_element $fileidsave
	# Write pixportaddress
	sxml::writer::element $fileidsave "register"
	sxml::writer::attribute \
	    $fileidsave "name" "port"
	sxml::writer::attribute \
	    $fileidsave "value" "$pixportaddress"
	sxml::writer::attribute \
	    $fileidsave "address" "133"
	sxml::writer::data $fileidsave \
	    "port = $pixportaddress"
	sxml::writer::pop_element $fileidsave
	# Write currenttbm
	sxml::writer::element $fileidsave "register"
	sxml::writer::attribute \
	    $fileidsave "name" "tbm"
	sxml::writer::attribute \
	    $fileidsave "value" "$currenttbm"
	sxml::writer::attribute \
	    $fileidsave "address" "132"
	sxml::writer::data $fileidsave \
	    "tbm = $currenttbm"
	sxml::writer::pop_element $fileidsave
	# Write wbc	
	sxml::writer::element $fileidsave "register"
	sxml::writer::attribute \
	    $fileidsave "name" "wbc"
	sxml::writer::attribute \
	    $fileidsave "value" "$wbc"
	sxml::writer::attribute \
	    $fileidsave "address" "130"
	sxml::writer::data $fileidsave \
	    "wbc = $wbc"
	sxml::writer::pop_element $fileidsave
	# Write pix chip address
	sxml::writer::element $fileidsave "register"
	sxml::writer::attribute \
	    $fileidsave "name" "pixchipaddress"
	sxml::writer::attribute \
	    $fileidsave "value" "$pixchipaddress"
	sxml::writer::attribute \
	    $fileidsave "address" "0"
	sxml::writer::data $fileidsave \
	    "Pixel Chip Address=$pixchipaddress"
	sxml::writer::pop_element $fileidsave

	# Write a cosmo register (a hint for cosmo, not for the chip)
	sxml::writer::element $fileidsave "register"
	sxml::writer::attribute \
	    $fileidsave "name" "cosmostate"
	if {$chiptouched($command) == "1"} {
	    sxml::writer::attribute \
		$fileidsave "value" "1"
	    sxml::writer::data $fileidsave \
		"Included in module"
	} else {
	    sxml::writer::attribute \
		$fileidsave "value" "-1"
	    sxml::writer::data $fileidsave \
		"Not included in module"
	}
	sxml::writer::attribute \
	    $fileidsave "address" "0"
	sxml::writer::pop_element $fileidsave

    }

    # Write (10000:button(1..16)) (10001:line) (10002:row)
    # upon reading file the 10002 causes save of data in working "register"
    if {$type == "flat"} {
	puts $fileidsave "10000 $chipcounter\n10001 $myy\n10002 $myx"
    }

    # Set a flag (10003 only) so that button lights up blue
    if {$type == "flat"} {
	if {$chiptouched($command) == "1"} {
	    puts $fileidsave "10003 1"
	} else {
	    puts $fileidsave "10003 -1"
	}    
    }


    if {$type == "xml"} {
	sxml::writer::pop_element $fileidsave
    }

    
#    foreach idac $dac {
#	sxml::writer::element $myfd "dac"
#	sxml::writer::attribute $myfd "name" "$dacscalelabel($"
#	sxml::writer::attribute $myfd "value" "$wbc"
#	sxml::writer::pop_element $myfd
#    }
    incr chipcounter
}
proc diskloadchip { command myfilename } {

    global dacarray
    global dacscale
    global dacscalelabel

#    set dacscale($mydac) $myscalename
#    set dacscalelabel($mydac) $mylabel

    global pixarray
    global maskbitstate
    global calibrationon
    global resetcalibration
    global resetstatus
    global pixarraycal
    global pixarrayenabled
    global pixarraytrim
    global trim
    global wbc
    global ctrl_reg
    global dacarray
    global pixreadoutspeed
    global pixportaddress
    global pixchipaddress
    global pixstopdaq
    global dcolstate

    global calrowlist
    global calcollist

    global currenttbm

# Load the chip state into active memory
    memload $command 1
#    puts dacscale($mydac) $myscalename
#    set dacscalelabel($mydac) $mylabel

    puts "load chip: $command; file: $myfilename"

}



proc xset_dac {cstack cdata saved_data cattr saved_attr args} {
    upvar $saved_data sdata
    upvar $saved_attr sattr
    global itemno
    global dacarray
    global dacscale
    global dacscalelabel
    global verbose2
    
    if {$verbose2=="1"} {
	puts "$itemno. cattr:$cattr ; cdata:$cdata ; args: $args"
    }

    incr itemno
    
    set result [scan $cattr "%s %s %s %s %s %s" \
		    var1 var2 var3 var4 var5 var6]
    if {$verbose2=="1"} {
	puts "($var1,$var2,$var3,$var4,$var5,$var6)"
    }

    if {$var5=="address"} {
	set vid $var6
    } elseif {$var3=="address"} {
	set vid $var4
    } elseif {$var1=="address"} {
	set vid $var2
    } else {
	puts "ERROR: Parsing xml file. address of dac not found.\n"
	bell
	set vid -1
    }

    if {$var3=="value"} {
	set vvalue $var4
    } elseif {$var5=="value"} {
	set vvalue $var6
    } elseif {$var1=="value"} {
	set vvalue $var2
    } else {
	puts "ERROR: Parsing xml file. value of dac not found.\n"
	bell
	set vvalue -1
    }
    
    set dacarray($vid) $vvalue

    if {$verbose2=="1"} {
	puts "Set (via xml) DAC:$vid to value:$vvalue"
    }

	return SXML_PURGE
}
proc xset_location {cstack cdata saved_data cattr saved_attr args} {
    upvar $saved_data sdata
    upvar $saved_attr sattr
    global itemno
    global xmllastx
    global xmllasty
    global verbose2

    if {$verbose2=="1"} {
	puts "LOADMOD set chip num =$cdata $itemno. cattr:$cattr ; cdata:$cdata ; args: $args"
    }

    incr itemno
    set result [scan $cattr "%s %s %s %s %s %s" \
		    var1 var2 var3 var4 var5 var6]
    if {$var5=="x"} {
	set vx $var6
    } elseif {$var3=="x"} {
	set vx $var4
    } elseif {$var1=="x"} {
	set vx $var2
    } else {
	puts "ERROR: Parsing xml file. x position not found.\n"
	bell
	set vx -1
    }
    
    if {$var3=="y"} {
	set vy $var4
    } elseif {$var5=="y"} {
	set vy $var6
    } elseif {$var1=="y"} {
	set vy $var2
    } else {
	puts "ERROR: Parsing xml file. y position not found.\n"
	bell
	set vy -1
    }

    if {$verbose2=="1"} {
	puts "MODULE COL:$vx  ROW:$vy\n"
    }
    set xmllastx $vx
    set xmllasty $vy
    return SXML_PURGE
}
proc xset_setall {cstack cdata saved_data cattr saved_attr args} {
    upvar $saved_data sdata
    upvar $saved_attr sattr
    global xmllastx
    global xmllasty
    global itemno
    global chipcounter
    global verbose2

    incr itemno
    memsave  .win0.fr2.g2.line$xmllasty.bb$xmllastx
    
    incr chipcounter
    if {$chipcounter <= 16} {
	gauge_value .moduleload.g [ expr 100*($chipcounter/16.0) ]
    }
    
    return SXML_PURGE
}
proc xset_register {cstack cdata saved_data cattr saved_attr args} {
    upvar $saved_data sdata
    upvar $saved_attr sattr
    global xmllastx
    global xmllasty
    global wbc
    global itemno
    global chiptouched
    global pixchipaddress
    global verbose2

    incr itemno
    set result [scan $cattr "%s %s %s %s %s %s" \
		    var1 var2 var3 var4 var5 var6]
    if {$var5=="address"} {
	set vaddr $var6
    } elseif {$var3=="address"} {
	set vaddr $var4
    } elseif {$var1=="address"} {
	set vaddr $var2
    } else {
	puts "ERROR: Parsing xml file. address of register not found.\n"
	bell
	set vaddr -1
    }
    
    if {$var3=="value"} {
	set vval $var4
    } elseif {$var5=="value"} {
	set vval $var6
    } elseif {$var1=="value"} {
	set vval $var2
    } else {
	puts "ERROR: Parsing xml file. register value not found.\n"
	bell
	set vval -1
    }
    if {$var3=="name"} {
	set vname $var4
    } elseif {$var5=="name"} {
	set vname $var6
    } elseif {$var1=="name"} {
	set vname $var2
    } else {
	puts "ERROR: Parsing xml file. name not found.\n"
	bell
	set vname -1
    }
    
    
    if {$verbose2=="1"} {
	puts "MODULE REGISTER vaddr:$vaddr,  value:$vval"
    }

    if {$vname == "pixchipaddress"} {
	set pixchipaddress $vval
    } elseif {$vname == "wbc"} {
	set wbc $vval
    } elseif {$vname == "cosmostate"} {
	set chiptouched(.win0.fr2.g2.line$xmllasty.bb$xmllastx) $vval
	if {$vval=="1"} {
	    .win0.fr2.g2.line$xmllasty.bb$xmllastx configure -fg red \
		-bg blue \
		-activebackground blue
	    enumdacs 1
	} elseif {$vval=="-1"} {
	    .win0.fr2.g2.line$xmllasty.bb$xmllastx configure \
		-fg gray \
		-bg lightgray \
		-activebackground gray
	}

    }

    return SXML_PURGE
}

set itemno 0

proc loadxmlmodule { myfilename } {
    global itemno
    global chiptouched
    global globalmodchipbutton
    global chipcounter
    global initialdir

    set id [sxml::init $myfilename]
    if { $id == -1 } {
	puts "Error: Unable to open test file!"
	exit 1
    }
 
    toplevel .moduleload -class Dialog 
    wm title .moduleload "Loading pixel chip module settings..."
    set width [winfo screenwidth .]
    set height [winfo screenheight .]
    set x [expr ($width/2) - 200]
    set y [expr ($height/2) - 200]
    
    wm geometry .moduleload  +$x+$y
    gauge_create .moduleload.g SkyBlue
    pack .moduleload.g -expand yes -fill both -padx 50 -pady 10
    
    
    set chipcounter 0
    
    memsave $globalmodchipbutton
    
    sxml::register_routine $id "vhdi:detector" xset_setall
    sxml::register_routine $id "vhdi:detector:location" xset_location
    sxml::register_routine $id "vhdi:detector:dac" xset_dac
    sxml::register_routine $id "vhdi:detector:register" xset_register
    
    set itemno 1
    set x [sxml::parse $id]
    sxml::end $id
    
    memload $globalmodchipbutton 1
    chipload $globalmodchipbutton

#    set iline 1
#   while {$iline <= 2} {
#	set ibutton 1
#	while {$ibutton <= 8} {
#	    set buttext [expr (($iline-1)*8 + $ibutton)]
#	    set iiline $iline
#	    set chipbuttontext(.win0.fr2.g2.line$iiline.bb$ibutton) $buttext

#	    diskloadchip .win0.fr2.g2.line$iiline.bb$ibutton "testfilename"
	    
#	    incr ibutton
#	}
#	incr iline
#    }

    destroy .moduleload

}
proc loadcosmoscript { command } {
    global initialdir
    global verbose3

    global vmeslot mfec

    set file_types {
	{ "Cosmo Script" { .script }}
	{ "Cosmo Script TCL " { .scr }}
	{ "All Files" * }
    }
    set mfilename [tk_getOpenFile -initialdir $initialdir \
		      -filetypes $file_types \
		      -title "Execute Cosmo Script "\
		       -parent .]

    set lasts [string last ".script" $mfilename]
    set laststcl [string last ".scr" $mfilename]
#    puts "Filename:$mfilename    lasts:$lasts"
    if {$lasts > 0} {
#	puts "Filename2:$mfilename    lasts:$lasts"

	if { $mfilename != "" } {
	    set initialdir [ file dirname $mfilename ]
	    set fileidread [ open $mfilename "r" ]
	    set result "START"
	    while { [gets $fileidread inputline] >= 0 } {
		if {$verbose3=="1"} {
		    puts $inputline
		}
#		regsub "script_" $inputline "" inputline2
		if {$inputline > " "} {
		    set inputline2 "script_$inputline"
		    set inputline $inputline2
		}
#		regsub "_ " $inputline "_" inputline2
#		set inputline $inputline2
		set result [eval $inputline]
#		after 1000
		bell
		update idletasks
#		puts "RESULT: $result"
#		set result [scan $inputline "%d %d" var1 var2]
	    }
	    close $fileidread
	}

#	source $mfilename
	
	return $result
    } elseif {$laststcl > 0} {
	source $mfilename
    }

    toplevel .scriptload -class Dialog 
    wm title .scriptload "Execute Cosmo Script..."
    set width [winfo screenwidth .]
    set height [winfo screenheight .]
    set x [expr ($width/2) - 200]
    set y [expr ($height/2) - 200]

    wm geometry .scriptload  +$x+$y


    destroy .scriptload

}
proc loadmodule { command } {
    global initialdir
    global dacarray
    global dacscale
    global dacscalelabel

    global globalmodchipbutton

#    set dacscale($mydac) $myscalename
#    set dacscalelabel($mydac) $mylabel

    global pixarray
    global maskbitstate
    global calibrationon
    global resetcalibration
    global resetstatus
    global pixarraycal
    global pixarrayenabled
    global pixarraytrim
    global trim
    global wbc
    global ctrl_reg
    global dacarray
    global pixreadoutspeed
    global pixportaddress
    global pixchipaddress
    global pixstopdaq
    global dcolstate

    global verbose2

    global calrowlist
    global calcollist

    global calibratelatency
    global currenttbm

    global chiptouched

    memsave $globalmodchipbutton

    set file_types {
	{ "Cosmo Pixel Chip Module File" { .mod }}
	{ "Cosmo Pixel Chip Module XML File" { .xml }}
	{ "All Files" * }
    }
    set mfilename [tk_getOpenFile -initialdir $initialdir \
		      -filetypes $file_types \
		      -title "Read Pixel Module Settings "\
		       -parent .]
    set chipcounter 0

    set lasts [string last ".xml" $mfilename]
    puts "Filename:$mfilename    lasts:$lasts"
    if {$lasts > 0} {
	loadxmlmodule $mfilename
	return 0
    }

    toplevel .moduleload -class Dialog 
    wm title .moduleload "Loading pixel chip module settings..."
    set width [winfo screenwidth .]
    set height [winfo screenheight .]
    set x [expr ($width/2) - 200]
    set y [expr ($height/2) - 200]

    wm geometry .moduleload  +$x+$y
    gauge_create .moduleload.g SkyBlue
    pack .moduleload.g -expand yes -fill both -padx 50 -pady 10


    if { $mfilename != "" } {
	set initialdir [ file dirname $mfilename ]
	set fileidread [ open $mfilename "r" ]
	
	while { [gets $fileidread inputline] >= 0 } {
	    set result [scan $inputline "%d %d" var1 var2]
	    # Normal DACS are 0x04-0x81(129)	    
	    if {$var1 < 130} {
		set dacarray($var1) $var2 
	    } elseif {$var1 == 130} {
		set wbc $var2
	    } elseif {$var1 == 131} {
		set calibratelatency $var2
	    } elseif {$var1 == 132} {
		set currenttbm $var2
	    } elseif {$var1 == 133} {
		set pixportaddress $var2
	    } elseif {$var1 == 999} {
		set pixchipaddress $var2
		puts "chipid:$var2"
	    } elseif {$var1 == 10000} {
		set chipcounter $var2
		if {$chipcounter <= 16} {
		    gauge_value .moduleload.g [ expr 100*($chipcounter/16.0) ]
		}
	    } elseif {$var1 == 10001} {
		set templine $var2
	    } elseif {$var1 == 10002} {
		set tempbut $var2
		memsave  .win0.fr2.g2.line$templine.bb$tempbut
	    } elseif {$var1 == 10003} {
		if {$var2 == "1" } {
		    set chiptouched(.win0.fr2.g2.line$templine.bb$tempbut) 1
		    .win0.fr2.g2.line$templine.bb$tempbut configure -fg red \
			-bg blue \
			-activebackground blue
		    # enumdacs to send i2c to setup current dacarray values
		    enumdacs 1
		} elseif {$var2 == "-1"} {
		    .win0.fr2.g2.line$templine.bb$tempbut configure \
			-fg gray \
			-bg lightgray \
			-activebackground gray
		}
	    }
	}
    } else {
	if {$verbose2 == "1"} {
	    puts "ERROR: what mfilename?"
	}
    }

    memload $globalmodchipbutton 1
    chipload $globalmodchipbutton

    destroy .moduleload
}

proc savemodule { command } {
    global initialdir
    global fileidsave
    global chipcounter
    global globalmodchipbutton
    
    set chipcounter 1
    
    set file_types {
	{ "Cosmo Pixel Chip Module File" { .mod }}
	{ "Cosmo Pixel Chip Module XML File" { .xml }}
	{ "All Files" * }
    }
    set filename [tk_getSaveFile -initialdir $initialdir \
		      -filetypes $file_types \
		      -title "Pixel Chip Module Settings " \
		      -parent .]
    if { $filename != "" } {

	set lasts [string last ".xml" $filename]
	if {$lasts > 0} {
	    set type "xml"
	} else {
	    set type "flat"
	    set lasts [string last ".mod" $filename]
	    if {$lasts < 0} {
		append filename ".mod"
	    }
	}
	

	set initialdir [ file dirname $filename ]

	puts "FILENAME:$filename"

	if {$type == "xml"} {
	    set fileidsave [sxml::writer::init $filename 1]
	    if {$fileidsave < 0} {
		puts stderr "[sxml::get_error $fileidsave]"
		exit 2
	    }
	    set ret [sxml::writer::set_attr $fileidsave compact 1]
	    sxml::writer::element $fileidsave "vhdi"
	    sxml::writer::data $fileidsave "Detector Module"
	} elseif {$type=="flat"} {
	    set fileidsave [ open $filename "w" ]
	}
	
	toplevel .modulesave -class Dialog 
	wm title .modulesave "Saving pixel chip module settings..."
	set width [winfo screenwidth .]
	set height [winfo screenheight .]
	set x [expr ($width/2) - 200]
	set y [expr ($height/2) - 200]
	
	wm geometry .modulesave  +$x+$y
	gauge_create .modulesave.g SkyBlue
	pack .modulesave.g -expand yes -fill both -padx 50 -pady 10
	
	memsave $globalmodchipbutton
	set iline 1
	while {$iline <= 2} {
	    set ibutton 1
	    while {$ibutton <= 8} {
		set buttext [expr (($iline-1)*8 + $ibutton)]
		if {$buttext <= 16} {
		    gauge_value .modulesave.g [ expr 100*($buttext/16.0) ]
		}
		set iiline $iline
		set chipbuttontext(.win0.fr2.g2.line$iiline.bb$ibutton) \
		    $buttext
		disksavechip .win0.fr2.g2.line$iiline.bb$ibutton \
		    $ibutton $iiline $type
		incr ibutton
	    }
	    incr iline
	}

	if {$type == "xml"} {
	    #pop top element vhdi tag, the entire module
#	    sxml::writer::pop_element $fileidsave

	    set retx [ sxml::writer::finalize $fileidsave ]
	    if {$retx < 0} {
		puts "retx:$retx"
		puts stderr "[sxml::writer::get_error $fileidsave]"
		exit 2
	    }


	} elseif {$type == "flat"} {

	    close $fileidsave
	}

	memload $globalmodchipbutton 1
	chipload $globalmodchipbutton
	destroy .modulesave
    }
}

proc dumpfecregisters { command } {
    global initialdir
    
    set file_types {
	{ "FEC dump" { .dump }}
	{ "All Files" * }
    }
    set filename [tk_getSaveFile -initialdir $initialdir \
		      -filetypes $file_types \
		      -title "FEC Internal Register Dump " \
		      -parent .]
    if { $filename != "" } {

	set lasts [string last ".dump" $filename]
	if {$lasts < 0} {
	    append filename ".dump"
	}

	set initialdir [ file dirname $filename ]

	puts "FILENAME:$filename"

	    set fileidsave [ open $filename "w" ]
	
	toplevel .dumpfec -class Dialog 
	wm title .dumpfec "Saving FEC internal registers..."
	set width [winfo screenwidth .]
	set height [winfo screenheight .]
	set x [expr ($width/2) - 200]
	set y [expr ($height/2) - 200]
	
	wm geometry .dumpfec  +$x+$y
	gauge_create .dumpfec.g SkyBlue
	pack .dumpfec.g -expand yes -fill both -padx 50 -pady 10
	
	puts $fileidsave "FEC Memory HEX Dump"
	puts $fileidsave "OUTBuf\tINBuf\tOUTBuf\tINBuf"

# reset-all	pcifeccmdreg 3 1

	pcifeccmdreg 3 1
	puts "Performed RstAll"

# reset WR
#	pcifeccmdreg 1 1
#	pcifeccmdreg 2 1
#	puts "Performed Rst WR and Rst Rd Buf"

	after 500
	set hextemp1 [padhexword [pcifecreadword 8]]
	set hextemp2 [padhexword [pcifecreadword 12]]
	puts $fileidsave "StatusREG2: $hextemp1    ControlStatus: $hextemp2"

	for {set i 0} {$i < 512} {incr i} {
	    if {$i < 256} {
		set hextemp0 [padhexword [pcifecreadword 0]]
	    }
	    set hextemp1 [padhexword [pcifecreadword 4]]
#	    set hextemp2 [padhexword [pcifecreadword 8]]
	    if {$i < 256} {
		set hextemp3 [padhexword [pcifecreadword 16]]
	    }
	    set hextemp4 [padhexword [pcifecreadword 20]]
#	    set hextemp5 [padhexword [pcifecreadword 24]]

	    if {$i < 256} {
		set hextemp0a [string range $hextemp0 6 7]
		set hextemp0b [string range $hextemp0 4 5]
		set hextemp0c [string range $hextemp0 2 3]
		set hextemp0d [string range $hextemp0 0 1]
		set hextemp3a [string range $hextemp3 6 7]
		set hextemp3b [string range $hextemp3 4 5]
		set hextemp3c [string range $hextemp3 2 3]
		set hextemp3d [string range $hextemp3 0 1]
	    }

	    set hextemp1a [string range $hextemp1 6 7]
	    set hextemp1b [string range $hextemp1 4 5]
	    set hextemp1c [string range $hextemp1 2 3]
	    set hextemp1d [string range $hextemp1 0 1]

#	    set hextemp2a [string range $hextemp2 6 7]
#	    set hextemp2b [string range $hextemp2 2 3]
#	    set hextemp2c [string range $hextemp2 4 5]
#	    set hextemp2d [string range $hextemp2 0 1]

	    set hextemp4a [string range $hextemp4 6 7]
	    set hextemp4b [string range $hextemp4 4 5]
	    set hextemp4c [string range $hextemp4 2 3]
	    set hextemp4d [string range $hextemp4 0 1]

#	    set hextemp5a [string range $hextemp5 6 7]
#	    set hextemp5b [string range $hextemp5 2 3]
#	    set hextemp5c [string range $hextemp5 4 5]
#	    set hextemp5d [string range $hextemp5 0 1]

	    if {$i < 256} {
		puts $fileidsave "$hextemp0a\t$hextemp1a\t$hextemp3a\t$hextemp4a"
		puts $fileidsave "$hextemp0b\t$hextemp1b\t$hextemp3b\t$hextemp4b"
		puts $fileidsave "$hextemp0c\t$hextemp1c\t$hextemp3c\t$hextemp4c"
		puts $fileidsave "$hextemp0d\t$hextemp1d\t$hextemp3d\t$hextemp4d"
	    } else {
		puts $fileidsave "     \t$hextemp1a\t     \t$hextemp4a"
		puts $fileidsave "     \t$hextemp1b\t     \t$hextemp4b"
		puts $fileidsave "     \t$hextemp1c\t     \t$hextemp4c"
		puts $fileidsave "     \t$hextemp1d\t     \t$hextemp4d"
	    }

#	    puts $fileidsave "$hextemp0a\t$hextemp1a\t$hextemp2a\t$hextemp3a\t$hextemp4a\t$hextemp5a"
#	    puts $fileidsave "$hextemp0b\t$hextemp1b\t$hextemp2b\t$hextemp3b\t$hextemp4b\t$hextemp5b"
#	    puts $fileidsave "$hextemp0c\t$hextemp1c\t$hextemp2c\t$hextemp3c\t$hextemp4c\t$hextemp5c"
#	    puts $fileidsave "$hextemp0d\t$hextemp1d\t$hextemp2d\t$hextemp3d\t$hextemp4d\t$hextemp5d"
	    gauge_value .dumpfec.g [ expr 100*($i/512.0) ]
	    
	}

	destroy .dumpfec.g

	close $fileidsave

	destroy .dumpfec
    }
}
#----------------MAIN-------------------------------------------------


# now set up a graphical pixel selection window
frame .win0
frame .win0.fr1
canvas .win0.fr1.can1 -width 700 -height 720

#set pixwidth 12
#set pixheight 12
if {$roctype == "psi46"} {
    set pixwidth 12
    set pixheight 8
}
set mincolsep 2
set xoffset 25
set yoffset 15

#init trims
set ic 0
while { $ic < $maxcols } {
    set ir 0
    while { $ir < $maxrows } {
	set aindex "$ic $ir"
	set pixarraytrim($aindex) Unknown
	set pixarraycal($aindex) 0
	set pixarrayenabled($aindex) 0
	incr ir
    }
	incr ic    
}


#draw the long rect for the background of each dcol
set ic 0
while { $ic < $maxcols } {
    set ir 0
    set icpixels [ expr ($ic * $pixwidth)+$xoffset+3 ]
    set irpixels [ expr ($ir * $pixheight)+$yoffset-5 ]
    set icpixels2 [ expr ($icpixels + 2*$pixwidth*0.75-6) + $mincolsep ]
#orig    set irpixels2 [ expr ($irpixels + $pixheight*0.75 \
\#			      + $maxcols*$pixheight + 10) ]
    set irpixels2 [ expr (($maxrows+1)*$pixheight + 10) ]
   .win0.fr1.can1 create rectangle \
	$icpixels $irpixels $icpixels2 $irpixels2 \
	-fill white -tag "$ic-back"


# and draw a box around the dcol to use for highlighting
#    set icpixels [expr ($ic*$pixwidth+$xoffset-2)]
#    set irpixels [expr ($ir*$pixheight+$yoffset-7)]
#    set icpixels2 [ expr ($icpixels + 2*$pixwidth*0.75+4) + $mincolsep ]
#    set irpixels2 [ expr ($irpixels + $pixheight*0.75 \
\#			      + $maxcols*$pixheight + 10 +2) ]
#    .win0.fr1.can1 create rectangle \
\#	$icpixels $irpixels $icpixels2 $irpixels2 \
\#	-fill "" -tag "$ic-back-highlight"



    incr ic    
    incr ic
}


#draw the line through the cal rows and cols
#first the rows
set ir 0
while {$ir < $maxrows} {
    set invertedir [expr ($maxrows-1)-$ir]
    set ic 0
    while {$ic < $maxcols} {
	set x0 [expr $ic*$pixwidth+$xoffset-3]
	set y0 [expr $ir*$pixheight + $yoffset+($pixheight/2-1)]
	set x1 [expr $x0 + $pixwidth-2]
	set calrowlinearray($ir) [.win0.fr1.can1 create line \
				      $x0 $y0 $x1 $y0 \
				      -fill "" -smooth 0 -width 4 \
				      -tag "$invertedir-calrow" ]
	incr ic
    }
    incr ir
}
#now the cols
set ic 0
while {$ic < $maxcols} {
    set ir 0
    while {$ir < $maxrows} {
	set x0 [expr $ic*$pixwidth+$xoffset+$pixwidth/2-2]
	set y0 [expr $ir*$pixheight + $yoffset]
	set y1 [expr $y0 + $pixheight]
	set calcollinearray($ic) [.win0.fr1.can1 create line \
				      $x0 $y0 $x0 $y1 \
				      -fill "" -smooth 0 -width 4 \
				      -tag "$ic-calcol" ]
	incr ir
    }
    incr ic
}




set ic 0
while { $ic < $maxcols } {
    set ir 0
    while { $ir < $maxrows } {
	set icpixels [ expr ($ic * $pixwidth)+$xoffset ]
	set irpixels [ expr (($maxrows - 1 - $ir) * $pixheight)+$yoffset ]
	set icpixels2 [ expr ($icpixels + $pixwidth*0.75) ]
	set irpixels2 [ expr ($irpixels + $pixheight*0.75) ]
	set aindex "$ic $ir"
	set pixarray($aindex) [.win0.fr1.can1 create rectangle \
		$icpixels $irpixels $icpixels2 $irpixels2 \
		-fill darkgray ]
##		-fill gray -tag "$ic-back" ]
    
#----
	.win0.fr1.can1 bind $pixarray($aindex) <Motion> {
	# the left hand of a double column
	    set ixcol [expr (%x-$xoffset)/$pixwidth ]
	    set iyrow [expr ($maxrows-1) - (%y-$yoffset)/$pixheight ]
##	    puts "x=%x  y=%y  ixcol=$ixcol iycol=$iyrow"
	    if { ($ixcol>=0)&&($ixcol<$maxcols)&&($iyrow>=0)&&($iyrow<$maxrows) } {
		set aindex2 "$ixcol $iyrow"
#######		.win0.fr1.can1 itemconfigure $pixarray($aindex2) -fill black
#### Turn on a box around the double col
#### First turn off all boxes currently on
#		set tempcol 0
#		while {$tempcol<52} {
#		    set textixcol $tempcol
#		    append textixcol "-back-highlight"
#		    .win0.fr1.can1 itemconfigure $textixcol -fill "" -fill green
#		    incr tempcol
#		    incr tempcol
#		}
#### Now set the one we're over
#		set textixcol $ixcol
#		append textixcol "-back-highlight"
#		.win0.fr1.can1 itemconfigure $textixcol -fill "" -fill blue

		set mouseovercol $ixcol
		set mouseoverdcol [expr ($mouseovercol/2)]
		set mouseoverrow $iyrow
		set mouseovertrim $pixarraytrim($aindex2) 
		set mouseovercal $pixarraycal($aindex2) 
		set mouseoverenabled $pixarrayenabled($aindex2) 

		if {$roctype == "psi46"} {
		    set columnanalog [inttobase6 $mouseoverdcol 2]
		    set irtemp [expr (79-$mouseoverrow)*2 + 2]
		    set rowanalog [inttobase6 $irtemp 3]
		}


	    }
	}

#---------
	.win0.fr1.can1 bind $pixarray($aindex) <ButtonPress> {
	    set ixcolv [expr (%x-$xoffset)/$pixwidth ]
	    set iyrowv [expr ($maxrows-1) - (%y-$yoffset)/$pixheight ]
	    ##	    puts "P x=%x  y=%y  ixcol=$ixcol iycol=$iyrow"
	    if { ($ixcolv>=0)&&($ixcolv<$maxcols)&&($iyrowv>=0)&&($iyrowv<$maxrows) } {

		set fwin [focus]
		focus .busylock
		grab set .busylock
		set cursor [. cget -cursor]
		. configure -cursor watch
#may26		update

		set aindex2 "$ixcol $iyrow"
		set mouseovercolv $ixcolv
		set mouseoverrowv $iyrowv
		set colv $mouseovercolv
		set rowv $mouseoverrowv
		set pixarraytrim($aindex2) $trim
		if {$textstatus=="ENABLED"} {
		    set maskbitstate 1
		} else {
		    set maskbitstate 0
		}
		set mouseoverenabled $maskbitstate
		set pixarrayenabled($aindex2) $mouseoverenabled

		if {($textcalibration=="ON") || ($textcalibration=="CAP") \
		    || ($textcalibration=="BUMP")} {
		    set calibrationon 1
		    if {$textcalibration=="BUMP"} { set calibrationon 2 }
		} else {
		    set calibrationon 0
		}

		set flippedcalstate 0
# calon: return 1 if coord is in a calibd row or column
		set icalon [calon $colv $rowv]
		if {($calibrationon==0) && (($pixarraycal($aindex2)>0)||($icalon==1))} {
		    # turn off a single cal pixel by clrcal and reset rest
		    set pixarraycal($aindex2) 0
		    ClrCal 0
		    dopixarraycal $colv $rowv 0
		    docaldelete $colv $rowv
###		    findcals 1
		    reassertcals 1
		    set flippedcalstate 1
		    if {$verbose3==1} {
			puts "CAL turned OFF via CLRCAL"
		    }
		}

		if {($textcalibration=="ON") || ($textcalibration=="CAP") \
		    || ($textcalibration=="BUMP")} {
#doro

		    set pixarraycal($aindex2) 1
		    if {$textcalibration=="BUMP"} { 
			set pixarraycal($aindex2) 2 
		    }
		    docalcolrow $colv $rowv
######checkpoint		    dopixarraycal $colv $rowv 1
		    dopixarraycal $colv $rowv $pixarraycal($aindex2)
		    findcals 1

		}

		set mouseovertrim $pixarraytrim($aindex2) 
		set mouseovercal $pixarraycal($aindex2) 
		set mouseoverenabled $pixarrayenabled($aindex2) 


		if {($mouseovercal>0) && ($mouseoverenabled==1)} {
		    pix_transfer_byte "0x44" $ixcolv $iyrowv 1 0 $mouseovertrim 0
		    pix_transfer_byte "0x45" $ixcolv $iyrowv 1 $mouseovercal \
			$mouseovertrim 0
		    .win0.fr1.can1 itemconfigure $pixarray($aindex2) \
			-fill red
		}
		if {($mouseovercal>0) && ($mouseoverenabled==0)} {
		    pix_transfer_byte "0x44" $ixcolv $iyrowv 0 0 $mouseovertrim 0
		    pix_transfer_byte "0x45" $ixcolv $iyrowv 0 $mouseovercal $mouseovertrim 0
		    .win0.fr1.can1 itemconfigure $pixarray($aindex2) \
			-fill pink
		}
		if {($mouseovercal==0) && ($mouseoverenabled==0)} {
		    pix_transfer_byte "0x44" $ixcolv $iyrowv 0 0 $mouseovertrim 0
		    .win0.fr1.can1 itemconfigure $pixarray($aindex2) \
			-fill darkgray
		}
		if {($mouseovercal==0) && ($mouseoverenabled==1)} {
		    pix_transfer_byte "0x44" $ixcolv $iyrowv 1 0 $mouseovertrim 0
		    .win0.fr1.can1 itemconfigure $pixarray($aindex2) \
			-fill [trimcolor $mouseovertrim]
		}

	    }

		. configure -cursor $cursor
		grab release .busylock
		focus $fwin


	}
	incr ir
    }
    incr ic



    set ir 0
    set icpixels [ expr $icpixels2+$mincolsep ]
    while { $ir < $maxrows } {
	set icpixels2 [ expr ($icpixels + $pixwidth*0.75) ]
	set irpixels [ expr ((($maxrows-1) - $ir) * $pixheight)+$yoffset ]
	set irpixels2 [ expr ($irpixels + $pixheight*0.75) ]
	set aindex "$ic $ir"
	set pixarray($aindex) [.win0.fr1.can1 create rectangle \
		$icpixels $irpixels $icpixels2 $irpixels2 \
		-fill darkgray ]
##		-fill gray -tag "$ic-back" ]
#----
	.win0.fr1.can1 bind $pixarray($aindex) <Motion> {
	# the right hand of a double column
	    set ixcol [expr (%x-$xoffset)/$pixwidth ]
	    set iyrow [expr ($maxrows-1) - (%y-$yoffset)/$pixheight ]
##	    puts "x=%x  y=%y  ixcol=$ixcol iycol=$iyrow"
	    if { ($ixcol>=0)&&($ixcol<$maxcols)&&($iyrow>=0)&&($iyrow<$maxrows) } {
		set aindex2 "$ixcol $iyrow"
####		.win0.fr1.can1 itemconfigure $pixarray($aindex2) -fill black
		set mouseovercol $ixcol
		set mouseoverdcol [expr ($mouseovercol/2)]
		set mouseoverrow $iyrow
#mouseovertrim
		set mouseovertrim $pixarraytrim($aindex2) 
		set mouseovercal $pixarraycal($aindex2) 
		set mouseoverenabled $pixarrayenabled($aindex2) 

		if {$roctype == "psi46"} {
		    set columnanalog [inttobase6 $mouseoverdcol 2]
		    set irtemp [expr (79-$mouseoverrow)*2 + 3]
		    set rowanalog [inttobase6 $irtemp 3]
		}

	    }
	}

#---------

	.win0.fr1.can1 bind $pixarray($aindex) <ButtonPress> {
	    set ixcolv [expr (%x-$xoffset)/$pixwidth ]
	    set iyrowv [expr ($maxrows-1) - (%y-$yoffset)/$pixheight ]
##	    puts "P x=%x  y=%y  ixcol=$ixcol iycol=$iyrow"
	    if { ($ixcolv>=0)&&($ixcolv<$maxcols)&&($iyrowv>=0)&&($iyrowv<$maxrows) } {

		set fwin [focus]
		focus .busylock
		grab set .busylock
		set cursor [. cget -cursor]
		. configure -cursor watch
#may26		update

		set aindex2 "$ixcol $iyrow"
		set mouseovercolv $ixcolv
		set mouseoverrowv $iyrowv
		set colv $mouseovercolv
		set rowv $mouseoverrowv
		set pixarraytrim($aindex2) $trim
		if {$textstatus=="ENABLED"} {
		    set maskbitstate 1
		} else {
		    set maskbitstate 0
		}
		set mouseoverenabled $maskbitstate
		set pixarrayenabled($aindex2) $mouseoverenabled

		if {($textcalibration=="ON") || ($textcalibration=="CAP") \
		    || ($textcalibration=="BUMP")} {
		    if {$textcalibration=="BUMP"} { set calibrationon 2 }
		    set calibrationon 1
		    
		} else {
		    set calibrationon 0
		}

		set flippedcalstate 0
		set icalon [calon $colv $rowv]
		if {($calibrationon==0) && (($pixarraycal($aindex2)>0)||($icalon==1))} {
		    # turn off a single cal pixel by clrcal and reset rest
		    set pixarraycal($aindex2) 0
		    ClrCal 0
		    dopixarraycal $colv $rowv 0
		    docaldelete $colv $rowv
###		    findcals 1
		    reassertcals 1
		    set flippedcalstate 1
		    if {$verbose3==1} {
			puts "CAL turned OFF via CLRCAL"
		    }
		}

		if {($textcalibration=="ON") || ($textcalibration=="CAP") \
		    || ($textcalibration=="BUMP")} {

		    set pixarraycal($aindex2) 1
		    if {$textcalibration=="BUMP"} {
			set pixarraycal($aindex2) 2
		    }

		    docalcolrow $colv $rowv
		    dopixarraycal $colv $rowv $pixarraycal($aindex2)
		    findcals 1
		}

		set mouseovertrim $pixarraytrim($aindex2) 
		set mouseovercal $pixarraycal($aindex2) 
		set mouseoverenabled $pixarrayenabled($aindex2) 

		if {($mouseovercal>0) && ($mouseoverenabled==1)} {
		    pix_transfer_byte "0x44" $ixcolv $iyrowv 1 0 $mouseovertrim 0
		    pix_transfer_byte "0x45" $ixcolv $iyrowv 1 $mouseovercal \
			$mouseovertrim 0
		    .win0.fr1.can1 itemconfigure $pixarray($aindex2) \
			-fill red
		}
		if {($mouseovercal>0) && ($mouseoverenabled==0)} {
		    pix_transfer_byte "0x44" $ixcolv $iyrowv 0 0 $mouseovertrim 0
		    pix_transfer_byte "0x45" $ixcolv $iyrowv 0 $mouseovercal \
			$mouseovertrim 0
		    .win0.fr1.can1 itemconfigure $pixarray($aindex2) \
			-fill pink
		}
		if {($mouseovercal==0) && ($mouseoverenabled==0)} {
		    pix_transfer_byte "0x44" $ixcolv $iyrowv 0 0 $mouseovertrim 0
		    .win0.fr1.can1 itemconfigure $pixarray($aindex2) \
			-fill darkgray
		}
		if {($mouseovercal==0) && ($mouseoverenabled==1)} {
		    pix_transfer_byte "0x44" $ixcolv $iyrowv 1 0 $mouseovertrim 0
		    .win0.fr1.can1 itemconfigure $pixarray($aindex2) \
			-fill [trimcolor $mouseovertrim]
		}



	    }

		. configure -cursor $cursor
		grab release .busylock
		focus $fwin

	}
      incr ir
    }
    incr ic
}




set mycol 0
set myrow 0

## tk version 8 or higher for font syntax
font create textfont -family Helvetica -size 6
set fontname textfont
#font create textfont2 -family Helvetica -size 18
#set fontname2 textfont2

#.win0.fr1.can1 create text 10 50 -text "10" -font $fontname

frame .win0.fr2 -relief sunken -borderwidth 3

frame .win0.fr2.g0 -relief sunken -borderwidth 2


if {$roctype == "psi46"} {
    set tempstring "PSI46"
} else {
    set tempstring "?????"
}

set redledoff "
R0lGODlhEAAPAOcAAAAAAIAAAACAAICAAAAAgIAAgACAgICAgMDcwKbK8JkA
AMQAJv9mZv+Zmf/MzFMAXYFUkKWGs9S+3gAoVgA9hFxqpo+SwMzM/wBBawBj
pFmEvY+n0cbW8wAAiAAAqjOZ/47N8Nnu+SRNDDJsEU2mGYzAbLPTm0RZAF99
AJLAALjSa9HfnFBQHmZmM5mZZszMmf//1nlwAKqdAOjSWP/rAP/wnm5HAJpk
AP9mAP+ZAPezV//MM1Y1DW5KIKZ+Tsefbf/Mmebm5tnZ2b+/v6amppmZmYyM
jGZmZkBAQG+MnP//////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
///////////////////////////////////////////78KCgpICAgP8AAAD/
AP///////////////////yH5BAEAAP8ALAAAAAAQAA8AAAhTAP8JHEiwoMGD
CDcAWAggCcJ/SQAEmBigocOCCilStFjwyMQFGgMcSXJRoMSQEzkOPIlSpUCP
KEWSJJgxpMWSAiNqvHlQIUOSOA0CDfqwqNGBAQEAOw==
"
set greenled "
R0lGODlhEAAPAPf1AAAAAIAAAACAAICAAAAAgIAAgACAgICAgMDcwKbK8JkA
AMQAJv9mZv+Zmf/MzFMAXYFUkKWGs9S+3gAoVgA9hFxqpo+SwMzM/wBBawBj
pFmEvY+n0cbW8wAAiAAAqjOZ/47N8Nnu+SRNDDJsEU2mGYzAbLPTm0RZAF99
AJLAALjSa9HfnFBQHmZmM5mZZszMmf//1nlwAKqdAOjSWP/rAP/wnm5HAJpk
AP9mAP+ZAPezV//MM1Y1DW5KIKZ+Tsefbf/Mmebm5tnZ2b+/v6amppmZmYyM
jGZmZkBAQG+MnP//////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
///////////////////////////////////////////78KCgpICAgP8AAAD/
AP//AAAA//8A/wD//wAAACH5BAEAAP0ALAAAAAAQAA8AAAheAPsJHEiwoMGD
CDcAWAggCcJ+SQAImCigocOCCgXo06exYpKLA49oVKKEI8cjHwlK1Edy40SL
KjVuNOkRZD+RNDkKQGkzI82aNiFKpGgxqECFDD8aJah06cOnUPsFBAA7
"
set redled "
R0lGODlhEAAPAOcAAAAAAIAAAACAAICAAAAAgIAAgACAgICAgMDcwKbK8JkA
AMQAJv9mZv+Zmf/MzFMAXYFUkKWGs9S+3gAoVgA9hFxqpo+SwMzM/wBBawBj
pFmEvY+n0cbW8wAAiAAAqjOZ/47N8Nnu+SRNDDJsEU2mGYzAbLPTm0RZAF99
AJLAALjSa9HfnFBQHmZmM5mZZszMmf//1nlwAKqdAOjSWP/rAP/wnm5HAJpk
AP9mAP+ZAPezV//MM1Y1DW5KIKZ+Tsefbf/Mmebm5tnZ2b+/v6amppmZmYyM
jGZmZkBAQG+MnP//////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
///////////////////////////////////////////78KCgpICAgP8AAAD/
AP///////////////////yH5BAEAAP8ALAAAAAAQAA8AAAheAP8JHEiwoMGD
CDcAWAggCcJ/SQAsmLigocOCChfgwKGxYpKLA49oVKKEI8cjHwlKxEFy40SL
KjVuNOkR5D+RNDkuQGkzI82aNiFKpGgxqECFDD8aJah06cOnUP8FBAA7
"


image create photo redoffimg -format GIF -data $redledoff
image create photo redonimg -format GIF -data $redled
image create photo greenonimg -format GIF -data $greenled

label .win0.fr2.g0.heading -text "$tempstring" -padx 14 \
    -font { Helvetica 24 {bold italic} } -fg "blue"

frame .win0.fr2.g0.ch1 -relief sunken -borderwidth 2
frame .win0.fr2.g0.ch2 -relief sunken -borderwidth 2
#    label .win0.frameledch1.led0 -image redledoffimage
#    label .win0.frameledch2.led0 -image redledonimage
    label .win0.fr2.g0.ch1.label1 -text "CH1" -fg darkgray
    label .win0.fr2.g0.ch2.label2 -text "CH2" -fg darkgray
    label .win0.fr2.g0.ch1.led0 -image redoffimg
    label .win0.fr2.g0.ch2.led0 -image redoffimg
pack .win0.fr2.g0.ch1.label1 .win0.fr2.g0.ch1.led0
pack .win0.fr2.g0.ch2.label2 .win0.fr2.g0.ch2.led0

pack .win0.fr2.g0.ch1 .win0.fr2.g0.heading .win0.fr2.g0.ch2 -side left

pack .win0.fr2.g0


frame .win0.fr2.g1
label .win0.fr2.g1.label1 -text "Chip Address Selection: " -fg blue
pack .win0.fr2.g1.label1
pack .win0.fr2.g1

frame .win0.fr2.g2 -borderwidth 2 -relief raised 
balloon_help .win0.fr2.g2 "Each button is a possible pixel chip
within a module.  Click on a button to set pixel and
DAC values for a particular chip.
The button will be green for the active chip.
Buttons that are not part of the module are gray.
Once any grayed chip is selected, it will display
blue or green."
label .win0.fr2.g2.l1 -text "Chip Select:" -fg blue
pack .win0.fr2.g2.l1
frame .win0.fr2.g2.line1
frame .win0.fr2.g2.line2

set globalchip 1
#button .win0.fr2.g2.line1.bb1 -text "1" -width 1 -fg darkgray -command { 
#    memsave $globalchip
#    .win0.fr2.g2.line1.bb$globalchip configure -fg red -bg blue \
\#	-activebackground blue
#    .win0.fr2.g2.line1.bb1 configure -fg red -bg green -activebackground green
#    memload 1
#    set globalchip 1
#}

# reset arrays in global pixel area, then save all
resetarrays 1

#Setup multiple module chip selection
set iline 1
while {$iline <= 2} {
    set ibutton 1
    while {$ibutton <= 8} {
	set buttext [expr (($iline-1)*8 + $ibutton)]
	set iiline $iline
	set chipbuttontext(.win0.fr2.g2.line$iiline.bb$ibutton) $buttext
	set chiptouched(.win0.fr2.g2.line$iiline.bb$ibutton) 0
	set modulechip(.win0.fr2.g2.line$iiline.bb$ibutton) \
	    [button .win0.fr2.g2.line$iiline.bb$ibutton -text "$buttext" \
		 -width 1 -fg darkgray ]
	bind .win0.fr2.g2.line$iiline.bb$ibutton <ButtonRelease> {
	    chipload %W
	    set chiptouched(%W) 1
	}
	set pixchipaddress 15
	memsave .win0.fr2.g2.line$iiline.bb$ibutton

	incr ibutton
    }
    incr iline
}

.win0.fr2.g2.line1.bb1 configure -fg red -bg green -activebackground green
set globalmodchipbutton .win0.fr2.g2.line1.bb1

pack  .win0.fr2.g2.line1.bb1 .win0.fr2.g2.line1.bb2 .win0.fr2.g2.line1.bb3 .win0.fr2.g2.line1.bb4 -side left 
pack  .win0.fr2.g2.line1.bb5 .win0.fr2.g2.line1.bb6 .win0.fr2.g2.line1.bb7 .win0.fr2.g2.line1.bb8 -side left 

pack  .win0.fr2.g2.line2.bb1 .win0.fr2.g2.line2.bb2 .win0.fr2.g2.line2.bb3 .win0.fr2.g2.line2.bb4 -side left 
pack  .win0.fr2.g2.line2.bb5 .win0.fr2.g2.line2.bb6 .win0.fr2.g2.line2.bb7 .win0.fr2.g2.line2.bb8 -side left 

pack .win0.fr2.g2.line1
pack .win0.fr2.g2.line2

frame .win0.fr2.line2
frame .win0.fr2.line2.g2
label .win0.fr2.line2.g2.label1 -text "Hub: " 
tk_optionMenu .win0.fr2.line2.g2.option1 hubaddress \
	0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 \
	25 26 27 28 29 30 31
set hubaddress 0
balloon_help .win0.fr2.line2.g2.option1 "Select Hub"
balloon_help .win0.fr2.line2.g2.label1 "Select Hub"
pack .win0.fr2.line2.g2.label1 .win0.fr2.line2.g2.option1 -side left

frame .win0.fr2.line2.g3
label .win0.fr2.line2.g3.label1 -text "Port: " 
tk_optionMenu .win0.fr2.line2.g3.option1 pixportaddress \
	0 1 2 3 4 5 6 7
balloon_help .win0.fr2.line2.g3.label1 "Select Port"
balloon_help .win0.fr2.line2.g3.option1 "Select Port"
set pixportaddress 0
pack .win0.fr2.line2.g3.label1 .win0.fr2.line2.g3.option1 -side left
pack .win0.fr2.line2.g2 .win0.fr2.line2.g3 -side left
pack .win0.fr2.line2

frame .win0.fr2.g4
label .win0.fr2.g4.label1 -text "ROC I2C ID:"
tk_optionMenu .win0.fr2.g4.option1 pixchipaddress \
	0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15
set pixchipaddress 15

balloon_help .win0.fr2.g4.label1 "Select\nRead Out Chip\nAddress"
balloon_help .win0.fr2.g4.option1 "Select\nRead Out Chip\nAddress"
label .win0.fr2.g4.label2 -text " TBM:"
tk_optionMenu .win0.fr2.g4.option2 currenttbm \
	A B
set currenttbm A

balloon_help .win0.fr2.g4.option2 "Select\nTBM A or B"
pack .win0.fr2.g4.label1 .win0.fr2.g4.option1 .win0.fr2.g4.label2 .win0.fr2.g4.option2 -side left
pack .win0.fr2.g4


frame .win0.fr2.g6
#frame .win0.fr2.g6.box2 -relief sunken -borderwidth 2
frame .win0.fr2.g6.box2
label .win0.fr2.g6.box2.l1 -fg blue -text ".                    Pixel Information:                     ."
balloon_help .win0.fr2.g6.box2.l1 "The settings for the pixel\n under the pointer are\ndisplayed below."


frame .win0.fr2.g6.box2.line1
frame .win0.fr2.g6.box2.line2
if {$roctype =="psi46"} {
    frame .win0.fr2.g6.box2.line3
}
label .win0.fr2.g6.box2.line1.l1 -text "Double-Column:"
balloon_help .win0.fr2.g6.box2.line1.l1 "This is the Double-Column \nof the pixel the pointer is\n currently over."

label .win0.fr2.g6.box2.line1.l3 -text "Column:"
balloon_help .win0.fr2.g6.box2.line1.l3 "This is the column coordinate \nof the pixel the pointer is\n currently over."

label .win0.fr2.g6.box2.line1.l5 -text "  Row:"
balloon_help .win0.fr2.g6.box2.line1.l5 "This is the row coordinate \nof the pixel the pointer is\n currently over."

label .win0.fr2.g6.box2.line1.l2 -textvariable mouseoverdcol
balloon_help .win0.fr2.g6.box2.line1.l2 "This is the Double-Column coordinate of\n the pixel the pointer is\n currently over."

label .win0.fr2.g6.box2.line1.l4 -textvariable mouseovercol
balloon_help .win0.fr2.g6.box2.line1.l4 "This is the column coordinate of\n the pixel the pointer is\n currently over."

label .win0.fr2.g6.box2.line1.l6 -textvariable mouseoverrow
balloon_help .win0.fr2.g6.box2.line1.l6 "This is the row coordinate of\n the pixel the pointer is\n currently over."

label .win0.fr2.g6.box2.line2.l1 -text "Trim:"
label .win0.fr2.g6.box2.line2.l2 -textvariable mouseovertrim
balloon_help .win0.fr2.g6.box2.line2.l1 "Shows the trim setting\n of the pixel the\npointer is currently over.\n 0: Least sensitive\n 7: Most sensitive\n Unknown: Hasn't been \nsent to pixel."
balloon_help .win0.fr2.g6.box2.line2.l2 "Shows the trim setting\n of the pixel the\npointer is currently over.\n 0: Least sensitive\n 7: Most sensitive\n Unknown: Hasn't been \nsent to pixel."

label .win0.fr2.g6.box2.line2.l3 -text "Calibrate:"
label .win0.fr2.g6.box2.line2.l4 -textvariable mouseovercal
balloon_help .win0.fr2.g6.box2.line2.l3 "Shows the state (calibrate or  \nnot calibrate) of the pixel the\npointer is currently over.\n 0: Calibration OFF\n 1: Calibration ON"
balloon_help .win0.fr2.g6.box2.line2.l4 "Shows the state (calibrate or  \nnot calibrate), of the pixel the\npointer is currently over.\n 0: Calibration OFF\n 1: Calibration ON"

label .win0.fr2.g6.box2.line2.l5 -text "  Enabled:"
balloon_help .win0.fr2.g6.box2.line2.l5 "Shows the state (masked or  \nunmasked), of the pixel the\npointer is currently over. \nUnmasked means ENABLED.\n 0: Disabled or Masked\n 1: Enabled or Unmasked"
label .win0.fr2.g6.box2.line2.l6 -textvariable mouseoverenabled
balloon_help .win0.fr2.g6.box2.line2.l6 "Shows the state (masked or  \nunmasked), of the pixel the\npointer is currently over. \nUnmasked means ENABLED.\n 0: Disabled or Masked\n 1: Enabled or Unmasked"

if {$roctype=="psi46"} {
    label .win0.fr2.g6.box2.line3.l1 -text "DCol Analog:"
    label .win0.fr2.g6.box2.line3.l2 -textvariable columnanalog
    label .win0.fr2.g6.box2.line3.l3 -text "    Pixel Analog:"
    label .win0.fr2.g6.box2.line3.l4 -textvariable rowanalog
    balloon_help .win0.fr2.g6.box2.line3 "The DCol and Pixel Analog Readout display\nthe analog encoded digital\nlevels that should be\nseen in the ROC analog\nreadout (Base 6)."
}

frame .win0.fr2.g6.box3 -relief sunken -borderwidth 2
frame .win0.fr2.g6.box3.line5b
label .win0.fr2.g6.box3.line5b.l1 -fg blue \
    -text "      Pixel Write Change:     "
balloon_help .win0.fr2.g6.box3.line5b.l1 "Click on a pixel \nat the left to\nset the values below."

frame .win0.fr2.g6.box3.line6
    set ttemp 5
    set ttemp2 15

scale .win0.fr2.g6.box3.line6.s1 -from 0 -to $ttemp2 \
    -tickinterval $ttemp \
    -label "Trim Setting:" \
    -length 100 -orient horizontal -command "updatetrimslide"

if {$roctype == "psi46"} {
    balloon_help .win0.fr2.g6.box3.line6.s1 "This Trim setting will\nbe set on the next pixel\nyou select.\n  0 : Most sensitive \n $ttemp2 : Least sensitive" 
} else {
    balloon_help .win0.fr2.g6.box3.line6.s1 "This Trim setting will\nbe set on the next pixel\nyou select.\n  0 : Least sensitive \n $ttemp2 : Most sensitive"
}

##
    frame .win0.fr2.g6.box3.line7
    label .win0.fr2.g6.box3.line7.s1 -text "Calibration:"
    tk_optionMenu .win0.fr2.g6.box3.line7.option1 textcalibration \
	CAP BUMP OFF
    set textcalibration OFF

if {$roctype == "psi46"} {
    balloon_help .win0.fr2.g6.box3.line7.option1 "Set Calibration here \nthen click on a pixel to  \nset the Calibration bits for  \nthat pixel's column  \n and row.  \nCAP (Cd) is set to calibrate via calibrate\n capacitor and BUMP is set to\ncalibrate via sensor bumps"
}
    pack .win0.fr2.g6.box3.line7.s1 .win0.fr2.g6.box3.line7.option1 -side left

##
    frame .win0.fr2.g6.box3.line8
    label .win0.fr2.g6.box3.line8.s1 -text "Pixel:"
tk_optionMenu .win0.fr2.g6.box3.line8.option1 textstatus \
    ENABLED DISABLED
pack .win0.fr2.g6.box3.line8.s1 .win0.fr2.g6.box3.line8.option1 -side left
    set textstatus ENABLED
balloon_help .win0.fr2.g6.box3.line8.option1 "Select DISABLED to mask,  \nor ENABLED to unmask the  \npixel you select next.  "

#frame .win0.fr2.g6.box2.line7
#scale .win0.fr2.g6.box2.line7.s1 -from 0 -to 1 -label "Calibration:" \
\#    -length 100 -orient horizontal -tickinterval 1 -command "updatecalslide"
#
#frame .win0.fr2.g6.box2.line8
#scale .win0.fr2.g6.box2.line8.s1 -from 0 -to 1 -label "Disabled:" \
\#    -length 100 -orient horizontal -tickinterval 1 -command "updateenabledslide"

frame .win0.fr2.g6.box1 -relief sunken -borderwidth 2

frame .win0.fr2.g6.box1.f0
# -relief groove -borderwidth 4
label .win0.fr2.g6.box1.f0.l1 -text "System Settings:" -fg blue 
#    -font { Helvetica 12 {bold} }
#frame .win0.fr2.g6.box1.f2
#label .win0.fr2.g6.box1.f2.l1 -text "Readout speed "
#tk_optionMenu .win0.fr2.g6.box1.f2.option1 textreadoutspeed 20MHz 40MHz 
#button .win0.fr2.g6.box1.f2.b1 -text "Set" \
\#    -command {
#dochangereadoutspeed 1 
#}
frame .win0.fr2.g6.box1.f1
frame .win0.fr2.g6.box1.f1.left
label .win0.fr2.g6.box1.f1.left.l1 -text "Readout:\n speed"
frame .win0.fr2.g6.box1.f1.right -relief ridge -borderwidth 4
button .win0.fr2.g6.box1.f1.right.b1 -borderwidth 4 -text "20MHz" -command {
    set textreadoutspeed 20MHz
    dochangereadoutspeed 1 
}
balloon_help .win0.fr2.g6.box1.f1 "Push either the 20MHz or 40MHz\nbutton to set the readout speed."
button .win0.fr2.g6.box1.f1.right.b2 -borderwidth 4 -text "40MHz" -command {
    set textreadoutspeed 40MHz
    dochangereadoutspeed 1 
}

#.win0.fr2.g6.box1.f1.right.b1 configure -relief sunken \
\#    -fg darkgreen -activeforeground darkgreen
#.win0.fr2.g6.box1.f1.right.b1 configure -relief sunken \
\#    -fg darkgray -activeforeground black


pack .win0.fr2.g6.box1.f1.right.b1 .win0.fr2.g6.box1.f1.right.b2 -side left
pack .win0.fr2.g6.box1.f1.left.l1
pack .win0.fr2.g6.box1.f1.left .win0.fr2.g6.box1.f1.right -side left




#    -font { Helvetica 10 {bold} }
#balloon_help .win0.fr2.g6.box1.f2.option1 "Click 'Set' to send  \nthe 20MHz or 40MHz\nselection. "
#balloon_help .win0.fr2.g6.box1.f2.b1 "Click 'Set' to send  \nthe 20MHz or 40MHz\nselection. "
frame .win0.fr2.g6.box1.f3
button .win0.fr2.g6.box1.f3.b1 -text "Reset System" \
    -command {
doresetsystem 1
}
balloon_help .win0.fr2.g6.box1.f3.b1 "Sends a ClrCal, then\ndisables all the columns \nthen does a TBM reset and\nsets the system speed to that\nselected below."
button .win0.fr2.g6.box1.f3.b2 -text "FindHub" \
    -command {
discoverhub2 1
}
balloon_help .win0.fr2.g6.box1.f3.b2 "  Probes the attached Test-Station\nto automatically set the hub address"



frame .win0.fr2.g6.box1.f4
frame .win0.fr2.g6.box1.f4.left
label .win0.fr2.g6.box1.f4.left.l1 -text "DAQ:"
frame .win0.fr2.g6.box1.f4.right -relief ridge -borderwidth 4
button .win0.fr2.g6.box1.f4.right.b1 -borderwidth 4 -text "ON" -command {
    set pixstopdaq 0
    dochangereadoutspeed 1 
}


balloon_help .win0.fr2.g6.box1.f4 "Turns ROC DAQ ON or OFF"
button .win0.fr2.g6.box1.f4.right.b2 -borderwidth 4 -text "OFF" -command {
    set pixstopdaq 1
    dochangereadoutspeed 1 
}
set textreadoutspeed 20MHz
set pixstopdaq 0

pack .win0.fr2.g6.box1.f4.right.b1 .win0.fr2.g6.box1.f4.right.b2 -side left
pack .win0.fr2.g6.box1.f4.left.l1
pack .win0.fr2.g6.box1.f4.left .win0.fr2.g6.box1.f4.right -side left

frame .win0.fr2.g6.box4 -relief flat -borderwidth 20
button .win0.fr2.g6.box4.b1 \
    -text "Clear Cal" \
    -command {
# doro	
	clrcalintersections 1
	set calrowlist ""
	set calcollist ""
	ClrCal 1
	reassertcals 1
    }
balloon_help .win0.fr2.g6.box4.b1 "Send a 'Clear Cal' \nto the ROC.\nThis will turn off Calibration on \npixels which have been \nset to Calibration."
button .win0.fr2.g6.box4.b15 \
    -text "TBM Reset" \
    -command {
	if {$verbose3=="1"} {
	    puts "SENDING RTBM TO FEC"
	}
	set tt1 [wordtobits 00000004]
	set tt2 [bitwordset $tt1 5 $fec_trigger_source]
	set tt2 [bitwordset $tt2 6 $norm_loop_on]
	set tt3 [bitstoword $tt2]
	sendpcifectriggerregister 2
#old	#TBM Reset
#old	set savedreset $InjectTBMReset
#old	set InjectTBMReset 1
#old	tbm2_write_f1 1
#old	set InjectTBMReset $savedreset
    }
balloon_help .win0.fr2.g6.box4.b15 "Reset TBM."

button .win0.fr2.g6.box4.b2 \
    -text "ROC Reset" \
    -command {
	if {$verbose3=="1"} {
	    puts "SENDING ResetROC TO FEC"
	}
	set temp2 "0111000"
##npci	sendbytewparity $temp2
	set tt1 [wordtobits 00000001]
	set tt2 [bitwordset $tt1 7 $encalpulse]
	set tt2 [bitwordset $tt2 5 $fec_trigger_source]
	set tt2 [bitwordset $tt2 6 $norm_loop_on]
	set tt3 [bitstoword $tt2]
	sendpcifectriggerregister 0

# ROC reset 00000100
#	set bytewrite "00000100"
#	transfer_byte $bytewrite "0x2" "write"    
    }
balloon_help .win0.fr2.g6.box4.b2 "Send command to \nInject Reset to \nthe readout chip."
pack .win0.fr2.g6.box4.b1 .win0.fr2.g6.box4.b15 .win0.fr2.g6.box4.b2 -side left


frame .win0.fr2.g6.box1.f5
button .win0.fr2.g6.box1.f5.b1 \
    -text "DAC Settings" \
    -command {
	if { [winfo exists .windac] == 1} {
	    wm withdraw .windac
	    wm deiconify .windac
	} else {
	    dacsettings 1
	}
    }
balloon_help .win0.fr2.g6.box1.f5.b1 "Show the controls \nfor setting the \nDACs of the selected \nRead Out Chip (ROC)."

button .win0.fr2.g6.box1.f5.b2 \
    -text "ROC Init" \
    -command {
	if { [winfo exists .winreset] == 1} {
	    wm withdraw .winreset
	    wm deiconify .winreset
	} else {
	    resetchip 1 
	}
    }
balloon_help .win0.fr2.g6.box1.f5.b2 "Bring up window to \nset initial values \nfor the entire chip."

frame .win0.fr2.g6.box1.f6
frame .win0.fr2.g6.box1.f6.left
balloon_help .win0.fr2.g6.box1.f6 "Set the mode to:   \nSync or ClearTriggerCount\nor Calibration"
label .win0.fr2.g6.box1.f6.left.l1 -text "Mode:"
frame .win0.fr2.g6.box1.f6.right -relief ridge -borderwidth 4
button .win0.fr2.g6.box1.f6.right.b1 -borderwidth 4 -text "Sync" -command {
    set tbm2mode "Sync"
    .win0.fr2.g6.box1.f6.right.b3 configure -relief raised \
	-fg black -activeforeground black
    .win0.fr2.g6.box1.f6.right.b2 configure -relief raised \
	-fg black -activeforeground black
    .win0.fr2.g6.box1.f6.right.b1 configure -relief sunken \
	-fg darkgreen -activeforeground darkgreen
    tbm2_write_f2 1
}
button .win0.fr2.g6.box1.f6.right.b2 -borderwidth 4 -text "ClrTrigCnt" -command {
    set tbm2mode "ClrTrigCount"
    .win0.fr2.g6.box1.f6.right.b3 configure -relief raised \
	-fg black -activeforeground black
    .win0.fr2.g6.box1.f6.right.b1 configure -relief raised \
	-fg black -activeforeground black
    .win0.fr2.g6.box1.f6.right.b2 configure -relief sunken \
	-fg darkgreen -activeforeground darkgreen
    tbm2_write_f2 1
}
button .win0.fr2.g6.box1.f6.right.b3 -borderwidth 4 -text "Cal" -command {
    set tbm2mode "Calibration"
    .win0.fr2.g6.box1.f6.right.b1 configure -relief raised \
	-fg black -activeforeground black
    .win0.fr2.g6.box1.f6.right.b2 configure -relief raised \
	-fg black -activeforeground black
    .win0.fr2.g6.box1.f6.right.b3 configure -relief sunken \
	-fg darkgreen -activeforeground darkgreen
    tbm2_write_f2 1
}




#frame .win0.fr2.g6.box1.f6
#label .win0.fr2.g6.box1.f6.label1 -text "Mode: " -width 6
#tk_optionMenu .win0.fr2.g6.box1.f6.option1 tbm2mode  "Sync" "ClrTrigCount" "Calibration"
#button .win0.fr2.g6.box1.f6.b1 -text "Set"  -command {
#	tbm2_write_f2 1
#    }

##set tbm2mode "Sync"
#pack .win0.fr2.g6.box1.f6.label1 .win0.fr2.g6.box1.f6.option1 \
\#    .win0.fr2.g6.box1.f6.b1 -side left

pack .win0.fr2.g6.box1.f6.left.l1
pack .win0.fr2.g6.box1.f6.right.b1 .win0.fr2.g6.box1.f6.right.b2 \
    .win0.fr2.g6.box1.f6.right.b3 -side left

pack .win0.fr2.g6.box1.f6.left .win0.fr2.g6.box1.f6.right -side left

# pack multichip selection area
pack .win0.fr2.g2

pack .win0.fr2.g6.box1.f0.l1
pack .win0.fr2.g6.box1.f0

#pack .win0.fr2.g6.box1.f2.l1 .win0.fr2.g6.box1.f2.option1 \
\#    .win0.fr2.g6.box1.f2.b1 -side left
# temp eliminate find hub 
#pack .win0.fr2.g6.box1.f3.b1 .win0.fr2.g6.box1.f3.b2 -side left
pack .win0.fr2.g6.box1.f3.b1 -side left


#pack .win0.fr2.g6.box1.f4.b1 .win0.fr2.g6.box1.f4.b2 -side left
pack .win0.fr2.g6.box1.f5.b1 .win0.fr2.g6.box1.f5.b2 -side left
#pack .win0.fr2.g6.box1.f1 .win0.fr2.g6.box1.f2 .win0.fr2.g6.box1.f3 jd 
pack .win0.fr2.g6.box1.f3
pack .win0.fr2.g6.box1.f5
pack .win0.fr2.g6.box1.f1 
pack .win0.fr2.g6.box1.f6
pack .win0.fr2.g6.box1.f4

pack .win0.fr2.g6.box1

pack .win0.fr2.g6.box2.line1.l1 .win0.fr2.g6.box2.line1.l2 \
    .win0.fr2.g6.box2.line1.l3 .win0.fr2.g6.box2.line1.l4 \
    .win0.fr2.g6.box2.line1.l5 .win0.fr2.g6.box2.line1.l6 -side left
pack .win0.fr2.g6.box2.line2.l1 .win0.fr2.g6.box2.line2.l2 \
    .win0.fr2.g6.box2.line2.l3 .win0.fr2.g6.box2.line2.l4 \
    .win0.fr2.g6.box2.line2.l5 .win0.fr2.g6.box2.line2.l6 -side left
if {$roctype=="psi46"} {
    pack .win0.fr2.g6.box2.line3.l1 .win0.fr2.g6.box2.line3.l2 \
	.win0.fr2.g6.box2.line3.l3 .win0.fr2.g6.box2.line3.l4 \
	-side left
}
pack .win0.fr2.g6.box3.line5b.l1 -side left
pack .win0.fr2.g6.box3.line6.s1
pack .win0.fr2.g6.box3.line7.s1
pack .win0.fr2.g6.box3.line8.s1

pack .win0.fr2.g6.box2.l1
pack .win0.fr2.g6.box2.line1
pack .win0.fr2.g6.box2.line2
if {$roctype=="psi46"} {
    pack .win0.fr2.g6.box2.line3
}
pack .win0.fr2.g6.box2

pack .win0.fr2.g6.box3.line5b \
    .win0.fr2.g6.box3.line6 \
    .win0.fr2.g6.box3.line7 .win0.fr2.g6.box3.line8 

pack .win0.fr2.g6.box3
pack .win0.fr2.g6.box4

frame .win0.fr2.g6.boxbottom -borderwidth 3
label .win0.fr2.g6.boxbottom.l1 -text "Version: $softwareversion" -fg "\#1F1F1F" \
    -font { Helvetica 10 {bold italic} }
pack .win0.fr2.g6.boxbottom.l1

pack .win0.fr2.g6.boxbottom -side right

pack .win0.fr1.can1

pack .win0.fr2.g6

set idcol 0
set tempnumcols [expr $maxcols / 2]
while { $idcol < $tempnumcols } {
    set dcolstateid(.win0.fr1.b$idcol) \
	[button .win0.fr1.b$idcol \
	     -foreground lightblue -background darkgray \
	     -width 0 -height 0]
#    .win0.fr1.b$idcol configure -t
    set dcolstate(.win0.fr1.b$idcol) 0
    bind .win0.fr1.b$idcol <ButtonRelease> {
	changestatebutton %W
    }
    
    pack .win0.fr1.b$idcol

    balloon_help .win0.fr1.b$idcol "Click to turn double-column\nperiphery on and off for\ndouble column $idcol"

    set icpixels [ expr (2*$idcol * $pixwidth)+$xoffset - 2 ]
    place configure .win0.fr1.b$idcol -x $icpixels -y 660

# Place column numbers along bottom, under buttons
    label .win0.fr1.dcol$idcol -font { Helvetica 8 } -text $idcol
    pack .win0.fr1.dcol$idcol
    place configure .win0.fr1.dcol$idcol -x [expr ($icpixels+5)] -y 690

    incr idcol
}


# Place row numbers along left side
if {$roctype == "psi46"} {
    set rowlist { 0 5 10 15 20 25 30 35 40 45 50 55 60 65 70 75 }
}

foreach irow $rowlist {
    set irpixels [ expr ((($maxrows-1) - $irow) * $pixheight)+$yoffset-3 ]
    label .win0.fr1.rowl$irow -font { Helvetica 8 } -text $irow
    pack .win0.fr1.rowl$irow
    if {$irow<10} {
	place configure .win0.fr1.rowl$irow -x 9 -y [expr ($irpixels-2)]
    } else {
	place configure .win0.fr1.rowl$irow -x 5 -y [expr ($irpixels-2)]
    }
    incr irow
}

# Place row numbers along right side
if {$roctype == "psi46"} {
    set rowlist { 159 155 150 145 140 135 130 125 120 115 110 105 100 95 90 85 80 }
}

foreach irow $rowlist {
#    set iirow [expr (105 - $irow)]
    set firstitem [lindex $rowlist 0]
    set iirow [expr ($firstitem - $irow)]
    set irpixels [ expr (($maxrows-1)*$pixheight)-($iirow * $pixheight)+$yoffset ]
    set icol [ expr ( $xoffset + $pixwidth*$maxcols + 6) ]
    label .win0.fr1.rowlright$irow -font { Helvetica 8 } -text $irow
    pack .win0.fr1.rowlright$irow
    place configure .win0.fr1.rowlright$irow -x 650 -y [expr ($irpixels-6)]
    incr irow
}


pack .win0.fr1 .win0.fr2 -side left -fill y



pack .win0

set iter 0
while { $iter < 0 } {
    set ic 0
    while { $ic < $maxcols } {
	.win0.fr1.can1 itemconfigure "$ic-back-highlight" -fill violet
#may26	update
	.win0.fr1.can1 itemconfigure "$ic-back" -fill red
#may26	update
	.win0.fr1.can1 itemconfigure "$ic-back" -fill darkgray
#may26	update
	incr ic
    }
    incr iter
}


##-----------procedures primarily for pixsel window above

frame .busylock
bind .busylock <Motion> break
place .busylock -x -2 -y -2

##--create the dac window and hide it
dacsettings 1
wm withdraw .windac

#### -----MAIN BEGIN FROM HERE -------------
# Started a cosmo-startup earlier.  The following doesnt really do anything
# but pauses for the user to see the version.  

update
cosmo-startup_update "Loading..."
after 100
update

CreateMenubar . .menubar

global analogcurrent
set analogcurrent 128


set menu_name [CreateMenu .menubar filemenu "Mode" 0]

#$menu_name add command -label "TBM Version 1 Fast I2C Communicator" -command { 
#    pack forget .win0
#    pack forget .wintbm2
#    pack .wintbm
#} -underline 0
$menu_name add command -label "TBM Control" -command { 
    pack forget .win0
#    pack forget .wintbm
    pack .wintbm2
} -underline 0

if {$roctype == "psi46"} {
    set tempstring "PSI46"
} else {
    set tempstring "?????"
}

$menu_name add command -label "$tempstring ROC Control" -command { 
#    pack forget .wintbm
    pack forget .wintbm2
    pack .win0
} -underline 0
$menu_name add command -label "FEC Control" -command {
    if { [winfo exists .fec] == 1} {
	wm withdraw .fec
	wm deiconify .fec
    } else {
	feccontrol 1 
    }
} -underline 0
$menu_name add command -label "Slow Hub Control" -command {
    if { [winfo exists .slowhub] == 1} {
	wm withdraw .slowhub
	wm deiconify .slowhub
    } else {
	slowhubcontrol 1 
    }
} -underline 0
$menu_name add command -label "Open ROC Pixel state file" \
    -command { getpixarray 1 } -underline 0
$menu_name add command -label "Save ROC Pixel state file" \
    -command { putpixarray 1 } -underline 0
$menu_name add command -label "Open Module" \
    -command { loadmodule 1 } -underline 0
$menu_name add command -label "Save Module" \
    -command { savemodule 1 } -underline 0
#$menu_name add command -label "SEU tests" \
\#    -command {
#	source "seutest.scr"
#    }
$menu_name add command -label "Dump FEC registers to file" \
    -command { dumpfecregisters 1 } -underline 0
$menu_name add command -label "Toggle FEC debug controls" \
    -command { togglefecexpert } -underline 0
$menu_name add command -label "Properties" -command { getproperties 1 } -underline 0

$menu_name add command -label "Exit" -command { 
    exit } -underline 1

set menu_name [CreateMenu .menubar adcmenu "ADC" 0]
$menu_name add command -label "Start ADC Tool" -command {

    if {$adcloaded == 0} {
	puts "init of adc card..."
	set iii [cyberinit 1]
	puts "returned $iii"
	if {$iii < 0} {
	    puts "Error loading ADC driver"
	} else {
	    set adcloaded 1
	}
    }
    if { [winfo exists .adc] == 1} {
	wm withdraw .adc
	wm deiconify .adc
	set dostop 0
    } else {
	set dostop 0
	doadc 1
    }
}

$menu_name add command -label "Stop ADC" -command {
    if { [winfo exists .adc] == 1} {
	wm withdraw .adc
	wm deiconify .adc
	set dostop 1
    } else {
	set dostop 1
    }
}
$menu_name add command -label "Set Data Output Filename" -command {
    if { [winfo exists .adc] != 1} {
	doadc 1
    }

    if {$adcloaded == 0} {
	puts "init of cyber card..."
	set iii [cyberinit 1]
	puts "returned $iii"
	if {$iii < 0} {
	    puts "Error loading ADC Cyber driver"
	} else {
	    set adcloaded 1
	}
    }

    set saveddataoutputfile $dataoutputfile
    selectoutputfile 1
    if {$dataoutputfile != $saveddataoutputfile} {
	if {$saveddataoutputfile != ""} {
	    # had a file already, need to close it and open new one
	    set iii [cyberinitfileclose 0]
	    set adcfileopen 0
	    puts "Close output file returned: $iii"
	    set iii [cyberinitfile "$dataoutputfile"]
	    puts "Open output file returned: $iii"
	    set adcfileopen 1

#	    puts "OUT changed to:$dataoutputfile, should close $saveddataoutputfile"
####	    close $fdataout
	    if {$verbose3==1} {
		puts "Closefile: oldfilestream"
	    }
####	    set fdataout [open $dataoutputfile "w"]
####	    if {$outformat=="binary"} {
####		fconfigure $fdataout -translation binary 
####	    }
	    set nfileevents 0
	    .adc.f6.l1 config -text $nfileevents -fg black
	    if {$verbose3==1} {
		puts "Openfile1: $dataoutputfile"
	    }
	} else {
	    # new file
	    set iii [cyberinitfile "$dataoutputfile"]
	    puts "Open output file returned: $iii"
	    set adcfileopen 1

####	    set fdataout [open $dataoutputfile "w"]
####	    if {$outformat=="binary"} {
####		fconfigure $fdataout -translation binary
####	    }
	    set nfileevents 0
	    .adc.f6.l1 config -text $nfileevents -fg black
	    if {$verbose3==1} {
		puts "Openfile2: $dataoutputfile"
	    }
	}
    } else {
	# selected same file, close and open, rewrite from beginning
	set iii [cyberinitfileclose 0]
	set adcfileopen 0
	puts "Close output file returned: $iii"
	set iii [cyberinitfile "$dataoutputfile"]
	puts "Open output file returned: $iii"
	set adcfileopen 1
	
####	close $fdataout
	if {$verbose3==1} {
	    puts "Closefile2: oldfilestream"
	}
####	set fdataout [open $dataoutputfile "w"]
####	    if {$outformat=="binary"} {
####		fconfigure $fdataout -translation binary
####	    }
	set nfileevents 0
	.adc.f6.l1 config -text $nfileevents -fg black
	if {$verbose3==1} { 
	    puts "Openfile4: $dataoutputfile"
	}
    }
}



$menu_name add command -label "Close Data Output Filename" -command {
    if {$dataoutputfile != ""} {
	set iii [cyberinitfileclose 0]
	puts "Close output file returned: $iii"
	set adcfileopen 0

	# close file
####	close $fdataout
	if {$verbose3==1} {
	    puts "Closefile3: $dataoutputfile"
	}
	set dataoutputfile ""
	if { [winfo exists .adc.f5.l1] } {
	    .adc.f5.l1 config -text "Output filename:                     " \
		-fg gray
	}
    }
}
#set parser [interp create -safe]
#$parser eval {
#    proc add { x y cmd} {
#	set num [expr $x+$y]
#	set response $num
#	return $response
#    }
#    proc subtract {x y cmd} {
#	set num [expr $x-$y]
#	set response $num
#	return $response
#    }
#}



if {$COSMOTCPSCRIPT == 1} {
    $menu_name add command -label "Enable CosmoScript Server" -command {
	if {$serverstarted < 0} {
	    # removed
	}

	if { [winfo exists .adc] != 1} {
	    doadc 1
	}
	
	if { [winfo exists .windac] == 1} {
	    wm withdraw .windac
	    wm deiconify .windac
	} else {
	    dacsettings 1
	}
	set cosmotcpscripting 1


	tk_messageBox -icon warning -title Warning -type ok -message \
	    "Please note that you have just\nenabled the CosmoScript Server\nand that it is now waiting for instructions\nfrom the network.\nAnother program running on this\nmachine or another machine on the network\ncan now remotely script Cosmo.\n  If you do not use this feature,\n do not enable it.  \n"

#####	vwait enter-mainloop
    }
}

#if {$COSMOTCPSCRIPT == 1} {
#    $menu_name add command -label "Disable CosmoScript Server" -command {
#	eof $server_sock
#	close $server_sock
#	set server_sock ""
#	set serverstarted -1
#    }
#}





$menu_name add command -label "Load and run cosmo script" \
    -command {
	if { [winfo exists .adc] != 1} {
	    doadc 1
	}
	
	if { [winfo exists .windac] == 1} {
	    wm withdraw .windac
	    wm deiconify .windac
	} else {
	    dacsettings 1
	}
	set myresult [loadcosmoscript 1]
	#	    puts "MYRESULT: $myresult"
	tk_messageBox -icon info -title Info -type ok -message \
	    "Script Completed"
	
    }

#$menu_name add command -label "ADC Options" -command {
#    if { [winfo exists .adc] == 1} {
#	wm withdraw .adc
#	wm deiconify .adc
#    } else {
#	selectoutputfile 1
#    }
#}

$menu_name add command -label "Start Internal Test Procedure" -command {
    test_pix_transfer_byte 1
}





set tstring "Test Station Software v$softwareversion"
if {$helpavailable == 1} {
    set menu_name [CreateMenu .menubar historymenu "Help" 0]
} else {
    set menu_name [CreateMenu .menubar historymenu "About" 0]
}

$menu_name add command -label $tstring \
    -command {
	if { [winfo exists .helpwin1] == 1} {
	    wm withdraw .helpwin1
	    wm deiconify .helpwin1
	} else {
	    cosmoabout 1
	}
    }

if {$helpavailable == 1} {
$menu_name add command -label "Test Station Help" \
    -command { 
	if {$PCTARGET == "devel"} {
	    exec /home/doroshen/tbmhelp/help1/tbmhelp1 &
	} else {
	    exec /usr/local/cosmo/bin/help/tbmhelp1 &
	}
    }
}
#	puts "exec help"
##	exec_cmd {/usr/local/ppfec/tbmhelp}
##	blt::bgexec bgexecStatus -output bgexecOutput /usr/local/ppfec/tbmhelp
#    }


################################################################
## Now setup the TBM2 window
frame .wintbm2
frame .wintbm2.fg2
frame .wintbm2.fg2.ft1

# row of bits on screen
# base + 0x0

frame .wintbm2.fg2.ft1.f07 -relief groove -borderwidth 4
#label .wintbm2.fg2.ft1.f07.l07 -text "Disable\nAnalog\nOutput" -width 10 -height 3
label .wintbm2.fg2.ft1.f07.l07 -text " " -width 10 -height 3
frame .wintbm2.fg2.ft1.f06 -relief groove -borderwidth 4
label .wintbm2.fg2.ft1.f06.l06 -text "Disable\nTrigger\nOut" -width 10 -height 3
frame .wintbm2.fg2.ft1.f05 -relief groove -borderwidth 4
label .wintbm2.fg2.ft1.f05.l05 -text "Stack\nReadback\nMode" -width 10 -height 3
frame .wintbm2.fg2.ft1.f04 -relief groove -borderwidth 4
label .wintbm2.fg2.ft1.f04.l04 -text "Ignore\nIncoming\nTriggers" -width 10 -height 3
frame .wintbm2.fg2.ft1.f03 -relief groove -borderwidth 4
label .wintbm2.fg2.ft1.f03.l03 -text "Pause\nReadout" -width 10 -height 3
frame .wintbm2.fg2.ft1.f02 -relief groove -borderwidth 4
label .wintbm2.fg2.ft1.f02.l02 -text "Stack Full\nOn Count\n=24/32" -width 10 -height 3
frame .wintbm2.fg2.ft1.f01 -relief groove -borderwidth 4
label .wintbm2.fg2.ft1.f01.l01 -text "Disable TBM\nClock" -width 10 -height 3
frame .wintbm2.fg2.ft1.f00 -relief groove -borderwidth 4
label .wintbm2.fg2.ft1.f00.l00 -text "40/20MHz\nRead Out" -width 10 -height 3
frame .wintbm2.fg2.ft1.f000 -relief groove -borderwidth 4
label .wintbm2.fg2.ft1.f000.l000 -width 10 -height 3

# DisableAnalogOutput   DisableTriggerOutput  StackReadbackMode  IgnoreIncomingTriggers
# PauseReadout  ResetOnStackCount  DisableTBMCLK  Readout40MHz 

checkbutton .wintbm2.fg2.ft1.f07.c07 -variable DisableAnalogOutput -onvalue "1"	-offvalue "0" 
checkbutton .wintbm2.fg2.ft1.f06.c06 -variable DisableTriggerOutput -onvalue "1" \
	-offvalue "0"
checkbutton .wintbm2.fg2.ft1.f05.c05 -variable StackReadbackMode -onvalue "1" \
	-offvalue "0"
checkbutton .wintbm2.fg2.ft1.f04.c04 -variable IgnoreIncomingTriggers -onvalue "1" \
	-offvalue "0"
checkbutton .wintbm2.fg2.ft1.f03.c03 -variable PauseReadout -onvalue "1" \
	-offvalue "0"
checkbutton .wintbm2.fg2.ft1.f02.c02  -variable ResetOnStackCount -onvalue "1" \
	-offvalue "0"
checkbutton .wintbm2.fg2.ft1.f01.c01  -variable DisableTBMCLK -onvalue "1" \
	-offvalue "0"
checkbutton .wintbm2.fg2.ft1.f00.c00  -variable Readout40MHz -onvalue "1" \
	-offvalue "0"
button .wintbm2.fg2.ft1.f000.b1 -text "WRITE" -command { tbm2_write_f0 1 }

checkbutton .wintbm2.fg2.ft1.f07.rc07 
checkbutton .wintbm2.fg2.ft1.f06.rc06 
checkbutton .wintbm2.fg2.ft1.f05.rc05 
checkbutton .wintbm2.fg2.ft1.f04.rc04 
checkbutton .wintbm2.fg2.ft1.f03.rc03 
checkbutton .wintbm2.fg2.ft1.f02.rc02 
checkbutton .wintbm2.fg2.ft1.f01.rc01 
checkbutton .wintbm2.fg2.ft1.f00.rc00 
button .wintbm2.fg2.ft1.f000.rb1 -text "READ" -command { tbm2_read_f0 1 }


pack .wintbm2.fg2.ft1.f07.l07 .wintbm2.fg2.ft1.f06.l06 .wintbm2.fg2.ft1.f05.l05 .wintbm2.fg2.ft1.f04.l04 .wintbm2.fg2.ft1.f03.l03 .wintbm2.fg2.ft1.f02.l02 .wintbm2.fg2.ft1.f01.l01 .wintbm2.fg2.ft1.f00.l00 .wintbm2.fg2.ft1.f000.l000
pack .wintbm2.fg2.ft1.f07.c07 .wintbm2.fg2.ft1.f06.c06 .wintbm2.fg2.ft1.f05.c05 .wintbm2.fg2.ft1.f04.c04 .wintbm2.fg2.ft1.f03.c03 .wintbm2.fg2.ft1.f02.c02 .wintbm2.fg2.ft1.f01.c01 .wintbm2.fg2.ft1.f00.c00 .wintbm2.fg2.ft1.f000.b1
pack .wintbm2.fg2.ft1.f07.rc07 .wintbm2.fg2.ft1.f06.rc06 .wintbm2.fg2.ft1.f05.rc05 .wintbm2.fg2.ft1.f04.rc04 .wintbm2.fg2.ft1.f03.rc03 .wintbm2.fg2.ft1.f02.rc02 .wintbm2.fg2.ft1.f01.rc01 .wintbm2.fg2.ft1.f00.rc00 .wintbm2.fg2.ft1.f000.rb1
pack .wintbm2.fg2.ft1.f07 .wintbm2.fg2.ft1.f06 .wintbm2.fg2.ft1.f05 .wintbm2.fg2.ft1.f04 .wintbm2.fg2.ft1.f03 .wintbm2.fg2.ft1.f02 .wintbm2.fg2.ft1.f01 .wintbm2.fg2.ft1.f00 .wintbm2.fg2.ft1.f000 -side left

pack .wintbm2.fg2.ft1

 # row of bits on screen

frame .wintbm2.fg2.ft2

#

frame .wintbm2.fg2.ft2.f17 -relief groove -borderwidth 4

frame .wintbm2.fg2.ft2.f17.f1
label .wintbm2.fg2.ft2.f17.f1.l1 \
    -text "Mode:\n\n(Unknown)" -width 20 -height 3
pack .wintbm2.fg2.ft2.f17.f1.l1

frame .wintbm2.fg2.ft2.f17.f2
label .wintbm2.fg2.ft2.f17.f2.l1 -text "Stack\nCount:\n(Unknown)" -width 10 -height 3
button .wintbm2.fg2.ft2.f17.f2.b1 -text "READ" -command { tbm2_read_f1 1 }
pack .wintbm2.fg2.ft2.f17.f2.l1
pack .wintbm2.fg2.ft2.f17.f2.b1
pack .wintbm2.fg2.ft2.f17.f1 .wintbm2.fg2.ft2.f17.f2  -side left



frame .wintbm2.fg2.ft2.f27 -relief groove -borderwidth 4
# label .wintbm2.fg2.ft3.f27.t1 -font { Helvetica 18 } -text "           Test0          "
label .wintbm2.fg2.ft2.f27.l1 -text "EventNum:\n(Unknown)" -width 10 -height 3
button .wintbm2.fg2.ft2.f27.b1 -text "READ" -command { tbm2_read_f2 1 }

pack .wintbm2.fg2.ft2.f27.l1  
pack .wintbm2.fg2.ft2.f27.b1  

#
frame .wintbm2.fg2.ft2.f28 -relief groove -borderwidth 4 
frame .wintbm2.fg2.ft2.f28.f1
label .wintbm2.fg2.ft2.f28.f1.label1 -text "Mode: " -width 18
tk_optionMenu .wintbm2.fg2.ft2.f28.f1.option tbm2mode  \
	"Sync" "ClrTrigCount" "Calibration"
set tbm2mode "Sync"
pack .wintbm2.fg2.ft2.f28.f1.label1
pack .wintbm2.fg2.ft2.f28.f1.option

frame .wintbm2.fg2.ft2.f28.f2
label .wintbm2.fg2.ft2.f28.f2.l1 -text "Disable\nCalibration\nDelayedTrig" \
    -width 10 -height 3
checkbutton .wintbm2.fg2.ft2.f28.f2.c1 -variable DisableCalDelayedTrig \
    -onvalue "1" -offvalue "0" 
pack .wintbm2.fg2.ft2.f28.f2.l1  
pack .wintbm2.fg2.ft2.f28.f2.c1  

frame .wintbm2.fg2.ft2.f28.f3
button .wintbm2.fg2.ft2.f28.f3.b1 -text "WRITE" -command { tbm2_write_f2 1 }
pack .wintbm2.fg2.ft2.f28.f3.b1  
pack .wintbm2.fg2.ft2.f28.f1 .wintbm2.fg2.ft2.f28.f2 \
    .wintbm2.fg2.ft2.f28.f3 -side left
#pack .wintbm2.fg2.ft2.f28
#

#
frame .wintbm2.fg2.ft2.f29 -relief groove -borderwidth 4
set fecinject .wintbm2.fg2.ft2.f29
frame .wintbm2.fg2.ft2.f29.f1 -relief flat
frame .wintbm2.fg2.ft2.f29.f1.ftitle
frame .wintbm2.fg2.ft2.f29.f1.ftop
frame .wintbm2.fg2.ft2.f29.f1.fbot
label .wintbm2.fg2.ft2.f29.f1.ftitle.l1 -text "FEC Inject"
button .wintbm2.fg2.ft2.f29.f1.ftop.b1 -text "CSRE" -width 5 \
    -command {
	if {$verbose3=="1"} {
	    puts "SENDING CSR REQUEST TO FEC"
	}


#	set tt1 [wordtobits 00080000]
#	set tt2 [bitwordset $tt1 14 $fec_trigger_source]
#	set tt2 [bitwordset $tt2 13 $norm_loop_on]
#	set tt3 [bitstoword $tt2]

	set tt1 [wordtobits 00000008]
	set tt2 [bitwordset $tt1 5 $fec_trigger_source]
	set tt2 [bitwordset $tt2 6 $norm_loop_on]
	set tt3 [bitstoword $tt2]

	sendpcifectriggerregister 3

#	pcifecwriteword 28 $tt3
#	pcifecwriteword 24 00084000

##npci	set temp2 "0110001"
##npci	sendbytewparity $temp2
    }
button .wintbm2.fg2.ft2.f29.f1.ftop.b2 -text "RTBM" -width 5 \
    -command {
	if {$verbose3=="1"} {
	    puts "SENDING RTBM TO FEC"
	}
#	set tt1 [wordtobits 00040000]
#	set tt2 [bitwordset $tt1 14 $fec_trigger_source]
#	set tt2 [bitwordset $tt2 13 $norm_loop_on]
#	set tt3 [bitstoword $tt2]
	set tt1 [wordtobits 00000004]
	set tt2 [bitwordset $tt1 5 $fec_trigger_source]
	set tt2 [bitwordset $tt2 6 $norm_loop_on]
	set tt3 [bitstoword $tt2]
	sendpcifectriggerregister 2
##	pcifecwriteword 28 $tt3
#	pcifecwriteword 24 00044000
##npci	set temp2 "0110010"
##npci	sendbytewparity $temp2
    }

button .wintbm2.fg2.ft2.f29.f1.ftop.b3 -text "ENCALPULSE" -width 10 \
    -command {
	toggle_encalpulse 1
	if {$verbose3=="1"} {
	    puts "Changing encalpulse"
	}
	if {$callatencybit == "enabled"} {
	    set temp1 "1"
	} else {
	    set temp1 "0"
	}

	set temp2 "100$temp1"
	set temp3 "001"
	set temp4 "$temp2$temp3"
##npci	sendbytewparity $temp4

	if {$callatencybit == "enabled"} {
#	set tt1 [wordtobits 00100000]
#	set tt2 [bitwordset $tt1 14 $fec_trigger_source]
#	set tt2 [bitwordset $tt2 13 $norm_loop_on]
#	set tt3 [bitstoword $tt2]
	set tt1 [wordtobits 00000010]
        set tt2 [bitwordset $tt1 7 $encalpulse]
	set tt2 [bitwordset $tt2 5 $fec_trigger_source]
	set tt2 [bitwordset $tt2 6 $norm_loop_on]
	set tt3 [bitstoword $tt2]
	sendpcifectriggerregister -1
####	pcifecwriteword 28 $tt3
#	    pcifecwriteword 24 00104000 
#00200400
	} else {
	set tt1 [wordtobits 00000000]
        set tt1 [bitwordset $tt1 7 $encalpulse]
	set tt2 [bitwordset $tt1 5 $fec_trigger_source]
	set tt2 [bitwordset $tt2 6 $norm_loop_on]
	set tt3 [bitstoword $tt2]
	sendpcifectriggerregister -1
##	pcifecwriteword 28 $tt3
#	    pcifecwriteword 24 00104000
#00200000
	}

    }

button .wintbm2.fg2.ft2.f29.f1.fbot.b1 -text "NORM" -width 5 \
    -command {
	if {$verbose3=="1"} {
	    puts "SENDING NORM TRIG REQUEST TO FEC"
	}
	    set temp2 "0110100"
	set tt1 [wordtobits 00000002]
	set tt2 [bitwordset $tt1 7 $encalpulse]
	set tt2 [bitwordset $tt2 5 $fec_trigger_source]
	set tt2 [bitwordset $tt2 6 $norm_loop_on]
	set tt3 [bitstoword $tt2]
	sendpcifectriggerregister 1
##	pcifecwriteword 28 $tt3
#	oldest    pcifecwriteword 24 00024000
##npci	    sendbytewparity $temp2
    }
button .wintbm2.fg2.ft2.f29.f1.fbot.b2 -text "RROC" -width 5 \
    -command {
	if {$verbose3=="1"} {
	    puts "SENDING ResetROC TO FEC"
	}
	set temp2 "0111000"
##npci	sendbytewparity $temp2
	set tt1 [wordtobits 00000001]
	set tt2 [bitwordset $tt1 7 $encalpulse]
	set tt2 [bitwordset $tt2 5 $fec_trigger_source]
	set tt2 [bitwordset $tt2 6 $norm_loop_on]
	set tt3 [bitstoword $tt2]
	sendpcifectriggerregister 0
#	pcifecwriteword 28 $tt3
#	pcifecwriteword 24 00014000
    }

button .wintbm2.fg2.ft2.f29.f1.fbot.b3 -text "NRMLOOP OFF" -width 10 \
    -command {
	toggle_norm_loop_on 1
	sendpcifectriggerregister -1
    }



pack .wintbm2.fg2.ft2.f29.f1.ftitle.l1 
pack .wintbm2.fg2.ft2.f29.f1.ftop.b1 .wintbm2.fg2.ft2.f29.f1.ftop.b2 .wintbm2.fg2.ft2.f29.f1.ftop.b3 -side left
pack .wintbm2.fg2.ft2.f29.f1.fbot.b1 .wintbm2.fg2.ft2.f29.f1.fbot.b2 .wintbm2.fg2.ft2.f29.f1.fbot.b3 -side left
pack .wintbm2.fg2.ft2.f29.f1.ftitle
pack .wintbm2.fg2.ft2.f29.f1.ftop
pack .wintbm2.fg2.ft2.f29.f1.fbot
#pack .wintbm2.fg2.ft2.f29.f1

balloon_help .wintbm2.fg2.ft2.f29.f1.ftitle "FEC Inject Commands\nCSRE: Cal/Sync/RST EVENT (D3)\nRTBM: Reset TBM (D2)\nNORM: Normal Trig (D1)\nRROC: Reset ROC (D0)"

if {$roctype == "psi46"} {
   pack .wintbm2.fg2.ft2.f17 .wintbm2.fg2.ft2.f27 .wintbm2.fg2.ft2.f28 .wintbm2.fg2.ft2.f29 -side left
} else {
    pack .wintbm2.fg2.ft2.f17 .wintbm2.fg2.ft2.f27 .wintbm2.fg2.ft2.f28 -side left
}
pack .wintbm2.fg2.ft2

# row of bits on screen

frame .wintbm2.fg2.ft6

frame .wintbm2.fg2.ft6.f67 -relief groove -borderwidth 4
label .wintbm2.fg2.ft6.f67.l67 -text "Disable\nTBM CLK" -width 10 -height 3
frame .wintbm2.fg2.ft6.f66 -relief groove -borderwidth 4
label .wintbm2.fg2.ft6.f66.l66 -text "40/20MHz\nReadout" -width 10 -height 3
frame .wintbm2.fg2.ft6.f65 -relief groove -borderwidth 4
label .wintbm2.fg2.ft6.f65.l65 -width 10 -height 3
frame .wintbm2.fg2.ft6.f64 -relief groove -borderwidth 4
label .wintbm2.fg2.ft6.f64.l64 -width 10 -height 3
frame .wintbm2.fg2.ft6.f63 -relief groove -borderwidth 4
label .wintbm2.fg2.ft6.f63.l63 -width 10 -height 3
frame .wintbm2.fg2.ft6.f62 -relief groove -borderwidth 4
label .wintbm2.fg2.ft6.f62.l62 -width 10 -height 3
frame .wintbm2.fg2.ft6.f61 -relief groove -borderwidth 4
label .wintbm2.fg2.ft6.f61.l61 -width 10 -height 3
frame .wintbm2.fg2.ft6.f60 -relief groove -borderwidth 4
label .wintbm2.fg2.ft6.f60.l60 -width 10 -height 3
frame .wintbm2.fg2.ft6.f600 -relief groove -borderwidth 4
label .wintbm2.fg2.ft6.f600.l600 -width 10 -height 3

checkbutton .wintbm2.fg2.ft6.f67.c67 -variable DisableTBMCLK -onvalue "1" \
	-offvalue "0" 
checkbutton .wintbm2.fg2.ft6.f66.c66 -variable Readout40MHz -onvalue "1" \
	-offvalue "0" 
checkbutton .wintbm2.fg2.ft6.f65.c65 -variable NotUsedLine65 -onvalue "1" \
	-offvalue "0" 
checkbutton .wintbm2.fg2.ft6.f64.c64 -variable NotUsedLine64 -onvalue "1" \
	-offvalue "0" 
checkbutton .wintbm2.fg2.ft6.f63.c63 -variable NotUsedLine63 -onvalue "1" \
	-offvalue "0" 
checkbutton .wintbm2.fg2.ft6.f62.c62 -variable NotUsedLine62 -onvalue "1" \
	-offvalue "0" 
checkbutton .wintbm2.fg2.ft6.f61.c61 -variable NotUsedLine61 -onvalue "1" \
	-offvalue "0" 
checkbutton .wintbm2.fg2.ft6.f60.c60 -variable NotUsedLine60 -onvalue "1" \
	-offvalue "0" 
button .wintbm2.fg2.ft6.f600.b1 -text "WRITE"  -command { tbm2_write_f2 1 }

pack .wintbm2.fg2.ft6.f67.l67 .wintbm2.fg2.ft6.f66.l66 .wintbm2.fg2.ft6.f65.l65 .wintbm2.fg2.ft6.f64.l64 .wintbm2.fg2.ft6.f63.l63 .wintbm2.fg2.ft6.f62.l62 .wintbm2.fg2.ft6.f61.l61 .wintbm2.fg2.ft6.f60.l60 .wintbm2.fg2.ft6.f600.l600
pack .wintbm2.fg2.ft6.f67.c67 .wintbm2.fg2.ft6.f66.c66 .wintbm2.fg2.ft6.f65.c65 .wintbm2.fg2.ft6.f64.c64 .wintbm2.fg2.ft6.f63.c63 .wintbm2.fg2.ft6.f62.c62 .wintbm2.fg2.ft6.f61.c61 .wintbm2.fg2.ft6.f60.c60 .wintbm2.fg2.ft6.f600.b1
pack .wintbm2.fg2.ft6.f67 .wintbm2.fg2.ft6.f66 .wintbm2.fg2.ft6.f65 .wintbm2.fg2.ft6.f64 .wintbm2.fg2.ft6.f63 .wintbm2.fg2.ft6.f62 .wintbm2.fg2.ft6.f61 .wintbm2.fg2.ft6.f60 .wintbm2.fg2.ft6.f600 -side left

#################pack .wintbm2.fg2.ft6

# row of bits on screen

frame .wintbm2.fg2.ft4

frame .wintbm2.fg2.ft4.f47 -relief groove -borderwidth 4
label .wintbm2.fg2.ft4.f47.l47 -text "Clear\nTrigger\nCounter" -width 10 -height 3
frame .wintbm2.fg2.ft4.f46 -relief groove -borderwidth 4
label .wintbm2.fg2.ft4.f46.l46 -text "Clear\nToken\nOut" -width 10 -height 3
frame .wintbm2.fg2.ft4.f45 -relief groove -borderwidth 4
label .wintbm2.fg2.ft4.f45.l45 -text "Clear\nStack" -width 10 -height 3
frame .wintbm2.fg2.ft4.f44 -relief groove -borderwidth 4
label .wintbm2.fg2.ft4.f44.l44 -text "Inject TBM\nReset" -width 10 -height 3
frame .wintbm2.fg2.ft4.f43 -relief groove -borderwidth 4
label .wintbm2.fg2.ft4.f43.l43 -text "Inject\nPreCalibrate\nTrigger" -width 10 -height 3
frame .wintbm2.fg2.ft4.f42 -relief groove -borderwidth 4
label .wintbm2.fg2.ft4.f42.l42 -text "Inject ROC\nReset" -width 10 -height 3
frame .wintbm2.fg2.ft4.f41 -relief groove -borderwidth 4
label .wintbm2.fg2.ft4.f41.l41 -text "Inject Sync\nTrigger" -width 10 -height 3
frame .wintbm2.fg2.ft4.f40 -relief groove -borderwidth 4
label .wintbm2.fg2.ft4.f40.l40 -text "Inject\nNormal\nTrigger" -width 10 -height 3
frame .wintbm2.fg2.ft4.f400 -relief groove -borderwidth 4
label .wintbm2.fg2.ft4.f400.l400 -width 10 -height 3


# ClearTriggerCounter ClearTokenOut ClearStack InjectTBMReset InjectPreCalTrigger  
# InjectROCReset InjectSyncTrigger InjectNormalTrigger ClearTriggerCounter  ClearTokenOut

checkbutton .wintbm2.fg2.ft4.f47.c47 -variable ClearTriggerCounter -onvalue "1" \
	-offvalue "0" 
checkbutton .wintbm2.fg2.ft4.f46.c46  -variable ClearTokenOut -onvalue "1" \
	-offvalue "0" 
checkbutton .wintbm2.fg2.ft4.f45.c45  -variable ClearStack -onvalue "1" \
	-offvalue "0" 
checkbutton .wintbm2.fg2.ft4.f44.c44  -variable InjectTBMReset -onvalue "1" \
	-offvalue "0" 
checkbutton .wintbm2.fg2.ft4.f43.c43  -variable InjectPreCalTrigger -onvalue "1" \
	-offvalue "0" 
checkbutton .wintbm2.fg2.ft4.f42.c42  -variable InjectROCReset -onvalue "1" \
	-offvalue "0" 
checkbutton .wintbm2.fg2.ft4.f41.c41  -variable InjectSyncTrigger -onvalue "1" \
	-offvalue "0" 
checkbutton .wintbm2.fg2.ft4.f40.c40  -variable InjectNormalTrigger -onvalue "1" \
	-offvalue "0" 
button .wintbm2.fg2.ft4.f400.b1 -text "WRITE"  -command { tbm2_write_f1 1 }

pack .wintbm2.fg2.ft4.f47.l47 .wintbm2.fg2.ft4.f46.l46 .wintbm2.fg2.ft4.f45.l45 .wintbm2.fg2.ft4.f44.l44 .wintbm2.fg2.ft4.f43.l43 .wintbm2.fg2.ft4.f42.l42 .wintbm2.fg2.ft4.f41.l41 .wintbm2.fg2.ft4.f40.l40 .wintbm2.fg2.ft4.f400.l400
pack .wintbm2.fg2.ft4.f47.c47 .wintbm2.fg2.ft4.f46.c46 .wintbm2.fg2.ft4.f45.c45 .wintbm2.fg2.ft4.f44.c44 .wintbm2.fg2.ft4.f43.c43 .wintbm2.fg2.ft4.f42.c42 .wintbm2.fg2.ft4.f41.c41 .wintbm2.fg2.ft4.f40.c40 .wintbm2.fg2.ft4.f400.b1
pack .wintbm2.fg2.ft4.f47 .wintbm2.fg2.ft4.f46 .wintbm2.fg2.ft4.f45 .wintbm2.fg2.ft4.f44 .wintbm2.fg2.ft4.f43 .wintbm2.fg2.ft4.f42 .wintbm2.fg2.ft4.f41 .wintbm2.fg2.ft4.f40 .wintbm2.fg2.ft4.f400 -side left

pack .wintbm2.fg2.ft4

# row of bits on screen

frame .wintbm2.fg2.ft5

frame .wintbm2.fg2.ft5.f57 -relief groove -borderwidth 4
label .wintbm2.fg2.ft5.f57.l57 -text "Stack D15\nNo Token\nPass" -width 10 -height 3
frame .wintbm2.fg2.ft5.f56 -relief groove -borderwidth 4
label .wintbm2.fg2.ft5.f56.l56 -text "Stack D14\nTBM Reset" -width 10 -height 3
frame .wintbm2.fg2.ft5.f55 -relief groove -borderwidth 4
label .wintbm2.fg2.ft5.f55.l55 -text "Stack D13\nROC Reset" -width 10 -height 3
frame .wintbm2.fg2.ft5.f54 -relief groove -borderwidth 4
label .wintbm2.fg2.ft5.f54.l54 -text "Stack D12\nSync Error" -width 10 -height 3
frame .wintbm2.fg2.ft5.f53 -relief groove -borderwidth 4
label .wintbm2.fg2.ft5.f53.l53 -text "Stack D11\nSync" -width 10 -height 3
frame .wintbm2.fg2.ft5.f52 -relief groove -borderwidth 4
label .wintbm2.fg2.ft5.f52.l52 -text "Stack D10\nCLR Trigger\nCounter" -width 10 -height 3
frame .wintbm2.fg2.ft5.f51 -relief groove -borderwidth 4
label .wintbm2.fg2.ft5.f51.l51 -text "Stack D9\nPreCalibrate\nTrigger" -width 10 -height 3
frame .wintbm2.fg2.ft5.f50 -relief groove -borderwidth 4
label .wintbm2.fg2.ft5.f50.l50 -text "Stack D8\nStack Full" -width 10 -height 3
frame .wintbm2.fg2.ft5.f500 -relief groove -borderwidth 4
label .wintbm2.fg2.ft5.f500.l500 -width 10 -height 3

checkbutton .wintbm2.fg2.ft5.f57.c57 
checkbutton .wintbm2.fg2.ft5.f56.c56 
checkbutton .wintbm2.fg2.ft5.f55.c55 
checkbutton .wintbm2.fg2.ft5.f54.c54 
checkbutton .wintbm2.fg2.ft5.f53.c53 
checkbutton .wintbm2.fg2.ft5.f52.c52 
checkbutton .wintbm2.fg2.ft5.f51.c51 
checkbutton .wintbm2.fg2.ft5.f50.c50 
button .wintbm2.fg2.ft5.f500.b1 -text "READ" -command { tbm2_read_f3 1 }

pack .wintbm2.fg2.ft5.f57.l57 .wintbm2.fg2.ft5.f56.l56 .wintbm2.fg2.ft5.f55.l55 .wintbm2.fg2.ft5.f54.l54 .wintbm2.fg2.ft5.f53.l53 .wintbm2.fg2.ft5.f52.l52 .wintbm2.fg2.ft5.f51.l51 .wintbm2.fg2.ft5.f50.l50 .wintbm2.fg2.ft5.f500.l500
pack .wintbm2.fg2.ft5.f57.c57 .wintbm2.fg2.ft5.f56.c56 .wintbm2.fg2.ft5.f55.c55 .wintbm2.fg2.ft5.f54.c54 .wintbm2.fg2.ft5.f53.c53 .wintbm2.fg2.ft5.f52.c52 .wintbm2.fg2.ft5.f51.c51 .wintbm2.fg2.ft5.f50.c50 .wintbm2.fg2.ft5.f500.b1
pack .wintbm2.fg2.ft5.f57 .wintbm2.fg2.ft5.f56 .wintbm2.fg2.ft5.f55 .wintbm2.fg2.ft5.f54 .wintbm2.fg2.ft5.f53 .wintbm2.fg2.ft5.f52 .wintbm2.fg2.ft5.f51 .wintbm2.fg2.ft5.f50 .wintbm2.fg2.ft5.f500 -side left

pack .wintbm2.fg2.ft5


frame .wintbm2.fg1
label .wintbm2.fg1.label1 -text "Hub Address: "
##entry .wintbm2.fg1.ent1 -width 2 -textvariable hubaddress
tk_optionMenu .wintbm2.fg1.option hubaddress \
	0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 \
	25 26 27 28 29 30 31
set hubaddress 0

label .wintbm2.fg1.label2 -text "Token Bit Manager           TBM:"
tk_optionMenu .wintbm2.fg1.option2 currenttbm \
	A B
# old fec1	{ set baseaddress 0x78 }

label .wintbm2.fg1.filler1 -text ""
label .wintbm2.fg1.filler2 -text ""
button .wintbm2.fg1.pb1 -text "Read Stack" -command { read_stack now }
##pack .wintbm2.fg1.label1 .wintbm2.fg1.ent1 .wintbm2.fg1.filler1 .wintbm2.fg1.label2 .wintbm2.fg1.rbtbm1 .wintbm2.fg1.rbtbm2 .wintbm2.fg1.filler2 .wintbm2.fg1.pb1 -side left
#pack .wintbm2.fg1.label1 .wintbm2.fg1.option .wintbm2.fg1.filler1 .wintbm2.fg1.label2 .wintbm2.fg1.rbtbm1 .wintbm2.fg1.rbtbm2 wintbm2.fg1.filler2 .wintbm2.fg1.pb0 .wintbm2.fg1.pb1 -side left
pack .wintbm2.fg1.label1 .wintbm2.fg1.option .wintbm2.fg1.filler1 .wintbm2.fg1.label2 .wintbm2.fg1.option2 .wintbm2.fg1.filler2 .wintbm2.fg1.pb1 -side left
pack configure .wintbm2.fg1.filler1  -padx 40
pack configure .wintbm2.fg1.filler2  -padx 40



pack .wintbm2.fg1
#.wintbm2.fg1.rbtbm1 invoke


frame .wintbm2.fg3
frame .wintbm2.fg4
frame .wintbm2.fg5
frame .wintbm2.fg6
frame .wintbm2.fg7
#########################################################################
# Base+0x4
# Reset Analog Registers to nominal
frame .wintbm2.fg3.f7 
frame .wintbm2.fg3.f7.ff1 -relief flat -borderwidth 4 
frame .wintbm2.fg3.f7.ff1.f07 -relief groove -borderwidth 4
label .wintbm2.fg3.f7.ff1.f07.l07 -text "Force\nReadout\nClock On" -width 10 -height 3
frame .wintbm2.fg3.f7.ff1.f06 -relief groove -borderwidth 4
label .wintbm2.fg3.f7.ff1.f06.l06 -text "Disable\nToken\nOut Drv" -width 10 -height 3
frame .wintbm2.fg3.f7.ff1.f05 -relief groove -borderwidth 4
label .wintbm2.fg3.f7.ff1.f05.l05 -text "Disable\nAnalog\nOutput Drv" -width 10 -height 3
checkbutton .wintbm2.fg3.f7.ff1.f07.c07 -variable ForceReadoutClockOn -onvalue "1" \
	-offvalue "0" 
checkbutton .wintbm2.fg3.f7.ff1.f06.c06 -variable DisableTokenOutDrv -onvalue "1" \
	-offvalue "0"
checkbutton .wintbm2.fg3.f7.ff1.f05.c05 -variable DisableAnalogOutputDrv -onvalue "1" \
	-offvalue "0"

frame .wintbm2.fg3.f7.ff1.f000 -relief flat -borderwidth 4
label .wintbm2.fg3.f7.ff1.f000.l000 -width 10 -height 3

button .wintbm2.fg3.f7.ff1.f000.b1 -text "WRITE" -command { tbm2_write_f4 1 }
checkbutton .wintbm2.fg3.f7.ff1.f07.rc07x 
checkbutton .wintbm2.fg3.f7.ff1.f06.rc06x 
checkbutton .wintbm2.fg3.f7.ff1.f05.rc05x 
button .wintbm2.fg3.f7.ff1.f000.rb1 -text "READ" -command { tbm2_read_f4 1 }


pack .wintbm2.fg3.f7.ff1.f07.l07 .wintbm2.fg3.f7.ff1.f06.l06 .wintbm2.fg3.f7.ff1.f05.l05  .wintbm2.fg3.f7.ff1.f000.b1

pack .wintbm2.fg3.f7.ff1.f07.c07 .wintbm2.fg3.f7.ff1.f06.c06 .wintbm2.fg3.f7.ff1.f05.c05 
#.wintbm2.fg3.f7.ff1.f000.rb1
pack .wintbm2.fg3.f7.ff1.f07.rc07x .wintbm2.fg3.f7.ff1.f06.rc06x .wintbm2.fg3.f7.ff1.f05.rc05x .wintbm2.fg3.f7.ff1.f000.rb1

pack .wintbm2.fg3.f7.ff1.f07 .wintbm2.fg3.f7.ff1.f06 .wintbm2.fg3.f7.ff1.f05 \
    .wintbm2.fg3.f7.ff1.f000 -side left

pack .wintbm2.fg3.f7.ff1
pack .wintbm2.fg3.f7

#########################################################################
### hiding slider.  now send 255 when pushed: pack .wintbm2.fg3.f7.ff1.c1
#pack .wintbm2.fg3.f7.ff2.b1
#pack .wintbm2.fg3.f7.ff1 .wintbm2.fg3.f7.ff2 -side left
#pack .wintbm2.fg3.f7

# Base+0x5
# Analog Output Gain Adjustment
frame .wintbm2.fg3.f8 
frame .wintbm2.fg3.f8.ff1 -relief groove -borderwidth 4 
scale .wintbm2.fg3.f8.ff1.c1 -from 0 -to 255 -label "TBMs Analog Output Dac Gain" \
  -length 250 -orient horizontal -tickinterval 50 -command "updateslide2"
.wintbm2.fg3.f8.ff1.c1 set $AnalogOutputGain
frame .wintbm2.fg3.f8.ff2 
button .wintbm2.fg3.f8.ff2.b1 -text "WRITE" \
	-command { write_slider .wintbm2.fg3.f8.ff1.c1 $AnalogOutputGain "GainWrite" }
button .wintbm2.fg3.f8.ff2.b2 -text "READ" \
	-command { 
	    write_slider .wintbm2.fg3.f8.ff1.c1 $AnalogOutputGain "GainRead" 
#	    getusbreaddata 1
	        clearnibblearray "0000"
    if {$slowfecchannel == "1"} {
	pcifeccmdreg 2 1
    } elseif {$slowfecchannel == "2"} {
	pcifeccmdreg 18 1
    }

    if {$slowfecchannel == "1"} {
	set hextemp [pcifecreadword 4]
    } else {
	set hextemp [pcifecreadword 20]
    }
    set hextemp [padhexword $hextemp]
#    set retdata [string range $hextemp 2 3]
    set retdata [string range $hextemp 4 5]
    
    set temphigh [hexhigh $retdata]
    set templow [hexlow $retdata]
    set byteread "$temphigh$templow"

#	    set byteread $usbreaddata
	    set AnalogOutputGain [bitstoint $byteread]
	    .wintbm2.fg3.f8.ff1.c1 set $AnalogOutputGain
	}

pack .wintbm2.fg3.f8.ff1.c1
pack .wintbm2.fg3.f8.ff2.b1 .wintbm2.fg3.f8.ff2.b2
pack .wintbm2.fg3.f8.ff1 .wintbm2.fg3.f8.ff2 -side left
#pack  .wintbm2.fg3.f7 .wintbm2.fg3.f8 -side left -fill x

# Base+0x6
# Write Differential Offset Adjustment
frame .wintbm2.fg4.f9 
frame .wintbm2.fg4.f9.ff1 -relief groove -borderwidth 4 
scale .wintbm2.fg4.f9.ff1.c1 -from 0 -to 255 -label "Analog Input Amplifier Bias" \
   -length 250 -orient horizontal -tickinterval 50 -command "updateslide3"
.wintbm2.fg4.f9.ff1.c1 set $AnalogInputAmplifierBias
frame .wintbm2.fg4.f9.ff2

button .wintbm2.fg4.f9.ff2.b1 -text "WRITE" \
    -command { write_slider .wintbm2.fg4.f9.ff1.c1 $AnalogInputAmplifierBias "InputAmpBiasWrite" }
button .wintbm2.fg4.f9.ff2.b2 -text "READ" \
	-command { 
	    write_slider .wintbm2.fg4.f9.ff1.c1 $AnalogInputAmplifierBias "InputAmpBiasRead"
#	    getusbreaddata 1
	        clearnibblearray "0000"
    if {$slowfecchannel == "1"} {
	pcifeccmdreg 2 1
    } elseif {$slowfecchannel == "2"} {
	pcifeccmdreg 18 1
    }
    if {$slowfecchannel == "1"} {
	set hextemp [pcifecreadword 4]
    } else {
	set hextemp [pcifecreadword 20]
    }
	    set hextemp [padhexword $hextemp]
#    set retdata [string range $hextemp 2 3]
    set retdata [string range $hextemp 4 5]
    
    set temphigh [hexhigh $retdata]
    set templow [hexlow $retdata]
    set byteread "$temphigh$templow"

#	    set byteread $usbreaddata
	    set AnalogInputAmplifierBias [bitstoint $byteread]
	    .wintbm2.fg4.f9.ff1.c1 set $AnalogInputAmplifierBias
	}

pack .wintbm2.fg4.f9.ff1.c1
pack .wintbm2.fg4.f9.ff2.b1 .wintbm2.fg4.f9.ff2.b2
pack .wintbm2.fg4.f9.ff1 .wintbm2.fg4.f9.ff2 -side left
#pack .wintbm2.fg3.f9

# Base+0x7
# Write Analog Output Driver Current
frame .wintbm2.fg4.f10 
frame .wintbm2.fg4.f10.ff1 -relief groove -borderwidth 4 
scale .wintbm2.fg4.f10.ff1.c1 -from 0 -to 255 -label "Analog Output Driver Bias" \
   -length 250 -orient horizontal -tickinterval 50 -command "updateslide4"
.wintbm2.fg4.f10.ff1.c1 set $AnalogOutputDriverBias
frame .wintbm2.fg4.f10.ff2
button .wintbm2.fg4.f10.ff2.b1 -text "WRITE" \
	-command { write_slider .wintbm2.fg4.f10.ff1.c1 $AnalogOutputDriverBias "OutputDriverBiasWrite" }
button .wintbm2.fg4.f10.ff2.b2 -text "READ" \
	-command { 
	    write_slider .wintbm2.fg4.f10.ff1.c1 $AnalogOutputDriverBias "OutputDriverBiasRead"
	        clearnibblearray "0000"
	    if {$slowfecchannel == "1"} {
	pcifeccmdreg 2 1
    } elseif {$slowfecchannel == "2"} {
	pcifeccmdreg 18 1
    }
	    if {$slowfecchannel == "1"} {
	set hextemp [pcifecreadword 4]
    } else {
	set hextemp [pcifecreadword 20]
    }
    set hextemp [padhexword $hextemp]
#    set retdata [string range $hextemp 2 3]
    set retdata [string range $hextemp 4 5]
    
    set temphigh [hexhigh $retdata]
    set templow [hexlow $retdata]
    set byteread "$temphigh$templow"

#	    getusbreaddata 1
#	    set byteread $usbreaddata
	    set AnalogOutputDriverBias [bitstoint $byteread]
	    .wintbm2.fg4.f10.ff1.c1 set $AnalogOutputDriverBias
	}

label .wintbm2.fg3.filler1
label .wintbm2.fg4.filler1
pack .wintbm2.fg4.f10.ff1.c1
pack .wintbm2.fg4.f10.ff2.b1 .wintbm2.fg4.f10.ff2.b2
pack .wintbm2.fg4.f10.ff1 .wintbm2.fg4.f10.ff2 -side left

pack  .wintbm2.fg3.f7 .wintbm2.fg3.filler1 .wintbm2.fg3.f8  -side left -fill x
pack  .wintbm2.fg4.f9 .wintbm2.fg4.filler1 .wintbm2.fg4.f10 -side left -fill x
pack .wintbm2.fg1
pack .wintbm2.fg2
pack .wintbm2.fg3
pack .wintbm2.fg4
pack configure .wintbm2.fg3.filler1  -padx 40
pack configure .wintbm2.fg4.filler1  -padx 40



proc loadrawdata { command } {
    global dataentry
    global inum
    global num
    global initialdir

    set file_types {
	{ "CR delim TBM Hex Data" { .hex }}
	{ "All Files" * }
    }
    set rawfilename [tk_getOpenFile -initialdir $initialdir \
		      -filetypes $file_types \
		      -title "Load Raw TBM data "\
		       -parent .]

#    set rlasts [string rlast ".hex" $rawfilename]
    set itemplen [string length $rawfilename]
    if {$itemplen > 0} {

	if { $rawfilename != "" } {

	    set i 0
	    while { $i < 512 } {
		set dataentry($i) "00"
		incr i
	    }


	    set initialdir [ file dirname $rawfilename ]
	    set rawfileidread [ open $rawfilename "r" ]
	    set result "START"
	    set i 0
	    while { [gets $rawfileidread inputline] >= 0 } {
		if {$inputline > " "} {
		    set dataentry($i) "$inputline"
		    incr i
		}
	    }
	    if {$i > 0} {
		if {$i > 255} {
		    set result [tk_messageBox -parent . \
			-title { Error } -type ok \
			-icon info \
			-message "Error:  HEX file limited to 256 Bytes\nas 2 hex digits per line, CR delimited\nRead:$i"]
		    return
		}

		set inum $i
		set myhex [format "%x" $inum]
		set itmp [string length $myhex]
		if {$itmp < 2} {
		    set myhex "0$myhex"
		}
		set num $myhex
		puts "READ of $inum hex bytes (0x$num)"
	    }
	    close $rawfileidread
	}

	return $result
    }

    toplevel .rawhexload -class Dialog 
    wm title .rawhexload "Load <CR> delimited HEX data"
    set width [winfo screenwidth .]
    set height [winfo screenheight .]
    set x [expr ($width/2) - 200]
    set y [expr ($height/2) - 200]

    wm geometry .rawhexload  +$x+$y


    destroy .rawhexload

}

proc makeslowhubdata { command } {
    global num
    global dataentry

    toplevel .slowhubdata -class Dialog 
    wm title .slowhubdata "Slow Hub Data"
    label .slowhubdata.label -text "Slow Hub Data"
    tk_optionMenu .slowhubdata.option iport 0x378 0x278 0x3BC


#    pack .slowhubdata.label .slowhubdata.option


    label .slowhubdata.label2 -text "Slow Hub Address:"
    tk_optionMenu .slowhubdata.option2 slowhubaddress 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31
    set slowhubaddress 0
    pack .slowhubdata.label2 .slowhubdata.option2

    label .slowhubdata.label3 -text "Slow Hub Port Address:"
    tk_optionMenu .slowhubdata.option3 slowhubport 0 1 2 3 4 5 6 7
    set slowhubport 0
    pack .slowhubdata.label3 .slowhubdata.option3

    label .slowhubdata.label31 -text "Start Shift:"
    tk_optionMenu .slowhubdata.option31 slowhubstartshift 0 -1 -2 -3 -4 -5 -6 -7 -8 -9 -10 1 2
    set slowhubstartshift 0
    pack .slowhubdata.label31 .slowhubdata.option31

    label .slowhubdata.label32 -text "Stop Shift:"
    tk_optionMenu .slowhubdata.option32 slowhubstopshift  0 1 2 3 4 5 6 7 8 9 10 -1 -2
    set slowhubportstopshift 0
    pack .slowhubdata.label32 .slowhubdata.option32



    label .slowhubdata.label4 -text "Data:" -font { Helvetica 10 {bold}}
    pack .slowhubdata.label4
    set i 0
    while { $i < 4 } {

	entry .slowhubdata.data$i -textvariable slowhubdata($i) -width 2 -font { Helvetica 10 {bold}}
	pack .slowhubdata.data$i -side left
	incr i
    }

    label .slowhubdata.label5 -text "N Bytes:"
    tk_optionMenu .slowhubdata.option5 nbytes  1 2 3 4
    set nbytes 1
    pack .slowhubdata.label5 .slowhubdata.option5
    
    button .slowhubdata.button -text "Build data and export" -command { 

    
    set myhub [expr $slowhubport + (8 * $slowhubaddress)]
    set imyhub [inttobits $myhub 8]
    set myhubhigh [string range $imyhub 0 3]
    set myhubhigh [bitstohex $myhubhigh]
    set myhublow [string range $imyhub 4 7]
    set myhublow [bitstohex $myhublow]
    
    set dataentry(0) "$myhubhigh$myhublow"

    puts "Data0=$dataentry(0)"

	set slowdata ""
# serial stream define 0="01" 1="10"
# insert a start, 24 x 0
    if {$slowhubstartshift != 0} {
	puts "START SHIFT for slowhub=$slowhubstartshift"
	set ilimit [expr $slowhubstartshift + 23]
	for {set ii 0} {$ii < $ilimit} {incr ii} {
	    append slowdata "01"
	}
    } else {
	for {set ii 0} {$ii < 23} {incr ii} {
	    append slowdata "01"
	}
    }
	for {set i 0} {$i < $nbytes} {incr i} {
	    set dbyte $slowhubdata($i)
	    set slowhubhigh [string index $dbyte 0]
	    set slowhublow [string index $dbyte 1]
	    set slowhubhigh [hextoint $slowhubhigh]
	    set slowhublow [hextoint $slowhublow]
	    set islowhubdata [expr $slowhublow + 16*$slowhubhigh]
	    set dbyte [inttobits $islowhubdata 8]
	    puts "DBYTE:$dbyte"
	    for {set j 0} {$j < 8} {incr j} {
		# now put the data in, 32 x N(i)
		set dbit [string index $dbyte $j]
		for {set ii 0} {$ii < 32} {incr ii} {
		    if {$dbit == 1} {
			append slowdata "10"
		    } else {
			append slowdata "01"
		    }
		}
	    }
	    # add in the 9th ACK data, 32 1s is 32 x 10
	    for {set ii 0} {$ii < 32} {incr ii} {
		append slowdata "10"
	    }
	    
	}
	

# insert a stop, M x 0 where m=19 for 1 byte, m=16 for 2B,m=18 for 3B,m=15
# for 4 bytes
	switch $nbytes {
	    1 { set nzeros 19 }
	    2 { set nzeros 16 }
	    3 { set nzeros 18 }
	    4 { set nzeros 15 }
	}
    if {$slowhubstopshift != 0} {
	set nzeros [expr $nzeros + $slowhubstopshift]
	puts "STOP SHIFT for slowhub=$slowhubstopshift   Stop M=$nzeros"
    }
	for {set ii 0} {$ii < $nzeros} {incr ii} {
	    append slowdata "01"
	}

	puts "SLOWDATA RAW: $slowdata"
	set newslowdata ""
	set ilen [string length $slowdata]
	for {set k 0} {$k < $ilen} {incr k} {
	    set mychar [string index $slowdata $k]
	    set mev [expr ($k+1) % 5]
	    if {$mev != 0} {
		append newslowdata $mychar
	    }
	}
	set slength [string length $newslowdata]
	puts "SLOWDATA length:$slength NEW: $newslowdata"

	set thisentry 1
	for {set m 0} {$m < [string length $newslowdata]} {set m [expr $m+8]} {
	    set thishigh [string range $newslowdata $m [expr $m+3]]
	    set thishigh [bitstohex $thishigh]
	    set thislow [string range $newslowdata [expr $m+4] [expr $m+7]]
	    set thislow [bitstohex $thislow]
	    set thishex "$thishigh$thislow"
	    set dataentry($thisentry) $thishex
	    incr thisentry
	}

	set inum $thisentry
	puts "INUM:$inum"
	set myhex [format "%x" $inum]
	set itmp [string length $myhex]
	if {$itmp < 2} {
	    set myhex "0$myhex"
	}
	set num $myhex

	destroy .slowhubdata
    }
    pack .slowhubdata.button


    set width [winfo screenwidth .]
    set height [winfo screenheight .]
    set x [expr ($width/2) - 200]
    set y [expr ($height/2) - 200]

    wm geometry .slowhubdata  +$x+$y
    wm resizable .slowhubdata 0 0
}

proc updatefeccmdindicators { command } {
    global verbose3
    global ch1receivetimeout ch1receivecomplete ch1hubaddresserror \
	ch1sendstarted ch1rcvcounterr
    global ch2receivetimeout ch2receivecomplete ch2hubaddresserror \
	ch2sendstarted ch2rcvcounterr
    global fecleds


    if { ![winfo exists .fec] } { 
	return
    }

    set hextemp [pcifecreadword 12]
    set tt0 [padhexword $hextemp]
    set tt2 [wordtobits $tt0]
    if {$verbose3 > 0} { puts "UpdateFECleds: $tt2" }
# now extract the status bits for the GUI status indicators
    set ch1receivetimeout [string index $tt2 23]
    set ch1receivecomplete [string index $tt2 22]
    set ch1hubaddresserror [string index $tt2 21]
    set ch1sendstarted [string index $tt2 20]
    set ch1rcvcounterr [string index $tt2 19]
    set ch2receivetimeout [string index $tt2 7]
    set ch2receivecomplete [string index $tt2 6]
    set ch2hubaddresserror [string index $tt2 5]
    set ch2sendstarted [string index $tt2 4]
    set ch2rcvcounterr [string index $tt2 3]

    if {($ch1receivetimeout == "1") || ($ch1rcvcounterr == "1")} {
	$fecleds.r4.led0 config  -image redledonimage
    } else {
	$fecleds.r4.led0 config -image redledoffimage
    }
    if {$ch1receivecomplete == "1"} {
	$fecleds.r2.led0 config  -image greenledonimage
    } else {
	$fecleds.r2.led0 config -image greenledoffimage
    }
    if {$ch1hubaddresserror == "1"} {
	$fecleds.r3.led0 config  -image yellowledonimage
    } else {
	$fecleds.r3.led0 config -image yellowledoffimage
    }
    if {$ch1sendstarted == "1"} {
	$fecleds.r1.led0 config  -image greenledonimage
    } else {
	$fecleds.r1.led0 config -image greenledoffimage
    }

# update the indicators on the main ROC window
    if {($ch1receivecomplete == "1") || ($ch1sendstarted == "1")} {
	.win0.fr2.g0.ch1.led0 config -image greenonimg
    } elseif {($ch1receivetimeout == "1") || ($ch1hubaddresserror == "1") || ($ch1rcvcounterr == "1")} {
	.win0.fr2.g0.ch1.led0 config -image redonimg
    } else {
	.win0.fr2.g0.ch1.led0 config -image redoffimg
    }
	
    

    if {($ch2receivetimeout == "1") || ($ch2rcvcounterr == "1")} {
	$fecleds.r4.led1 config  -image redledonimage
    } else {
	$fecleds.r4.led1 config -image redledoffimage
    }
    if {$ch2receivecomplete == "1"} {
	$fecleds.r2.led1 config  -image greenledonimage
    } else {
	$fecleds.r2.led1 config -image greenledoffimage
    }
    if {$ch2hubaddresserror == "1"} {
	$fecleds.r3.led1 config  -image yellowledonimage
    } else {
	$fecleds.r3.led1 config -image yellowledoffimage
    }
    if {$ch2sendstarted == "1"} {
	$fecleds.r1.led1 config  -image greenledonimage
    } else {
	$fecleds.r1.led1 config -image greenledoffimage
    }
# update the indicators on the main ROC window
    if {($ch2receivecomplete == "1") || ($ch2sendstarted == "1")} {
	.win0.fr2.g0.ch2.led0 config -image greenonimg
    } elseif {($ch2receivetimeout == "1") || ($ch2hubaddresserror == "1") || ($ch2rcvcounterr == "1")} {
	.win0.fr2.g0.ch2.led0 config -image redonimg
    } else {
	.win0.fr2.g0.ch2.led0 config -image redoffimg
    }

}

set edserrors 0
set edstotal 0

proc edstest { } {
    global edserrors
    global edstotal

    # read status word
    set stat1 [pcifecreadword 12]
    # read status word2
    set stat2 [pcifecreadword 8]
    # reset all
    pcifecwriteword 12 8
    # read 4 words from wr buffer
    set wr1 [pcifecreadword 0]
    set wr2 [pcifecreadword 0]
    set wr3 [pcifecreadword 0]
    set wr4 [pcifecreadword 0]
    # read 4 words from rd buffer
    set rd1 [pcifecreadword 4]
    set rd2 [pcifecreadword 4]
    set rd3 [pcifecreadword 4]
    set rd4 [pcifecreadword 4]

    set wr1 [padhexword $wr1]
    set wr2 [padhexword $wr2]
    set rd1 [padhexword $rd1]

    set bwr1 [wordtobits $wr1]
    set bwr2 [wordtobits $wr2]
    set brd1 [wordtobits $rd1]

    set wrbyte0 [string range $bwr1 24 31]
    set wrbyte2 [string range $bwr1 8 15]
    set wrbyte3 [string range $bwr1 0 7]
    set wrbyte4 [string range $bwr2 24 31]
    set rdbyte0 [string range $brd1 24 31]
    set rdbyte1 [string range $brd1 16 23]
    set rdbyte2 [string range $brd1 8 15]
    set rdbyte3 [string range $brd1 0 7]

    set merrors 0
    set mystatus "OK       "
    if {$stat1 != 400240} {
	set mystatus "NORCVCMPL"
    }
    if {[string compare $wrbyte0 $rdbyte3] != 0} {
	incr merrors
    }
    if {[string compare $wrbyte2 $rdbyte0] != 0} {
	incr merrors
    }
    if {[string compare $wrbyte3 $rdbyte1] != 0} {
	incr merrors
    }
    if {[string compare $wrbyte4 $rdbyte2] != 0} {
	incr merrors
    }
    if {$merrors > 0} {
	set mystatus "MISMATCH "
    }
	
    if {($stat1 != "400240") || ( $merrors > 0)} {
	set wr3 [pcifecreadword 0]
	set wr3 [padhexword $wr3]
	set wr4 [pcifecreadword 0]
	set wr4 [padhexword $wr4]
	set rd3 [pcifecreadword 4]
	set rd3 [padhexword $rd3]
	set rd4 [pcifecreadword 4]
	set rd4 [padhexword $rd4]
#	puts "STATUS $mystatus stat1 stat2 wr1234 rd1234: $stat1 $stat2 $wr1 $wr2 $wr3 $wr4 $rd1 $rd2 $rd3 $rd4 $wrbyte0 $wrbyte2 $wrbyte3 $wrbyte4 $rdbyte3 $rdbyte0 $rdbyte1 $rdbyte2"

	set strwr1 [revprinthexword $wr1]
	set strwr2 [revprinthexword $wr2]
	set strwr3 [revprinthexword $wr3]
	set strwr4 [revprinthexword $wr4]
	set strrd1 [revprinthexword $rd1]
	set strrd2 [revprinthexword $rd2]
	set strrd3 [revprinthexword $rd3]
	set strrd4 [revprinthexword $rd4]
	set stat1 [padhexword $stat1]
	set stat2 [padhexword $stat2]
	puts "\n$mystatus"
	puts "$stat1 $stat2"
	puts " "
	puts "$strwr1 $strwr2 $strwr3 $strwr4"
	puts "      $strrd1 $strrd2 $strrd3 $strrd4"
	puts " "
	puts "$wrbyte0 $wrbyte2 $wrbyte3 $wrbyte4"
	puts "$rdbyte3 $rdbyte0 $rdbyte1 $rdbyte2"
	incr edserrors

    }

    # reset all
    pcifecwriteword 12 8
    for {set k 0} {$k < 4} {incr k} {
	pcifecwriteword 4 0
    }
    # reset all
    pcifecwriteword 12 8
    update 
    after 150
    update
    
    incr edstotal
    puts "$edserrors fail  / $edstotal total"
}

proc revprinthexword { myword } {
    set hword [padhexword $myword]
    set b0 [string range $hword 6 7]
    set b1 [string range $hword 4 5]
    set b2 [string range $hword 2 3]
    set b3 [string range $hword 0 1]
    set myret "$b0 $b1 $b2 $b3"
}

proc padhexword { myword } {
    set i [string length $myword]
    set paddedstring $myword
    if {$i < 8} {
	for {set j $i} {$j < 8} {incr j} {
	    set paddedstring "0$paddedstring"
	}
    }
    return $paddedstring
}

set i 0
while { $i < 10 } {
    set dataentry($i) "0$i"
#    set dataentry($i) "00"
    incr i
}

set num 01
resetparitybyte

#--- end debug lowest level usb tx commands

####################pack .wintbm2.f8 .wintbm2.f10 -side left

##set textwindow .wintbm2.f8.text

#########################.wintbm2.f8.b1 config -activeforeground "#0000FF" -fg "#0000FF" -bg "#0000FF"


#image create photo ledonimage -format GIF -file redled.gif
#image create photo ledoffimage -format GIF -file redledoff.gif

# setup a binding to erase read window on right mouse click
#bind .wintbm2 <ButtonPress-3> { .wintbm2.f9.b1 invoke }

wm withdraw .cosmo-startup
wm deiconify .cosmo-startup

cosmo-startup_update "Initialization completing..."

pack forget .win0
pack forget .wintbm2
pack .win0
after 200
update

cosmo-startup_update "Load Complete."
update

# Initial state of DAQ is on and sent within same byte as speed to tbm
.win0.fr2.g6.box1.f4.right.b2 configure -relief raised \
    -fg black -activeforeground black
.win0.fr2.g6.box1.f4.right.b1 configure -relief sunken \
    -fg darkgreen -activeforeground darkgreen
#


after 1000 cosmo-startup_remove

if {$verbose3=="1"} {
    puts "DONE INIT."
}

# Add keyboard shortcuts

bind . <KeyPress-d><e><b><u><g><KeyPress-1> { 
    set verbose3 1
    bell
    console show
    puts "\nSetting debug 1.\n"
}
bind . <KeyPress-d><e><b><u><g><KeyPress-2> {
    set verbose3 1
    set verbose2 1
    bell
    console show
    puts "\nSetting debug 2 <Very verbose).\n"
}
bind . <KeyPress-d><e><b><u><g><KeyPress-3> { 
    set verbose3 1
    set verbose2 1
    set verbose1 1
    bell
    console show
    puts "\nSetting debug 3 (Massively verbose).\n"
}
bind . <KeyPress-d><e><b><u><g><KeyPress-4> { 
    set verbose4 1
    bell
    console show
    puts "\nSetting debug 4 (USB I/O).\n"
}
bind . <KeyPress-n><o><d><e><b><u><g> { 
    set verbose3 0
    set verbose2 0
    set verbose1 0
    set verbose4 0
    bell
    console hide
    puts "\nTurned OFF debug output.\n"
}


puts "DONE startup."


#puts "TEMP debug commands"


set mfec 5

#-----READ IN ANY STARTUP PARAM CHANGES AS TCL
set modfilename "/usr/local/cosmo/bin/cosmoparams.tcl"
set modfileexists [file exists $modfilename]
puts "MODFILEexists: $modfileexists"
if {$modfileexists > 0} {
    puts "COSMO PARAM MOD FILE FOUND."
    source $modfilename
}
#-----



setvmeslotandssid


###pcifecwriteword 12 e
#pcifecwriteword 0 1
#pcifecwriteword 0 2
#pcifecwriteword 0 4
#pcifecwriteword 0 1f1f
###############4-26-06 pcifecwriteword 772 00001000
###set ttemp [pcifecreadword 768]
###puts "$ttemp"
###set ttemp [pcifecreadword 772]
###puts "$ttemp"


# a test for searching for vme fec boards 
for {set vslot 11111} {$vslot < 22} {incr vslot} {
puts "unreach"
    # set the 
    if {$usevme == 1} {
	vmefecsetaddresses $vslot 8
    }
    set toriginal [pcifecreadword 772]
    puts "FEC configreg for slot $vslot: $toriginal"
    # set ssid register to tracker on fec board
##    pcifecwriteword 772 00001000
    # read back ssid from trig register
    set ttemp [pcifecreadword 768]
    set ttemp [padhexword $ttemp]
    set ttemp [string index $ttemp 4]
    # set ssid register to pixels on fec board
##    pcifecwriteword 772 00004000
    # read back ssid from trig register
    set ttemp2 [pcifecreadword 768]
    set ttemp2 [padhexword $ttemp2]
    set ttemp2 [string index $ttemp2 4]
    # set back original value
##    pcifecwriteword 772 $toriginal
    # read back original
    set ttemp3 [pcifecreadword 772]
    if {$toriginal != $ttemp3} {
	puts "Modified CONFIG0 during probing.  Device exists at target."
    }
    if {($ttemp == "1") && ($ttemp2 == "4")} {
	puts "SETTING SSID to PIXELS for card in slot $vslot"
	# set ssid register to pixels on fec board
#	pcifecwriteword 772 00004000
# mask just the bits we need set to 100 (pixels)
    set ttemp5 [pcifecreadword 772]
    set tt0 [padhexword $ttemp5]
    set tt1 [wordtobits $tt0]
    set tt1 [bitwordset $tt1 14 1]
    set tt1 [bitwordset $tt1 13 0]
    set tt2 [bitwordset $tt1 12 0]
    set tt3 [bitstoword $tt2]
    pcifecwriteword 772 $tt3

	puts "SSID write test succeeds.  FEC found in VME SLOT $vslot."
	puts "\nNow looking for mFECs installed in VME FEC in Slot $vslot"
	for {set myfec 1} {$myfec < 9} {incr myfec} {
	    vmefecsetaddresses $vslot $myfec
	    # reset the mfec WR pointer
	    pcifecwriteword 12 e
	    set temporiginal [pcifecreadword 0]
	    pcifecwriteword 12 e
	    pcifecwriteword 0 0
	    pcifecwriteword 12 e
	    set ttemp1 [pcifecreadword 0]
	    set ttemp1 [padhexword $ttemp1]
	    set ttemp1 [string index $ttemp1 7]
	    pcifecwriteword 12 e
	    pcifecwriteword 0 1
	    pcifecwriteword 12 e
	    set ttemp2 [pcifecreadword 0]
	    set ttemp2 [padhexword $ttemp2]
	    set ttemp2 [string index $ttemp2 7]

	    if {($ttemp1 == "0") && ($ttemp2 == "1")} {
		puts "   Found mFEC $myfec in VME FEC in slot $vslot"
	    }
	    

	    # set back to what we found
	    pcifecwriteword 12 e
	    pcifecwriteword 0 $temporiginal
	}
	    

    } else {
	puts "SSID register write fails readback."
    }
#    puts "ttemp:$ttemp  ttemp2:$ttemp2  ttemp3:$ttemp3"
}

# searching for active vme slot leaves last slot set incorrectly
# switch to current setting as default


#set readout speed to 20MHz initially
#set textreadoutspeed 20MHz
#dochangereadoutspeed 1 
# set mode to sync initially
#set tbm2mode "Sync"
#.win0.fr2.g6.box1.f6.right.b3 configure -relief raised \
    -fg black -activeforeground black
#.win0.fr2.g6.box1.f6.right.b2 configure -relief raised \
    -fg black -activeforeground black
#.win0.fr2.g6.box1.f6.right.b1 configure -relief sunken \
    -fg darkgreen -activeforeground darkgreen
#tbm2_write_f2 1
#


# The end, thanks.

